package br.com.centralit.api.dao;

import java.util.Collection;
import java.util.List;

import br.com.centralit.api.model.ClassificacaoMaterial;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAO;

public interface ClassificacaoMaterialDao extends CitGenericDAO {

	/**
	 * Método responsável por listar Grupos de acordo com o codigo eviado
	 * 
	 * @author wilker.machado
	 * @param descricao
	 * @param codigo
	 * 
	 * @return <code>List<ClassificacaoMaterial></code>
	 */
	List<ClassificacaoMaterial> listarClassificacaoMaterialPorDescricaoECodigo(String descricao, Long codigo);

	/**
	 * 
	 * Método responsável por listar Classificação Material codigo 1
	 * 
	 * @author wilker.machado
	 * @param idOrganizacao 
	 * 
	 * @return <code>List<ClassificacaoMaterial></code>
	 */
	List<ClassificacaoMaterial> listarGruposClassificacaoMaterial(Long idOrganizacao);

	/**
	 * Método responsável por listar filhos de no selecionado
	 * 
	 * @author wilker.machado
	 * 
	 * @param id
	 * @return <code>List<ClassificacaoMaterial></code>
	 */
	List<ClassificacaoMaterial> listarFilhos(Long id);

	/**
	 * Método responsável por listar Arvore de classificação material de acordo com descricao
	 * 
	 * @author wilker.machado
	 * 
	 * @param descricao
	 * @param idOrganizacao 
	 * @return <code>List<ClassificacaoMaterial></code>
	 */
	List<ClassificacaoMaterial> listarClassificacaoMaterialTree(String descricao, Long idOrganizacao);

	/**
	 * 
	 * Método responsável por listar nos filhos de determinado nó
	 * 
	 * @author wilker.machado
	 * 
	 * @param id
	 */
	List<Long> buscarIdsFilhosDeClassificacaoNo(Long id);

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por listar através da descrição e id do parent
	 * 
	 * @author rogerio.costa
	 * 
	 * @param id
	 * @param descricao
	 * @return Collection<ClassificacaoMaterial>
	 */
	Collection<ClassificacaoMaterial> listarFilhosPorDescricaoIdParent(Long id, String descricao);

	/**
	 *
	 * Método responsável por verificar se já existe codigo de uma nova classificação material
	 *
	 * @author wilker.machado
	 *
	 * @param entity
	 * @return <code>Boolean</code>
	 */
	boolean existeCodigo(ClassificacaoMaterial entity);
}
