package br.com.centralit.api.dao;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import br.com.centralit.api.model.ContaContabil;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAO;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Organizacao;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 29/12/2014 - 17:19:54
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
public interface ContaContabilDao extends CitGenericDAO {

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por pesquisar conta contabil por código ou descrição a partir da chave
	 * 
	 * @author rogerio.costa
	 * 
	 * @param chave
	 * @return
	 */
	Collection<ContaContabil> findContaContabil(String chave);

	/**
	 * 
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por encontrar todas as contas contabeis de um determinado orgão.
	 * 
	 * @author geovane.filho
	 * 
	 * @param organizacao
	 *            <code>EstruturaOrganizacional</code> a se buscar suas contas contabeis.
	 * 
	 * @return <code>Collection<ContaContabil></code> contendo todas as contas contabeis do orgão.
	 */
	Collection<ContaContabil> findAllByOrganizacao(Organizacao organizacao);

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por lista conta contábil por código e órgão do usuário logado
	 * 
	 * @author rogerio.cassimiro
	 * 
	 * @param codigo
	 * @param idOrganizacao
	 * @return Collection<ContaContabil>
	 */
	Collection<ContaContabil> findContaContabilPorOrganizacao(String codigo, Long idOrganizacao);

	/**
	 * 
	 * Método responsável por
	 * 
	 * @author wilker.machado
	 * 
	 * @param contaContabil
	 * @param organizacaoId 
	 * @return
	 */
	boolean existeCodigo(ContaContabil contaContabil, Long organizacaoId);

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por verificar se existe uma contaContabil vinculado a estrutura
	 * 
	 * @author rogerio.costa
	 * 
	 * @param localizacaoId
	 * 
	 * @return boolean
	 */
	boolean existeContaContabilVinculadaAEstrutura(Long idEstrutura);

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter conta contábil por código, órgão, domínio
	 * 
	 * @author rogerio.cassimiro
	 * 
	 * @param codigo
	 * @param idOrganizacao
	 * @param idDominio
	 * @return Collection<ContaContabil>
	 */
	Collection<ContaContabil> findContaContabilPorOrganizacaoEDominio(String codigo, Long idOrganizacao, Long idDominio);

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter conta contábil por código, órgão, domínio que não esteja em uso em algum material
	 * 
	 * @author rogerio.cassimiro
	 * 
	 * @param codigo
	 * @param idOrganizacao
	 * @param idDominio
	 * @return Collection<ContaContabil>
	 */
	Collection<ContaContabil> findContaContabilPorOrganizacaoEDominioSemUsoMaterial(String codigo, Long idOrganizacao, Long idDominio, Set<Long> idsEmUso);
	
	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por obter conta contábil por código, órgão, domínio que não esteja em uso em algum almoxarifado
	 * 
	 * @author rogerio.cassimiro
	 * 
	 * @param codigo
	 * @param idOrganizacao
	 * @param idDominio
	 * @param idContaEmUso
	 * @return Collection<ContaContabil>
	 */
	Collection<ContaContabil> findContaContabilPorOrganizacaoEDominioSemUsoAlmoxarifado(String codigo, Long idOrganizacao, Long idDominio, Long idContaEmUso);
	
	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por listar a entidade <code>ContaContabil</code> através do orgão, tipoMaterial e que não esteja em uso pela estrutura.
	 * 
	 * @author rogerio.costa
	 * 
	 * @param str
	 * @param idOrganizacao
	 * @param codigoTipoMaterial
	 * @param idContaContabilEmUso
	 * @return Collection<ContaContabil>
	 */
	Collection<ContaContabil> findContaContabilPorOrganizacaoTipoMaterial(String str, Long idOrganizacao, Long codigoTipoMaterial, Long idContaContabilEmUso);

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por recuperar o valor atual total de uma determinada conta
	 * 
	 * @author geovane.filho
	 * 
	 * @param contaId
	 * @param dataReferenciaVigente data da vigencia atual
	 * @return
	 */
	public BigDecimal getValorAtual(Long contaId, Calendar dataReferenciaVigente);

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 *  Método responsável por obter a conta contábil por inventário
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param busca
	 * @param idInventario
	 * @return
	 */
	List<ContaContabil> findContaContabilPorInventario(String busca, Long idInventario);

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por retornar uma conta contabil referenciada em um determinado material
	 *
	 * @author geovane.filho
	 *
	 * @param materialId
	 * @return
	 */
	ContaContabil findByMaterial(Long materialId);

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por encontrar todas as contas contábeis de um determinado tipo de material para um orgão
	 *
	 * @author geovane.filho
	 *
	 * @param organizacao
	 * @param tipoMaterial
	 * @return
	 */
	Collection<ContaContabil> findAllByOrganizacaoAndTipoMaterial(Organizacao organizacao, Dominio tipoMaterial);
}
