package br.com.centralit.api.dao;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;
import java.util.List;

import br.com.centralit.api.model.ContaContabil;
import br.com.centralit.api.model.ContaContabilMovimento;
import br.com.centralit.api.model.Entrada;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAO;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Organizacao;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 13/01/2015 - 15:37:32
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *
 */
public interface ContaContabilMovimentoDao extends CitGenericDAO {

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por trazer a soma dos valores de todos os movimentos de uma determinada conta em um mes.
	 *
	 * @author geovane.filho
	 *
	 * @param contaContabil
	 *            Conta Contabil a se procurar a soma dos valores dos movimentos
	 * @param mesReferencia
	 *            Mês de referencia a se filtrar os movimentos
	 * @return BigDecimal contendo o valor da soma de todos os movimentos do mês.
	 */
	BigDecimal somaValoresMovimentos(ContaContabil contaContabil, Calendar mesReferencia);

	/**
	 * Método responsável por
	 *
	 * @author wilker.machado
	 *
	 * @param organizacao
	 * @param dataReferencia
	 * @return
	 */
	Boolean existeMovimentoOrganizacaoEDataReferencia(Organizacao organizacao, Calendar dataReferencia);
	
	/**
	 * Método responsável por
	 *
	 * @author geovane.filho
	 *
	 * @param organizacao
	 * @param dataRefInicial
	 * @param dataRefFinal
	 * @param tipoMaterialId
	 * @return
	 */
	Boolean existeMovimentoOrganizacaoDataReferenciaETipoMaterial(Organizacao organizacao, Calendar dataRefInicial, Calendar dataRefFinal, Long tipoMaterialId);

	/**
	 * Método responsável por
	 *
	 * @author wilker.machado
	 *
	 * @param entity
	 * @return
	 */
	List<ContaContabilMovimento> listarMovimentosEntrada(Entrada entity);

	/**
	 * Método responsável por
	 *
	 * @author wilker.machado
	 *
	 * @param idOrganizacao
	 * @return
	 */
	Collection<Long> getIdsContaContabilEmUso(Long idOrganizacao);

	/**
	 * Método responsável por buscar o movimento gerado em uma conta contábil por uma determinada classe de referencia.
	 *
	 * @author geovane.filho
	 *
	 * @param contaContabil Conta Contábil a se bucar o movimento
	 * @param classeReferencia Classe de referencia que gerou o movimento
	 * @param idClasseReferencia Id da classe de referencia que gerou o movimento
	 * @return
	 */
	ContaContabilMovimento findByContaAndClasseReferencia(ContaContabil contaContabil, Dominio classeReferencia, Long idClasseReferencia);
	
	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por buscar os movimentos que devem ser inativados baseados nos bens que os originaram
	 *
	 * @author juliana.barbosa
	 *
	 * @param idEntradaItem
	 * @param bens
	 * @return
	 */
	List<ContaContabilMovimento> buscaContasContabeisMovimentoEntradaItem(List<Long> bens);	
	
	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por buscar os movimentos baseado no dominio classe referencia e no id classe referencia
	 *
	 * @author juliana.barbosa
	 *
	 * @param idRef
	 * @param idDominioClasseRef
	 * @return
	 */
	List<ContaContabilMovimento> listarMovimentosPorClasseRefIdRef(Long idRef, Long idDominioClasseRef);

}
