package br.com.centralit.api.dao.impl;

import java.util.Collection;
import java.util.List;

import javax.persistence.Query;

import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.ClassificacaoMaterialDao;
import br.com.centralit.api.model.ClassificacaoMaterial;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;
import br.com.centralit.framework.util.UtilDataBase;

import com.googlecode.genericdao.search.Sort;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * @since 23/12/2014 - 10:15:17
 * 
 * @version 1.0.0
 * 
 * @author wilker.machado
 * 
 */
@Repository("classificacaoMaterialDao")
public class ClassificacaoMaterialDaoHibernate extends CitGenericDAOImpl implements ClassificacaoMaterialDao {

	public ClassificacaoMaterialDaoHibernate() {

		super(ClassificacaoMaterial.class);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public List<ClassificacaoMaterial> listarClassificacaoMaterialPorDescricaoECodigo(String descricao, Long codigo) {

		SearchSeven search = new SearchSeven(this.persistentClass);

		search.addFilterILike("descricao", "%" + descricao + "%");

		search.addFilterEqual("dominioTipoClassificacaoMaterial.codigo", codigo);

		search.setDistinct(Boolean.TRUE);

		search.addSort(Sort.asc("descricao"));
		
		search.setMaxResults(10);

		return this.search(search, this.persistentClass);

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public List<ClassificacaoMaterial> listarGruposClassificacaoMaterial(Long idOrganizacao) {

		SearchSeven search = new SearchSeven(this.persistentClass);

		search.addFilterEqual("organizacao.id", idOrganizacao);

		search.addFilterEqual("dominioTipoClassificacaoMaterial.codigo", 0L);

		search.addSort(Sort.asc("descricao"));

		return this.search(search, this.persistentClass);

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public List<ClassificacaoMaterial> listarFilhos(Long id) {

		SearchSeven search = new SearchSeven(this.persistentClass);

		search.addFilterEqual("classificacaoMaterialParent.id", id);

		search.addSort(Sort.asc("descricao"));

		return this.search(search, this.persistentClass);
	}

	/**
	 * Método responsável por listar através da descrição e id do parent
	 * 
	 * @author rogerio.costa
	 * 
	 * @param id
	 * @param descricao
	 * @return Collection<ClassificacaoMaterial>
	 */
	public Collection<ClassificacaoMaterial> listarFilhosPorDescricaoIdParent(Long id, String descricao) {

		SearchSeven search = new SearchSeven(this.persistentClass);

		search.addFilterEqual("classificacaoMaterialParent.id", id);

		search.addFilterILike("descricao", "%" + descricao + "%");

		search.addFilterEqual("dominioTipoClassificacaoMaterial.codigo", 2);

		search.addSort(Sort.asc("descricao"));

		return this.search(search, this.persistentClass);
	}

	/**
	 * {@inheritDoc}
	 */
	@SuppressWarnings("unchecked")
	@Override
	public List<ClassificacaoMaterial> listarClassificacaoMaterialTree(String descricao, Long idOrganizacao) {

		StringBuilder strQuery = new StringBuilder();
		Query query;

		switch (UtilDataBase.getDataBaseName(em())) {

		// ////////////////////////////////////// POSTGRESQL //////////////////////////////////////////
			case POSTGRESQL:

				strQuery.append("WITH RECURSIVE rel_tree AS (SELECT id, descricao, codigo, organizacao_id, classificacaomaterialparent_id, dominiotipomaterial_id, tipoclassificacaomaterial_id, dataBloqueio,  datacriacao, dataedicao, datainativo, autor_id, editor_id, version, inativador_id, 1 AS level, array[id] AS path_info ");
				strQuery.append("FROM classificacaomaterial ");
				strQuery.append("WHERE organizacao_id = " + idOrganizacao + " AND LOWER(descricao) LIKE LOWER('%" + descricao + "%') AND datainativo IS NULL ");
				strQuery.append("UNION ALL ");
				strQuery.append("SELECT c.id, c.descricao, c.codigo, c.organizacao_id, c.classificacaomaterialparent_id, c.dominiotipomaterial_id, c.tipoclassificacaomaterial_id, c.dataBloqueio, c.datacriacao, c.dataedicao, c.datainativo, c.autor_id, c.editor_id, c.version, c.inativador_id, p.level + 1, p.path_info||c.id ");
				strQuery.append("FROM classificacaomaterial c ");
				strQuery.append("JOIN rel_tree p ON p.classificacaomaterialparent_id = c.id ) ");
				strQuery.append("SELECT DISTINCT ON (id) id, descricao, codigo, organizacao_id, dominiotipomaterial_id, tipoclassificacaomaterial_id, level, classificacaomaterialparent_id, dataBloqueio, datacriacao, dataedicao, datainativo, autor_id, editor_id, inativador_id, version ");
				strQuery.append("FROM rel_tree WHERE organizacao_id = " + idOrganizacao + " ORDER BY id, level DESC");

				query = em().createNativeQuery(strQuery.toString(), ClassificacaoMaterial.class);

				return query.getResultList();

				// ////////////////////////////////////SQL SERVER //////////////////////////////////////////
			case SQL_SERVER:

				strQuery.append("WITH rel_tree AS (SELECT id, descricao, codigo, organizacao_id, classificacaomaterialparent_id, dominiotipomaterial_id, tipoclassificacaomaterial_id, dataBloqueio,  datacriacao, dataedicao, datainativo, autor_id, editor_id, version, inativador_id, 1 AS level ");
				strQuery.append("FROM classificacaomaterial ");
				strQuery.append("WHERE organizacao_id = " + idOrganizacao + " AND LOWER(descricao) LIKE LOWER('%" + descricao + "%') AND datainativo IS NULL ");
				strQuery.append("UNION ALL ");
				strQuery.append("SELECT c.id, c.descricao, c.codigo, c.organizacao_id, c.classificacaomaterialparent_id, c.dominiotipomaterial_id, c.tipoclassificacaomaterial_id, c.dataBloqueio, c.datacriacao, c.dataedicao, c.datainativo, c.autor_id, c.editor_id, c.version, c.inativador_id, p.level + 1 ");
				strQuery.append("FROM classificacaomaterial c ");
				strQuery.append("JOIN rel_tree p ON p.classificacaomaterialparent_id = c.id ) ");
				// strQuery.append("SELECT DISTINCT (id) id, descricao, codigo, organizacao_id, dominiotipomaterial_id, tipoclassificacaomaterial_id, level, classificacaomaterialparent_id, dataBloqueio, datacriacao, dataedicao, datainativo, autor_id, editor_id, inativador_id, version ");
				strQuery.append("SELECT DISTINCT (id) id, descricao, codigo, organizacao_id, dominiotipomaterial_id, tipoclassificacaomaterial_id, classificacaomaterialparent_id, dataBloqueio, datacriacao, dataedicao, datainativo, autor_id, editor_id, inativador_id, version ");
				// strQuery.append("FROM rel_tree WHERE organizacao_id = " + idOrganizacao + " ORDER BY id, level DESC");
				strQuery.append("FROM rel_tree WHERE organizacao_id = " + idOrganizacao + " ORDER BY id DESC");

				query = em().createNativeQuery(strQuery.toString(), ClassificacaoMaterial.class);

				return query.getResultList();

				// TODO
				/**
				 * Implementar sql nativa para os outros bancos
				 */
				// ////////////////////////////////////ORACLE//////////////////////////////////////////
			case ORACLE:
				throw new IllegalArgumentException("SQL inexistente para o SGBD Oracle");

				// ////////////////////////////////////MySQL//////////////////////////////////////////
			case MYSQL:
				throw new IllegalArgumentException("SQL inexistente para o SGBD MySQL");

				// ////////////////////////////////////DB2//////////////////////////////////////////
			case DB2:
				throw new IllegalArgumentException("SQL inexistente para o SGBD MySQL");

			default:
				throw new IllegalArgumentException("SQL inexistente para o SGBD");

		}

	}

	/**
	 * {@inheritDoc}
	 */
	@SuppressWarnings("unchecked")
	@Override
	public List<Long> buscarIdsFilhosDeClassificacaoNo(Long id) {

		StringBuilder strQuery = new StringBuilder();
		Query query;

		switch (UtilDataBase.getDataBaseName(em())) {

		// ////////////////////////////////////// POSTGRESQL //////////////////////////////////////////
			case POSTGRESQL:

				strQuery.append("WITH RECURSIVE rel_tree AS ");
				strQuery.append("(SELECT id, descricao, codigo, classificacaomaterialparent_id, 1 AS level, array[id] AS path_info ");
				strQuery.append("FROM classificacaomaterial WHERE id = " + id + " ");
				strQuery.append("UNION ALL ");
				strQuery.append("SELECT c.id, c.descricao, c.codigo, c.classificacaomaterialparent_id, p.level + 1, p.path_info||c.id ");
				strQuery.append("FROM classificacaomaterial c JOIN rel_tree p ON p.id = c.classificacaomaterialparent_id ) ");
				strQuery.append("SELECT DISTINCT ON (id) id FROM rel_tree");

				query = em().createNativeQuery(strQuery.toString());

				return query.getResultList();

				// ////////////////////////////////////SQL SERVER //////////////////////////////////////////
			case SQL_SERVER:

				strQuery.append("WITH rel_tree AS ");
				strQuery.append("(SELECT id, descricao, codigo, classificacaomaterialparent_id, 1 AS level ");
				strQuery.append("FROM classificacaomaterial WHERE id = " + id + " ");
				strQuery.append("UNION ALL ");
				strQuery.append("SELECT c.id, c.descricao, c.codigo, c.classificacaomaterialparent_id, p.level + 1 ");
				strQuery.append("FROM classificacaomaterial c JOIN rel_tree p ON p.id = c.classificacaomaterialparent_id ) ");
				strQuery.append("SELECT DISTINCT ON (id) id FROM rel_tree");

				query = em().createNativeQuery(strQuery.toString(), ClassificacaoMaterial.class);

				return query.getResultList();

				// TODO
				/**
				 * Implementar sql nativa para os outros bancos
				 */
				// ////////////////////////////////////ORACLE//////////////////////////////////////////
			case ORACLE:
				throw new IllegalArgumentException("SQL inexistente para o SGBD Oracle");

				// ////////////////////////////////////MySQL//////////////////////////////////////////
			case MYSQL:
				throw new IllegalArgumentException("SQL inexistente para o SGBD MySQL");

				// ////////////////////////////////////DB2//////////////////////////////////////////
			case DB2:
				throw new IllegalArgumentException("SQL inexistente para o SGBD MySQL");

			default:
				throw new IllegalArgumentException("SQL inexistente para o SGBD");

		}

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean existeCodigo(ClassificacaoMaterial entity) {

		SearchSeven search = new SearchSeven(this.persistentClass);

		search.addFilterEqual("organizacao.id", entity.getOrganizacao().getId());

		search.addFilterEqual("codigo", entity.getCodigo());

		return this.count(search) > 0;
	}
}
