package br.com.centralit.api.dao.impl;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import javax.persistence.Query;

import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.ContaContabilDao;
import br.com.centralit.api.model.ContaContabil;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Organizacao;
import br.com.centralit.framework.util.UtilColecao;
import br.com.centralit.framework.util.UtilObjeto;

import com.googlecode.genericdao.search.Filter;
import com.googlecode.genericdao.search.Search;
import com.googlecode.genericdao.search.Sort;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 29/12/2014 - 17:22:47
 *
 * @version 1.0.0
 *
 * @author rogerio.costa
 *
 */
@SuppressWarnings("unchecked")
@Repository("contaContabilDao")
public class ContaContabilDaoHibernate extends CitGenericDAOImpl implements ContaContabilDao {

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 *
	 * @param classs
	 */
	public ContaContabilDaoHibernate() {

		super(ContaContabil.class);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por
	 *
	 * @author rogerio.costa
	 * 
	 * @param chave
	 * @return
	 */
	public Collection<ContaContabil> findContaContabil(String chave) {

		SearchSeven search = new SearchSeven();

		search.addFilterOr(Filter.ilike("codigo", "%" + chave + "%"), Filter.ilike("descricao", "%" + chave + "%"));

		search.addFilterEmpty("dataInativo");

		search.addSort(Sort.asc("codigo"));

		search.setMaxResults(10);

		return this.search(search, ContaContabil.class);

	}

	/**
	 *
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por encontrar todas as contas contabeis de um determinado orgão.
	 *
	 * @author geovane.filho
	 * 
	 * @param organizacao
	 *            <code>EstruturaOrganizacional</code> a se buscar suas contas contabeis.
	 *
	 * @return <code>Collection<ContaContabil></code> contendo todas as contas contabeis do orgão.
	 */
	@Override
	public Collection<ContaContabil> findAllByOrganizacao(Organizacao organizacao) {

		Search search = new Search();

		search.addFilterEqual("organizacao.id", organizacao.getId());

		return this.search(search, ContaContabil.class);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por lista conta contábil por código e órgão do usuário logado
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param codigo
	 * @param idOrganizacao
	 * @return Collection<ContaContabil>
	 */
	@Override
	public Collection<ContaContabil> findContaContabilPorOrganizacao(String str, Long idOrganizacao) {

		SearchSeven search = new SearchSeven();

		search.addFilterOr(Filter.ilike("codigo", "%" + str + "%"), Filter.ilike("descricao", "%" + str + "%"));

		search.addFilterEmpty("dataInativo");

		search.addSort(Sort.asc("codigo"));

		search.addFilterEqual("organizacao.id", idOrganizacao);

		search.setMaxResults(10);

		return this.search(search, ContaContabil.class);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por listar a entidade <code>ContaContabil</code> através do orgão, tipoMaterial e que não esteja em uso pela estrutura.
	 *
	 * @author rogerio.costa
	 *
	 * @param str
	 * @param idOrganizacao
	 * @param codigoTipoMaterial
	 * @param idContaContabilEmUso
	 * @return Collection<ContaContabil>
	 */
	public Collection<ContaContabil> findContaContabilPorOrganizacaoTipoMaterial(String str, Long idOrganizacao, Long codigoTipoMaterial, Long idContaContabilEmUso) {

		SearchSeven search = new SearchSeven();

		search.addFilterOr(Filter.ilike("codigo", "%" + str + "%"), Filter.ilike("descricao", "%" + str + "%"));

		search.addFilterEmpty("dataInativo");

		search.addSort(Sort.asc("codigo"));

		search.addFilterEqual("organizacao.id", idOrganizacao);

		search.addFilterEqual("dominioTipoMaterial.codigo", codigoTipoMaterial);

		if (UtilObjeto.isReferencia(idContaContabilEmUso)) {

			search.addFilterNotEqual("id", idContaContabilEmUso);
		}

		search.setMaxResults(10);

		return this.search(search, ContaContabil.class);

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean existeCodigo(ContaContabil contaContabil, Long organizacaoId) {

		SearchSeven search = new SearchSeven();

		search.addFilterEqual("organizacao.id", organizacaoId);

		search.addFilterEqual("codigo", contaContabil.getCodigo());

		search.addFilterNotEqual("id", contaContabil.getId());

		return this.count(search) > 0;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por verificar se existe uma contaContabil vinculado a estrutura
	 *
	 * @author rogerio.costa
	 *
	 * @param localizacaoId
	 *
	 * @return boolean
	 */
	public boolean existeContaContabilVinculadaAEstrutura(Long idEstrutura) {

		SearchSeven search = new SearchSeven();

		search.addFilterIn("organizacao.id", idEstrutura);

		return this.count(search) > 0 ? Boolean.TRUE : Boolean.FALSE;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por obter conta contábil por código, órgão, domínio
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param codigo
	 * @param idOrganizacao
	 * @param idDominio
	 * @return Collection<ContaContabil>
	 */
	@Override
	public Collection<ContaContabil> findContaContabilPorOrganizacaoEDominio(String codigo, Long idOrganizacao, Long idDominio) {

		SearchSeven searchSeven = new SearchSeven();

		searchSeven.addFilterOr(Filter.ilike("codigo", "%" + codigo + "%"), Filter.ilike("descricao", "%" + codigo + "%"));

		searchSeven.addFilterEmpty("dataInativo");

		searchSeven.addSort(Sort.asc("codigo"));

		searchSeven.addFilterEqual("organizacao.id", idOrganizacao);

		searchSeven.addFilterEqual("dominioTipoMaterial.id", idDominio);

		searchSeven.setMaxResults(10);

		return this.search(searchSeven, ContaContabil.class);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por obter conta contábil por código, órgão, domínio que não esteja em uso em algum material e sem contacontabilmovimento
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param codigo
	 * @param idOrganizacao
	 * @param idDominio
	 * @return Collection<ContaContabil>
	 */
	@Override
	public Collection<ContaContabil> findContaContabilPorOrganizacaoEDominioSemUsoMaterial(String codigo, Long idOrganizacao, Long idDominio, Set<Long> idsContaContabilEmUso) {

		SearchSeven searchSeven = new SearchSeven(this.persistentClass);
		searchSeven.addFilterEqual("dominioTipoMaterial.id", idDominio);
		searchSeven.addFilterEqual("organizacao.id", idOrganizacao);
		searchSeven.addFilterOr(Filter.ilike("codigo", "%" + codigo + "%"), Filter.ilike("descricao", "%" + codigo + "%"));
		searchSeven.addSort(Sort.asc("codigo"));
		searchSeven.setMaxResults(10);

		if (!UtilColecao.isVazio(idsContaContabilEmUso)) {
			searchSeven.addFilterNotIn("id", idsContaContabilEmUso);
		}

		return this.search(searchSeven);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por obter conta contábil por código, órgão, domínio que não esteja em uso em algum almoxarifado
	 *
	 * @author geovane.filho
	 *
	 * @param codigo
	 * @param idOrganizacao
	 * @param idDominio
	 * @return Collection<ContaContabil>
	 */
	@Override
	public Collection<ContaContabil> findContaContabilPorOrganizacaoEDominioSemUsoAlmoxarifado(String codigo, Long idOrganizacao, Long idDominio, Long idContaEmUso) {

		SearchSeven searchSeven = new SearchSeven();
		searchSeven.addFilterOr(Filter.ilike("codigo", "%" + codigo + "%"), Filter.ilike("descricao", "%" + codigo + "%"));
		searchSeven.addFilterEqual("organizacao.id", idOrganizacao);
		searchSeven.addFilterEqual("dominioTipoMaterial.id", idDominio);
		if(UtilObjeto.isReferencia(idContaEmUso)){
			searchSeven.addFilterNotIn("id", idContaEmUso);
		}
		searchSeven.setMaxResults(10);

		return this.search(searchSeven, ContaContabil.class);

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public BigDecimal getValorAtual(Long contaId, Calendar dataReferenciaVigente) {

		Query querySaldoAnterior = em().createQuery("SELECT saldo.valor FROM ContaContabil as conta INNER JOIN conta.contaContabilSaldoAnterior as saldo where conta.id = :contaId");

		querySaldoAnterior.setParameter("contaId", contaId);

		BigDecimal saldoAnterior = (BigDecimal) querySaldoAnterior.getSingleResult();

		if (saldoAnterior == null) {
			saldoAnterior = BigDecimal.ZERO;
		}

		Query querySaldoMovimentos = em().createQuery("SELECT sum(mov.valor) FROM ContaContabilMovimento as mov where mov.contaContabil.id = :contaId" + " AND mov.dataReferencia BETWEEN :referenciaInicial AND :referenciaFinal AND mov.dataInativo IS NULL");

		querySaldoMovimentos.setParameter("contaId", contaId);
		
		Calendar mesReferenciaInicial = (Calendar) dataReferenciaVigente.clone();
		mesReferenciaInicial.set(Calendar.DAY_OF_MONTH, mesReferenciaInicial.getActualMinimum(Calendar.DAY_OF_MONTH));
		mesReferenciaInicial.set(Calendar.HOUR_OF_DAY, mesReferenciaInicial.getActualMinimum(Calendar.HOUR_OF_DAY));
		mesReferenciaInicial.set(Calendar.MINUTE, mesReferenciaInicial.getActualMinimum(Calendar.MINUTE));
		mesReferenciaInicial.set(Calendar.SECOND, mesReferenciaInicial.getActualMinimum(Calendar.SECOND));
		mesReferenciaInicial.set(Calendar.MILLISECOND, mesReferenciaInicial.getActualMinimum(Calendar.MILLISECOND));

		Calendar mesReferenciaFinal = (Calendar) dataReferenciaVigente.clone();
		mesReferenciaFinal.set(Calendar.DAY_OF_MONTH, mesReferenciaFinal.getActualMaximum(Calendar.DAY_OF_MONTH));
		mesReferenciaFinal.set(Calendar.HOUR_OF_DAY, mesReferenciaFinal.getActualMaximum(Calendar.HOUR_OF_DAY));
		mesReferenciaFinal.set(Calendar.MINUTE, mesReferenciaFinal.getActualMaximum(Calendar.MINUTE));
		mesReferenciaFinal.set(Calendar.SECOND, mesReferenciaFinal.getActualMaximum(Calendar.SECOND));
		mesReferenciaFinal.set(Calendar.MILLISECOND, mesReferenciaFinal.getActualMaximum(Calendar.MILLISECOND));

		querySaldoMovimentos.setParameter("referenciaInicial", mesReferenciaInicial);
		querySaldoMovimentos.setParameter("referenciaFinal", mesReferenciaFinal);

		BigDecimal saldoMovimentos = (BigDecimal) querySaldoMovimentos.getSingleResult();

		if (saldoMovimentos == null) {
			saldoMovimentos = BigDecimal.ZERO;
		}

		return saldoAnterior.add(saldoMovimentos);
	}
	
	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 *  Método responsável por obter a conta contábil por inventário
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param busca
	 * @param idInventario
	 * @return List<ContaContabil>
	 */
	@Override
	public List<ContaContabil> findContaContabilPorInventario(String busca, Long idInventario) {
		
		StringBuilder query = new StringBuilder();
		query.append("select * ")
		.append("from contacontabil cc ")
		.append("where cc.id in ")
		.append("(")
		.append("select distinct (conta.id) ")
		.append("from dadosbempatrimonial dbp ")
		.append("INNER JOIN bempatrimonial bp on bp.id = dbp.bempatrimonial_id ")
		.append("INNER JOIN material material on material.id = bp.material_id ")
		.append("INNER JOIN contacontabil conta on conta.id = material.contacontabil_id ")
		.append("INNER JOIN inventestruturaorganizacional ieo on ieo.id = dbp.inventariounidade_id ")
		.append("INNER JOIN inventario i on i.id = ieo.inventario_id ")
		.append("where  ")
		.append("i.id = :idInventario ")
		.append("and dbp.copiadedadosfinal = true ")
		.append("and dbp.datainativo is null ")
		.append(" AND (lower(conta.descricao) LIKE lower('%" + busca + "%') OR lower(conta.codigo) LIKE lower('%" + busca + "%') )" )
		.append(")");
		
		Query resultado = em().createNativeQuery(query.toString(), ContaContabil.class);
		resultado.setParameter("idInventario", idInventario);
		resultado.setMaxResults(10);
		return resultado.getResultList();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public ContaContabil findByMaterial(Long materialId) {
		StringBuilder query = new StringBuilder();
		query.append("select * ")
		.append("from contacontabil cc ")
		.append("where cc.id = ")
		.append("(")
		.append("select mat.contaContabil_id ")
		.append("from Material mat ")
		.append("where ")
		.append("mat.id = :idMaterial ")
		.append(")");
		
		Query resultado = em().createNativeQuery(query.toString(), ContaContabil.class);
		resultado.setParameter("idMaterial", materialId);
		resultado.setMaxResults(1);
		return (ContaContabil) resultado.getSingleResult();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Collection<ContaContabil> findAllByOrganizacaoAndTipoMaterial(Organizacao organizacao, Dominio tipoMaterial) {
		Search search = new Search();

		search.addFilterEqual("organizacao.id", organizacao.getId());
		search.addFilterEqual("dominioTipoMaterial.id", tipoMaterial.getId());

		return this.search(search, ContaContabil.class);
	}
}
