package br.com.centralit.api.dao.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.persistence.Query;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.ContaContabilMovimentoDao;
import br.com.centralit.api.dao.DominioDao;
import br.com.centralit.api.model.ContaContabil;
import br.com.centralit.api.model.ContaContabilMovimento;
import br.com.centralit.api.model.Entrada;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Organizacao;
import br.com.centralit.framework.util.UtilDate;

import com.googlecode.genericdao.search.Search;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 13/01/2015 - 15:40:30
 * 
 * @version 1.0.0
 * 
 * @author iago.almeida
 * 
 */
@Repository("contaContabilMovimentoDao")
public class ContaContabilMovimentoDaoHibernate extends CitGenericDAOImpl implements ContaContabilMovimentoDao {

	@Autowired
	private DominioDao dominioDao;	

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * 
	 * @param classs
	 */
	public ContaContabilMovimentoDaoHibernate() {

		super(ContaContabilMovimento.class);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por trazer a soma dos valores de todos os movimentos de uma determinada conta em um mes.
	 * 
	 * @author geovane.filho
	 * 
	 * @param contaContabil
	 *            Conta Contabil a se procurar a soma dos valores dos movimentos
	 * @param mesReferencia
	 *            Mês de referencia a se filtrar os movimentos
	 * @return BigDecimal contendo o valor da soma de todos os movimentos do mês.
	 */
	@Override
	public BigDecimal somaValoresMovimentos(ContaContabil contaContabil, Calendar mesReferencia) {

		Query query = em().createQuery("SELECT sum(valor) FROM ContaContabilMovimento as mov where mov.contaContabil = :conta"
				+ " AND mov.dataReferencia BETWEEN :referenciaInicial AND :referenciaFinal AND mov.dataInativo IS NULL");

		query.setParameter("conta", contaContabil);

		Calendar mesReferenciaInicial = (Calendar) mesReferencia.clone();
		mesReferenciaInicial.set(Calendar.DAY_OF_MONTH, mesReferenciaInicial.getActualMinimum(Calendar.DAY_OF_MONTH));
		mesReferenciaInicial.set(Calendar.HOUR_OF_DAY, mesReferenciaInicial.getActualMinimum(Calendar.HOUR_OF_DAY));
		mesReferenciaInicial.set(Calendar.MINUTE, mesReferenciaInicial.getActualMinimum(Calendar.MINUTE));
		mesReferenciaInicial.set(Calendar.SECOND, mesReferenciaInicial.getActualMinimum(Calendar.SECOND));
		mesReferenciaInicial.set(Calendar.MILLISECOND, mesReferenciaInicial.getActualMinimum(Calendar.MILLISECOND));

		Calendar mesReferenciaFinal = (Calendar) mesReferencia.clone();
		mesReferenciaFinal.set(Calendar.DAY_OF_MONTH, mesReferenciaFinal.getActualMaximum(Calendar.DAY_OF_MONTH));
		mesReferenciaFinal.set(Calendar.HOUR_OF_DAY, mesReferenciaFinal.getActualMaximum(Calendar.HOUR_OF_DAY));
		mesReferenciaFinal.set(Calendar.MINUTE, mesReferenciaFinal.getActualMaximum(Calendar.MINUTE));
		mesReferenciaFinal.set(Calendar.SECOND, mesReferenciaFinal.getActualMaximum(Calendar.SECOND));
		mesReferenciaFinal.set(Calendar.MILLISECOND, mesReferenciaFinal.getActualMaximum(Calendar.MILLISECOND));

		query.setParameter("referenciaInicial", mesReferenciaInicial);
		query.setParameter("referenciaFinal", mesReferenciaFinal);

		BigDecimal valor = (BigDecimal) query.getSingleResult();

		if (valor == null) {
			valor = BigDecimal.ZERO;
		}

		return valor;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Boolean existeMovimentoOrganizacaoEDataReferencia(Organizacao organizacao, Calendar dataReferencia) {

		SearchSeven search = new SearchSeven();

		search.addFilterEqual("contaContabil.organizacao.id", organizacao.getId());

		search.addFilterGreaterOrEqual("dataReferencia", dataReferencia);

		search.addFilterLessOrEqual("dataReferencia", UtilDate.calendarUltimoDiaDoMes(dataReferencia));

		return this.count(search) > 0;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Boolean existeMovimentoOrganizacaoDataReferenciaETipoMaterial(Organizacao organizacao, Calendar dataReferenciaInicial, Calendar dataRefFinal, Long tipoMaterial) {

		SearchSeven search = new SearchSeven();

		search.addFilterEqual("contaContabil.organizacao.id", organizacao.getId());

		search.addFilterEqual("contaContabil.dominioTipoMaterial.id", tipoMaterial);

		search.addFilterGreaterOrEqual("dataReferencia", dataReferenciaInicial);

		search.addFilterLessOrEqual("dataReferencia", dataRefFinal);

		return this.count(search) > 0;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public List<ContaContabilMovimento> listarMovimentosEntrada(Entrada entity) {

		SearchSeven search = new SearchSeven();

		search.addFilterEqual("entrada.id", entity.getId());

		return search(search);
	}

	
	public List<ContaContabilMovimento> listarMovimentosPorClasseRefIdRef(Long idRef, Long idDominioClasseRef){
		SearchSeven search = new SearchSeven();

		search.addFilterEqual("dominioClasseReferencia.id", idDominioClasseRef);
		search.addFilterEqual("idClasseReferencia", idRef);

		return search(search);				
	}	

	@Override
	public Collection<Long> getIdsContaContabilEmUso(Long idOrganizacao) {

		SearchSeven searchSevenContaMovimento = new SearchSeven();

		searchSevenContaMovimento.setResultMode(Search.RESULT_MAP);

		searchSevenContaMovimento.addField("contaContabil.id");

		searchSevenContaMovimento.addFilterNotEmpty("contaContabil");

		List<Map<String, Long>> consulta = this.search(searchSevenContaMovimento);

		List<Long> result = new ArrayList<Long>();

		for (Map<String, Long> valores : consulta) {

			result.add(valores.get("contaContabil.id"));

		}

		return result;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public ContaContabilMovimento findByContaAndClasseReferencia(ContaContabil contaContabil, Dominio classeReferencia, Long idClasseReferencia) {
		
		SearchSeven search = new SearchSeven();

		search.addFilterEqual("contaContabil.id", contaContabil.getId());
		
		search.addFilterEqual("dominioClasseReferencia.id", classeReferencia.getId());
		
		search.addFilterEqual("idClasseReferencia", idClasseReferencia);
		
		search.addSortDesc("id");

		search.setMaxResults(1);

		return this.searchUnique(search);
	}
	
	public List<ContaContabilMovimento> buscaContasContabeisMovimentoEntradaItem(List<Long> bens){
		Query query = em().createQuery("SELECT mov FROM ContaContabilMovimento mov where (bemPatrimonial.id in (:bens))");

		query.setParameter("bens", bens);
		
		return (List<ContaContabilMovimento>) query.getResultList();
	
	}
}
