package br.com.centralit.api.dao.impl;

import java.util.Calendar;

import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.ContaContabilSaldoDao;
import br.com.centralit.api.model.ContaContabilSaldo;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;
import br.com.centralit.framework.model.Organizacao;

import com.googlecode.genericdao.search.Filter;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 13/01/2015 - 15:40:30
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *
 */
@Repository("contaContabilSaldoDao")
public class ContaContabilSaldoDaoHibernate extends CitGenericDAOImpl implements ContaContabilSaldoDao {

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 *
	 * @param class
	 */
	public ContaContabilSaldoDaoHibernate() {

		super(ContaContabilSaldo.class);
	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por
	 *
	 * @author iago.almeida
	 *
	 * @param idContaContabil
	 * @param dataReferencia
	 * @return
	 */
	@Override
	public ContaContabilSaldo obterContaContabilSaldoPorContaContabilEDataReferencia(Long idContaContabil, Calendar dataReferencia){

		Calendar dataInicial = dataReferencia;
		dataInicial.set(Calendar.DAY_OF_MONTH, dataInicial.getActualMinimum(Calendar.DAY_OF_MONTH));
		dataInicial.set(Calendar.HOUR, 0);
		dataInicial.set(Calendar.MINUTE, 0);
		dataInicial.set(Calendar.SECOND, 0);

		Calendar dataFinal = dataReferencia;
		dataFinal.set(Calendar.DAY_OF_MONTH, dataFinal.getActualMaximum(Calendar.DAY_OF_MONTH));
		dataFinal.set(Calendar.HOUR, 23);
		dataFinal.set(Calendar.MINUTE, 59);
		dataFinal.set(Calendar.SECOND, 59);

		SearchSeven search = new SearchSeven();

		search.addFilterIn("contaContabil.id", idContaContabil);

		Filter betweenDataReferencia = Filter.or(Filter.greaterOrEqual("dataReferencia", dataInicial), Filter.lessOrEqual("dataReferencia", dataFinal));

		search.addFilter(betweenDataReferencia);

		return this.searchUnique(search, ContaContabilSaldo.class);

	}

	@Override
	public ContaContabilSaldo obterUltimaContaContabilSaldo(Long idContaContabil) {

		SearchSeven search = new SearchSeven();
		search.addFilterIn("contaContabil.id", idContaContabil);

		search.addSortDesc("dataReferencia");

		search.setMaxResults(1);

		return this.searchUnique(search, ContaContabilSaldo.class);

	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por verificar a existencia de saldo fechado para as contas contábeis de um orgão em uma dada referência.
	 *
	 * @author geovane.filho
	 *
	 * @param organizacao Organização a se verificar a existencia de saldo fechado para suas contas.
	 * @param dataReferencia Data de referência para filtro.
	 * 
	 * @return <code>true</code> caso exista saldo fechado para alguma conta deste orgão na data de referência passada, <code>false</code> caso contrário.
	 */
	@Override
	public Boolean existeSaldoOrganizacaoEDataReferencia(Organizacao organizacao, Calendar dataReferencia) {
		SearchSeven search = new SearchSeven();
		
		search.addFilterEqual("contaContabil.organizacao.id", organizacao.getId());
		
		Calendar dataInicial = (Calendar) dataReferencia.clone();
		dataInicial.set(Calendar.DAY_OF_MONTH, dataInicial.getActualMinimum(Calendar.DAY_OF_MONTH));
		dataInicial.set(Calendar.HOUR, 0);
		dataInicial.set(Calendar.MINUTE, 0);
		dataInicial.set(Calendar.SECOND, 0);

		Calendar dataFinal = (Calendar) dataReferencia.clone();
		dataFinal.set(Calendar.DAY_OF_MONTH, dataFinal.getActualMaximum(Calendar.DAY_OF_MONTH));
		dataFinal.set(Calendar.HOUR, 23);
		dataFinal.set(Calendar.MINUTE, 59);
		dataFinal.set(Calendar.SECOND, 59);
		
		Filter betweenDataReferencia = Filter.and(Filter.greaterOrEqual("dataReferencia", dataInicial), Filter.lessOrEqual("dataReferencia", dataFinal));

		search.addFilter(betweenDataReferencia);
		
		if (this.count(search) > 0) {
			search = new SearchSeven();
			search.addFilterEqual("contaContabil.organizacao.id", organizacao.getId());
			
			dataInicial.add(Calendar.MONTH, -1);
			dataFinal.add(Calendar.MONTH, -1);
			Filter betweenDataReferenciaAnterior = Filter.and(Filter.greaterOrEqual("dataReferencia", dataInicial), Filter.lessOrEqual("dataReferencia", dataFinal));
			search.addFilter(betweenDataReferenciaAnterior);
			return this.count(search) > 0 ? Boolean.TRUE : Boolean.FALSE;
		} else {
			return false;
		}
	}

}
