package br.com.centralit.api.dao.impl;

import java.util.ArrayList;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.stereotype.Repository;

import br.com.centralit.api.dao.MaterialDao;
import br.com.centralit.api.model.Material;
import br.com.centralit.framework.dao.arquitetura.CitGenericDAOImpl;
import br.com.centralit.framework.dao.arquitetura.SearchSeven;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;

import com.googlecode.genericdao.search.Filter;
import com.googlecode.genericdao.search.Search;
import com.googlecode.genericdao.search.Sort;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>MaterialDaoHibernate
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * @since 15/12/2014 - 08:51:24
 *
 * @version 1.0.0
 *
 * @author rogerio.costa
 *
 */
@Repository("materialDao")
public class MaterialDaoHibernate extends CitGenericDAOImpl implements MaterialDao {

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 *
	 * @param classs
	 */
	public MaterialDaoHibernate() {

		super(Material.class);
	}

	/**
	 * Método responsável por listar a entidade <code>Material</code> através do nome
	 *
	 * @author rogerio.costa
	 *
	 * @param codigoOuDescricao
	 *
	 * @return Collection<Material>
	 */
	public Collection<Material> listarMaterial(String codigoOuDescricao, Long dominioCodigoTipoMaterial, Long idOrganizacao) {

		SearchSeven search = new SearchSeven();

		search.addFilterOr(Filter.ilike("descricao", "%" + codigoOuDescricao + "%"), Filter.ilike("classificacaoMaterial.codigo", "%" + codigoOuDescricao + "%"));

		if (dominioCodigoTipoMaterial != null) {
			search.addFilterEqual("dominioTipoMaterial.chave", Dominio.TIPO_MATERIAL);
			search.addFilterEqual("dominioTipoMaterial.codigo", dominioCodigoTipoMaterial);
		}

		search.addFilterEqual("organizacao.id", idOrganizacao);

		search.addSort(Sort.asc("descricao"));

		search.setMaxResults(10);

		return this.search(search, Material.class);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por obter a Entidade<code>Material</code> através do ID da classificacaoMaterial
	 *
	 * @author rogerio.costa
	 *
	 * @return Material
	 */
	public Material obterPorIdClassificacaoMaterial(Long idClassificacaoMaterial) {

		SearchSeven search = new SearchSeven(true);

		search.addFilterEqual("classificacaoMaterial.id", idClassificacaoMaterial);

		search.setMaxResults(1);

		return searchUnique(search, Material.class);

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean existeMaterialVinculadoAClassificacao(Set<Long> idsClassificacoes, Usuario usuarioLogado) {

		SearchSeven search = new SearchSeven();

		search.addFilterEqual("organizacao.id", usuarioLogado.getOrganizacao().getId());

		search.addFilterIn("classificacaoMaterial.id", idsClassificacoes);

		return this.count(search) > 0 ? Boolean.TRUE : Boolean.FALSE;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por
	 *
	 * @author iago.almeida
	 *
	 * @param idContaContabil
	 * @return
	 */
	@Override
	public boolean existeMaterialVinculadoAContaContabil(Long idContaContabil, Usuario usuarioLogado) {

		SearchSeven search = new SearchSeven();

		search.addFilterEqual("organizacao.id", usuarioLogado.getOrganizacao().getId());

		search.addFilterIn("contaContabil.id", idContaContabil);

		return this.count(search) > 0 ? Boolean.TRUE : Boolean.FALSE;

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por setar filter de contas contábeis que estão em uso em materiais
	 *
	 * @author rogerio.cassimiro
	 *
	 */
	@Override
	public Collection<Long> getIdsContasContabeisEmMateriais(Long idOrganizacao) {

		SearchSeven searchSevenMaterial = new SearchSeven();

		searchSevenMaterial.setResultMode(Search.RESULT_MAP);

		searchSevenMaterial.addField("contaContabil.id");

		searchSevenMaterial.addFilterNotEmpty("contaContabil");

		searchSevenMaterial.addFilterEqual("organizacao.id", idOrganizacao);

		List<Map<String, Long>> consulta = this.search(searchSevenMaterial);

		List<Long> result = new ArrayList<Long>();

		for(Map<String, Long> valores : consulta){

			result.add(valores.get("contaContabil.id"));

		}

		return result;
	}

	@Override
	public List<Material> findMateriaisByOrganizacao(String descricaoOuCodigo, Long idOrganizacao) {
		
		SearchSeven search = new SearchSeven(); 

		search.addFilterOr(Filter.ilike("descricao", "%" + descricaoOuCodigo + "%"), Filter.ilike("classificacaoMaterial.codigo", "%" + descricaoOuCodigo + "%"));

		search.addFilterEqual("organizacao.id", idOrganizacao);

		search.addSort(Sort.asc("descricao"));

		search.setMaxResults(10);

		return this.search(search, Material.class);

	}

}
