package br.com.centralit.api.model;

import java.math.BigDecimal;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import br.com.centralit.api.framework.json.ViewsAdmMateriais;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Organizacao;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAuditOrganizacao;

import com.fasterxml.jackson.annotation.JsonView;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 30/12/2014 - 08:30:33
 *
 * @version 1.0.0
 *
 * @author rogerio.costa
 *
 */
@Audited
@Entity
@Table(indexes={@Index(columnList="organizacao_id"),
				@Index(columnList="dominiotipomaterial_id")})
public class ContaContabil extends PersistentObjectAuditOrganizacao {

	/** Atributo serialVersionUID. */
	private static final long serialVersionUID = 5571004288947362498L;

	/** Atributo id. */
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@JsonView({ Views.GenericView.class})
	private Long id;

	/** Atributo codigo. */
	@Column(length = 30)
	@JsonView({ Views.ContaContabilAutoCompleteView.class, Views.MaterialEditView.class, Views.ContaContabilListView.class, Views.EstruturaOrganizacionalEditView.class,
		ViewsAdmMateriais.TransferenciaContaContabilEditView.class, ViewsAdmMateriais.MaterialTransferenciaView.class, ViewsAdmMateriais.EstruturaTransferenciaView.class, Views.MaterialConsultaInfo.class })
	private String codigo;

	/** Atributo codigo. */
	@Column(length = 30)
	@JsonView({ Views.ContaContabilAutoCompleteView.class, Views.MaterialEditView.class, Views.ContaContabilListView.class })
	private String codigoSistemaOrigem;

	/** Atributo descricao. */
	@Column(length = 100)
	@JsonView({Views.ContaContabilListView.class, Views.ContaContabilMovimentoListView.class, Views.ContaContabilAutoCompleteView.class, Views.MaterialEditView.class, Views.EstruturaOrganizacionalEditView.class,
		ViewsAdmMateriais.MaterialTransferenciaView.class, ViewsAdmMateriais.EstruturaTransferenciaView.class, ViewsAdmMateriais.TransferenciaContaContabilEditView.class })
	private String descricao;

	/** Atributo dominioTipoContaContabil. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	@JsonView({ Views.ContaContabilListView.class })
	private Dominio dominioTipoContaContabil;

	/** Atributo dominioTipoMaterial. */
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ Views.ContaContabilEditView.class })
	private Dominio dominioTipoMaterial;

	/** Atributo taxaDepreciacao. */
	@Column(nullable = true, precision = 20, scale = 4)
	@JsonView({ Views.ContaContabilListView.class, Views.MaterialConsultaInfo.class })
	private BigDecimal taxaDepreciacaoMensal;

	/** Atributo percentualNaoDepreciavel. */
	@Column(nullable = true, precision = 20, scale = 4)
	@JsonView({ Views.ContaContabilListView.class, Views.MaterialAutoCompleteView.class, Views.MaterialConsultaInfo.class })
	private BigDecimal percentualNaoDepreciavel;

	/** Atributo vidaUtil. */
	@Column(nullable = true)
	@JsonView({ Views.ContaContabilListView.class , Views.BemPatrimonialEditView.class, Views.BemPatrimonialListView.class, Views.MaterialConsultaInfo.class})
	private Integer vidaUtil;

	/** Atributo contaContabilSaldoAnterior. */
	@ManyToOne(fetch = FetchType.LAZY, cascade = CascadeType.ALL)
	@JsonView({ Views.ContaContabilListView.class, Views.MaterialConsultaInfo.class })
	private ContaContabilSaldo contaContabilSaldoAnterior;

	/** Atributo configuracoesReferencia. */
	@NotAudited
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "contaContabil", cascade = CascadeType.ALL)
	@JsonView({ Views.ContaContabilEditView.class})
	private Collection<ContaContabilConfiguracaoReferencia> configuracoesReferencia;

	/**
	 * Retorna o valor do atributo <code>id</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getId() {

		return id;
	}


	/**
	 * Define o valor do atributo <code>id</code>.
	 *
	 * @param id
	 */
	public void setId(Long id) {

		this.id = id;
	}


	/**
	 * Retorna o valor do atributo <code>codigo</code>
	 *
	 * @return <code>String</code>
	 */
	public String getCodigo() {

		return codigo;
	}


	/**
	 * Define o valor do atributo <code>codigo</code>.
	 *
	 * @param codigo
	 */
	public void setCodigo(String codigo) {

		this.codigo = codigo;
	}


	/**
	 * Retorna o valor do atributo <code>codigoSistemaOrigem</code>
	 *
	 * @return <code>String</code>
	 */
	public String getCodigoSistemaOrigem() {

		return codigoSistemaOrigem;
	}


	/**
	 * Define o valor do atributo <code>codigoSistemaOrigem</code>.
	 *
	 * @param codigoSistemaOrigem
	 */
	public void setCodigoSistemaOrigem(String codigoSistemaOrigem) {

		this.codigoSistemaOrigem = codigoSistemaOrigem;
	}


	/**
	 * Retorna o valor do atributo <code>descricao</code>
	 *
	 * @return <code>String</code>
	 */
	public String getDescricao() {

		return descricao;
	}


	/**
	 * Define o valor do atributo <code>descricao</code>.
	 *
	 * @param descricao
	 */
	public void setDescricao(String descricao) {

		this.descricao = descricao;
	}


	/**
	 * Retorna o valor do atributo <code>dominioTipoContaContabil</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getDominioTipoContaContabil() {

		return dominioTipoContaContabil;
	}


	/**
	 * Define o valor do atributo <code>dominioTipoContaContabil</code>.
	 *
	 * @param dominioTipoContaContabil
	 */
	public void setDominioTipoContaContabil(Dominio dominioTipoContaContabil) {

		this.dominioTipoContaContabil = dominioTipoContaContabil;
	}

	/**
	 * Retorna o valor do atributo <code>taxaDepreciacaoMensal</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getTaxaDepreciacaoMensal() {

		return taxaDepreciacaoMensal;
	}


	/**
	 * Define o valor do atributo <code>taxaDepreciacaoMensal</code>.
	 *
	 * @param taxaDepreciacaoMensal
	 */
	public void setTaxaDepreciacaoMensal(BigDecimal taxaDepreciacaoMensal) {

		this.taxaDepreciacaoMensal = taxaDepreciacaoMensal;
	}


	/**
	 * Retorna o valor do atributo <code>percentualNaoDepreciavel</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getPercentualNaoDepreciavel() {

		return percentualNaoDepreciavel;
	}


	/**
	 * Define o valor do atributo <code>percentualNaoDepreciavel</code>.
	 *
	 * @param percentualNaoDepreciavel
	 */
	public void setPercentualNaoDepreciavel(BigDecimal percentualNaoDepreciavel) {

		this.percentualNaoDepreciavel = percentualNaoDepreciavel;
	}


	/**
	 * Retorna o valor do atributo <code>vidaUtil</code>
	 *
	 * @return <code>Integer</code>
	 */
	public Integer getVidaUtil() {

		return vidaUtil;
	}


	/**
	 * Define o valor do atributo <code>vidaUtil</code>.
	 *
	 * @param vidaUtil
	 */
	public void setVidaUtil(Integer vidaUtil) {

		this.vidaUtil = vidaUtil;
	}


	public ContaContabilSaldo getContaContabilSaldoAnterior() {
		return contaContabilSaldoAnterior;
	}


	public void setContaContabilSaldoAnterior(
			ContaContabilSaldo contaContabilSaldoAnterior) {
		this.contaContabilSaldoAnterior = contaContabilSaldoAnterior;
	}


	/**
	 * Retorna o valor do atributo <code>organizacao</code>
	 *
	 * @return <code>Organizacao</code>
	 */
	@JsonView({ Views.ContaContabilListView.class })
	public Organizacao getOrganizacao() {

		return organizacao;
	}



	public static long getSerialversionuid() {
		return serialVersionUID;
	}


	/**
	 * Retorna o valor do atributo <code>configuracoesReferencia</code>
	 *
	 * @return <code>Collection<ContaContabilConfiguracaoReferencia></code>
	 */
	public Collection<ContaContabilConfiguracaoReferencia> getConfiguracoesReferencia() {
		return configuracoesReferencia;
	}


	/**
	 * Define o valor do atributo <code>configuracoesReferencia</code>.
	 *
	 * @param configuracoesReferencia
	 */
	public void setConfiguracoesReferencia(Collection<ContaContabilConfiguracaoReferencia> configuracoesReferencia) {
		this.configuracoesReferencia = configuracoesReferencia;
	}

	/**
	 * Retorna o valor do atributo <code>dominioTipoMaterial</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getDominioTipoMaterial() {
		return dominioTipoMaterial;
	}


	/**
	 * Define o valor do atributo <code>dominioTipoMaterial</code>.
	 *
	 * @param dominioTipoMaterial
	 */
	public void setDominioTipoMaterial(Dominio dominioTipoMaterial) {
		this.dominioTipoMaterial = dominioTipoMaterial;
	}
}