package br.com.centralit.api.model;

import java.util.Calendar;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAudit;

import com.fasterxml.jackson.annotation.JsonView;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: Configuração de Conta Contabil na referência </b>
 * </p>
 * 
 * <p>
 * <b>Description: Configurações de uma conta contabil em uma determinada referência </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 25/02/2015 - 12:02:35
 * 
 * @version 1.0.0
 * 
 * @author geovane.filho
 * 
 */
@Entity
@Table(name="ContaContabilConfigReferencia")
public class ContaContabilConfiguracaoReferencia extends PersistentObjectAudit {

	/** Atributo serialVersionUID. */
	private static final long serialVersionUID = 8018934828655291994L;

	/** Atributo id. */
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@JsonView({ Views.GenericView.class })
	private Long id;

	/** Atributo mesReferencia. */
	@Column(name = "mesReferencia", nullable = false)
	@Temporal(TemporalType.DATE)
	@JsonView({ Views.ContaContabilEditView.class })
	private Calendar mesReferencia;

	/** Atributo apresentaRelatorioMovimentacaoBem. */
	@JsonView({ Views.ContaContabilListView.class })
	private Boolean apresentaRMB;
	
	/** Atributo apresentaRMA. */
	@JsonView({ Views.ContaContabilListView.class })
	private Boolean apresentaRMA;

	/** Atributo apresentaRelatorioInventario. */
	@JsonView({ Views.ContaContabilListView.class })
	private Boolean apresentaRelatorioInventario;
	
	/** Atributo apresentaRelatorioDepreciacao. */
	@JsonView({ Views.ContaContabilListView.class })
	private Boolean apresentaRelatorioDepreciacao;

	/** Atributo depreciacao. */
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	private ContaContabil contaContabil;

	/**
	 * 
	 * Responsável pela criação de novas instâncias desta classe.
	 */
	protected ContaContabilConfiguracaoReferencia() {

		super();
	}

	/**
	 * 
	 * Responsável pela criação de novas instâncias desta classe com valores obrigatorios
	 * 
	 * @param contaContabil
	 *            <code>ContaContabil</code> Conta contabil da configuração
	 * @param mesReferencia
	 *            Mes de referencia da configuração a se criar
	 * @param apresentaRMB
	 *            <code>true</code> se a conta deve aparecer nesse mes no RMB e <code>false</code> caso contrario.
	 * @param apresentaRelatorioInventario
	 *            <code>true</code> se a conta deve aparecer nesse mes no relatório de inventário e <code>false</code> caso contrario.
	 */
	public ContaContabilConfiguracaoReferencia( ContaContabil contaContabil, Calendar mesReferencia, Boolean apresentaRMB, Boolean apresentaRelatorioInventario ) {

		super();
		this.mesReferencia = mesReferencia;
		this.apresentaRMB = apresentaRMB;
		this.apresentaRelatorioInventario = apresentaRelatorioInventario;
		this.contaContabil = contaContabil;
	}
	
	/**
	 * 
	 * Responsável pela criação de novas instâncias desta classe com valores obrigatorios
	 * 
	 * @param contaContabil
	 *            <code>ContaContabil</code> Conta contabil da configuração
	 * @param mesReferencia
	 *            Mes de referencia da configuração a se criar
	 * @param apresentaRMB
	 *            <code>true</code> se a conta deve aparecer nesse mes no RMB e <code>false</code> caso contrario.
	 *            
	 * @param apresentaRelatorioInventario
	 *            <code>true</code> se a conta deve aparecer nesse mes no relatorio de inventário e <code>false</code> caso contrario.
	 *            
	 * @param apresentaRMA
	 *            <code>true</code> se a conta deve aparecer nesse mes no RMA e <code>false</code> caso contrario.
	 *            
	 * @param apresentaRelatorioDepreciacao
	 * 			  <code>true</code> se a conta deve aparecer nesse mes no relatório de depreciação e <code>false</code> caso contrario.
	 */
	public ContaContabilConfiguracaoReferencia(ContaContabil contaContabil, Calendar mesReferencia, Boolean apresentaRMB, Boolean apresentaRelatorioInventario, Boolean apresentaRMA,
			Boolean apresentaRelatorioDepreciacao) {

		super();
		this.mesReferencia = mesReferencia;
		this.apresentaRMB = apresentaRMB;
		this.apresentaRelatorioInventario = apresentaRelatorioInventario;
		this.apresentaRMA = apresentaRMA;
		this.apresentaRelatorioDepreciacao = apresentaRelatorioDepreciacao;
		this.contaContabil = contaContabil;
	}

	/**
	 * Retorna o valor do atributo <code>id</code>
	 * 
	 * @return <code>Long</code>
	 */
	public Long getId() {

		return id;
	}

	/**
	 * Define o valor do atributo <code>id</code>.
	 * 
	 * @param id
	 */
	public void setId(Long id) {

		this.id = id;
	}

	/**
	 * Retorna o valor do atributo <code>mesReferencia</code>
	 * 
	 * @return <code>Calendar</code>
	 */
	public Calendar getMesReferencia() {

		return mesReferencia;
	}

	/**
	 * Define o valor do atributo <code>mesReferencia</code>.
	 * 
	 * @param mesReferencia
	 */
	public void setMesReferencia(Calendar mesReferencia) {

		this.mesReferencia = mesReferencia;
	}

	/**
	 * Retorna o valor do atributo <code>contaContabil</code>
	 * 
	 * @return <code>ContaContabil</code>
	 */
	public ContaContabil getContaContabil() {

		return contaContabil;
	}

	/**
	 * Define o valor do atributo <code>contaContabil</code>.
	 * 
	 * @param contaContabil
	 */
	public void setContaContabil(ContaContabil contaContabil) {

		this.contaContabil = contaContabil;
	}

	/**
	 * Retorna o valor do atributo <code>apresentaRMB</code>
	 * 
	 * @return <code>Boolean</code>
	 */
	public Boolean getApresentaRMB() {

		return apresentaRMB;
	}

	/**
	 * Define o valor do atributo <code>apresentaRMB</code>.
	 * 
	 * @param apresentaRMB
	 */
	public void setApresentaRMB(Boolean apresentaRMB) {

		this.apresentaRMB = apresentaRMB;
	}

	/**
	 * Retorna o valor do atributo <code>apresentaRelatorioInventario</code>
	 * 
	 * @return <code>Boolean</code>
	 */
	public Boolean getApresentaRelatorioInventario() {

		return apresentaRelatorioInventario;
	}

	/**
	 * Define o valor do atributo <code>apresentaRelatorioInventario</code>.
	 * 
	 * @param apresentaRelatorioInventario
	 */
	public void setApresentaRelatorioInventario(Boolean apresentaRelatorioInventario) {

		this.apresentaRelatorioInventario = apresentaRelatorioInventario;
	}

	/**
	 * Retorna o valor do atributo <code>apresentaRMA</code>
	 * 
	 * @return <code>Boolean</code>
	 */
	public Boolean getApresentaRMA() {
		return apresentaRMA;
	}

	/**
	 * Define o valor do atributo <code>apresentaRMA</code>.
	 * 
	 * @param apresentaRMA
	 */
	public void setApresentaRMA(Boolean apresentaRMA) {
		this.apresentaRMA = apresentaRMA;
	}

	
	/**
	 * Retorna o valor do atributo <code>apresentaRelatorioDepreciacao</code>
	 *
	 * @return <code>Boolean</code>
	 */
	public Boolean getApresentaRelatorioDepreciacao() {
	
		return apresentaRelatorioDepreciacao;
	}

	
	/**
	 * Define o valor do atributo <code>apresentaRelatorioDepreciacao</code>.
	 *
	 * @param apresentaRelatorioDepreciacao 
	 */
	public void setApresentaRelatorioDepreciacao(Boolean apresentaRelatorioDepreciacao) {
	
		this.apresentaRelatorioDepreciacao = apresentaRelatorioDepreciacao;
	}

}
