package br.com.centralit.api.model;

import java.math.BigDecimal;
import java.util.Calendar;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;
import br.com.centralit.framework.json.MoneyDeserializer;
import br.com.centralit.framework.json.MoneySerializer;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAudit;

import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 05/12/2014 - 16:47:30
 *
 * @version 1.0.0
 *
 * @author ally.barra
 *
 */
@Entity
@Inheritance(strategy = InheritanceType.JOINED)
@Table(indexes={@Index(columnList="contacontabil_id"),
				@Index(columnList="dataReferencia,contacontabil_id"),
				@Index(columnList="tipomovimentocontacontabil_id"),
				@Index(columnList="dominioclassereferencia_id,idClasseReferencia")})
public class ContaContabilMovimento extends PersistentObjectAudit {

	/** Atributo serialVersionUID. */
	private static final long serialVersionUID = -4423939286490338516L;

	/** Atributo id. */
	@Id
//	@SequenceGenerator(name = "contaContabilMovimento_id_seq", sequenceName = "contaContabilMovimento_id_seq")
//	@GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "contaContabilMovimento_id_seq")
	@GeneratedValue(strategy = GenerationType.AUTO)
	@JsonView({ Views.GenericView.class })
	private Long id;

	/** Atributo dataReferencia. */
	@Column(name = "dataReferencia", nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ Views.EntradaListView.class })
	private Calendar dataReferencia;

	/** Atributo valor. */
	@Column(precision = 20, scale = 4)
	@JsonDeserialize(using = MoneyDeserializer.class)
	@JsonSerialize(using = MoneySerializer.class)
	@JsonView({ Views.ContaContabilMovimentoEditView.class })
	private BigDecimal valor;

	/** Atributo contaContabil. */
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ Views.ContaContabilMovimentoListView.class })
	private ContaContabil contaContabil;

	/** Atributo dominioTipoMovimentoContaContabil. */
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JoinColumn(name="tipomovimentocontacontabil_id")
	@JsonView({ Views.ContaContabilMovimentoListView.class })
	private Dominio dominioTipoMovimentoContaContabil;

	/** Atributo entrada. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	private Entrada entrada;

	/** Atributo baixa. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	private Baixa baixa;

	/** Atributo transferenciaConta. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	private TransferenciaContaContabil transferenciaConta;

	/** Atributo dominioClasseReferencia. */
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	private Dominio dominioClasseReferencia;

	@Column()
	private Long idClasseReferencia;
	
	public ContaContabilMovimento() {
		super();
	}

	/**
	 * 
	 * Responsável pela criação de novas instâncias desta classe.
	 * @param dataReferencia
	 * @param valor
	 * @param contaContabil
	 * @param dominioTipoMovimentoContaContabil
	 * @param dominioClasseReferencia
	 * @param idClasseReferencia
	 */
	public ContaContabilMovimento(Calendar dataReferencia, BigDecimal valor, ContaContabil contaContabil, Dominio dominioTipoMovimentoContaContabil,
			Dominio dominioClasseReferencia, Long idClasseReferencia ) {
		super();
		this.dataReferencia = dataReferencia;
		this.valor = valor;
		this.contaContabil = contaContabil;
		this.dominioTipoMovimentoContaContabil = dominioTipoMovimentoContaContabil;
		this.dominioClasseReferencia = dominioClasseReferencia;
		this.idClasseReferencia = idClasseReferencia;
	}

	/**
	 * Retorna o valor do atributo <code>id</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getId() {

		return id;
	}

	/**
	 * Define o valor do atributo <code>id</code>.
	 *
	 * @param id
	 */
	public void setId(Long id) {

		this.id = id;
	}

	/**
	 * Retorna o valor do atributo <code>dataReferencia</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataReferencia() {

		return dataReferencia;
	}

	/**
	 * Define o valor do atributo <code>dataReferencia</code>.
	 *
	 * @param dataReferencia
	 */
	public void setDataReferencia(Calendar dataReferencia) {

		this.dataReferencia = dataReferencia;
	}

	/**
	 * Retorna o valor do atributo <code>valor</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getValor() {

		return valor;
	}

	/**
	 * Define o valor do atributo <code>valor</code>.
	 *
	 * @param valor
	 */
	public void setValor(BigDecimal valor) {

		this.valor = valor;
	}

	/**
	 * Retorna o valor do atributo <code>contaContabil</code>
	 *
	 * @return <code>ContaContabil</code>
	 */
	public ContaContabil getContaContabil() {

		return contaContabil;
	}

	/**
	 * Define o valor do atributo <code>contaContabil</code>.
	 *
	 * @param contaContabil
	 */
	public void setContaContabil(ContaContabil contaContabil) {

		this.contaContabil = contaContabil;
	}

	/**
	 * Retorna o valor do atributo <code>dominioTipoMovimentoContaContabil</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getDominioTipoMovimentoContaContabil() {

		return dominioTipoMovimentoContaContabil;
	}

	/**
	 * Define o valor do atributo <code>dominioTipoMovimentoContaContabil</code>.
	 *
	 * @param dominioTipoMovimentoContaContabil
	 */
	public void setDominioTipoMovimentoContaContabil(Dominio dominioTipoMovimentoContaContabil) {

		this.dominioTipoMovimentoContaContabil = dominioTipoMovimentoContaContabil;
	}

	/**
	 * Retorna o valor do atributo <code>entrada</code>
	 *
	 * @return <code>Entrada</code>
	 */
	public Entrada getEntrada() {

		return entrada;
	}

	/**
	 * Define o valor do atributo <code>entrada</code>.
	 *
	 * @param entrada
	 */
	public void setEntrada(Entrada entrada) {

		this.entrada = entrada;
	}

	/**
	 * Retorna o valor do atributo <code>baixa</code>
	 *
	 * @return <code>Baixa</code>
	 */
	public Baixa getBaixa() {

		return baixa;
	}

	/**
	 * Define o valor do atributo <code>baixa</code>.
	 *
	 * @param baixa
	 */
	public void setBaixa(Baixa baixa) {

		this.baixa = baixa;
	}

	/**
	 * Retorna o valor do atributo <code>serialversionuid</code>
	 *
	 * @return <code>long</code>
	 */
	public static long getSerialversionuid() {

		return serialVersionUID;
	}

	/**
	 * Retorna o valor do atributo <code>transferenciaConta</code>
	 *
	 * @return <code>TransferenciaContaContabil</code>
	 */
	public TransferenciaContaContabil getTransferenciaConta() {
		return transferenciaConta;
	}

	/**
	 * Define o valor do atributo <code>transferenciaConta</code>.
	 *
	 * @param transferenciaConta
	 */
	public void setTransferenciaConta(TransferenciaContaContabil transferenciaConta) {
		this.transferenciaConta = transferenciaConta;
	}

	/**
	 * Retorna o valor do atributo <code>dominioClasseReferencia</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getDominioClasseReferencia() {
		return dominioClasseReferencia;
	}

	/**
	 * Define o valor do atributo <code>dominioClasseReferencia</code>.
	 *
	 * @param dominioClasseReferencia
	 */
	public void setDominioClasseReferencia(Dominio dominioClasseReferencia) {
		this.dominioClasseReferencia = dominioClasseReferencia;
	}

	/**
	 * Retorna o valor do atributo <code>idClasseReferencia</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getIdClasseReferencia() {
		return idClasseReferencia;
	}

	/**
	 * Define o valor do atributo <code>idClasseReferencia</code>.
	 *
	 * @param idClasseReferencia
	 */
	public void setIdClasseReferencia(Long idClasseReferencia) {
		this.idClasseReferencia = idClasseReferencia;
	}

}
