package br.com.centralit.api.model;

import java.math.BigDecimal;
import java.util.Calendar;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.Audited;

import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;
import br.com.centralit.framework.json.MoneyDeserializer;
import br.com.centralit.framework.json.MoneySerializer;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAudit;

import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 05/12/2014 - 16:47:30
 *
 * @version 1.0.0
 *
 * @author ally.barra
 *
 */
@Audited
@Entity
public class ContaContabilSaldo extends PersistentObjectAudit {

	/** Atributo serialVersionUID. */
	private static final long serialVersionUID = 1657504272879531049L;

	/** Atributo id. */
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@JsonView({ Views.GenericView.class })
	private Long id;

	/** Atributo dataReferencia. */
	@Column(name = "dataReferencia", nullable = false, updatable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	private Calendar dataReferencia;

	/** Atributo valor. */
	@Column(precision = 20, scale = 2)
	@JsonDeserialize(using = MoneyDeserializer.class)
	@JsonSerialize(using = MoneySerializer.class)
	private BigDecimal valor;

	@Column(precision = 20, scale = 4)
	private BigDecimal depreciacaoMensal;

	@Column(precision = 20, scale = 4)
	private BigDecimal depreciacaoAcumulada;

	@Column(precision = 20, scale = 4)
	private BigDecimal reavaliacaoPeriodo;

	@Column(precision = 20, scale = 4)
	private BigDecimal reavaliacaoAcumulada;

	@Column(precision = 20, scale = 4)
	private BigDecimal reducaoPeriodo;

	@Column(precision = 20, scale = 4)
	private BigDecimal reducaoAcumulada;

	/** Atributo contaContabil. */
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	public ContaContabil contaContabil;

	/**
	 *
	 * Responsável pela criação de novas instâncias desta classe.
	 */
	protected ContaContabilSaldo() {
		super();
	}

	/**
	 *
	 * Responsável pela criação de novas instâncias desta classe com seus atributos obrigatorios.
	 * @param contaContabil <code>ContaContabil</code> referente a este saldo
	 * @param dataReferencia <code>Calendar</code> contendo a data de referencia deste saldo
	 * @param valor Valor do Saldo
	 */
	public ContaContabilSaldo(ContaContabil contaContabil, Calendar dataReferencia, BigDecimal valor) {
		super();
		this.dataReferencia = dataReferencia;
		this.valor = valor;
		this.contaContabil = contaContabil;
	}


	/**
	 * Retorna o valor do atributo <code>id</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getId() {

		return id;
	}


	/**
	 * Define o valor do atributo <code>id</code>.
	 *
	 * @param id
	 */
	public void setId(Long id) {

		this.id = id;
	}


	/**
	 * Retorna o valor do atributo <code>dataReferencia</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataReferencia() {

		return dataReferencia;
	}


	/**
	 * Define o valor do atributo <code>dataReferencia</code>.
	 *
	 * @param dataReferencia
	 */
	public void setDataReferencia(Calendar dataReferencia) {

		this.dataReferencia = dataReferencia;
	}


	/**
	 * Retorna o valor do atributo <code>valor</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getValor() {

		return valor;
	}


	/**
	 * Define o valor do atributo <code>valor</code>.
	 *
	 * @param valor
	 */
	public void setValor(BigDecimal valor) {

		this.valor = valor;
	}


	/**
	 * Retorna o valor do atributo <code>contaContabil</code>
	 *
	 * @return <code>ContaContabil</code>
	 */
	public ContaContabil getContaContabil() {

		return contaContabil;
	}


	/**
	 * Retorna o valor do atributo <code>depreciacaoMensal</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getDepreciacaoMensal() {

		return depreciacaoMensal;
	}


	/**
	 * Define o valor do atributo <code>depreciacaoMensal</code>.
	 *
	 * @param depreciacaoMensal
	 */
	public void setDepreciacaoMensal(BigDecimal depreciacaoMensal) {

		this.depreciacaoMensal = depreciacaoMensal;
	}


	/**
	 * Retorna o valor do atributo <code>depreciacaoAcumulada</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getDepreciacaoAcumulada() {

		return depreciacaoAcumulada;
	}


	/**
	 * Define o valor do atributo <code>depreciacaoAcumulada</code>.
	 *
	 * @param depreciacaoAcumulada
	 */
	public void setDepreciacaoAcumulada(BigDecimal depreciacaoAcumulada) {

		this.depreciacaoAcumulada = depreciacaoAcumulada;
	}


	/**
	 * Retorna o valor do atributo <code>reavaliacaoPeriodo</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getReavaliacaoPeriodo() {

		return reavaliacaoPeriodo;
	}


	/**
	 * Define o valor do atributo <code>reavaliacaoPeriodo</code>.
	 *
	 * @param reavaliacaoPeriodo
	 */
	public void setReavaliacaoPeriodo(BigDecimal reavaliacaoPeriodo) {

		this.reavaliacaoPeriodo = reavaliacaoPeriodo;
	}


	/**
	 * Retorna o valor do atributo <code>reavaliacaoAcumulada</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getReavaliacaoAcumulada() {

		return reavaliacaoAcumulada;
	}


	/**
	 * Define o valor do atributo <code>reavaliacaoAcumulada</code>.
	 *
	 * @param reavaliacaoAcumulada
	 */
	public void setReavaliacaoAcumulada(BigDecimal reavaliacaoAcumulada) {

		this.reavaliacaoAcumulada = reavaliacaoAcumulada;
	}


	/**
	 * Retorna o valor do atributo <code>reducaoPeriodo</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getReducaoPeriodo() {

		return reducaoPeriodo;
	}


	/**
	 * Define o valor do atributo <code>reducaoPeriodo</code>.
	 *
	 * @param reducaoPeriodo
	 */
	public void setReducaoPeriodo(BigDecimal reducaoPeriodo) {

		this.reducaoPeriodo = reducaoPeriodo;
	}


	/**
	 * Retorna o valor do atributo <code>reducaoAcumulada</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getReducaoAcumulada() {

		return reducaoAcumulada;
	}


	/**
	 * Define o valor do atributo <code>reducaoAcumulada</code>.
	 *
	 * @param reducaoAcumulada
	 */
	public void setReducaoAcumulada(BigDecimal reducaoAcumulada) {

		this.reducaoAcumulada = reducaoAcumulada;
	}

	/**
	 * Define o valor do atributo <code>contaContabil</code>.
	 *
	 * @param contaContabil
	 */
	public void setContaContabil(ContaContabil contaContabil) {

		this.contaContabil = contaContabil;
	}


	/**
	 * Retorna o valor do atributo <code>serialversionuid</code>
	 *
	 * @return <code>long</code>
	 */
	public static long getSerialversionuid() {

		return serialVersionUID;
	}


}
