package br.com.centralit.api.model;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import br.com.centralit.framework.json.JsonCalendarSimpleDateDeserializer;
import br.com.centralit.framework.json.JsonCalendarSimpleDateSerializer;
import br.com.centralit.framework.json.MoneyDeserializer;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.arquitetura.PersistentObjectAuditOrganizacao;
import br.com.centralit.framework.util.UtilObjeto;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonView;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

@Audited
@Entity
@Inheritance(strategy = InheritanceType.JOINED)
@JsonIgnoreProperties({ "$checked", "$selected" })
public class Entrada extends PersistentObjectAuditOrganizacao implements Cloneable {

	private static final long serialVersionUID = 6912491341217371160L;

	/** Atributo id. */
	@Id
//	@SequenceGenerator(name = "entrada_id_seq", sequenceName = "entrada_id_seq")
//	@GeneratedValue(strategy = GenerationType.AUTO, generator = "entrada_id_seq")
	@GeneratedValue(strategy = GenerationType.AUTO)
	@JsonView({ Views.GenericView.class })
	private Long id;

	/** Atributo dataReferencia. */
	@Column(name = "dataReferencia", nullable = false, updatable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ Views.EntradaListView.class })
	private Calendar dataReferencia;

	/** Atributo dataContabil. */
	@Column(name = "dataContabil", nullable = true)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ Views.EntradaListView.class, Views.BemPatrimonialEditView.class, Views.BemPatrimonialListView.class, Views.BemPatrimonialAutoCompleteView.class, Views.GenericView.class })
	private Calendar dataContabil;

	/** Atributo dataRecebimento. */
	@Column(name = "dataRecebimento", nullable = true)
	@Temporal(TemporalType.TIMESTAMP)
	@JsonSerialize(using = JsonCalendarSimpleDateSerializer.class)
	@JsonDeserialize(using = JsonCalendarSimpleDateDeserializer.class)
	@JsonView({ Views.EntradaListView.class, Views.BemPatrimonialEditView.class  })
	private Calendar dataRecebimento;

	/** Atributo codigo. */
	@Column(length = 30)
	@GeneratedValue(strategy = GenerationType.AUTO)
	@JsonView({ Views.EntradaAutoCompleteView.class })
	private String codigo;

	/** Atributo valorTotalNota. */
	@Column(precision = 20, scale = 2)
	@JsonDeserialize(using = MoneyDeserializer.class)
	@JsonView({ Views.EntradaListView.class })
	private BigDecimal valorTotalNota;

	/** Atributo dominioTipoEntrada. */
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ Views.EntradaAutoCompleteView.class })
	private Dominio dominioTipoEntrada;

	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ Views.EntradaListView.class })
	private Dominio dominioTipoRecebimento;

	/** Atributo fornecedor. */
	@NotAudited
	@ManyToOne(fetch = FetchType.LAZY, optional = false)
	@JsonView({ Views.EntradaListView.class })
	private Parceiro fornecedor;

	/** Atributo observacoes. */
	@NotAudited
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "entrada", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ Views.EntradaEditView.class })
	private Collection<EntradaObservacao> observacoes;

	/** Atributo documentos. */
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "entrada", cascade = CascadeType.ALL, orphanRemoval = true)
	@JsonView({ Views.EntradaEditView.class })
	private Collection<EntradaDocumento> documentos;

	/** Atributo codigo. */
	@Column(length = 30)
	private String codigoAsi;
	
	/** Atributo entrada concluida. */
	@Column(name = "isentradaconcluida")
	@JsonView({ Views.EntradaListView.class })
	private Boolean isEntradaConcluida;
	
	/** Atributo entrada contabilizada. */
	@Column(name = "iscontabilizada")
	@JsonView({ Views.EntradaListView.class })
	private Boolean isContabilizada;
	
	/**
	 * Retorna o valor do atributo <code>id</code>
	 *
	 * @return <code>Long</code>
	 */
	public Long getId() {

		return id;
	}

	/**
	 * Define o valor do atributo <code>id</code>.
	 *
	 * @param id
	 */
	public void setId(Long id) {

		this.id = id;
	}

	/**
	 * Retorna o valor do atributo <code>observacoes</code>
	 *
	 * @return <code>Collection<EntradaObservacao></code>
	 */
	public Collection<EntradaObservacao> getObservacoes() {

		if (!UtilObjeto.isReferencia(observacoes)) {

			this.observacoes = new ArrayList<EntradaObservacao>();
		}

		return observacoes;
	}

	/**
	 * Define o valor do atributo <code>observacoes</code>.
	 *
	 * @param observacoes
	 */
	public void setObservacoes(Collection<EntradaObservacao> observacoes) {

		this.observacoes = observacoes;
	}

	/**
	 * Retorna o valor do atributo <code>documentos</code>
	 *
	 * @return <code>Collection<EntradaDocumento></code>
	 */
	public Collection<EntradaDocumento> getDocumentos() {

		if (!UtilObjeto.isReferencia(documentos)) {

			this.documentos = new ArrayList<EntradaDocumento>();
		}

		return documentos;
	}

	/**
	 * Define o valor do atributo <code>documentos</code>.
	 *
	 * @param documentos
	 */
	public void setDocumentos(Collection<EntradaDocumento> documentos) {

		this.documentos = documentos;
	}

	/**
	 * Retorna o valor do atributo <code>dataContabil</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataContabil() {

		return dataContabil;
	}

	/**
	 * Define o valor do atributo <code>dataContabil</code>.
	 *
	 * @param dataContabil
	 */
	public void setDataContabil(Calendar dataContabil) {

		this.dataContabil = dataContabil;
	}

	/**
	 * Retorna o valor do atributo <code>dataRecebimento</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataRecebimento() {

		return dataRecebimento;
	}

	/**
	 * Define o valor do atributo <code>dataRecebimento</code>.
	 *
	 * @param dataRecebimento
	 */
	public void setDataRecebimento(Calendar dataRecebimento) {

		this.dataRecebimento = dataRecebimento;
	}

	/**
	 * Retorna o valor do atributo <code>codigo</code>
	 *
	 * @return <code>String</code>
	 */
	public String getCodigo() {

		return codigo;
	}

	/**
	 * Define o valor do atributo <code>codigo</code>.
	 *
	 * @param codigo
	 */
	public void setCodigo(String codigo) {

		this.codigo = codigo;
	}

	/**
	 * Retorna o valor do atributo <code>valorTotalNota</code>
	 *
	 * @return <code>BigDecimal</code>
	 */
	public BigDecimal getValorTotalNota() {

		return valorTotalNota;
	}

	/**
	 * Define o valor do atributo <code>valorTotalNota</code>.
	 *
	 * @param valorTotalNota
	 */
	public void setValorTotalNota(BigDecimal valorTotalNota) {

		this.valorTotalNota = valorTotalNota;
	}

	/**
	 * Retorna o valor do atributo <code>dominioTipoEntrada</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getDominioTipoEntrada() {

		return dominioTipoEntrada;
	}

	/**
	 * Define o valor do atributo <code>dominioTipoEntrada</code>.
	 *
	 * @param dominioTipoEntrada
	 */
	public void setDominioTipoEntrada(Dominio dominioTipoEntrada) {

		this.dominioTipoEntrada = dominioTipoEntrada;
	}

	/**
	 * Retorna o valor do atributo <code>fornecedor</code>
	 *
	 * @return <code>Parceiro</code>
	 */
	public Parceiro getFornecedor() {

		return fornecedor;
	}

	/**
	 * Define o valor do atributo <code>fornecedor</code>.
	 *
	 * @param fornecedor
	 */
	public void setFornecedor(Parceiro fornecedor) {

		this.fornecedor = fornecedor;
	}

	/**
	 * Retorna o valor do atributo <code>dominioTipoRecebimento</code>
	 *
	 * @return <code>Dominio</code>
	 */
	public Dominio getDominioTipoRecebimento() {

		return dominioTipoRecebimento;
	}

	/**
	 * Define o valor do atributo <code>dominioTipoRecebimento</code>.
	 *
	 * @param dominioTipoRecebimento
	 */
	public void setDominioTipoRecebimento(Dominio dominioTipoRecebimento) {

		this.dominioTipoRecebimento = dominioTipoRecebimento;
	}

	/**
	 * Retorna o valor do atributo <code>dataReferencia</code>
	 *
	 * @return <code>Calendar</code>
	 */
	public Calendar getDataReferencia() {

		return dataReferencia;
	}

	/**
	 * Retorna o valor do atributo <code>codigoAsi</code>
	 *
	 * @return <code>String</code>
	 */
	public String getCodigoAsi() {

		return codigoAsi;
	}

	/**
	 * Define o valor do atributo <code>codigoAsi</code>.
	 *
	 * @param codigoAsi
	 */
	public void setCodigoAsi(String codigoAsi) {

		this.codigoAsi = codigoAsi;
	}

	/**
	 * Define o valor do atributo <code>dataReferencia</code>.
	 *
	 * @param dataReferencia
	 */
	public void setDataReferencia(Calendar dataReferencia) {

		this.dataReferencia = dataReferencia;
	}
	
	/**
	 * Define o valor do atributo <code>isEntradaConcluida</code>.
	 *
	 * @param isEntradaConcluida
	 */
	public void setIsEntradaConcluida(Boolean isEntradaConcluida) {
		
		this.isEntradaConcluida = isEntradaConcluida;
	}
	
	/**
	 * Retorna o valor do atributo <code>isEntradaConcluida</code>
	 *
	 * @return <code>IsEntradaConcluida</code>
	 */
	public Boolean getIsEntradaConcluida() {

		return isEntradaConcluida;
	}
	
	/**
	 * Define o valor do atributo <code>isContabilizada</code>.
	 *
	 * @param isContabilizada
	 */
	public void setIsContabilizada(Boolean isContabilizada) {
		
		this.isContabilizada = isContabilizada;
	}
	
	/**
	 * Retorna o valor do atributo <code>isContabilizada</code>
	 *
	 * @return <code>IsContabilizada</code>
	 */
	public Boolean getIsContabilizada() {
		
		return isContabilizada;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Entrada clone() {

		try {

			return (Entrada) super.clone();

		} catch (CloneNotSupportedException ex) {

			return this;
		}
	}
}
