package br.com.centralit.api.service;

import java.util.Collection;
import java.util.List;

import br.com.centralit.api.model.ClassificacaoMaterial;
import br.com.centralit.framework.service.arquitetura.GenericService;

public interface ClassificacaoMaterialService extends GenericService<ClassificacaoMaterial, Long> {

	/**
	 * Método responsável por listar Grupos de acordo com o codigo eviado. Sempre listara o codigo acima do selecionado
	 * 
	 * @author wilker.machado
	 * @param descricao
	 * @param codigo
	 * 
	 * @return <code>List<ClassificacaoMaterial></code>
	 */
	List<ClassificacaoMaterial> listarClassificacaoMaterialPorDescricaoECodigo(String descricao, Long codigo);

	/**
	 * Método responsável por salvar a entidade <code>classificacaoMaterial</code>
	 * 
	 * @author rogerio.costa
	 * 
	 * @param classificacaoMaterial
	 */
	ClassificacaoMaterial salvarClassificacaoMaterialTipoMaterial(ClassificacaoMaterial classificacaoMaterial);

	/**
	 * Método responsável por salvar a entidade <code>classificacaoMaterial</code>
	 * 
	 * @author rogerio.costa
	 * 
	 * @param classificacaoMaterial
	 */
	ClassificacaoMaterial salvarClassificacaoMaterialDetalhe(ClassificacaoMaterial classificacaoMaterial);

	/**
	 * 
	 * Método responsável por listar Grupos codigo 1 de classificação material
	 * 
	 * @author wilker.machado
	 * 
	 * @return
	 */
	List<ClassificacaoMaterial> listarGruposClassificacaoMaterial();

	/**
	 * Método responsável por listar filhos de no selecionado
	 * 
	 * @author wilker.machado
	 * 
	 * @param id
	 * @return <code>List<ClassificacaoMaterial></code>
	 */
	List<ClassificacaoMaterial> listarFilhos(Long id);

	/**
	 * Método responsável por arvore de classificação material e retornar a arvore com a arvore de acordo com a busca
	 * 
	 * @author wilker.machado
	 * 
	 * @param descricao
	 * @return <code>List<ClassificacaoMaterial></code>
	 */
	Collection<ClassificacaoMaterial> listarClassificacaoMaterialTree(String descricao);

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por listar através da descrição e id do parent
	 * 
	 * @author rogerio.costa
	 * 
	 * @param id
	 * @param descricao
	 * @return Collection<ClassificacaoMaterial>
	 */
	Collection<ClassificacaoMaterial> listarFilhosPorDescricaoIdParent(Long id, String descricao);
	
	//Metodo que chama um salvar simplificado, usado no startup listener
	ClassificacaoMaterial saveSimples(ClassificacaoMaterial classificacao);
}
