package br.com.centralit.api.service;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;
import java.util.List;

import br.com.centralit.api.model.Baixa;
import br.com.centralit.api.model.ContaContabil;
import br.com.centralit.api.model.ContaContabilMovimento;
import br.com.centralit.api.model.Entrada;
import br.com.centralit.api.model.TransferenciaContaContabil;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Organizacao;
import br.com.centralit.framework.service.arquitetura.GenericService;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 13/01/2015 - 15:30:00
 * 
 * @version 1.0.0
 * 
 * @author iago.almeida
 * 
 */
public interface ContaContabilMovimentoService extends GenericService<ContaContabilMovimento, Long> {

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por trazer a soma dos valores de todos os movimentos de uma determinada conta em um mes.
	 * 
	 * @author geovane.filho
	 * 
	 * @param contaContabil
	 *            Conta Contabil a se procurar a soma dos valores dos movimentos
	 * @param mesReferencia
	 *            Mês de referencia a se filtrar os movimentos
	 * @return BigDecimal contendo o valor da soma de todos os movimentos do mês.
	 */
	BigDecimal somaValoresMovimentos(ContaContabil contaContabil, Calendar mesReferencia);

	/**
	 * 
	 * Método responsável por
	 * 
	 * @author wilker.machado
	 * 
	 * @param organizacao
	 * @param calendarDaData
	 * @return
	 */
	Boolean existeMovimentoOrganizacaoEDataReferencia(Organizacao organizacao, Calendar dataReferencia);

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por criar e salvar um movimento de baixa em uma conta
	 * 
	 * @author geovane.filho
	 * 
	 * @param conta ContaContabil a se criar um movimento de baixa
	 * @param valor Valor do movimento
	 * @param baixa <code>Baixa</code> referente ao movimento de baixa;
	 * @return
	 */
	public ContaContabilMovimento salvaMovimentoBaixa(ContaContabil conta, BigDecimal valor, Baixa baixa);

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por criar e salvar um movimento de entrada em uma conta
	 * 
	 * @author geovane.filho
	 * 
	 * @param conta ContaContabil a se criar um movimento de entrada
	 * @param valor Valor do movimento
	 * @param entrada <code>Entrada</code> referente ao movimento de entrada;
	 * @return
	 */
	public ContaContabilMovimento salvaMovimentoEntrada(ContaContabil conta, BigDecimal valor, Entrada entrada);
	
	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por criar e salvar um movimento de entrada em uma conta
	 * 
	 * @author geovane.filho
	 * 
	 * @param conta ContaContabil a se criar um movimento de entrada
	 * @param valor Valor do movimento
	 * @param dominioClasseReferencia Dominio referenciando a classe de referencia que realizou o movimento na conta
	 * @param idClasseReferencia Id da classe que realizou o movimento.
	 * 
	 * @return
	 */
	public ContaContabilMovimento salvaMovimentoEntrada(ContaContabil conta, BigDecimal valor, Dominio dominioClasseReferencia, Long idClasseReferencia);
	
	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por criar e salvar um movimento de saida em para os atendimentos de requisicoes de consumo
	 * 
	 * @author geovane.filho, juliana.barbosa
	 * 
	 * @param conta ContaContabil a se criar um movimento de saida
	 * @param valor Valor do movimento
	 * @param dominioClasseReferencia Dominio referenciando a classe de referencia que realizou o movimento na conta
	 * @param idClasseReferencia Id da classe que realizou o movimento.
	 * 
	 * @return
	 */
	public ContaContabilMovimento salvaMovimentoSaidaAtendimento(ContaContabil conta, BigDecimal valor, Dominio dominioClasseReferencia, Long idClasseReferencia);
	
	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por criar e salvar os movimentos de saida e entrada referente a uma transferencia entre contas
	 * 
	 * @author geovane.filho
	 * 
	 * @param contaOrigem ContaContabil a se criar um movimento de baixa
	 * @param contaDestino ContaContabil a se criar um movimento de entrada
	 * @param valor Valor do movimento
	 * @param dominioClasseReferencia Dominio referenciando a classe de referencia que realizou o movimento na conta
	 * @param idClasseReferencia Id da classe que realizou o movimento.
	 * 
	 */
	public void salvaMovimentoTransferencia(ContaContabil contaOrigem, ContaContabil contaDestino, BigDecimal valor, Dominio dominioClasseReferencia, Long idClasseReferencia);

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por criar e salvar os movimentos de baixa e entrada referente a uma transferencia entre contas
	 * 
	 * @author geovane.filho
	 * 
	 * @param contaOrigem ContaContabil a se criar um movimento de baixa
	 * @param contaDestino ContaContabil a se criar um movimento de entrada
	 * @param valor Valor do movimento
	 * @param transferencia <code>TransferenciaContaContabil</code> referente aos movimentos de transferencia;
	 * 
	 */
	public void salvaMovimentoTransferencia(ContaContabil contaOrigem, ContaContabil contaDestino, BigDecimal valor, TransferenciaContaContabil transferencia);
	
	/**
	 *
	 * Método responsável por
	 *
	 * @author wilker.machado
	 *
	 * @param organizacao
	 * @param dataRefInicial
	 * @param dataRefFinal
	 * @param tipoMaterial
	 * @return
	 */
	Boolean existeMovimentoOrganizacaoDataReferenciaTipoMaterial(Organizacao organizacao, Calendar dataRefInicial, Calendar dataRefFinal, Long tipoMaterial);

	/**
	 * Método responsável por
	 *
	 * @author wilker.machado
	 *
	 * @param entity
	 */
	void removerMovimentosEntrada(Entrada entity);

	/**
	 * Método responsável por
	 *
	 * @author wilker.machado
	 *
	 * @param idOrganizacao
	 * @return
	 */
	Collection<Long> getIdsContaContabilEmUso(Long idOrganizacao);

	/**
	 * Método responsável por buscar o movimento gerado em uma conta contábil por uma determinada classe de referencia.
	 *
	 * @author geovane.filho
	 *
	 * @param contaContabil Conta Contábil a se bucar o movimento
	 * @param classeReferencia Classe de referencia que gerou o movimento
	 * @param idClasseReferencia Id da classe de referencia que gerou o movimento
	 * @return
	 */
	ContaContabilMovimento findByContaAndClasseReferencia(ContaContabil contaContabil, Dominio classeReferencia, Long idClasseReferencia);
	
	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por remover movimentos dos bens de uma determinada entrada item
	 *
	 * @author juliana.barbosa
	 *
	 * @param entradaItemId
	 * @return
	 */
	void removeMovimentosPorEntradaItem(List<Long> bens);
	
	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por salvar o movimento das entradas do almoxarifado por item de entrada, onde o idClasseReferencia e o id da entrada item
	 *
	 * @author juliana.barbosa
	 *
	 * @param conta
	 * @param valor
	 * @param idEntradaItem
	 * @param dominioTipoEntrada
	 * @return
	 */
	public ContaContabilMovimento salvaMovimentoEntradaAlmoxarifado(ContaContabil conta, BigDecimal valor, Long idEntradaItem, Dominio dominioTipoEntrada);
	
	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por salvar os movimentos de baixa do almoxarifado onde o idClasseReferencia e o id da baixaalmoxarifadoitem
	 *
	 * @author juliana.barbosa
	 *
	 * @param conta
	 * @param valor
	 * @param dominioClasseReferencia
	 * @param idClasseReferencia
	 * @param baixa
	 * @return
	 */
	public ContaContabilMovimento salvaMovimentoSaidaBaixaAlmoxarifado(ContaContabil conta, BigDecimal valor, Dominio dominioClasseReferencia, Long idClasseReferencia, Baixa baixa);
	
	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por remover movimentos da baixa
	 *
	 * @author juliana.barbosa
	 *
	 * @param entity
	 */
	public void removerMovimentosBaixa(Long idBaixaItem);
	
	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por movimentos do atendimento por item de atendimento
	 *
	 * @author juliana.barbosa
	 *
	 * @param idAtendimentoItem
	 */
	public void removerMovimentosAtendimento(Long idAtendimentoItem);
	
	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por salvar os movimentos da devolucao
	 *
	 * @author juliana.barbosa
	 *
	 * @param conta
	 * @param valor
	 * @param dominioClasseReferencia
	 * @param idClasseReferencia
	 * @return
	 */
	public ContaContabilMovimento salvaMovimentoDevolucao(ContaContabil conta, BigDecimal valor, Dominio dominioClasseReferencia, Long idClasseReferencia);
	
	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por remover movimentos de devolucao
	 *
	 * @author juliana.barbosa
	 *
	 * @param idAtendimentoItem
	 */
	public void removerMovimentosDevolucao(Long idAtendimentoItem);
	
	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por remover movimentos de entrada de materiais (almoxarifado)
	 *
	 * @author geovane.filho
	 *
	 * @param idEntradaAlmoxarifado
	 */
	public void removerMovimentosEntradaAlmoxarifado(Long idEntradaAlmoxarifado);

}
