package br.com.centralit.api.service.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.ClassificacaoMaterialDao;
import br.com.centralit.api.model.ClassificacaoMaterial;
import br.com.centralit.api.service.ClassificacaoMaterialService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.FornecedorRamoAtividadeService;
import br.com.centralit.api.service.MaterialService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * @since 29/12/2014 - 10:46:50
 * 
 * @version 1.0.0
 * 
 * @author wilker.machado
 * 
 */
@Service("classificacaoMaterialService")
public class ClassificacaoMaterialServiceImpl extends GenericServiceImpl<ClassificacaoMaterial, Long> implements ClassificacaoMaterialService {

	private static final String VALIDACAO_CODIGO_CLASSIFICACAO_UNICO = "VALIDACAO.CODIGO_CLASSIFICACAO_UNICO";

	/** Atributo MSG_ERRO_VALIDACAO_EXCLUSAO_CLASSIFICACAO. */
	private static final String VALIDACAO_ERRO_EXCLUSAO_CLASSIFICACAO = "VALIDACAO.ERRO_EXCLUSAO_CLASSIFICACAO";

	/** Atributo VALIDACAO_ERRO_RAMO_ATIVIDADE. */
	private static final String VALIDACAO_ERRO_RAMO_ATIVIDADE = "VALIDACAO.ERRO_EXCLUSAO_CLASSIFICACAO_RAMO_ATIVIDADE";

	/** Atributo classificacaoMaterialDao. */
	private ClassificacaoMaterialDao classificacaoMaterialDao;

	/** Atributo dominioService. */
	@Autowired
	private DominioService dominioService;

	/** Atributo materialService. */
	@Autowired
	private MaterialService materialService;

	/** Atributo usuarioService. */
	@Autowired
	private UsuarioService usuarioService;

	/** Atributo fornecedorRamoAtividadeService. */
	@Autowired
	private FornecedorRamoAtividadeService fornecedorRamoAtividadeService;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * 
	 * @param classificacaoMaterialDao
	 * @param validator
	 */
	@Autowired
	public ClassificacaoMaterialServiceImpl( ClassificacaoMaterialDao classificacaoMaterialDao, @Qualifier("classificacaoMaterialValidator") Validator validator ) {

		this.dao = classificacaoMaterialDao;

		this.classificacaoMaterialDao = classificacaoMaterialDao;

		this.validator = validator;
	}

	/**
	 * Método responsável por salvar a entidade <code>classificacaoMaterial</code>
	 * 
	 * @author rogerio.costa
	 * 
	 * @param classificacaoMaterial
	 */
	public ClassificacaoMaterial salvarClassificacaoMaterialTipoMaterial(ClassificacaoMaterial classificacaoMaterial) {

		validarCodigoClassificacao(classificacaoMaterial);

		classificacaoMaterial.setDominioTipoClassificacaoMaterial(this.dominioService.findByChaveAndCodigo(Dominio.TIPO_CLASSIFICACAO_MATERIAL, Dominio.TIPO_CLASSIFICACAO_MATERIAL_MATERIAL));

		classificacaoMaterial.setDominioTipoMaterial(this.dominioService.findByChaveAndCodigo(Dominio.TIPO_MATERIAL, Dominio.TIPO_MATERIAL_PERMANENTE));

		return super.save(classificacaoMaterial);

	}

	/**
	 * Método responsável por salvar a entidade <code>classificacaoMaterial</code>
	 * 
	 * @author rogerio.costa
	 * 
	 * @param classificacaoMaterial
	 */
	public ClassificacaoMaterial salvarClassificacaoMaterialDetalhe(ClassificacaoMaterial classificacaoMaterial) {

		validarCodigoClassificacao(classificacaoMaterial);

		classificacaoMaterial.setDominioTipoClassificacaoMaterial(this.dominioService.findByChaveAndCodigo(Dominio.TIPO_CLASSIFICACAO_MATERIAL, Dominio.TIPO_CLASSIFICACAO_MATERIAL_DETALHE));

		classificacaoMaterial.setDominioTipoMaterial(this.dominioService.findByChaveAndCodigo(Dominio.TIPO_MATERIAL, Dominio.TIPO_MATERIAL_PERMANENTE));

		return super.save(classificacaoMaterial);

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public ClassificacaoMaterial save(ClassificacaoMaterial entity) {

		// Validação para nao acontecer duplicidade de codigo
		validarCodigoClassificacao(entity);

		entity.setDominioTipoMaterial(this.dominioService.findByChaveAndCodigo(Dominio.TIPO_MATERIAL, Dominio.TIPO_MATERIAL_PERMANENTE));

		return super.save(entity);
	}

	public ClassificacaoMaterial saveSimples(ClassificacaoMaterial entity) {

		return super.save(entity);
	}

	/**
	 * Método responsável por Validação para nao acontecer duplicidade de codigo
	 * 
	 * @author wilker.machado
	 * 
	 * @param entity
	 */
	private void validarCodigoClassificacao(ClassificacaoMaterial entity) {

		Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());

		entity.setOrganizacao(usuarioLogado.getOrganizacao());

		if (this.classificacaoMaterialDao.existeCodigo(entity)) {

			throw new BusinessException(VALIDACAO_CODIGO_CLASSIFICACAO_UNICO, CodigoErro.REGRA_NEGOCIO.getValue());

		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public ClassificacaoMaterial merge(ClassificacaoMaterial entity) {

		ClassificacaoMaterial classificacaoAnterior = this.find(entity.getId());

		if (!classificacaoAnterior.getCodigo().equals(entity.getCodigo())) {

			validarCodigoClassificacao(entity);

		}

		classificacaoAnterior.setCodigo(entity.getCodigo());

		classificacaoAnterior.setDescricao(entity.getDescricao());

		return super.merge(classificacaoAnterior);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public List<ClassificacaoMaterial> listarClassificacaoMaterialPorDescricaoECodigo(String descricao, Long codigo) {

		return classificacaoMaterialDao.listarClassificacaoMaterialPorDescricaoECodigo(descricao, codigo);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public List<ClassificacaoMaterial> listarGruposClassificacaoMaterial() {

		Usuario usuarioLogado = ( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() );

		return classificacaoMaterialDao.listarGruposClassificacaoMaterial(usuarioLogado.getOrganizacao().getId());

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public List<ClassificacaoMaterial> listarFilhos(Long id) {

		return classificacaoMaterialDao.listarFilhos(id);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Collection<ClassificacaoMaterial> listarClassificacaoMaterialTree(String descricao) {

		List<ClassificacaoMaterial> result = new ArrayList<ClassificacaoMaterial>();

		Usuario usuarioLogado = ( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() );

		List<ClassificacaoMaterial> resultBusca = this.classificacaoMaterialDao.listarClassificacaoMaterialTree(descricao, usuarioLogado.getOrganizacao().getId());

		for (ClassificacaoMaterial classificacaoMaterial : resultBusca) {

			if (!UtilObjeto.isReferencia(classificacaoMaterial.getClassificacaoMaterialParent())) {

				ClassificacaoMaterial classificacao = classificacaoMaterial.clone();

				this.preecherFilhosClassificacaoMaterial(classificacao, resultBusca);

				result.add(classificacao);

			}

		}

		return result;
	}

	/**
	 * Método responsável por preencher filhos de classificação material recursivamente
	 * 
	 * @author wilker.machado
	 * 
	 * @param classificacaoMaterial
	 * @param resultBusca
	 */
	private void preecherFilhosClassificacaoMaterial(ClassificacaoMaterial classificacaoMaterial, List<ClassificacaoMaterial> resultBusca) {

		// percorre todos os itens da busca ou percorrer os filhos da classificação enviado da chamada recursiva
		for (ClassificacaoMaterial classificacao : resultBusca) {

			// verifica se o item existe a referencia da classficiação pai e verifica se o pai e o no raiz da arvore
			if (UtilObjeto.isReferencia(classificacao.getClassificacaoMaterialParent()) && classificacao.getClassificacaoMaterialParent().getId().equals(classificacaoMaterial.getId())) {

				ClassificacaoMaterial temp1 = classificacao.clone();

				// chamada recursiva para preencher os filhos do item da iteração
				preecherFilhosClassificacaoMaterial(temp1, resultBusca);

				// adiciona o item como filho da classificação material
				classificacaoMaterial.getClassificacaoMaterialFilhos().add(temp1);
			}
		}
	}

	/**
	 * Método responsável por listar através da descrição e id do parent
	 * 
	 * @author rogerio.costa
	 * 
	 * @param id
	 * @param descricao
	 * @return Collection<ClassificacaoMaterial>
	 */
	public Collection<ClassificacaoMaterial> listarFilhosPorDescricaoIdParent(Long id, String descricao) {

		return this.classificacaoMaterialDao.listarFilhosPorDescricaoIdParent(id, descricao);
	}

	/**
	 * Método responsável por exluir uma classificação, porem se está já estar vinculado a um material deve se seguir a Regra RN003 do documento de caso de uso de classificação de material
	 * 
	 * @author wilker.machado
	 * 
	 * @param classificacaoMaterial
	 * 
	 * @param resultBusca
	 */
	public boolean removeById(Long id) {

		// lista de ids não pode ser duplicados por isso o uso de HashSet
		Set<Long> idsClassificacoesImpactantes = new HashSet<Long>();

		// adiciona o id do item a ser removido
		idsClassificacoesImpactantes.add(id);

		// se o item tiver filhos essa busca ira retornar o id dos filhos da classificação selecionada
		idsClassificacoesImpactantes.addAll(this.classificacaoMaterialDao.buscarIdsFilhosDeClassificacaoNo(id));

		Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());

		// verifica se existe material vinculado a classificação de material
		if (this.materialService.existeMaterialVinculadoAClassificacao(idsClassificacoesImpactantes, usuarioLogado)) {
			// nao Excluir apresentar erro de regra de negocio
			throw new BusinessException(VALIDACAO_ERRO_EXCLUSAO_CLASSIFICACAO, CodigoErro.REGRA_NEGOCIO.getValue());
		}

		// verifica se existe ramo atividade de algum fornecedor vinculado a alguma das classificacoes
		if (this.fornecedorRamoAtividadeService.existeRamoAtividadeVinculadoAClassificacao(idsClassificacoesImpactantes, usuarioLogado)) {
			// nao Excluir apresentar erro de regra de negocio
			throw new BusinessException(VALIDACAO_ERRO_RAMO_ATIVIDADE, CodigoErro.REGRA_NEGOCIO.getValue());
		}

		ClassificacaoMaterial classificacaoPai = this.find(id);

		List<ClassificacaoMaterial> filhos = this.listarFilhos(id);

		if (filhos != null && !filhos.isEmpty()) {

			this.removeList(filhos);

		}
		return this.remove(classificacaoPai);
	}
}
