package br.com.centralit.api.service.impl;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import br.com.centralit.api.dao.ContaContabilMovimentoDao;
import br.com.centralit.api.model.Baixa;
import br.com.centralit.api.model.ContaContabil;
import br.com.centralit.api.model.ContaContabilMovimento;
import br.com.centralit.api.model.DominioMaterial;
import br.com.centralit.api.model.Entrada;
import br.com.centralit.api.model.TransferenciaContaContabil;
import br.com.centralit.api.service.ContaContabilMovimentoService;
import br.com.centralit.api.service.ContaContabilSaldoService;
import br.com.centralit.api.service.ContaContabilService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.EstruturaOrganizacionalService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Organizacao;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 13/01/2015 - 15:34:59
 * 
 * @version 1.0.0
 * 
 * @author iago.almeida
 * 
 */
@Service("contaContabilMovimentoService")
public class ContaContabilMovimentoServiceImpl extends GenericServiceImpl<ContaContabilMovimento, Long> implements ContaContabilMovimentoService {

	/** Atributo contaContabilMovimentoDao. */
	@Autowired
	private ContaContabilMovimentoDao contaContabilMovimentoDao;

	/** Atributo dominioService. */
	@Autowired
	private DominioService dominioService;

	/** Atributo contaContabilService. */
	@Autowired
	private ContaContabilService contaContabilService;

	/** Atributo estruturaOrganizacionalService. */
	@Autowired
	private EstruturaOrganizacionalService estruturaOrganizacionalService;

	/** Atributo contaContabilSaldoService. */
	@Autowired
	private ContaContabilSaldoService contaContabilSaldoService;
	
	@Autowired
	private UsuarioService usuarioService;
	

	

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * 
	 * @param contaContabilMovimentoDao
	 */
	@Autowired
	public ContaContabilMovimentoServiceImpl( ContaContabilMovimentoDao contaContabilMovimentoDao ) {

		this.dao = contaContabilMovimentoDao;
		this.contaContabilMovimentoDao = contaContabilMovimentoDao;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por trazer a soma dos valores de todos os movimentos de uma determinada conta em um mes.
	 * 
	 * @author geovane.filho
	 * 
	 * @param contaContabil
	 *            Conta Contabil a se procurar a soma dos valores dos movimentos
	 * @param mesReferencia
	 *            Mês de referencia a se filtrar os movimentos
	 * @return BigDecimal contendo o valor da soma de todos os movimentos do mês.
	 */
	@Override
	public BigDecimal somaValoresMovimentos(ContaContabil contaContabil, Calendar mesReferencia) {

		return this.contaContabilMovimentoDao.somaValoresMovimentos(contaContabil, mesReferencia);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Boolean existeMovimentoOrganizacaoEDataReferencia(Organizacao organizacao, Calendar dataReferencia) {

		return this.contaContabilMovimentoDao.existeMovimentoOrganizacaoEDataReferencia(organizacao, dataReferencia);
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public void salvaMovimentoTransferencia(ContaContabil contaOrigem, ContaContabil contaDestino, BigDecimal valor, TransferenciaContaContabil transferencia) {
		ContaContabilMovimento contaTransBaixa = criaMovimentoBaixa(contaOrigem, valor, null);
		contaTransBaixa.setTransferenciaConta(transferencia);
		this.save(contaTransBaixa);
		
		ContaContabilMovimento contaTransEntrada = criaMovimentoEntrada(contaDestino, valor, null);
		contaTransEntrada.setTransferenciaConta(transferencia);
		this.save(contaTransEntrada);
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public ContaContabilMovimento salvaMovimentoEntrada(ContaContabil conta, BigDecimal valor, Entrada entrada) {
		ContaContabilMovimento movimentoEntrada = criaMovimentoEntrada(conta, valor, entrada);
		return this.save(movimentoEntrada);
	}
	
	@Override
	public ContaContabilMovimento salvaMovimentoEntradaAlmoxarifado(ContaContabil conta, BigDecimal valor, Long idEntradaItem, Dominio dominioTipoEntrada) {
		ContaContabilMovimento movimentoEntrada = criaMovimentoEntradaAlmoxarifado(conta, valor, idEntradaItem, dominioTipoEntrada);
		return this.save(movimentoEntrada);
	}	
	
	/**
	 * Método responsável por criar um movimento de entrada
	 * 
	 * @param conta
	 * @param valor
	 * @param baixa
	 * @return
	 */
	private ContaContabilMovimento criaMovimentoEntrada(ContaContabil conta, BigDecimal valor, Entrada entrada) {
		Long tipoMovimento = entrada == null ? Dominio.TIPO_MOVIMENTO_CONTA_CONTABIL_ENTRADA_EXTRAORCAMENTARIA_CODIGO : Dominio.TIPO_MOVIMENTO_CONTA_CONTABIL_ENTRADA_ORCAMENTARIA_CODIGO;
		Dominio dominioEntrada = this.dominioService.findByChaveAndCodigo(Dominio.TIPO_MOVIMENTO_CONTA_CONTABIL, tipoMovimento);
		ContaContabilMovimento movimentoEntrada;
		if (valor.compareTo(BigDecimal.ZERO) >= 0) {
			movimentoEntrada = criaContaContabilMovimento(conta, valor, dominioEntrada);
		} else {
			movimentoEntrada = criaContaContabilMovimento(conta, valor.negate(), dominioEntrada);
		}
		movimentoEntrada.setEntrada(entrada);
		return movimentoEntrada;
	}
	
	/**
	 * Método responsável por criar um movimento de entrada
	 * 
	 * @param conta
	 * @param valor
	 * @param baixa
	 * @return
	 */
	private ContaContabilMovimento criaMovimentoEntradaAlmoxarifado(ContaContabil conta, BigDecimal valor, Long idEntradaItem, Dominio dominioTipoEntrada) {
		Dominio classeReferencia = this.dominioService.findByChaveAndCodigo(DominioMaterial.CLASSE_REFERENCIA, DominioMaterial.REFERENCIA_ENTRADA_ALMOXARIFADO_ITEM);
		ContaContabilMovimento movimentoEntrada;
		if (valor.compareTo(BigDecimal.ZERO) >= 0) {
			movimentoEntrada = criaContaContabilMovimento(conta, valor, dominioTipoEntrada);
		} else {
			movimentoEntrada = criaContaContabilMovimento(conta, valor.negate(), dominioTipoEntrada);
		}
		movimentoEntrada.setDominioClasseReferencia(classeReferencia);
		movimentoEntrada.setIdClasseReferencia(idEntradaItem);
		return movimentoEntrada;
	}	
	
	/**
	 * Método responsável por criar um movimento de entrada
	 * 
	 * @param conta
	 * @param valor
	 * @param dominioClasseReferencia
	 * @param idClasseReferencia
	 * @return
	 */
	private ContaContabilMovimento criaMovimentoEntrada(ContaContabil conta, BigDecimal valor, Dominio dominioClasseReferencia, Long idClasseReferencia, Long tipoMovimento) {
		Dominio dominioEntrada = this.dominioService.findByChaveAndCodigo(Dominio.TIPO_MOVIMENTO_CONTA_CONTABIL, tipoMovimento);
		ContaContabilMovimento movimentoEntrada;
		if (valor.compareTo(BigDecimal.ZERO) >= 0) {
			movimentoEntrada = criaContaContabilMovimento(conta, valor, dominioEntrada);
		} else {
			movimentoEntrada = criaContaContabilMovimento(conta, valor.negate(), dominioEntrada);
		}
		movimentoEntrada.setDominioClasseReferencia(dominioClasseReferencia);
		movimentoEntrada.setIdClasseReferencia(idClasseReferencia);
		return movimentoEntrada;
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public ContaContabilMovimento salvaMovimentoBaixa(ContaContabil conta, BigDecimal valor, Baixa baixa) {
		ContaContabilMovimento movimentoBaixa = criaMovimentoBaixa(conta, valor, baixa);
		return this.save(movimentoBaixa);
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public ContaContabilMovimento salvaMovimentoEntrada(ContaContabil conta, BigDecimal valor, Dominio dominioClasseReferencia, Long idClasseReferencia) {
		ContaContabilMovimento movimentoEntrada = criaMovimentoEntrada(conta, valor, dominioClasseReferencia, idClasseReferencia, Dominio.TIPO_MOVIMENTO_CONTA_CONTABIL_ENTRADA_ORCAMENTARIA_CODIGO);
		return this.save(movimentoEntrada);
	}
	
	public ContaContabilMovimento salvaMovimentoDevolucao(ContaContabil conta, BigDecimal valor, Dominio dominioClasseReferencia, Long idClasseReferencia) {
		ContaContabilMovimento movimentoEntrada = criaMovimentoEntrada(conta, valor, dominioClasseReferencia, idClasseReferencia, Dominio.TIPO_MOVIMENTO_CONTA_CONTABIL_ENTRADA_ORCAMENTARIA_CODIGO);
		return this.save(movimentoEntrada);
	}	

	/**
	 * {@inheritDoc}
	 */
	@Override
	public ContaContabilMovimento salvaMovimentoSaidaAtendimento(ContaContabil conta, BigDecimal valor, Dominio dominioClasseReferencia, Long idClasseReferencia) {
		ContaContabilMovimento movimentoSaida = criaMovimentoSaida(conta, valor, dominioClasseReferencia, idClasseReferencia, Dominio.TIPO_MOVIMENTO_CONTA_CONTABIL_BAIXA_ORCAMENTARIA_CODIGO);
		return this.save(movimentoSaida);
	}

	public ContaContabilMovimento salvaMovimentoSaidaBaixaAlmoxarifado(ContaContabil conta, BigDecimal valor, Dominio dominioClasseReferencia, Long idClasseReferencia, Baixa baixa) {
		ContaContabilMovimento movimentoSaida = criaMovimentoBaixaAlmoxarifado(conta, valor, dominioClasseReferencia, idClasseReferencia, Dominio.TIPO_MOVIMENTO_CONTA_CONTABIL_BAIXA_ORCAMENTARIA_CODIGO, baixa);
		return this.save(movimentoSaida);
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public void salvaMovimentoTransferencia(ContaContabil contaOrigem, ContaContabil contaDestino, BigDecimal valor, Dominio dominioClasseReferencia, Long idClasseReferencia) {
		ContaContabilMovimento contaTransSaida = criaMovimentoSaida(contaOrigem, valor, dominioClasseReferencia, idClasseReferencia, Dominio.TIPO_MOVIMENTO_CONTA_CONTABIL_BAIXA_EXTRAORCAMENTARIA_CODIGO);
		this.save(contaTransSaida);
		
		ContaContabilMovimento contaTransEntrada = criaMovimentoEntrada(contaDestino, valor, dominioClasseReferencia, idClasseReferencia, Dominio.TIPO_MOVIMENTO_CONTA_CONTABIL_ENTRADA_EXTRAORCAMENTARIA_CODIGO);
		this.save(contaTransEntrada);
	}
	
	/**
	 * Método responsável por criar um movimento de baixa
	 * 
	 * @param conta
	 * @param valor
	 * @param baixa
	 * @return
	 */
	private ContaContabilMovimento criaMovimentoBaixa(ContaContabil conta, BigDecimal valor, Baixa baixa) {
		Long tipoMovimento = baixa == null ? Dominio.TIPO_MOVIMENTO_CONTA_CONTABIL_BAIXA_EXTRAORCAMENTARIA_CODIGO : Dominio.TIPO_MOVIMENTO_CONTA_CONTABIL_BAIXA_ORCAMENTARIA_CODIGO;
		Dominio dominioBaixa = this.dominioService.findByChaveAndCodigo(Dominio.TIPO_MOVIMENTO_CONTA_CONTABIL, tipoMovimento);
		ContaContabilMovimento movimentoBaixa;
		if (valor.compareTo(BigDecimal.ZERO) >= 0) {
			movimentoBaixa = criaContaContabilMovimento(conta, valor.negate(), dominioBaixa);
		} else {
			movimentoBaixa = criaContaContabilMovimento(conta, valor, dominioBaixa);
		}
		movimentoBaixa.setBaixa(baixa);
		return movimentoBaixa;
	}
	
	/**
	 * Método responsável por criar um movimento de saida
	 * 
	 * @param conta
	 * @param valor
	 * @param dominioClasseReferencia
	 * @param idClasseReferencia
	 * @return
	 */
	private ContaContabilMovimento criaMovimentoSaida(ContaContabil conta, BigDecimal valor, Dominio dominioClasseReferencia, Long idClasseReferencia, Long tipoMovimento) {
		Dominio dominioBaixa = this.dominioService.findByChaveAndCodigo(Dominio.TIPO_MOVIMENTO_CONTA_CONTABIL, tipoMovimento);
		ContaContabilMovimento movimentoBaixa;
		if (valor.compareTo(BigDecimal.ZERO) >= 0) {
			movimentoBaixa = criaContaContabilMovimento(conta, valor.negate(), dominioBaixa);
		} else {
			movimentoBaixa = criaContaContabilMovimento(conta, valor, dominioBaixa);
		}
		movimentoBaixa.setDominioClasseReferencia(dominioClasseReferencia);
		movimentoBaixa.setIdClasseReferencia(idClasseReferencia);
		return movimentoBaixa;
	}
	
	private ContaContabilMovimento criaMovimentoBaixaAlmoxarifado(ContaContabil conta, BigDecimal valor, Dominio dominioClasseReferencia, Long idClasseReferencia, Long tipoMovimento, Baixa baixa) {
		Dominio dominioBaixa = this.dominioService.findByChaveAndCodigo(Dominio.TIPO_MOVIMENTO_CONTA_CONTABIL, tipoMovimento);
		ContaContabilMovimento movimentoBaixa;
		if (valor.compareTo(BigDecimal.ZERO) >= 0) {
			movimentoBaixa = criaContaContabilMovimento(conta, valor.negate(), dominioBaixa);
		} else {
			movimentoBaixa = criaContaContabilMovimento(conta, valor, dominioBaixa);
		}
		movimentoBaixa.setDominioClasseReferencia(dominioClasseReferencia);
		movimentoBaixa.setIdClasseReferencia(idClasseReferencia);
		movimentoBaixa.setBaixa(baixa);
		return movimentoBaixa;
	}	
	
	/**
	 * Método responsável por criar um movimento basico
	 * 
	 * @param conta
	 * @param valor
	 * @param tipoMovimento
	 * @return
	 */
	private ContaContabilMovimento criaContaContabilMovimento(ContaContabil conta, BigDecimal valor, Dominio tipoMovimento) {
		Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
		
		ContaContabilMovimento movimento = new ContaContabilMovimento();
		movimento.setDominioTipoMovimentoContaContabil(tipoMovimento);
		movimento.setDataReferencia(usuarioLogado.getOrganizacao().getDataReferenciaVigente());
		movimento.setValor(valor);
		movimento.setContaContabil(conta);
		return movimento;
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public Boolean existeMovimentoOrganizacaoDataReferenciaTipoMaterial(Organizacao organizacao, Calendar dataRefInicial, Calendar dataRefFinal, Long tipoMaterial) {

		return this.contaContabilMovimentoDao.existeMovimentoOrganizacaoDataReferenciaETipoMaterial(organizacao, dataRefInicial, dataRefFinal, tipoMaterial);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void removerMovimentosEntrada(Entrada entity) {

		List<ContaContabilMovimento> movimentosEntrada = this.contaContabilMovimentoDao.listarMovimentosEntrada(entity);
		
		this.removeList(movimentosEntrada);
 	}
	
	public void removerMovimentosBaixa(Long idBaixaItem) {
		Dominio dominioClasseReferencia = this.dominioService.findByChaveAndCodigo(DominioMaterial.CLASSE_REFERENCIA, DominioMaterial.REFERENCIA_BAIXA_ALMOXARIFADO_ITEM);
		
		if (UtilObjeto.isReferencia(dominioClasseReferencia)){
			List<ContaContabilMovimento> movimentosBaixa = this.contaContabilMovimentoDao.listarMovimentosPorClasseRefIdRef(idBaixaItem, dominioClasseReferencia.getId());
			this.removeList(movimentosBaixa);
		}		
 	}	
	
	public void removerMovimentosAtendimento(Long idAtendimentoItem) {
		Dominio dominioClasseReferencia = this.dominioService.findByChaveAndCodigo(DominioMaterial.CLASSE_REFERENCIA, DominioMaterial.REFERENCIA_ATENDIMENTO_ALMOXARIFADO_ITEM);
		
		if (UtilObjeto.isReferencia(dominioClasseReferencia)){
			List<ContaContabilMovimento> movimentosAtendimento = this.contaContabilMovimentoDao.listarMovimentosPorClasseRefIdRef(idAtendimentoItem, dominioClasseReferencia.getId());
			this.removeList(movimentosAtendimento);
		}		
 	}	
	
	public void removerMovimentosDevolucao(Long idDevolucaoItem) {
		Dominio dominioClasseReferencia = this.dominioService.findByChaveAndCodigo(DominioMaterial.CLASSE_REFERENCIA, DominioMaterial.REFERENCIA_DEVOLUCAO_ITEM);
		
		if (UtilObjeto.isReferencia(dominioClasseReferencia)){
			List<ContaContabilMovimento> movimentosAtendimento = this.contaContabilMovimentoDao.listarMovimentosPorClasseRefIdRef(idDevolucaoItem, dominioClasseReferencia.getId());
			this.removeList(movimentosAtendimento);
		}		
 	}		

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Collection<Long> getIdsContaContabilEmUso(Long idOrganizacao) {

		return this.contaContabilMovimentoDao.getIdsContaContabilEmUso(idOrganizacao);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public ContaContabilMovimento findByContaAndClasseReferencia(ContaContabil contaContabil, Dominio classeReferencia, Long idClasseReferencia) {
		return this.contaContabilMovimentoDao.findByContaAndClasseReferencia(contaContabil, classeReferencia, idClasseReferencia);
	}
	
	public void removeMovimentosPorEntradaItem(List<Long> bens){				
		this.removeList(this.contaContabilMovimentoDao.buscaContasContabeisMovimentoEntradaItem(bens));			
	}

	@Override
	public void removerMovimentosEntradaAlmoxarifado(Long idEntradaAlmoxarifado) {
		Dominio dominioClasseReferencia = this.dominioService.findByChaveAndCodigo(DominioMaterial.CLASSE_REFERENCIA, DominioMaterial.REFERENCIA_ENTRADA_ALMOXARIFADO);
		
		if (UtilObjeto.isReferencia(dominioClasseReferencia)){
			List<ContaContabilMovimento> movimentosAtendimento = this.contaContabilMovimentoDao.listarMovimentosPorClasseRefIdRef(idEntradaAlmoxarifado, dominioClasseReferencia.getId());
			this.removeList(movimentosAtendimento);
		}
	}
}
