package br.com.centralit.api.service.impl;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import br.com.centralit.api.dao.ContaContabilSaldoDao;
import br.com.centralit.api.model.ContaContabil;
import br.com.centralit.api.model.ContaContabilConfiguracaoReferencia;
import br.com.centralit.api.model.ContaContabilSaldo;
import br.com.centralit.api.service.ContaContabilMovimentoService;
import br.com.centralit.api.service.ContaContabilSaldoService;
import br.com.centralit.api.service.ContaContabilService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Organizacao;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 * 
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
 * 	
 * @since 13/01/2015 - 15:34:59
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *	
 */
@Service("contaContabilSaldoService")
public class ContaContabilSaldoServiceImpl extends GenericServiceImpl<ContaContabilSaldo, Long> implements ContaContabilSaldoService {

	private ContaContabilSaldoDao contaContabilSaldoDao;
	
	@Autowired
	private ContaContabilService contaContabilService;
	
	@Autowired
	private DominioService dominioService;
	
	@Autowired
	private ContaContabilMovimentoService contaContabilMovimentoService;
	
	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * @param contaContabilSaldoDao
	 */
	@Autowired
	public ContaContabilSaldoServiceImpl(ContaContabilSaldoDao contaContabilSaldoDao) {
		this.dao = contaContabilSaldoDao;
		
		this.contaContabilSaldoDao = contaContabilSaldoDao;
	}

	@Override
	public ContaContabilSaldo obterContaContabilSaldoPorContaContabilEDataReferencia(Long idContaContabil, Calendar dataReferencia) {
		return this.contaContabilSaldoDao.obterContaContabilSaldoPorContaContabilEDataReferencia(idContaContabil, dataReferencia);
	}

	@Override
	public ContaContabilSaldo obterUltimaContaContabilSaldo(Long idContaContabil) {
		return this.contaContabilSaldoDao.obterUltimaContaContabilSaldo(idContaContabil);
	}

	@Override
	public boolean fechaSaldoContasContabeis(Organizacao organizacao) {
		Collection<ContaContabil> contasContabeis = this.contaContabilService.findAllByOrganizacao(organizacao);
		Calendar mesReferencia = (Calendar) organizacao.getDataReferenciaVigente().clone();
		
		for (ContaContabil conta : contasContabeis) {
			if (conta.getId() != null && isReferenciaMaiorQueAnterior(conta.getContaContabilSaldoAnterior().getDataReferencia(), mesReferencia)) {
				fechaSaldoConta(conta, mesReferencia);
			}
		}
		this.flush();
		
		organizacao.setDataReferenciaVigente(mesReferencia);
		return true;
	}
	
	@Override
	public boolean fechaSaldoContasContabeisTipoPermanente(Organizacao organizacao) {
		Dominio tipoMaterial = this.dominioService.findByChaveAndCodigo(Dominio.TIPO_MATERIAL, Dominio.TIPO_MATERIAL_PERMANENTE);
		Collection<ContaContabil> contasContabeis = this.contaContabilService.findAllByOrganizacaoAndTipoMaterial(organizacao, tipoMaterial);
		Calendar mesReferencia = (Calendar) organizacao.getDataReferenciaVigente().clone();
		
		for (ContaContabil conta : contasContabeis) {
			if (conta.getId() != null && isReferenciaMaiorQueAnterior(conta.getContaContabilSaldoAnterior().getDataReferencia(), mesReferencia)) {
				fechaSaldoConta(conta, mesReferencia);
			}
		}
		this.flush();
		
		organizacao.setDataReferenciaVigente(mesReferencia);
		return true;
	}
	
	/**
	 * 
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por verificar se a referencia atual de fechamento de conta é maior que o ultimo fechamento de conta.
	 *
	 * @author geovane.filho
	 *
	 * @param dataReferenciaAnterior Data de referencia do ultimo fechamento de conta.
	 * @param dataReferenciaAtual Data de referencia atual para fechar uma conta
	 * @return <code>true</code> se a data de referencia atual for maior que a ultima referencia fechada, <code>false</code> caso contrário.
	 */
	private boolean isReferenciaMaiorQueAnterior(Calendar dataReferenciaAnterior, Calendar dataReferenciaAtual) {
		int comparacaoData = UtilDate.comparaMesAnoReferencia(dataReferenciaAnterior, dataReferenciaAtual);
		if (comparacaoData >= 0) {
			return false;
		} else {
			return true;
		}
	}
	
	/**
	 * 
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por fechar o saldo de uma conta contabil em um determinado mes de referencia.
	 *
	 * @author geovane.filho
	 *
	 * @param contaContabil Conta Contabil a se fechar seu saldo.
	 * @param mesReferencia Mês de referencia a se fechar o saldo.
	 */
	private void fechaSaldoConta(ContaContabil contaContabil, Calendar mesReferencia) {
		BigDecimal valor = this.contaContabilMovimentoService.somaValoresMovimentos(contaContabil, (Calendar) mesReferencia.clone());
		valor = valor.add(UtilObjeto.isReferencia(contaContabil.getContaContabilSaldoAnterior().getValor()) ? contaContabil.getContaContabilSaldoAnterior().getValor() : BigDecimal.ZERO);
		ContaContabilSaldo saldo = new ContaContabilSaldo(contaContabil, mesReferencia, valor);
		contaContabil.setContaContabilSaldoAnterior(this.save(saldo));
		
		ContaContabilConfiguracaoReferencia confReferenciaSubsequente = createConfiguracaoSubsequente(contaContabil, (Calendar) mesReferencia.clone());
		contaContabil.getConfiguracoesReferencia().add(confReferenciaSubsequente);
		this.contaContabilService.save(contaContabil);
	}
	
	/**
	 * 
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por criar a configuração para o proximo mes da conta contabil
	 *
	 * @author geovane.filho
	 *
	 * @param conta <code>ContaContabil</code> a se criar sua nova configuração
	 * @param mesReferencia Mes de referencia atual
	 * @return <code>ContaContabilConfiguracaoReferencia</code> contendo as configurações padrões do proximo mes.
	 */
	private ContaContabilConfiguracaoReferencia createConfiguracaoSubsequente(ContaContabil conta, Calendar mesReferencia) {
		Boolean apresenteRMB = false;
		Boolean aprensetaInventario = false;
		Boolean apresentaRMA = false;
		Boolean apresentaDepreciacao = false;
		for (ContaContabilConfiguracaoReferencia confRef : conta.getConfiguracoesReferencia()) {
			if (UtilDate.isDentroMes(confRef.getMesReferencia(), mesReferencia)) {
				apresenteRMB = confRef.getApresentaRMB();
				aprensetaInventario = confRef.getApresentaRelatorioInventario();
				apresentaRMA = confRef.getApresentaRMA();
				apresentaDepreciacao = confRef.getApresentaRelatorioDepreciacao();
			}
		}
		Calendar mesSubsequente = (Calendar) mesReferencia.clone();
		mesSubsequente.add(Calendar.MONTH, 1);
		
		return new ContaContabilConfiguracaoReferencia(conta, mesSubsequente, apresenteRMB, aprensetaInventario, apresentaRMA, apresentaDepreciacao);
	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por verificar a existencia de saldo fechado para as contas contábeis de uma organização em uma dada referência.
	 *
	 * @author geovane.filho
	 *
	 * @param organizacao Organização a se verificar a existencia de saldo fechado para suas contas.
	 * @param dataReferencia Data de referência para filtro.
	 * 
	 * @return <code>true</code> caso exista saldo fechado para alguma conta desta organização na data de referência passada, <code>false</code> caso contrário.
	 */
	@Override
	public Boolean existeSaldoOrganizacaoEDataReferencia(Organizacao organizacao, Calendar dataReferencia) {
		
		return this.contaContabilSaldoDao.existeSaldoOrganizacaoEDataReferencia(organizacao, dataReferencia);
	}
	
}
