package br.com.centralit.api.service.impl;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.ContaContabilConfiguracaoReferenciaDao;
import br.com.centralit.api.dao.ContaContabilDao;
import br.com.centralit.api.model.ContaContabil;
import br.com.centralit.api.model.ContaContabilConfiguracaoReferencia;
import br.com.centralit.api.model.ContaContabilSaldo;
import br.com.centralit.api.service.ConfiguracaoParametroSistemaService;
import br.com.centralit.api.service.ConfiguracaoService;
import br.com.centralit.api.service.ContaContabilMovimentoService;
import br.com.centralit.api.service.ContaContabilSaldoService;
import br.com.centralit.api.service.ContaContabilService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.MaterialService;
import br.com.centralit.api.service.OrganizacaoService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Organizacao;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 29/12/2014 - 17:26:37
 *
 * @version 1.0.0
 *
 * @author rogerio.costa
 *
 */
@Service("contaContabilService")
public class ContaContabilServiceImpl extends GenericServiceImpl<ContaContabil, Long> implements ContaContabilService {

	/** Atributo contaContabilDao. */
	private ContaContabilDao contaContabilDao;

	/** Atributo dominioService. */
	@Autowired
	private DominioService dominioService;

	/** Atributo organizacaoService */
	@Autowired
	private OrganizacaoService organizacaoService;

	/** Atributo materialService. */
	@Autowired
	private MaterialService materialService;

	/** Atributo contaContabilSaldoService. */
	@Autowired
	private ContaContabilSaldoService contaContabilSaldoService;

	@Autowired
	private UsuarioService usuarioService;

	@Autowired
	private ContaContabilConfiguracaoReferenciaDao contaContabilConfiguracaoReferenciaDao;

	@Autowired
	private ConfiguracaoService configuracaoService;

	@Autowired
	private ConfiguracaoParametroSistemaService configuracaoParametroSistemaService;

	@Autowired
	private ContaContabilMovimentoService contabilMovimentoService;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 *
	 * @param contaContabilDao
	 */
	@Autowired
	public ContaContabilServiceImpl( ContaContabilDao contaContabilDao, @Qualifier("contaContabilValidator") Validator validator ) {

		this.dao = contaContabilDao;

		this.contaContabilDao = contaContabilDao;

		this.validator = validator;

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por listar contas contábeis disponíveis por codigo
	 *
	 * @author rogerio.costa
	 *
	 * @param codigo
	 * @return
	 */
	public Collection<ContaContabil> findContaContabil(String codigo) {

		return this.getContaContabilDao().findContaContabil(codigo);
	}

	/**
	 * Retorna o valor do atributo <code>contaContabilDao</code>
	 *
	 * @return <code>ContaContabilDao</code>
	 */
	public ContaContabilDao getContaContabilDao() {

		return contaContabilDao;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">563</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por salvar conta contábil
	 *
	 * @author iago.almeida
	 *
	 * @param contaContabil
	 * @return <ContaContabil>
	 */
	@Override
	public ContaContabil save(ContaContabil contaContabil) {

		contaContabil.setDominioTipoContaContabil(( contaContabil.getDominioTipoContaContabil() != null && contaContabil.getDominioTipoContaContabil().getId() != null ) ? this.dominioService.getReference(contaContabil.getDominioTipoContaContabil().getId()) : null);

		if (contaContabil.getDominioTipoMaterial() != null && contaContabil.getDominioTipoMaterial().getCodigo() != null) {
			contaContabil.setDominioTipoMaterial(this.dominioService.findByChaveAndCodigo(Dominio.TIPO_MATERIAL, contaContabil.getDominioTipoMaterial().getCodigo()));
		}

		if (contaContabil.getConfiguracoesReferencia() != null) { // Se houver configurações eu atualizo os atributos delas
			atualizarContaEmConfiguracao(contaContabil);
		}

		// VALIDA CAMPO OBRIGATÓRIO DA ENTIDADE
		if (UtilObjeto.isReferencia(this.validator)) {

			this.validarEntidade(contaContabil, this.validator);
		}

		if (!UtilObjeto.isReferencia(contaContabil.getId()) && this.contaContabilDao.existeCodigo(contaContabil, contaContabil.getOrganizacao().getId())) {

			throw new BusinessException("VALIDACAO.CODIGO_UNICO", CodigoErro.REGRA_NEGOCIO.getValue());

		}

		ContaContabil entity = super.save(contaContabil);

		if (!UtilObjeto.isReferencia(entity.getContaContabilSaldoAnterior())) {
			// GERA CONTA SALDO ZERADO PARA CONTA CONTÁBIL
			this.montaContaContabilSaldoAnterior(entity);
		}

		calcularTaxaDepreciacao(entity);

		return super.merge(entity);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">563</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por alterar conta contábil
	 *
	 * @author iago.almeida
	 *
	 * @param contaContabil
	 * @return <ContaContabil>
	 */
	@Override
	public ContaContabil merge(ContaContabil contaContabil) {

		contaContabil.setDominioTipoContaContabil(( contaContabil.getDominioTipoContaContabil() != null && contaContabil.getDominioTipoContaContabil().getId() != null ) ? this.dominioService.getReference(contaContabil.getDominioTipoContaContabil().getId()) : null);

		if (this.contaContabilDao.existeCodigo(contaContabil, contaContabil.getOrganizacao().getId())) {
			throw new BusinessException("VALIDACAO.CODIGO_UNICO", CodigoErro.REGRA_NEGOCIO.getValue());
		}

		if (contaContabil.getDominioTipoMaterial() != null && contaContabil.getDominioTipoMaterial().getCodigo() != null) {
			contaContabil.setDominioTipoMaterial(this.dominioService.findByChaveAndCodigo(Dominio.TIPO_MATERIAL, contaContabil.getDominioTipoMaterial().getCodigo()));
		}

		if (contaContabil.getConfiguracoesReferencia() != null) { // Se houver configurações eu atualizo os atributos delas
			atualizarContaEmConfiguracao(contaContabil);
		}

		if (!UtilObjeto.isReferencia(contaContabil.getContaContabilSaldoAnterior())) {
			// GERA CONTA SALDO ZERADO PARA CONTA CONTÁBIL EM EDIÇÃO
			this.montaContaContabilSaldoAnterior(contaContabil);
		} else {
			contaContabil.setContaContabilSaldoAnterior(this.contaContabilSaldoService.find(contaContabil.getContaContabilSaldoAnterior().getId()));
		}

		contaContabil = calcularTaxaDepreciacao(contaContabil);

		return super.merge(contaContabil);
	}

	/**
	 *
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por atualizar os atributos das configurações da Conta Contabil
	 *
	 * @author geovane.filho
	 *
	 * @param contaContabil
	 *            Conta Contabil a se atualizar os atributos de configurações
	 */
	private void atualizarContaEmConfiguracao(ContaContabil contaContabil) {

		for (Iterator<ContaContabilConfiguracaoReferencia> iterator = contaContabil.getConfiguracoesReferencia().iterator(); iterator.hasNext();) { // Atualizo os atributos para cada configuração
			ContaContabilConfiguracaoReferencia conf = iterator.next();
			conf.getMesReferencia().set(Calendar.HOUR_OF_DAY, conf.getMesReferencia().getActualMaximum(Calendar.HOUR_OF_DAY));
			conf.setContaContabil(contaContabil);
		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por gerar conta contabil saldo anterior
	 *
	 * @author iago.almeida
	 *
	 * @param contaContabil
	 */
	private void montaContaContabilSaldoAnterior(ContaContabil contaContabil) {

		if (contaContabil.getOrganizacao() != null) {
			Organizacao organizacao = this.organizacaoService.getReference(contaContabil.getOrganizacao().getId());

			Calendar dataReferencia = UtilDate.getCalendarDaData(organizacao.getDataReferenciaVigente().getTime());
			dataReferencia.add(Calendar.MONTH, -1);

			ContaContabilSaldo contaContabilSaldo = new ContaContabilSaldo(contaContabil, dataReferencia, BigDecimal.ZERO);
			this.contaContabilSaldoService.save(contaContabilSaldo);
			contaContabil.setContaContabilSaldoAnterior(contaContabilSaldo);
		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por inativar a conta contábil setando dataInativo.
	 *
	 * @author iago.almeida
	 *
	 * @param idContaContabil
	 * @return
	 */
	@Override
	public boolean removeById(Long idContaContabil) {

		Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
		Organizacao organizacao = this.organizacaoService.find(usuarioLogado.getOrganizacao().getId());

		Long idConta = getIdContaContabilEmUso(organizacao);

		if ((idConta.equals(idContaContabil)) || this.materialService.existeMaterialVinculadoAContaContabil(idContaContabil, usuarioLogado)) {
			throw new BusinessException("PORTAL.MSG.MN020", CodigoErro.REGRA_NEGOCIO.getValue());
		}

		ContaContabil conta = this.find(idContaContabil);

		if (conta.getConfiguracoesReferencia() != null) { // remove todas as configurações da conta contabil
			for (ContaContabilConfiguracaoReferencia confRef : conta.getConfiguracoesReferencia()) {
				this.contaContabilConfiguracaoReferenciaDao.remove(confRef);
			}
		}

		return this.dao.removeById(idContaContabil);
	}

	/**
	 *
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por encontrar todas as contas contabeis de uma determinada organização.
	 *
	 * @author geovane.filho
	 *
	 * @param organizacao
	 *            <code>Organizacao</code> a se buscar suas contas contabeis.
	 *
	 * @return <code>Collection<ContaContabil></code> contendo todas as contas contabeis da organização.
	 */
	@Override
	public Collection<ContaContabil> findAllByOrganizacao(Organizacao organizacao) {

		return this.contaContabilDao.findAllByOrganizacao(organizacao);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por lista conta contábil por código e órgão do usuário logado
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param codigo
	 * @param idOrganizacao
	 * @return Collection<ContaContabil>
	 */
	@Override
	public Collection<ContaContabil> findContaContabilPorOrganizacao(String codigo, Long idOrganizacao) {

		return this.contaContabilDao.findContaContabilPorOrganizacao(codigo, idOrganizacao);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por verificar se existe uma contaContabil vinculado a estrutura
	 *
	 * @author rogerio.costa
	 *
	 * @param localizacaoId
	 *
	 * @return boolean
	 */
	public boolean existeContaContabilVinculadaAEstrutura(Long idEstrutura) {

		return this.contaContabilDao.existeContaContabilVinculadaAEstrutura(idEstrutura);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por obter conta contábil por código, órgão, domínio
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param codigo
	 * @param idOrganizacao
	 * @param idDominio
	 * @return Collection<ContaContabil>
	 */
	@Override
	public Collection<ContaContabil> findContaContabilPorOrganizacaoEDominio(String codigo, Long idOrganizacao, Long idDominio) {

		return this.contaContabilDao.findContaContabilPorOrganizacaoEDominio(codigo, idOrganizacao, idDominio);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por obter conta contábil por código, órgão, domínio que não esteja em uso em algum material
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param codigo
	 * @param idOrganizacao
	 * @param idDominio
	 * @return Collection<ContaContabil>
	 */
	@Override
	public Collection<ContaContabil> findContaContabilPorOrganizacaoEDominioSemUsoMaterial(String codigo, Long idOrganizacao, Long idDominio) {

		Collection<Long> idsContaContabilEmUsoMaterial = this.materialService.getIdsContaContabilEmUso(idOrganizacao);

		Set<Long> result = new HashSet<Long>();

		result.addAll(new HashSet<Long>(idsContaContabilEmUsoMaterial));

		return this.contaContabilDao.findContaContabilPorOrganizacaoEDominioSemUsoMaterial(codigo, idOrganizacao, idDominio, result);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por obter conta contábil por código, órgão, domínio que não esteja em uso em algum almoxarifado
	 *
	 * @author geovane.filho
	 *
	 * @param codigo
	 * @param idOrganizacao
	 * @param idDominio
	 * @return Collection<ContaContabil>
	 */
	@Override
	public Collection<ContaContabil> findContaContabilPorOrganizacaoEDominioSemUsoAlmoxarifado(String codigo, Long idOrganizacao, Long idDominio) {

		Organizacao organizacao = organizacaoService.find(idOrganizacao);

		Long idConta = getIdContaContabilEmUso(organizacao);

		return this.contaContabilDao.findContaContabilPorOrganizacaoEDominioSemUsoAlmoxarifado(codigo, idOrganizacao, idDominio, idConta);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por listar a entidade <code>ContaContabil</code> através da organização, tipoMaterial e que não esteja em uso pela estrutura.
	 *
	 * @author rogerio.costa
	 *
	 * @param str
	 * @param idOrganizacao
	 * @param codigoTipoMaterial
	 * @param idContaContabilEmUso
	 * @return Collection<ContaContabil>
	 */
	public Collection<ContaContabil> findContaContabilPorOrganizacaoTipoMaterial(String str, Long idOrganizacao, Long codigoTipoMaterial) {

		Organizacao organizacao = this.organizacaoService.getReference(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getOrganizacao().getId());

		Long idConta = getIdContaContabilEmUso(organizacao);

		return this.contaContabilDao.findContaContabilPorOrganizacaoTipoMaterial(str, idOrganizacao, codigoTipoMaterial, idConta);
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por recuperar o valor atual total de uma determinada conta
	 *
	 * @author geovane.filho
	 *
	 * @param contaId id da conta
	 * @return
	 */
	@Override
	public BigDecimal getValorAtual(Long contaId) {

		Organizacao organizacao = this.organizacaoService.getReference(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getOrganizacao().getId());

		return this.contaContabilDao.getValorAtual(contaId, organizacao.getDataReferenciaVigente());
	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por obter id da conta contábil em uso para configuração de almoxarifado.
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param organizacao
	 * @return Long idConta
	 */
	private Long getIdContaContabilEmUso(Organizacao organizacao) {

		String idContaContabilEmUso = configuracaoParametroSistemaService.getParametro("CONTA_CONTABIL_ALMOXARIFADO", organizacao).getValor();
		Long idConta = null;

		if (UtilObjeto.isReferencia(idContaContabilEmUso)) {
			idConta = Long.valueOf(idContaContabilEmUso);
		}
		return idConta;
	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por obter a conta contábil definida na configuracaoparametrosistema
	 *
	 * @author rogerio.cassimiro
	 *
	 * @return ContaContabil
	 */
	@Override
	public ContaContabil obterContaContabilAlmoxarifadoConfiguracaoParametroSistema() {

		Organizacao organizacao = this.organizacaoService.getReference(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getOrganizacao().getId());

		Long idConta = this.getIdContaContabilEmUso(organizacao);

		return UtilObjeto.isReferencia(idConta) ? (ContaContabil) this.contaContabilDao.find(idConta) :  new ContaContabil();
	}


	/**
	 *
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por calcular a taxa de depreciacao
	 * Formula:
	 *
	 * TaxaDepreciacao = (1/VidaUtil)
	 *
	 * @author gilberto.nery
	 *
	 * @param contaContabil
	 */
	public ContaContabil calcularTaxaDepreciacao (ContaContabil contaContabil){

		if(contaContabil != null){

			if(contaContabil.getVidaUtil() != null && contaContabil.getVidaUtil() > 0){
				contaContabil.setTaxaDepreciacaoMensal(BigDecimal.ONE.divide(BigDecimal.valueOf(contaContabil.getVidaUtil()), 4, RoundingMode.HALF_EVEN));
			} else {
				contaContabil.setTaxaDepreciacaoMensal(BigDecimal.ZERO);
			}

		}

		return contaContabil;

	}
	
	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 *  Método responsável por obter a conta contábil por inventário
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param busca
	 * @param idInventario
	 * @return
	 */
	@Override
	public List<ContaContabil> findContaContabilPorInventario(String busca, Long idInventario) {
		return this.contaContabilDao.findContaContabilPorInventario(busca,idInventario);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public ContaContabil findByMaterial(Long materialId) {
		return this.contaContabilDao.findByMaterial(materialId);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Collection<ContaContabil> findAllByOrganizacaoAndTipoMaterial(Organizacao organizacao, Dominio tipoMaterial) {
		return this.contaContabilDao.findAllByOrganizacaoAndTipoMaterial(organizacao, tipoMaterial);
	}

}