package br.com.centralit.api.service.impl;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import br.com.centralit.api.dao.MaterialCaracteristicaDao;
import br.com.centralit.api.model.MaterialCaracteristica;
import br.com.centralit.api.service.MaterialCaracteristicaService;
import br.com.centralit.api.service.MaterialService;
import br.com.centralit.api.service.ModuloService;
import br.com.centralit.api.util.ClientRest;
import br.com.centralit.framework.exception.BusinessException;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Modulo;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 23/12/2014 - 15:54:28
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
@Service("materialCaracteristicaService")
public class MaterialCaracteristicaServiceImpl extends GenericServiceImpl<MaterialCaracteristica, Long> implements MaterialCaracteristicaService {

	private static final String MSG_EXCLUSAO_NEGADA_CARACTERISTICA = "MSG.EXCLUSAO_NEGADA_CARACTERISTICA";

	/** Atributo materialCaracteristicaDao. */
	private MaterialCaracteristicaDao materialCaracteristicaDao;

	/** Atributo materialService. */
	@Autowired
	private MaterialService materialService;

	/** Atributo moduloService. */
	@Autowired
	private ModuloService moduloService;

	/** Atributo request. */
	@Autowired
	private HttpServletRequest request;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * 
	 * @param materialCaracteristicaDao
	 */
	@Autowired
	public MaterialCaracteristicaServiceImpl( MaterialCaracteristicaDao materialCaracteristicaDao ) {

		this.dao = materialCaracteristicaDao;

		this.materialCaracteristicaDao = materialCaracteristicaDao;

	}

	@Override
	public MaterialCaracteristica merge(MaterialCaracteristica entity) {

		entity.setMaterial(this.getMaterialService().getReference(entity.getMaterial().getId()));

		return super.merge(entity);
	}

	@Override
	public boolean removeById(Long id) {

		Modulo moduloPatrimonio = moduloService.moduloEstaAtivo(ClientRest.MODULO_PATRIMONIO);

		if (UtilObjeto.isReferencia( moduloPatrimonio)) {

			ClientRest clientRestPatrimonio = new ClientRest(request, moduloPatrimonio);

			clientRestPatrimonio.addParametro("id", id.toString());

			clientRestPatrimonio.addParametro("joinClass", "caracteristicas.materialCaracteristica.id");

			boolean contemBemPatrimonial = clientRestPatrimonio.validarRegra("bemPatrimonial/existeVinculo");

			if (contemBemPatrimonial) {

				throw new BusinessException(MSG_EXCLUSAO_NEGADA_CARACTERISTICA, CodigoErro.REGRA_NEGOCIO.getValue());
			}
		}

		MaterialCaracteristica materialCaracteristica = this.getReference(id);

		materialCaracteristica.setMaterialRemocao(materialCaracteristica.getMaterial());

		materialCaracteristica.setMaterial(null);

		super.merge(materialCaracteristica);

		return super.removeById(id);

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 * 
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 * 
	 * Método responsável por validar se existem Material Caracteristica por caracteristica
	 * 
	 * @author rogerio.cassimiro
	 * 
	 * @param id
	 * @return
	 */
	@Override
	public Boolean existeMaterialCarateristicaPorCaracteristica(Long id) {

		return this.materialCaracteristicaDao.existeMaterialCarateristicaPorCaracteristica(id);
	}

	/**
	 * Retorna o valor do atributo <code>materialCaracteristicaDao</code>
	 * 
	 * @return <code>MaterialCaracteristicaDao</code>
	 */
	public MaterialCaracteristicaDao getMaterialCaracteristicaDao() {

		return materialCaracteristicaDao;
	}

	/**
	 * Retorna o valor do atributo <code>materialService</code>
	 * 
	 * @return <code>MaterialService</code>
	 */
	public MaterialService getMaterialService() {

		return materialService;
	}

}
