package br.com.centralit.api.service.impl;

import java.math.BigDecimal;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.MaterialConsumoDao;
import br.com.centralit.api.model.MaterialConsumo;
import br.com.centralit.api.service.ConfiguracaoParametroSistemaService;
import br.com.centralit.api.service.EstruturaOrganizacionalService;
import br.com.centralit.api.service.MaterialConsumoService;
import br.com.centralit.api.service.MaterialService;
import br.com.centralit.api.service.ModuloService;
import br.com.centralit.api.util.ClientRest;
import br.com.centralit.framework.model.Modulo;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 25/05/2015 - 10:25:41
 *
 * @version 1.0.0
 *
 * @author rogerio.costa
 *
 */
@Service("materialConsumoService")
public class MaterialConsumoServiceImpl extends GenericServiceImpl<MaterialConsumo, Long> implements MaterialConsumoService {

	/** Atributo materialConsumoDao. */
	private MaterialConsumoDao materialConsumoDao;

	/** Atributo materialService. */
	@Autowired
	private MaterialService materialService;

	/** Atributo moduloService. */
	@Autowired
	private ModuloService moduloService;

	/** Atributo estruturaOrganizacionalService. */
	@Autowired
	private EstruturaOrganizacionalService estruturaOrganizacionalService;
	
	/** Atributo configuracaoService. */
	@Autowired
	private ConfiguracaoParametroSistemaService configuracaoService;	

	@Autowired
	private HttpServletRequest request;

	@Override
	public MaterialConsumo save(MaterialConsumo entity) {

		this.validarEntidade(entity, this.validator);

		return (MaterialConsumo) this.materialService.save(entity);

	}

	@Override
	public MaterialConsumo merge(MaterialConsumo entity) {

		return (MaterialConsumo) this.materialService.merge(entity);
	}

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 *
	 * @param materialConsumoDao
	 */
	@Autowired
	public MaterialConsumoServiceImpl( MaterialConsumoDao materialConsumoDao, @Qualifier("materialConsumoValidator") Validator validator ) {

		this.dao = materialConsumoDao;

		this.materialConsumoDao = materialConsumoDao;

		this.validator = validator;

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">593</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por buscar os materiais de consumo de uma organizacao
	 *
	 * @author juliana.barbosa
	 *
	 * @param descricaoOuCodigo
	 * @param idOrganizacao
	 * @return
	 */
	@Override
	public List<MaterialConsumo> findMateriaisConsumoByOrganizacao(String descricaoOuCodigo, Long idOrganizacao) {

		return this.materialConsumoDao.findMateriaisConsumoByOrganizacao(descricaoOuCodigo, idOrganizacao);

	}

	/**
	 * Retorna o valor do atributo <code>materialConsumoDao</code>
	 *
	 * @return <code>MaterialConsumoDao</code>
	 */
	public MaterialConsumoDao getMaterialConsumoDao() {

		return materialConsumoDao;
	}

	/**
	 * Método responsável por calcular o valor atual total dos materiais de consumo em estoque de um determinado material.
	 *
	 * @param materialId
	 *            material a se pesquisar o valor atual total dos seus materiais de consumo em estoque.
	 * @param organizacaoId
	 * @return
	 */
	@Override
	public BigDecimal getValorAtualMaterial(Long materialId, Long organizacaoId) {

		Modulo moduloAlmoxarifado = moduloService.moduloEstaAtivo(ClientRest.MODULO_ALMOXARFADO);

		if (UtilObjeto.isReferencia(moduloAlmoxarifado)) {

			ClientRest clientRest = new ClientRest(request, moduloAlmoxarifado);

			clientRest.addParametro("materialId", materialId.toString());
			clientRest.addParametro("idOrganizacao", organizacaoId.toString());

			return clientRest.getResponse("materialEstoqueSaldo/buscaValorAtual", BigDecimal.class);
		}

		return BigDecimal.ZERO;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public BigDecimal getQuantidadeAtualMaterial(Long materialId, Long organizacaoId) {
		Modulo moduloAlmoxarifado = moduloService.moduloEstaAtivo(ClientRest.MODULO_ALMOXARFADO);

		if (UtilObjeto.isReferencia(moduloAlmoxarifado)) {

			ClientRest clientRest = new ClientRest(request, moduloAlmoxarifado);

			clientRest.addParametro("IdMaterial", materialId.toString());
			clientRest.addParametro("idOrganizacao", organizacaoId.toString());

			return clientRest.getResponse("materialEstoqueSaldo/buscaQuantidadeAtual", BigDecimal.class);
		}

		return BigDecimal.ZERO;
	}

	/**
	 *
	 * Método responsável por buscar a quantidade atual de um determinado material de consumo - chamada da job
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param materialId
	 * @param organizacaoId
	 * @param urlScheme
	 * @param urlHost
	 * @param urlPort
	 * @return {@link BigDecimal}
	 */
	@Override
	public BigDecimal getQuantidadeAtualMaterial(Long materialId, Long organizacaoId, String urlScheme, String urlHost, int urlPort) {
		
		Modulo moduloAlmoxarifado = moduloService.moduloEstaAtivo(ClientRest.MODULO_ALMOXARFADO);

		if (UtilObjeto.isReferencia(moduloAlmoxarifado)) {
			ClientRest clientRest = new ClientRest(urlScheme, urlHost, urlPort, moduloAlmoxarifado);
			clientRest.addParametro("IdMaterial", materialId.toString());
			clientRest.addParametro("idOrganizacao", organizacaoId.toString());

			return clientRest.getResponse("materialEstoqueSaldo/buscaQuantidadeAtual", BigDecimal.class);
		}

		return BigDecimal.ZERO;
	}

}
