package br.com.centralit.api.service.impl;

import java.math.BigDecimal;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.validation.Validator;

import br.com.centralit.api.dao.TransferenciaContaContabilDao;
import br.com.centralit.api.model.Material;
import br.com.centralit.api.model.TransferenciaContaContabil;
import br.com.centralit.api.service.ConfiguracaoParametroSistemaService;
import br.com.centralit.api.service.ContaContabilMovimentoService;
import br.com.centralit.api.service.ContaContabilService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.MaterialConsumoService;
import br.com.centralit.api.service.MaterialService;
import br.com.centralit.api.service.ModuloService;
import br.com.centralit.api.service.OrganizacaoService;
import br.com.centralit.api.service.TransferenciaContaContabilService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.api.util.ClientRest;
import br.com.centralit.framework.model.ConfiguracaoParametroSistema;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Modulo;
import br.com.centralit.framework.model.Organizacao;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.service.arquitetura.GenericServiceImpl;
import br.com.centralit.framework.util.UtilObjeto;

@Service("transferenciaContaContabilService")
public class TransferenciaContaContabilServiceImpl extends GenericServiceImpl<TransferenciaContaContabil, Long> implements TransferenciaContaContabilService {

    private TransferenciaContaContabilDao transferenciaContaContabilDao;

    @Autowired
    private ModuloService moduloService;

    @Autowired
    private UsuarioService usuarioService;

    @Autowired
    private DominioService dominioService;

    @Autowired
    private ContaContabilService contaContabilService;

    @Autowired
    private ContaContabilMovimentoService contaContabilMovimentoService;

    @Autowired
    private ConfiguracaoParametroSistemaService configuracaoParametroSistemaService;

    @Autowired
    private MaterialService materialService;

    @Autowired
    private OrganizacaoService organizacaoService;

    @Autowired
    private MaterialConsumoService materialConsumoService;

    @Autowired
	private HttpServletRequest request;

    @Autowired
	public TransferenciaContaContabilServiceImpl(TransferenciaContaContabilDao transferenciaContaContabilDao,  @Qualifier("transferenciaContaContabilValidator") Validator validator) {
    	this.dao = transferenciaContaContabilDao;
        this.transferenciaContaContabilDao = transferenciaContaContabilDao;
    	this.validator = validator;
	}

	@Override
	public boolean realizaTransferenciaMaterial(TransferenciaContaContabil transferenciaConta) {

		Material material = this.materialService.find(transferenciaConta.getMaterial().getId());
		BigDecimal valorTransferencia = null;
		BigDecimal qntTransferencia = null;
		BigDecimal valorDepreciacaoAcumulada = null;
		Boolean materialPermanente = Boolean.FALSE;
		String idUsuarioLogado = getUsuario().getId().toString();

		Modulo modulo = moduloService.moduloEstaAtivo(ClientRest.MODULO_PATRIMONIO);

		if (material.getDominioTipoMaterial().getCodigo() == Dominio.TIPO_MATERIAL_PERMANENTE) {

			if (UtilObjeto.isReferencia(modulo)) {
				materialPermanente = Boolean.TRUE;
				ClientRest clientRest = new ClientRest(request, modulo);
				clientRest.addParametro("materialId", material.getId().toString());
				clientRest.addParametro("idOrganizacao", material.getOrganizacao().getId().toString());

				valorTransferencia = clientRest.getResponse("bemPatrimonial/valorAtualMaterial", BigDecimal.class);
				qntTransferencia = BigDecimal.valueOf(clientRest.getResponse("bemPatrimonial/quantidadeAtualMaterial", Long.class));
				valorDepreciacaoAcumulada = clientRest.getResponse("bemPatrimonial/getDepreciacaoAcumulada", BigDecimal.class);
				
				transferenciaConta.setValorTransferencia(valorTransferencia);
				transferenciaConta.setValorDepreciacaoAcumulada(valorDepreciacaoAcumulada);
			}
		} else {
			valorTransferencia = materialConsumoService.getValorAtualMaterial(material.getId(), material.getOrganizacao().getId());
			qntTransferencia = materialConsumoService.getQuantidadeAtualMaterial(material.getId(), material.getOrganizacao().getId());
		}

		if (valorTransferencia != null) {
			
			transferenciaConta.setValorTransferencia(valorTransferencia);
			transferenciaConta.setQuantidadeMaterial(qntTransferencia);
			transferenciaConta.setValorDepreciacaoAcumulada(valorDepreciacaoAcumulada);

			this.save(transferenciaConta);

			configurarCodigo(transferenciaConta);

			if(materialPermanente){
				this.salvarMovimentoTransferenciaPatrimonio(transferenciaConta.getContaOrigem().getId().toString(), transferenciaConta.getContaDestino().getId().toString(), valorTransferencia, valorDepreciacaoAcumulada, transferenciaConta.getId().toString(), modulo, idUsuarioLogado);
			} else {
				this.contaContabilMovimentoService.salvaMovimentoTransferencia(transferenciaConta.getContaOrigem(), transferenciaConta.getContaDestino(), valorTransferencia, transferenciaConta);
			}

			material.setContaContabil(transferenciaConta.getContaDestino());
			this.materialService.merge(material);

			return true;
		} else {
			return false;
		}
	}

	/**
	 *
	 * Método responsável por salvar movimento de transferência entre contas para material permanente depreciação acumulada
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param idContaOrigem
	 * @param idContaDestino
	 * @param valorTransferencia
	 * @param valorDepreciacaoAcumulada
	 * @param idTransferenciaContaContabilMovimento
	 * @param modulo
	 * @param idUsuarioLogado
	 */
	private void salvarMovimentoTransferenciaPatrimonio(String idContaOrigem, String idContaDestino, BigDecimal valorTransferencia, BigDecimal valorDepreciacaoAcumulada, String idTransferenciaContaContabilMovimento, Modulo modulo, String idUsuarioLogado) {
		ClientRest clientRest = new ClientRest(request, modulo);
		clientRest.addParametro("idContaOrigem", idContaOrigem);
		clientRest.addParametro("idContaDestino", idContaDestino);
		clientRest.addParametro("valorTransferencia", valorTransferencia.toString());
		clientRest.addParametro("valorDepreciacaoAcumulada", valorDepreciacaoAcumulada.toString());
		clientRest.addParametro("idTransferenciaContaContabilMovimento", idTransferenciaContaContabilMovimento);
		clientRest.addParametro("idUsuarioLogado", idUsuarioLogado);
		clientRest.getResponse("contaContabilMovimentoPatrimonio/salvarMovimentoTransferenciaPatrimonio");
	}

	@Override
	public boolean realizaTransferenciaAlmoxarifado(TransferenciaContaContabil transferenciaConta) {

		Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
		Organizacao organizacao = this.organizacaoService.find(usuarioLogado.getOrganizacao().getId());

		BigDecimal valorTransferencia = this.contaContabilService.getValorAtual(transferenciaConta.getContaOrigem().getId());

		if (valorTransferencia != null) {
			this.save(transferenciaConta);

			configurarCodigo(transferenciaConta);

			this.contaContabilMovimentoService.salvaMovimentoTransferencia(transferenciaConta.getContaOrigem(), transferenciaConta.getContaDestino(), valorTransferencia, transferenciaConta);

			ConfiguracaoParametroSistema confContaAlmoxarifado = this.configuracaoParametroSistemaService.getParametro("CONTA_CONTABIL_ALMOXARIFADO", organizacao);
			confContaAlmoxarifado.setValor(transferenciaConta.getContaDestino().getId().toString());
			this.configuracaoParametroSistemaService.merge(confContaAlmoxarifado);

			return true;
		} else {
			return false;
		}
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por settar o atributo codigo.
	 *
	 * @author luis.camargo
	 *
	 * @param entity
	 */
	private void configurarCodigo(TransferenciaContaContabil entity) {

		TransferenciaContaContabil ultimoRegistro =
				(TransferenciaContaContabil) transferenciaContaContabilDao.buscarUltimoRegistroComOrdenadoParametrizada("codigo", entity.getOrganizacao().getId());

		Long sequencial = null;
		if (UtilObjeto.isReferencia(ultimoRegistro)) {
			sequencial = ultimoRegistro.getCodigo() == null ? null : Long.parseLong(ultimoRegistro.getCodigo());
		}
		String codigo = configuracaoParametroSistemaService.gerarNumeroIdentificacao(sequencial, entity.getOrganizacao());

		entity.setCodigo(codigo);
	}

}
