package br.com.centralit.api.service.validation;

import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import br.com.centralit.api.model.ClassificacaoMaterial;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * @since 29/12/2014 - 10:06:42
 * 
 * @version 1.0.0
 * 
 * @author wilker.machado
 * 
 */
@Component("classificacaoMaterialValidator")
public class ClassificacaoMaterialValidator implements Validator {

	/** Atributo TAMANHO_MAX_DESCRICAO. */
	private static final int TAMANHO_MAX_DESCRICAO = 100;

	/** Atributo TAMANHO_MAX_CODIGO. */
	private static final int TAMANHO_MAX_CODIGO = 30;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean supports(Class<?> clazz) {

		return ClassificacaoMaterial.class.isAssignableFrom(clazz);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void validate(Object target, Errors errors) {

		ValidationUtils.rejectIfEmpty(errors, "codigo", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.CODIGO");

		ValidationUtils.rejectIfEmpty(errors, "descricao", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.DESCRICAO");

		ValidationUtils.rejectIfEmpty(errors, "dominioTipoClassificacaoMaterial", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.TIPO_FEDERAL_SUPPLY");

		ValidationUtils.rejectIfEmpty(errors, "dominioTipoMaterial", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "ADMINISTRACAODEMATERIAIS.LABEL.TIPO_MATERIAL");

		ClassificacaoMaterial entity = (ClassificacaoMaterial) target;

		// validação de tamanho de campos do codigo  - codigo nao pode conter mais q 30 caracter
		if (UtilObjeto.isReferencia(entity.getCodigo()) && entity.getCodigo().length() > TAMANHO_MAX_CODIGO) {

			errors.rejectValue("codigo", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "VALIDACAO.TAMANHO_CODIGO_CLASSIFICACAO");
		}

		// validação de tamanho de campos da descricao  - descricao nao pode conter mais q 30 caracter
		if (UtilObjeto.isReferencia(entity.getDescricao()) && entity.getDescricao().length() > TAMANHO_MAX_DESCRICAO) {

			errors.rejectValue("descricao", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "VALIDACAO.TAMANHO_DESCRICAO_CLASSIFICACAO");

		}

		// se o tipo da classificação material for sub grupo seja qual for o nivel, e obrigatorio ter a classificação material PAI
		if (UtilObjeto.isReferencia(entity.getDominioTipoClassificacaoMaterial()) && entity.getDominioTipoClassificacaoMaterial().getNome().equalsIgnoreCase("SUB_GRUPO")) {

			ValidationUtils.rejectIfEmpty(errors, "classificacaoMaterialParent", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "LABEL.GRUPO");

		}
	}
}
