package br.com.centralit.api.service.validation;

import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import br.com.centralit.api.model.ContaContabil;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.util.UtilObjeto;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 * 
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
 * 	
 * @since 13/01/2015 - 08:03:35
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *	
 */
@Component("contaContabilValidator")
public class ContaContabilValidator implements Validator{

	@Override
	public boolean supports(Class<?> clazz) {

		return ContaContabilValidator.class.isAssignableFrom(clazz);
	}

	@Override
	public void validate(Object target, Errors errors) {
		
		ValidationUtils.rejectIfEmpty(errors, "codigo", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "LABEL.CODIGO");
		
		ValidationUtils.rejectIfEmpty(errors, "descricao", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "LABEL.DESCRICAO");

		ContaContabil entity = (ContaContabil) target;
		
		ValidationUtils.rejectIfEmpty(errors, "dominioTipoMaterial", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "ADMINISTRACAODEMATERIAIS.LABEL.TIPO_MATERIAL");
		
		if (entity.getDominioTipoMaterial() != null && entity.getDominioTipoMaterial().getCodigo() == Dominio.TIPO_MATERIAL_PERMANENTE) {
			
			if (!UtilObjeto.isReferencia(entity.getDominioTipoContaContabil())){
				
				ValidationUtils.rejectIfEmpty(errors, "dominioTipoContaContabil", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "ADMINISTRACAODEMATERIAIS.LABEL.TIPO_CONTA_CONTABIL");
				
			}
			
			ValidationUtils.rejectIfEmpty(errors, "taxaDepreciacaoMensal", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "ADMINISTRACAODEMATERIAIS.LABEL.TAXA_DEPRECIACAO_MENSAL");
			
			ValidationUtils.rejectIfEmpty(errors, "percentualNaoDepreciavel", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "ADMINISTRACAODEMATERIAIS.LABEL.PERCENTUAL_NAO_DEPRECIACAO");
			
			ValidationUtils.rejectIfEmpty(errors, "vidaUtil", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "LABEL.VIDA_UTIL");
		}
	}

}
