package br.com.centralit.api.service.validation;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import br.com.centralit.api.model.TransferenciaContaContabil;
import br.com.centralit.api.service.OrganizacaoService;
import br.com.centralit.api.service.UsuarioService;
import br.com.centralit.framework.exception.CodigoErro;
import br.com.centralit.framework.model.Organizacao;
import br.com.centralit.framework.model.Usuario;
import br.com.centralit.framework.util.UtilDate;
import br.com.centralit.framework.util.UtilObjeto;

@Component("transferenciaContaContabilValidator")
public class TransferenciaContaContabilValidator implements Validator {

	@Autowired
	private UsuarioService usuarioService;
	
	@Autowired
	private OrganizacaoService organizacaoService;
	
    @Override
	public boolean supports(Class<?> clazz) {

		return TransferenciaContaContabil.class.isAssignableFrom(clazz);
	}

	@Override
	public void validate(Object target, Errors errors) {
        
        ValidationUtils.rejectIfEmpty(errors, "dataTransferencia", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "ADMINISTRACAODEMATERIAIS.LABEL.DATATRANSFERENCIA");
                   
        ValidationUtils.rejectIfEmpty(errors, "contaOrigem", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "ADMINISTRACAODEMATERIAIS.LABEL.CONTAORIGEM");
                   
        ValidationUtils.rejectIfEmpty(errors, "contaDestino", CodigoErro.VALIDACAO_CAMPOS_OBRIGATORIOS.getValue().toString(), "ADMINISTRACAODEMATERIAIS.LABEL.CONTADESTINO");
        
        TransferenciaContaContabil transferencia = (TransferenciaContaContabil) target;
        
        Usuario usuarioLogado = (Usuario) usuarioService.find(( (Usuario) SecurityContextHolder.getContext().getAuthentication().getPrincipal() ).getId());
		Organizacao organizacao = this.organizacaoService.find(usuarioLogado.getOrganizacao().getId());
		
		// 	verificar se os valores da nota e da soma dos itens sao iguais, caso nao seja e lanada uma exceo
     	if(UtilObjeto.isReferencia(transferencia.getDataTransferencia()) && !UtilDate.isDentroMes(transferencia.getDataTransferencia(), organizacao.getDataReferenciaVigente())) {
   			errors.rejectValue("dataTransferencia", CodigoErro.VALIDACAO_CAMPOS.getValue().toString(), "ADMINISTRACAODEMATERIAIS.VALIDACAO.DATA_TRANSFERENCIA_FORA_REFERENCIA");
   		}
	}
}

