﻿--===========================================================================--
-- ##INI :: AJUSTES INICIAIS PARA A MIGRACAO                                 --
--===========================================================================--


-- #INI :: CRIA FUNCAO QUE BUSCA BAIRROS  >> USADA NO CADASTRO DE PESSOAS
    CREATE OR REPLACE FUNCTION getidbairro 
    (
      IN p_nm_bai character varying,
      IN p_nm_cid character varying,
      IN p_cd_cid character varying,
      IN p_dt_blq timestamp without time zone,
      IN p_dt_cad timestamp without time zone
    ) RETURNS integer 
    AS
    $BODY$DECLARE
        v_id_bai bigint;
        v_nm_bai text  := COALESCE(TRIM(p_nm_bai),'(MIGRACAO)');
      BEGIN
        IF (SELECT ID FROM public.bairro WHERE UPPER(TRIM(nome)) = v_nm_bai LIMIT 1) IS NULL THEN
            v_id_bai := (SELECT COALESCE(MAX(ID),0) AS MAX FROM public.bairro) + 1;
            INSERT INTO public.bairro
            (
              id, databloqueio, datainativo, datacriacao, dataedicao, version,
              codigo, nome, inativador_id, autor_id, editor_id, 
              cidade_id
            ) 
            VALUES 
            (
              v_id_bai, p_dt_blq, NULL, p_dt_cad, LOCALTIMESTAMP, 0,
              NULL, p_nm_bai, NULL, NULL, NULL,
              (SELECT COALESCE(MAX(ID),2) FROM PUBLIC.CIDADE WHERE CODIGO=p_cd_cid LIMIT 1) -- CONSIDERAMOS BSB A CIDADE DEFAULT
            );
        END IF;
        RETURN v_id_bai;
      END;$BODY$
    LANGUAGE plpgsql;
    ALTER FUNCTION public.getidbairro(IN character varying, IN character varying, IN character varying, IN timestamp without time zone, IN timestamp without time zone)
      OWNER TO postgres;
-- #FIM :: CRIA FUNCAO QUE BUSCA BAIRROS  >> USADA NO CADASTRO DE PESSOAS


-- #INI :: CRIA FUNCAO QUE BUSCA BAIRROS  >> USADA NO CADASTRO DE PESSOAS
    CREATE OR REPLACE FUNCTION hashCode
    (
      IN p_code character varying
    ) RETURNS bigint 
    AS 
    $BODY$DECLARE
        h bigint := 0;
        i int    := 0;
      BEGIN
        FOR i IN 1..LENGTH(p_code) LOOP
            h := 31 * h + ASCII(SUBSTRING(p_code,i,1));
        END LOOP;
        RETURN h;
      END;$BODY$
    LANGUAGE plpgsql;
    ALTER FUNCTION public.hashCode(IN p_code character varying)
      OWNER TO postgres;
-- #FIM :: CRIA FUNCAO QUE BUSCA BAIRROS  >> USADA NO CADASTRO DE PESSOAS  


-- #INI :: CRIA FUNCAO QUE RETIRA ACENTOS >> USADA NA MIGRACAO DE DOMINIOS
    CREATE OR REPLACE FUNCTION retiraAcentos
    (
      IN p_texto text
    ) RETURNS text AS  
     $BODY$  
     SELECT translate($1, 
       'áàâãäåaaaÁÂÃÄÅAAAÀéèêëeeeeeEEEÉEEÊÈìíîïìiiiÌÍÎÏÌIIIóôõöoooòÒÓÔÕÖOOOùúûüuuuuÙÚÛÜUUUUçÇñÑýÝ ',
       'aaaaaaaaaAAAAAAAAAeeeeeeeeeEEEEEEEEiiiiiiiiIIIIIIIIooooooooOOOOOOOOuuuuuuuuUUUUUUUUcCnNyY_');  
     $BODY$
     LANGUAGE sql VOLATILE  
     COST 100;    
-- #FIM :: CRIA FUNCAO QUE RETIRA ACENTOS >> USADA NA MIGRACAO DE DOMINIOS


-- #INI :: ALTERACOES ESPECIFICAS PARA MIGRACAO
    ALTER TABLE public.pessoa ADD COLUMN agente_id CHARACTER VARYING(30);                    -- ** HERBERT
    ALTER TABLE public.parceiro ADD COLUMN agente_id CHARACTER VARYING(30);                  -- ** HERBERT
    ALTER TABLE public.transferencia ADD COLUMN codigoasi CHARACTER VARYING(30);             -- ** JULIANA
    ALTER TABLE public.transferenciaitem ADD COLUMN vl_unitario NUMERIC(15,4);               -- ** JULIANA
    ALTER TABLE public.transferenciaitem ADD COLUMN cd_conta CHARACTER VARYING(10);          -- ** JULIANA
    ALTER TABLE public.baixa ADD COLUMN codigo NUMERIC(10,0);                                -- ** JULIANA
    ALTER TABLE public.baixapatrimonioitem ADD COLUMN vl_unitario NUMERIC(15,4);             -- ** JULIANA
    ALTER TABLE public.observacao ADD COLUMN codigo NUMERIC(10,0);                           -- ** JULIANA
    ALTER TABLE public.contacontabilmovimento ADD COLUMN bempatrimonial_id BIGINT;           -- ** JULIANA
    ALTER TABLE public.contacontabilmovimento ADD COLUMN transferencia_id BIGINT;            -- ** JULIANA
    ALTER TABLE public.baixa ALTER COLUMN codigo TYPE BIGINT;                                -- ** HERBERT
    ALTER TABLE public.requisicaoconsumoitem ADD COLUMN sq_requisicao_item NUMERIC(10,0);    -- ** JULIANA (PRECISO DESSE CAMPO PARA FAZER LINK COM O ATENDIMENTO DA REQUISICAO)
    ALTER TABLE public.baixaalmoxarifadoitem ADD COLUMN sq_baixa_item CHARACTER VARYING(10); -- ** JULIANA (PRECISO DESSE CAMPO PARA LINKAR A BAIXA ITEM COM O ASI NA HORA DE PEGAR O VALOR NO MOVIMENTOESTOQUE)
    ALTER TABLE public.dominio ALTER COLUMN ispadrao SET DEFAULT FALSE;                      -- ** HERBERT
    ALTER TABLE public.localizacao ALTER COLUMN nome TYPE CHARACTER VARYING(250);            -- ** HERBERT
	  ALTER TABLE public.entradaalmoxarifadoitem ADD COLUMN CD_CONTA CHARACTER VARYING(15);    -- ** JULIANA (PRECISO DESSE CAMPO PARA FALAR A CONTA CONTABIL QUE O MATERIAL PERTENCIA A EPOCA DA ENTRADA
	  ALTER TABLE public.baixaalmoxarifadoitem ADD COLUMN CD_CONTA CHARACTER VARYING(15);      -- ** JULIANA (PRECISO DESSE CAMPO PARA FALAR A CONTA CONTABIL QUE O MATERIAL PERTENCIA A EPOCA DA BAIXA
	  ALTER TABLE public.devolucaoitem ADD COLUMN CD_CONTA CHARACTER VARYING(15);              -- ** JULIANA (PRECISO DESSE CAMPO PARA FALAR A CONTA CONTABIL QUE O MATERIAL PERTENCIA A EPOCA DA devolucao
	  ALTER TABLE public.requisicaoconsumoitem ADD COLUMN CD_CONTA CHARACTER VARYING(15);      -- ** JULIANA (PRECISO DESSE CAMPO PARA FALAR A CONTA CONTABIL QUE O MATERIAL PERTENCIA A EPOCA DA requisicaoconsumo
	  ALTER TABLE public.atendimentoreqconsumoitem ADD COLUMN VALOR NUMERIC(15,4);             -- ** JULIANA (PRECISO DESSE CAMPO PRA SALVAR NO MOVIMENTO ESTOQUE)

    -- VERIFICAR SE ESSES INDICES SERARO MANTIDOS NO MODELO
    CREATE INDEX ix01_entrada_cd   ON public.entrada (codigo);
    CREATE INDEX ix01_entpat_eoid  ON public.entradapatrimonio (estruturaorganizacional_id);
    CREATE INDEX ix01_estrutura_id ON public.estruturaorganizacional (id);
    CREATE INDEX ix02_estrutura_cd ON public.estruturaorganizacional (codigo);
    CREATE INDEX ix01_baixa_cd     ON public.baixa (codigo);
    CREATE INDEX ix01_baixapat_cd  ON public.baixapatrimonio (codigo);
    CREATE INDEX ix01_histpat_dtop ON public.historicobempatrimonial (dataoperacao);
    CREATE INDEX ix02_histpat_bpid ON public.historicobempatrimonial (bempatrimonial_id);
    CREATE INDEX ix01_ctacont_cd   ON public.contacontabil (codigo);
    CREATE INDEX ix01_seq_bem_perm ON asiweb.bp_depreciacao (sq_bem_perm);
    CREATE INDEX ix01_ctacmov_mesctatp ON public.contacontabilmovimento (datareferencia, contacontabil_id, tipomovimentocontacontabil_id);
    CREATE INDEX ix02_ctacmov_dtref    ON public.contacontabilmovimento (datareferencia);
    CREATE INDEX ix03_ctacmov_ctaid    ON public.contacontabilmovimento (contacontabil_id);
    CREATE INDEX ix01_ctacsdo_mescta   ON public.contacontabilsaldo (datareferencia, contacontabil_id);
    CREATE UNIQUE INDEX ix02_entrada_cdasi ON public.entrada (codigoasi);
    CREATE UNIQUE INDEX ix01_dominio_chvcd ON public.dominio (chave, codigo);
    CREATE INDEX idx01_transfitem_ctacd ON public.transferenciaitem (cd_conta);
    CREATE INDEX idx02_transfitem_undid ON public.transferenciaitem (unidadeorigem_id);
    CREATE INDEX idx03_transfitem_bemid ON public.transferenciaitem (bempatrimonial_id);
-- #FIM :: ALTERACOES ESPECIFICAS PARA MIGRACAO


BEGIN;  -- #INI :: ACRESCENTANDO DOMINIOS

    -- ----------------------------------------------- --
    --  ATRIBUIR AO STARTUPLISTENER ESSAS ATUALIZACOES --
    -- ----------------------------------------------- --

    --> ATUALIZA TIPOMOVIMENTOCONTACONTABIL
    UPDATE public.dominio 
       SET DESCRICAO = 'Entrada orçamentária'
         , NOME      = 'ENTRADA_ORCAMENTARIA'
     WHERE CHAVE     = 'tipoMovimentoContaContabil'
       AND CODIGO    = 1;

    UPDATE public.dominio 
       SET DESCRICAO = 'Baixa orçamentária'
         , NOME      = 'BAIXA_ORCAMENTARIA'
     WHERE CHAVE     = 'tipoMovimentoContaContabil'
       AND CODIGO    = 2;


    --> ATUALIZA TIPOSTATUSDEPRECIACAO
    UPDATE public.dominio 
       SET DESCRICAO = 'Depreciação finalizada'
         , NOME      = 'NAO_DEPRECIAVEL_VALOR_RESIDUAL'
     WHERE CHAVE     = 'tipoStatusDepreciacao'
       AND CODIGO    = 3;
    
    UPDATE public.dominio 
       SET DESCRICAO = 'Não Depreciável'
         , NOME      = 'NAO_DEPRECIAVEL_MARCACAO_USUARIO'
     WHERE CHAVE     = 'tipoStatusDepreciacao'
       AND CODIGO    = 4;
    -- INSERT INTO public.dominio
    --    (id, datacriacao, dataedicao, datainativo, version, chave, descricao, nome, codigo) 
    -- VALUES 
    --    (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoStatusDepreciacao', 'Não Depreciado em exercício anterior', 'NAO_DEPRECIADO_EXERCICIO_ANTERIOR', 5);


    --> ACRESCENTA TIPODOCUMENTO
    INSERT INTO public.dominio
         (id, datacriacao, dataedicao, datainativo, version, chave, descricao, nome, codigo, ispadrao) 
    VALUES 
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoDocumento', 'Processo de Baixa', 'PROCESSO_BAIXA', 40, false),
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoDocumento', 'Programa de Trabalho', 'PROGRAMA_TRABALHO', 41, false);

    --> ACRESCENTA TIPOINVENTARIO
    INSERT INTO public.dominio 
         (id, datacriacao, dataedicao, datainativo, version, chave, descricao, nome, codigo, ispadrao) 
    VALUES 
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoInventario', 'POR U.A.', 'POR_UA', 6, false),
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoInventario', 'POR U.L.', 'POR_UL', 7, false);
       
    --> CRIA TIPOOBJETIVOINVENTARIO
    DELETE FROM public.dominio WHERE CHAVE='tipoObjetivoInventario';
    INSERT INTO public.dominio 
         (id, datacriacao, dataedicao, datainativo, version, chave, descricao, nome, codigo, ispadrao) 
    VALUES 
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoObjetivoInventario', 'INVENTÁRIO GERAL', 'INVENTARIO_GERAL', 1, false),
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoObjetivoInventario', 'CRIAÇÃO DE ÓRGÃO/UG', 'CRIACAO_ORGAO_UG', 2, false),
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoObjetivoInventario', 'EXTINÇAO DE ÓRGÃO/UG', 'EXTINCAO_ORGAO_UG', 3, false),  
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoObjetivoInventario', 'PARA LEVANTAMENTO', 'PARA_LEVANTAMENTO', 4, false),     
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoObjetivoInventario', 'PARA CONFERÊNCIA', 'PARA_CONFERENCIA', 5, false),     
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoObjetivoInventario', 'PARA INSPEÇÃO/AVALIAÇÃO', 'PARA_INSPECAO_AVALIACAO', 6, false),
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoObjetivoInventario', 'PASSAGEM DE CARGA', 'PASSAGEM_CARGA', 7, false),           
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoObjetivoInventario', 'PARA BAIXA', 'PARA_BAIXA', 8, false);           

    --> ACRESCENTA TIPOOBJETIVOSAIDATEMPORARIA
    INSERT INTO public.dominio
         (id, datacriacao, dataedicao, datainativo, version, chave, descricao, nome, codigo, ispadrao) 
    VALUES 
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoObjetivoSaidaTemporaria', 'Manutenção', 'MANUTENCAO', 6, false),
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoObjetivoSaidaTemporaria', 'Exposição externa', 'EXPOSICAO_EXTERNA', 9, false),
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoObjetivoSaidaTemporaria', 'Serviço', 'SERVIÇO', 18, false),
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoObjetivoSaidaTemporaria', 'Orçamento para execução de serviço', 'ORCAMENTO_EXECUCAO_SERVICO"', 20, false);

    --> ACRESCENTA TIPOBAIXA
    INSERT INTO public.dominio
         (id, datacriacao, dataedicao, datainativo, version, chave, descricao, nome, codigo, ispadrao) 
    VALUES 
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoBaixa', 'Incineração', 'INCINERACAO', 16, false),
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoBaixa', 'Desmembramento/Atualizacao', 'DESMEMBRAMENTO/ATUALIZACAO', 17, false),
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoBaixa', 'Devolução', 'DEVOLUCAO', 18, false);

    --> ACRESCENTA TIPOBAIXAALMOXARIFADO
    INSERT INTO public.dominio
         (id, datacriacao, dataedicao, datainativo, version, chave, descricao, nome, codigo, ispadrao) 
    VALUES 
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoBaixaAlmoxarifado', 'Desuso', 'DESUSO', 7, false);

    --> ACRESCENTA TIPORECEBIMENTO
    INSERT INTO public.dominio
         (id, datacriacao, dataedicao, datainativo, version, chave, descricao, nome, codigo, ispadrao) 
    VALUES
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoRecebimento', 'Devolução', 'DEVOLUCAO', 12, false),
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoRecebimento', 'Incorporação para integralização ou aumento de capital', 'INCORPORACAO_INTEGRALIZAÇAO_OU_AUMENTO_CAPITAL', 13, false),
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoRecebimento', 'Adjudicação por executivo fiscal', 'ADJUDICACAO_EXECUTIVO_FISCAL', 14, false),
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoRecebimento', 'Bens de terceiros', 'BENS_TERCEIROS', 15, false);

    --> ACRESCENTA TIPORECEBIMENTOALMOXARIFADO
    INSERT INTO public.dominio
         (id, datacriacao, dataedicao, datainativo, version, chave, descricao, nome, codigo, ispadrao) 
    VALUES
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoRecebimentoAlmoxarifado', 'Bens de terceiros', 'BENS_TERCEIROS', 7, false);

    --> ACRESCENTA TIPOSITUACAOFISICA
    INSERT INTO public.dominio
         (id, datacriacao, dataedicao, datainativo, version, chave, descricao, nome, codigo, ispadrao) 
    VALUES
         (NEXTVAL('hibernate_sequence'), LOCALTIMESTAMP, LOCALTIMESTAMP, NULL, 0, 'tipoSituacaoFisica', 'Péssimo', 'PESSIMO', 18, false);


    --> CRIA TABELA MAPEAMENTO DE DOMINIOS ASI X GRP
    /*
       ATRIBUTO_GRP               CHAVE_DE_PARA                    CODIGO_ASI                   TABELA_DOMINIO_ASI
       -------------------------  -------------------------------- ---------------------------- ------------------------------------
       dominiotiporecebimento_id  'tipoRecebimento'                E1.CD_TP_FORMA_OP            asi_homologa..bp_tp_forma_op
       dominiotipobem_id          'tipoBem'                        E1.CD_TP_BEM                 asi_homologa..bp_tp_bem
       dominiotipodocumento_id    'tipoDocumento'                  D.CD_TP_DOCUMENTO            asi_homologa..cr_tp_documento
       dominiosituacaofisica_id   'tipoSituacaoFisica'             B.CD_TP_SIT_FISICA           asi_homologa..bp_tp_sit_fisica
       dominiostatus_id           'tipoStatusBem'                  B.CD_TP_STATUS               asi_homologa..bp_tp_status
       dominiotipoobjetivo_id     'tipoObjetivoSaidaTemporaria'    TRANSF.CD_MOTIVO_SAIDA_TEMP  asi_homologa..bp_motivo_saida_temp
       dominiosituacaobaixa_id    'tipoSituacaoBaixa'              BAIXA.CD_TP_SIT_BAIXA        asi_homologa..bp_tp_sit_baixa
       dominiotipobaixa_id        'tipoBaixa'                      BAIXA.CD_TP_FORMA_OP         asi_homologa..bp_tp_forma_op
       dominiotipodocumento_id    'tipoDocumento'                  DOC.CD_TP_DOCUMENTO          asi_homologa..cr_tp_documento

       dominiotiporecebimento_id  'tipoRecebimentoAlmoxarifado'    NR.CD_TIPO_MOVIMENTO         asi_homologa..ax_tipo_movimento
       dominiotipodocumento_id    'tipoDocumento'                  DOC.CD_TP_DOCUMENTO          asi_homologa..cr_tp_documento
       dominiotipobaixa_id        'tipoBaixaAlmoxarifado'          BA.CD_TIPO_MOVIMENTO         asi_homologa..ax_tipo_movimento
       dominiotipodocumento_id    'tipoDocumento'                  DOC.CD_TP_DOCUMENTO          asi_homologa..cr_tp_documento
       tipostatusrequisicao_id    'tipoStatusRequisicao'           R.CD_SITUACAO_REQ_ATUAL      asi_homologa..ax_situacao_req
    */

    CREATE TABLE public.dominio_asi_grp
    AS
    SELECT id     AS id_grp
         , chave  AS tp_grp
         , codigo AS cd_grp
         , nome   AS nm_grp
         , chave  AS cd_asi
         , nome   AS nm_asi
      FROM public.dominio
     WHERE chave='tipoAbrangencia'
        OR chave='tipoPorte'
        OR chave='tipoBem'
        OR chave='tipoRecebimento'
        OR chave='tipoRecebimentoAlmoxarifado'
        OR chave='tipoSituacaoFisica'
        OR chave='tipoStatusBem'
        OR chave='tipoDocumento'
        OR chave='tipoObjetivoSaidaTemporaria'
        OR chave='tipoSituacaoBaixa'
        OR chave='tipoBaixa'
        OR chave='tipoBaixaAlmoxarifado'
        OR chave='tipoStatusRequisicao'
        OR chave='tipoInventario'
        OR chave='tipoObjetivoInventario';

    --> INICIALIZA OS VALORES [ASI] DO MAPEAMENTO
    UPDATE public.dominio_asi_grp 
    SET cd_asi = NULL, nm_asi = NULL;

    /*
    SELECT CONCAT (
    '    UPDATE public.dominio_asi_grp SET cd_asi = ', chr(39), cd_tp_forma_op, chr(39), ', nm_asi = ', chr(39), nm_tp_forma_op, chr(39), chr(13),
    '    WHERE  tp_grp = ', chr(39), 'tipoRecebimento', chr(39), ' AND cd_grp = ', cd_tp_forma_op::bigint
    )
    FROM  asiweb.bp_tp_forma_op
    ORDER BY cd_tp_operacao, cd_tp_forma_op
    */

    --===========================================================================--
    --> DE-PARA TIPOABRANGENCIA  -- "asiweb.cr_tp_for_abrang"
    --===========================================================================--
    UPDATE public.dominio_asi_grp SET cd_asi = '01', nm_asi = 'LOCAL' 
    WHERE  tp_grp = 'tipoAbrangencia' AND cd_grp = 1;

    UPDATE public.dominio_asi_grp SET cd_asi = '02', nm_asi = 'REGIONAL' 
    WHERE  tp_grp = 'tipoAbrangencia' AND cd_grp = 2;

    UPDATE public.dominio_asi_grp SET cd_asi = '03', nm_asi = 'NACIONAL' 
    WHERE  tp_grp = 'tipoAbrangencia' AND cd_grp = 3;


    --===========================================================================--
    --> DE-PARA TIPOPORTE  -- "asiweb.cr_tp_for_porte"
    --===========================================================================--
    UPDATE public.dominio_asi_grp SET cd_asi = '01', nm_asi = 'MICRO-EMPRESA' 
    WHERE  tp_grp = 'tipoPorte' AND cd_grp = 1;

    UPDATE public.dominio_asi_grp SET cd_asi = '02', nm_asi = 'PEQUENA' 
    WHERE  tp_grp = 'tipoPorte' AND cd_grp = 2;

    UPDATE public.dominio_asi_grp SET cd_asi = '03', nm_asi = 'MÉDIA' 
    WHERE  tp_grp = 'tipoPorte' AND cd_grp = 3;

    UPDATE public.dominio_asi_grp SET cd_asi = '04', nm_asi = 'GRANDE' 
    WHERE  tp_grp = 'tipoPorte' AND cd_grp = 4;


    --===========================================================================--
    --> DE-PARA TIPOBEM  -- "asiweb.bp_tp_bem"
    --===========================================================================--
    /*
      SELECT * FROM asiweb.bp_tp_bem 
      WHERE CD_TP_BEM IN (
           SELECT DISTINCT E1.CD_TP_BEM FROM ASIWEB.BP_ENTRADA E1
      )
      ORDER BY 1, 2
    */
    UPDATE public.dominio_asi_grp SET cd_asi = '01', nm_asi = 'PRÓPRIO CONTABILIZADO' 
    WHERE  tp_grp = 'tipoBem' AND cd_grp = 1;

    UPDATE public.dominio_asi_grp SET cd_asi = '02', nm_asi = 'PRÓPRIO CONTROLADO' 
    WHERE  tp_grp = 'tipoBem' AND cd_grp = 2;

    UPDATE public.dominio_asi_grp SET cd_asi = '03', nm_asi = 'DE TERCEIROS' 
    WHERE  tp_grp = 'tipoBem' AND cd_grp = 3;


    --===========================================================================--
    --> DE-PARA TIPORECEBIMENTO  -- "asiweb.bp_tp_forma_op"
    --===========================================================================--
    /*
      SELECT * FROM asiweb.bp_tp_forma_op 
      WHERE CD_TP_FORMA_OP IN (
           SELECT DISTINCT E1.CD_TP_FORMA_OP FROM ASIWEB.BP_ENTRADA E1
      )
      ORDER BY 1, 2
    */
    UPDATE public.dominio_asi_grp SET cd_asi = '01', nm_asi = 'COMPRA'
    WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = 1;

    UPDATE public.dominio_asi_grp SET cd_asi = '02', nm_asi = 'COMPRA EXTRA ORÇAMENTÁRIA'
    WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = 2;

    UPDATE public.dominio_asi_grp SET cd_asi = '03', nm_asi = 'DOAÇAO'
    WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = 3;

    UPDATE public.dominio_asi_grp SET cd_asi = '04', nm_asi = 'CESSAO DE USO'
    WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = 4;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '05', nm_asi = 'PERMUTA'
    -- WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = #;

    UPDATE public.dominio_asi_grp SET cd_asi = '06', nm_asi = 'TRANSFERENCIA FORA DO SISTEMA'
    WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = 5;

    UPDATE public.dominio_asi_grp SET cd_asi = '07', nm_asi = 'FABRICAÇÃO PRÓPRIA'
    WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = 7;

    UPDATE public.dominio_asi_grp SET cd_asi = '08', nm_asi = 'INCORPORAÇAO PARA INTEGRALIZAÇAO OU AUMENTO DE CAPITAL'
    WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = 13;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '09', nm_asi = 'REPOSIÇAO'
    -- WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = #;

    UPDATE public.dominio_asi_grp SET cd_asi = '10', nm_asi = 'ADJUDICACAO POR EXECUTIVO FISCAL'
    WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = 14;


    --===========================================================================--
    --> DE-PARA TIPORECEBIMENTOALMOXARIFADO  -- "asiweb.ax_tipo_movimento"
    --===========================================================================--
    /*
      SELECT * FROM ASIWEB.AX_TIPO_MOVIMENTO 
      WHERE CD_TIPO_MOVIMENTO IN (
           SELECT DISTINCT NR.CD_TIPO_MOVIMENTO FROM ASIWEB.AX_NOTA_REC NR
      )
      ORDER BY 1, 2
    */
    UPDATE public.dominio_asi_grp SET cd_asi = '001', nm_asi = 'COMPRA'
    WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = 1;

    UPDATE public.dominio_asi_grp SET cd_asi = '002', nm_asi = 'DOAÇÃO - ENTRADA'
    WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = 3;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '004', nm_asi = 'REQUISIÇÃO MATERIAL'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;

    UPDATE public.dominio_asi_grp SET cd_asi = '005', nm_asi = 'DEVOLUÇÃO'
    WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = 6;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '007', nm_asi = 'EXTRAVIO'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '008', nm_asi = 'ENTRADA POR TRANSFERENCIA'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '009', nm_asi = 'ENTRADA POR ALTERAÇÃO DE CONTA'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '010', nm_asi = 'SAIDA POR ALTERAÇÃO DE CONTA'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '011', nm_asi = 'ALTERAÇÃO DE CONTA'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '012', nm_asi = 'ESTORNO/AJUSTE DE MOV.'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;

    UPDATE public.dominio_asi_grp SET cd_asi = '013', nm_asi = 'CESSÃO - ENTRADA'
    WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = 4;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '014', nm_asi = 'FABRICAÇÃO PRÓPRIA'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '015', nm_asi = 'PERMUTA - ENTRADA'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;

    UPDATE public.dominio_asi_grp SET cd_asi = '016', nm_asi = 'COMPRA EXTRA-ORÇAMENTÁRIA'
    WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = 2;

    UPDATE public.dominio_asi_grp SET cd_asi = '017', nm_asi = 'BENS DE TERCEIROS'
    WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = 7;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '018', nm_asi = 'SUPRIMENTO DE FUNDOS'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '027', nm_asi = 'ESTORNO - ORCAMENTARIO'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '029', nm_asi = 'ENTRADA - IMPLANTACAO'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '032', nm_asi = 'ENTRADA - TRANSFERENCIA END.'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '033', nm_asi = 'D02'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '034', nm_asi = 'AQUISIÇÃO EXC. ANT.'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '035', nm_asi = 'TRANSF. ENTRE ÓRGÃO'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '100', nm_asi = 'COMPRA INTEGRADA'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '108', nm_asi = 'EMPRÉSTIMO'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;


    --===========================================================================--
    --> DE-PARA TIPOSITUACAOFISICA  -- "asiweb.bp_tp_sit_fisica"
    --===========================================================================--
    /*
      SELECT * FROM ASIWEB.BP_TP_SIT_FISICA 
      WHERE CD_TP_SIT_FISICA IN (
           SELECT DISTINCT B.CD_TP_SIT_FISICA FROM ASIWEB.BP_BEM B
      )
      ORDER BY 1, 2
    */
    UPDATE public.dominio_asi_grp SET cd_asi = '01', nm_asi = 'BOM'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 1;

    UPDATE public.dominio_asi_grp SET cd_asi = '02', nm_asi = 'REGULAR'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 2;

    UPDATE public.dominio_asi_grp SET cd_asi = '03', nm_asi = 'INSERVÍVEL'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 3;

    UPDATE public.dominio_asi_grp SET cd_asi = '04', nm_asi = 'PRECÁRIO'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 4;

    UPDATE public.dominio_asi_grp SET cd_asi = '05', nm_asi = 'RUIM'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 5;

    UPDATE public.dominio_asi_grp SET cd_asi = '06', nm_asi = 'ANTI ECONÔMICO'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 6;

    UPDATE public.dominio_asi_grp SET cd_asi = '07', nm_asi = 'DANIFICADO'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 7;

    UPDATE public.dominio_asi_grp SET cd_asi = '08', nm_asi = 'OBSOLETO'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 8;

    UPDATE public.dominio_asi_grp SET cd_asi = '09', nm_asi = 'OCIOSO'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 9;

    UPDATE public.dominio_asi_grp SET cd_asi = '10', nm_asi = 'RECUPERÁVEL'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 10;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '11', nm_asi = 'IRRECUPERÁVEL'
    -- WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 11;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '12', nm_asi = 'ÓTIMO'
    -- WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 12;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '13', nm_asi = 'COM DEFEITO'
    -- WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 13;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '14', nm_asi = 'CONSERVADO'
    -- WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 14;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '15', nm_asi = 'ESTRAGADO'
    -- WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 15;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '16', nm_asi = 'INUTILIZAVEL'
    -- WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 16;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '17', nm_asi = 'NOVO'
    -- WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 17;

    UPDATE public.dominio_asi_grp SET cd_asi = '18', nm_asi = 'PESSIMO'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 18;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '19', nm_asi = 'QUEBRADO'
    -- WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 19;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '20', nm_asi = 'QUEIMADO'
    -- WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 20;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '21', nm_asi = 'RASGADO'
    -- WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 21;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '22', nm_asi = 'REFORMADO'
    -- WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 22;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '23', nm_asi = 'SUCATA'
    -- WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 23;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '24', nm_asi = 'VELHO'
    -- WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 24;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '25', nm_asi = 'USADO'
    -- WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 25;


    --===========================================================================--
    --> DE-PARA TIPOSTATUSBEM  -- "asiweb.bp_tp_status"
    --===========================================================================--
    /*
      SELECT * FROM ASIWEB.BP_TP_STATUS 
      WHERE CD_TP_STATUS IN (
           SELECT DISTINCT B.CD_TP_STATUS FROM ASIWEB.BP_BEM B
      )
      ORDER BY 1, 2    
    */
    UPDATE public.dominio_asi_grp SET cd_asi = '01', nm_asi = 'UTILIZADO'
    WHERE  tp_grp = 'tipoStatusBem' AND cd_grp = 1;

    UPDATE public.dominio_asi_grp SET cd_asi = '02', nm_asi = 'BAIXADO'
    WHERE  tp_grp = 'tipoStatusBem' AND cd_grp = 2;

    UPDATE public.dominio_asi_grp SET cd_asi = '03', nm_asi = 'DISPONÍVEL'
    WHERE  tp_grp = 'tipoStatusBem' AND cd_grp = 3;

    UPDATE public.dominio_asi_grp SET cd_asi = '04', nm_asi = 'EM PROCESSO DE BAIXA'
    WHERE  tp_grp = 'tipoStatusBem' AND cd_grp = 4;

    UPDATE public.dominio_asi_grp SET cd_asi = '05', nm_asi = 'EM PROCESSO DE TRANSFERÊNCIA'
    WHERE  tp_grp = 'tipoStatusBem' AND cd_grp = 5;

    UPDATE public.dominio_asi_grp SET cd_asi = '06', nm_asi = 'EM SAÍDA TEMPORÁRIA'
    WHERE  tp_grp = 'tipoStatusBem' AND cd_grp = 6;

    UPDATE public.dominio_asi_grp SET cd_asi = '07', nm_asi = 'EXTRAVIADO'
    WHERE  tp_grp = 'tipoStatusBem' AND cd_grp = 7;

    UPDATE public.dominio_asi_grp SET cd_asi = '08', nm_asi = 'INDISPONÍVEL'
    WHERE  tp_grp = 'tipoStatusBem' AND cd_grp = 8;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '09', nm_asi = 'SINDICÂNCIA'
    -- WHERE  tp_grp = 'tipoStatusBem' AND cd_grp = 9;


    --===========================================================================--
    --> DE-PARA TIPODOCUMENTO -- "asiweb.cr_tp_documento"
    --===========================================================================--
    /*
      SELECT * FROM ASIWEB.CR_TP_DOCUMENTO 
      WHERE CD_TP_DOCUMENTO IN (
           SELECT DISTINCT D.CD_TP_DOCUMENTO FROM ASIWEB.BP_ENT_DOC D
      )
      UNION
      SELECT * FROM ASIWEB.CR_TP_DOCUMENTO 
      WHERE CD_TP_DOCUMENTO IN (
           SELECT DISTINCT DOC.CD_TP_DOCUMENTO FROM ASIWEB.BP_BAIXA_DOC DOC
      )
      ORDER BY 1, 2
    */
    UPDATE public.dominio_asi_grp SET cd_asi = '001', nm_asi = 'EMPENHO'
    WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 1;

    UPDATE public.dominio_asi_grp SET cd_asi = '002', nm_asi = 'PROCESSO DE ENTRADA'
    WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 2;

    UPDATE public.dominio_asi_grp SET cd_asi = '003', nm_asi = 'NOTA FISCAL'
    WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 3;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '004', nm_asi = 'REQUISICAO'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 9;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '005', nm_asi = 'CONTRATO'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 10;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '006', nm_asi = 'TERMO DE DOACAO'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 4;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '007', nm_asi = 'TERMO DE PERMUTA'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = #;

    UPDATE public.dominio_asi_grp SET cd_asi = '008', nm_asi = 'PROCESSO DE BAIXA'
    WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 40;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '009', nm_asi = 'TERMO DE CESSAO'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 5;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '010', nm_asi = 'Nº DO PROCESSO'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 12;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '011', nm_asi = 'NATUREZA DE DESPESA'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '012', nm_asi = 'PROCESSO DE COMPRA'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '013', nm_asi = 'EVENTO CONTÁBIL'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '014', nm_asi = 'MEMORANDO'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 13;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '015', nm_asi = 'PORTARIA'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 14;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '016', nm_asi = 'INCORPORACÃO'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '017', nm_asi = 'TERMO D ADESÃO'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '018', nm_asi = 'DESPACHO'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 15;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '019', nm_asi = 'OCORRENCIA'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 16;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '020', nm_asi = 'NOTA DE SISTEMA'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 17;

    UPDATE public.dominio_asi_grp SET cd_asi = '026', nm_asi = 'PROGRAMA DE TRABALHO'
    WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 41;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '120', nm_asi = 'EDITAL'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '121', nm_asi = 'OFÍCIO'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = #;


    --===========================================================================--
    --> DE-PARA TIPOOBJETIVOSAIDATEMPORARIA -- "asiweb.bp_motivo_saida_temp"
    --===========================================================================--
    /*
      SELECT * FROM ASIWEB.BP_MOTIVO_SAIDA_TEMP 
      WHERE CD_MOTIVO_SAIDA_TEMP IN (
           SELECT DISTINCT TRANSF.CD_MOTIVO_SAIDA_TEMP FROM ASIWEB.BP_TRANSF_EXTERNA TRANSF
      )
      ORDER BY 1, 2
    */
    UPDATE public.dominio_asi_grp SET cd_asi = '01', nm_asi = 'COMODATO'
    WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = 1;

    UPDATE public.dominio_asi_grp SET cd_asi = '04', nm_asi = 'EMPRÉSTIMO'
    WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = 3;

    UPDATE public.dominio_asi_grp SET cd_asi = '05', nm_asi = 'EVENTO'
    WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = 4;

    UPDATE public.dominio_asi_grp SET cd_asi = '06', nm_asi = 'MANUTENÇÃO'
    WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = 6;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '08', nm_asi = 'SERVIÇO EXTERNO'
    -- WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = #;

    UPDATE public.dominio_asi_grp SET cd_asi = '09', nm_asi = 'EXPOSIÇÃO EXTERNA'
    WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = 9;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '14', nm_asi = 'EMPRÉSTIMO'
    -- WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '15', nm_asi = 'EVENTO'
    -- WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '16', nm_asi = 'MANUTENÇÃO'
    -- WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = #;

    UPDATE public.dominio_asi_grp SET cd_asi = '18', nm_asi = 'SERVIÇO'
    WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = 18;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '19', nm_asi = 'RESIDÊNCIA'
    -- WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = #;

    UPDATE public.dominio_asi_grp SET cd_asi = '20', nm_asi = 'ORÇAMENTO PARA EXECUÇÃO DE SERVIÇO'
    WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = 20;


    --===========================================================================--
    --> DE-PARA TIPOSITUACAOBAIXA  -- "asiweb.bp_tp_sit_baixa"
    --===========================================================================--
    /*
      SELECT * FROM ASIWEB.BP_TP_SIT_BAIXA 
      WHERE CD_TP_SIT_BAIXA IN (
           SELECT DISTINCT BAIXA.CD_TP_SIT_BAIXA FROM ASIWEB.BP_BAIXA BAIXA
      )
      ORDER BY 1, 2    
    */
    UPDATE public.dominio_asi_grp SET cd_asi = '01', nm_asi = 'EM ANDAMENTO'
    WHERE  tp_grp = 'tipoSituacaoBaixa' AND cd_grp = 1;

    UPDATE public.dominio_asi_grp SET cd_asi = '02', nm_asi = 'AUTORIZADA'
    WHERE  tp_grp = 'tipoSituacaoBaixa' AND cd_grp = 2;

    UPDATE public.dominio_asi_grp SET cd_asi = '03', nm_asi = 'NÃO AUTORIZADA'
    WHERE  tp_grp = 'tipoSituacaoBaixa' AND cd_grp = 3;


    --===========================================================================--
    --> DE-PARA TIPOBAIXA  -- "asiweb.bp_tp_forma_op"
    --===========================================================================--
    /*
      SELECT * FROM ASIWEB.BP_TP_FORMA_OP 
      WHERE CD_TP_FORMA_OP IN (
           SELECT DISTINCT BAIXA.CD_TP_FORMA_OP FROM ASIWEB.BP_BAIXA BAIXA
      )
      ORDER BY 1, 2
    */
    UPDATE public.dominio_asi_grp SET cd_asi = '01', nm_asi = 'INCINERAÇÃO'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 16;

    UPDATE public.dominio_asi_grp SET cd_asi = '05', nm_asi = 'DESMEMBRAMENTO/ATUALIZACAO'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 17;

    UPDATE public.dominio_asi_grp SET cd_asi = '15', nm_asi = 'DOAÇAO'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 2;

    UPDATE public.dominio_asi_grp SET cd_asi = '16', nm_asi = 'CESSAO'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 15;

    UPDATE public.dominio_asi_grp SET cd_asi = '17', nm_asi = 'EXTRAVIO OU SINISTRO'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 4;

    UPDATE public.dominio_asi_grp SET cd_asi = '18', nm_asi = 'PERMUTA'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 5;

    UPDATE public.dominio_asi_grp SET cd_asi = '20', nm_asi = 'CADASTRAMENTO INDEVIDO'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 7;

    UPDATE public.dominio_asi_grp SET cd_asi = '26', nm_asi = 'OUTROS MOTIVOS'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 10;

    UPDATE public.dominio_asi_grp SET cd_asi = '79', nm_asi = 'DEVOLUÇÃO'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 18;


    --===========================================================================--
    --> DE-PARA TIPOBAIXAALMOXARIFADO  -- "asiweb.ax_tipo_movimento"
    --===========================================================================--
    /*
      SELECT * FROM ASIWEB.AX_TIPO_MOVIMENTO 
      WHERE CD_TIPO_MOVIMENTO IN (
           SELECT DISTINCT BA.CD_TIPO_MOVIMENTO FROM ASIWEB.AX_BAIXA BA
      )
      ORDER BY 1, 2
    */
    -- UPDATE public.dominio_asi_grp SET cd_asi = '001', nm_asi = 'COMPRA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '003', nm_asi = 'DOAÇÃO - ENTRADA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '004', nm_asi = 'REQUISIÇÃO MATERIAL'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '005', nm_asi = 'DEVOLUÇÃO'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '006', nm_asi = 'REQUISIÇÃO POR TRANSFERENCIA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    UPDATE public.dominio_asi_grp SET cd_asi = '007', nm_asi = 'EXTRAVIO'
    WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = 2;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '008', nm_asi = 'ENTRADA POR TRANSFERENCIA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '009', nm_asi = 'ENTRADA POR ALTERAÇÃO DE CONTA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '010', nm_asi = 'SAIDA POR ALTERAÇÃO DE CONTA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '011', nm_asi = 'ALTERAÇÃO DE CONTA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '012', nm_asi = 'ESTORNO/AJUSTE DE MOV.'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '013', nm_asi = 'CESSÃO - ENTRADA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '014', nm_asi = 'FABRICAÇÃO PRÓPRIA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '015', nm_asi = 'PERMUTA - ENTRADA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '016', nm_asi = 'COMPRA EXTRA-ORÇAMENTÁRIA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '018', nm_asi = 'SUPRIMENTO DE FUNDOS'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '020', nm_asi = 'LEILÃO'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    UPDATE public.dominio_asi_grp SET cd_asi = '021', nm_asi = 'DOAÇÃO - BAIXA'
    WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = 1;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '022', nm_asi = 'PERMUTA - BAIXA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    UPDATE public.dominio_asi_grp SET cd_asi = '023', nm_asi = 'CESSAO - BAIXA'
    WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = 6;

    UPDATE public.dominio_asi_grp SET cd_asi = '024', nm_asi = 'CADASTRO INDEVIDO'
    WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = 3;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '025', nm_asi = 'ALIENAÇÃO'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '026', nm_asi = 'BAIXA-LEVANTAMENTO'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '027', nm_asi = 'ESTORNO - ORCAMENTARIO'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '028', nm_asi = 'ESTORNO SAIDA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '029', nm_asi = 'ENTRADA - IMPLANTACAO'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '030', nm_asi = 'SALDO ANTERIOR'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '031', nm_asi = 'SAIDA - TRANSFERENCIA END.'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '032', nm_asi = 'ENTRADA - TRANSFERENCIA END.'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '033', nm_asi = 'D02'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '034', nm_asi = 'AQUISIÇÃO EXC. ANT.'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '035', nm_asi = 'TRANSF. ENTRE ÓRGÃO'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '036', nm_asi = 'DIVERSAS'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '037', nm_asi = 'DEVOLUÇÃO AO FORNECEDOR'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '044', nm_asi = 'INSERVÍVEL'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '045', nm_asi = 'PERDA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    UPDATE public.dominio_asi_grp SET cd_asi = '046', nm_asi = 'DESUSO'
    WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = 7;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '047', nm_asi = 'VALIDADE'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = 8;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '100', nm_asi = 'COMPRA INTEGRADA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    -- UPDATE public.dominio_asi_grp SET cd_asi = '108', nm_asi = 'EMPRÉSTIMO'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;

    
    --===========================================================================--
    --> DE-PARA TIPOSTATUSREQUISICAO  -- "asiweb.ax_situacao_req"
    --===========================================================================--
    /*
      SELECT * FROM ASIWEB.AX_SITUACAO_REQ 
      WHERE CD_SITUACAO_REQ IN (
           SELECT DISTINCT R.CD_SITUACAO_REQ_ATUAL FROM ASIWEB.AX_REQUISICAO R
      )
      ORDER BY 1, 2
    */
    -- UPDATE public.dominio_asi_grp SET cd_asi = '001', nm_asi = 'PRONTA PARA AUTORIZAR'
    -- WHERE  tp_grp = 'tipoStatusRequisicao' AND cd_grp = #;

    UPDATE public.dominio_asi_grp SET cd_asi = '002', nm_asi = 'PRONTA PARA ATENDER'
    WHERE  tp_grp = 'tipoStatusRequisicao' AND cd_grp = 2;

    UPDATE public.dominio_asi_grp SET cd_asi = '003', nm_asi = 'ATENDIDA'
    WHERE  tp_grp = 'tipoStatusRequisicao' AND cd_grp = 5;
    -- WHERE  tp_grp = 'tipoStatusRequisicao' AND cd_grp = 4; -- REDMINE #4715 :: ATENDIDA >> FINALIZADA 

    -- UPDATE public.dominio_asi_grp SET cd_asi = '004', nm_asi = 'SEPARADA'
    -- WHERE  tp_grp = 'tipoStatusRequisicao' AND cd_grp = #;

    UPDATE public.dominio_asi_grp SET cd_asi = '005', nm_asi = 'DEVOLVIDA'
    WHERE  tp_grp = 'tipoStatusRequisicao' AND cd_grp = 5;
    -- WHERE  tp_grp = 'tipoStatusRequisicao' AND cd_grp = 6; -- REDMINE #4715 :: DEVOLVIDA >> FINALIZADA 

    -- UPDATE public.dominio_asi_grp SET cd_asi = '006', nm_asi = 'PRONTA PARA RECEBER'
    -- WHERE  tp_grp = 'tipoStatusRequisicao' AND cd_grp = #;

    UPDATE public.dominio_asi_grp SET cd_asi = '007', nm_asi = 'FINALIZADA'
    WHERE  tp_grp = 'tipoStatusRequisicao' AND cd_grp = 5;
COMMIT; -- #FIM :: ACRESCENTANDO DOMINIOS


-- #INI :: CRIAÇÃO TABELA AUDITORIA DADOSBEMPATRIMONIAL :: NÃO ANOTADA NO JPA
    -- Table: dadosbempatrimonial_aud
    
    -- DROP TABLE dadosbempatrimonial_aud;
    
    CREATE TABLE dadosbempatrimonial_aud
    (
      id                            bigint NOT NULL,
      version                       bigint,
      responsavel_id                bigint,
      detentor_id                   bigint,
      databloqueio                  date,
      datainativo                   date,
      datacriacao                   timestamp without time zone,
      dataedicao                    timestamp without time zone,
      copiadedadosfinal             boolean,
      numeropatrimonial             character varying(50),
      valorliquido                  numeric(20,4),
      autor_id                      bigint,
      editor_id                     bigint,
      bempatrimonial_id             bigint,
      bempatrimonialprincipal_id    bigint,
      dominiolocalinconsistencia_id bigint,
      dominiosituacaofisica_id      bigint,
      dominiostatus_id              bigint,
      dominiostatusinventariobem_id bigint,
      unidadeatual_id               bigint,
      inventariobempatrimonial_id   bigint,
      inventariounidade_id          bigint,
      material_id                   bigint
    );
-- #FIM :: CRIAÇÃO TABELA AUDITORIA DADOSBEMPATRIMONIAL :: NÃO ANOTADA NO JPA


--===========================================================================--
-- ##FIM :: AJUSTES INICIAIS PARA A MIGRACAO                                 --
--===========================================================================--