--===========================================================================--
-- ##INI :: AJUSTES INICIAIS PARA A MIGRACAO                                 --
--===========================================================================--
SET NOCOUNT ON


-- #INI :: CRIA FUNCAO QUE BUSCA BAIRROS >> USADA NO CADASTRO DE PESSOAS
    BEGIN TRANSACTION
    GO 

    IF OBJECT_ID('dbo.CreateBairro') IS NOT NULL
        DROP PROCEDURE dbo.CreateBairro;
    GO

    CREATE PROCEDURE dbo.CreateBairro
    (
      @p_nm_bai nvarchar,
      @p_nm_cid nvarchar,
      @p_cd_cid nvarchar,
      @p_dt_blq datetime,
      @p_dt_cad datetime
    )
    AS
    BEGIN
      DECLARE @v_nm_bai nvarchar = COALESCE(RTRIM(LTRIM(@p_nm_bai)),'(MIGRACAO)')
      DECLARE @v_id_bai bigint = (SELECT TOP 1 ID FROM [dbo].[Bairro] WHERE UPPER(RTRIM(LTRIM(nome))) = @v_nm_bai)
      
      IF @v_id_bai IS NULL
      BEGIN
          INSERT INTO [dbo].[Bairro]
               ( databloqueio, datainativo, datacriacao, dataedicao, version,
                 codigo, nome, inativador_id, autor_id, editor_id, 
                 cidade_id ) 
          VALUES 
               ( @p_dt_blq, NULL, @p_dt_cad, GETDATE(), 0,
                 NULL, @p_nm_bai, NULL, NULL, NULL,
                 (SELECT TOP 1 COALESCE(MAX(ID),2) FROM [dbo].[Cidade] WHERE CODIGO=@p_cd_cid) -- CONSIDERAMOS BSB A CIDADE DEFAULT
               );
      END
    END;
    GO
  
    IF OBJECT_ID('dbo.GetIdBairro') IS NOT NULL
        DROP FUNCTION dbo.GetIdBairro;
    GO

    CREATE FUNCTION dbo.GetIdBairro
    (
      @p_nm_bai nvarchar,
      @p_nm_cid nvarchar,
      @p_cd_cid nvarchar,
      @p_dt_blq datetime,
      @p_dt_cad datetime
    )
    RETURNS bigint
    WITH EXECUTE AS CALLER
    AS
    BEGIN
      DECLARE @v_nm_bai nvarchar = COALESCE(RTRIM(LTRIM(@p_nm_bai)),'(MIGRACAO)')
      DECLARE @v_id_bai bigint = (SELECT TOP 1 ID FROM [dbo].[Bairro] WHERE UPPER(RTRIM(LTRIM(nome))) = @v_nm_bai)
      
      IF @v_id_bai IS NULL
      BEGIN
        EXECUTE @v_id_bai = [dbo].[CreateBairro] 
          @p_nm_bai
         ,@p_nm_cid
         ,@p_cd_cid
         ,@p_dt_blq
         ,@p_dt_cad
      END
      RETURN(@v_id_bai);
    END;
    GO

    COMMIT
-- #FIM :: CRIA FUNCAO QUE BUSCA BAIRROS >> USADA NO CADASTRO DE PESSOAS
GO


-- #INI :: CRIA FUNCAO QUE BUSCA BAIRROS >> USADA NO CADASTRO DE PESSOAS
    BEGIN TRANSACTION
    GO 

    IF OBJECT_ID('dbo.hashCode') IS NOT NULL
      DROP FUNCTION dbo.hashCode;
    GO

    CREATE FUNCTION dbo.hashCode
    (
      @p_code nvarchar
    )
    RETURNS bigint
    WITH EXECUTE AS CALLER
    AS
    BEGIN
      DECLARE @h bigint = 0
      DECLARE @i int = 0
      
      WHILE @i < LEN(@p_code)
      BEGIN
        SET @h = 31 * @h + ASCII(SUBSTRING(@p_code,@i,1));
        SET @i = @i + 1
      END;
      RETURN(@h);
    END;
    GO

    COMMIT
-- #FIM :: CRIA FUNCAO QUE BUSCA BAIRROS >> USADA NO CADASTRO DE PESSOAS  
GO


-- #INI :: CRIA FUNCAO QUE RETIRA ACENTOS >> USADA NA MIGRACAO DE DOMINIOS
    BEGIN TRANSACTION
    GO 

    IF OBJECT_ID('dbo.retiraAcentos') IS NOT NULL
      DROP FUNCTION dbo.retiraAcentos;
    GO

    CREATE FUNCTION dbo.retiraAcentos
    (
      @p_texto nvarchar
    )
    RETURNS nvarchar
    WITH EXECUTE AS CALLER
    AS
    BEGIN
      DECLARE @temp nvarchar = @p_texto
      
      SET @temp = REPLACE(@temp,'á','a')
      SET @temp = REPLACE(@temp,'à','a')
      SET @temp = REPLACE(@temp,'â','a')
      SET @temp = REPLACE(@temp,'ã','a')
      SET @temp = REPLACE(@temp,'ä','a')
      SET @temp = REPLACE(@temp,'å','a')
      SET @temp = REPLACE(@temp,'a','a')
      SET @temp = REPLACE(@temp,'a','a')
      SET @temp = REPLACE(@temp,'a','a')
      SET @temp = REPLACE(@temp,'Á','A')
      SET @temp = REPLACE(@temp,'Â','A')
      SET @temp = REPLACE(@temp,'Ã','A')
      SET @temp = REPLACE(@temp,'Ä','A')
      SET @temp = REPLACE(@temp,'Å','A')
      SET @temp = REPLACE(@temp,'A','A')
      SET @temp = REPLACE(@temp,'A','A')
      SET @temp = REPLACE(@temp,'A','A')
      SET @temp = REPLACE(@temp,'À','A')
      SET @temp = REPLACE(@temp,'é','e')
      SET @temp = REPLACE(@temp,'è','e')
      SET @temp = REPLACE(@temp,'ê','e')
      SET @temp = REPLACE(@temp,'ë','e')
      SET @temp = REPLACE(@temp,'e','e')
      SET @temp = REPLACE(@temp,'e','e')
      SET @temp = REPLACE(@temp,'e','e')
      SET @temp = REPLACE(@temp,'e','e')
      SET @temp = REPLACE(@temp,'e','e')
      SET @temp = REPLACE(@temp,'E','E')
      SET @temp = REPLACE(@temp,'E','E')
      SET @temp = REPLACE(@temp,'E','E')
      SET @temp = REPLACE(@temp,'É','E')
      SET @temp = REPLACE(@temp,'E','E')
      SET @temp = REPLACE(@temp,'E','E')
      SET @temp = REPLACE(@temp,'Ê','E')
      SET @temp = REPLACE(@temp,'È','E')
      SET @temp = REPLACE(@temp,'ì','i')
      SET @temp = REPLACE(@temp,'í','i')
      SET @temp = REPLACE(@temp,'î','i')
      SET @temp = REPLACE(@temp,'ï','i')
      SET @temp = REPLACE(@temp,'ì','i')
      SET @temp = REPLACE(@temp,'i','i')
      SET @temp = REPLACE(@temp,'i','i')
      SET @temp = REPLACE(@temp,'i','i')
      SET @temp = REPLACE(@temp,'Ì','I')
      SET @temp = REPLACE(@temp,'Í','I')
      SET @temp = REPLACE(@temp,'Î','I')
      SET @temp = REPLACE(@temp,'Ï','I')
      SET @temp = REPLACE(@temp,'Ì','I')
      SET @temp = REPLACE(@temp,'I','I')
      SET @temp = REPLACE(@temp,'I','I')
      SET @temp = REPLACE(@temp,'I','I')
      SET @temp = REPLACE(@temp,'ó','o')
      SET @temp = REPLACE(@temp,'ô','o')
      SET @temp = REPLACE(@temp,'õ','o')
      SET @temp = REPLACE(@temp,'ö','o')
      SET @temp = REPLACE(@temp,'o','o')
      SET @temp = REPLACE(@temp,'o','o')
      SET @temp = REPLACE(@temp,'o','o')
      SET @temp = REPLACE(@temp,'ò','o')
      SET @temp = REPLACE(@temp,'Ò','O')
      SET @temp = REPLACE(@temp,'Ó','O')
      SET @temp = REPLACE(@temp,'Ô','O')
      SET @temp = REPLACE(@temp,'Õ','O')
      SET @temp = REPLACE(@temp,'Ö','O')
      SET @temp = REPLACE(@temp,'O','O')
      SET @temp = REPLACE(@temp,'O','O')
      SET @temp = REPLACE(@temp,'O','O')
      SET @temp = REPLACE(@temp,'ù','u')
      SET @temp = REPLACE(@temp,'ú','u')
      SET @temp = REPLACE(@temp,'û','u')
      SET @temp = REPLACE(@temp,'ü','u')
      SET @temp = REPLACE(@temp,'u','u')
      SET @temp = REPLACE(@temp,'u','u')
      SET @temp = REPLACE(@temp,'u','u')
      SET @temp = REPLACE(@temp,'u','u')
      SET @temp = REPLACE(@temp,'Ù','U')
      SET @temp = REPLACE(@temp,'Ú','U')
      SET @temp = REPLACE(@temp,'Û','U')
      SET @temp = REPLACE(@temp,'Ü','U')
      SET @temp = REPLACE(@temp,'U','U')
      SET @temp = REPLACE(@temp,'U','U')
      SET @temp = REPLACE(@temp,'U','U')
      SET @temp = REPLACE(@temp,'U','U')
      SET @temp = REPLACE(@temp,'ç','c')
      SET @temp = REPLACE(@temp,'Ç','C')
      SET @temp = REPLACE(@temp,'ñ','n')
      SET @temp = REPLACE(@temp,'Ñ','N')
      SET @temp = REPLACE(@temp,'ý','y')
      SET @temp = REPLACE(@temp,'Ý','Y')
      SET @temp = REPLACE(@temp,' ','_')

      RETURN(@temp);
    END;
    GO

    COMMIT
-- #FIM :: CRIA FUNCAO QUE RETIRA ACENTOS >> USADA NA MIGRACAO DE DOMINIOS
GO


-- #INI :: CRIA FUNCAO QUE GERA O HASCODE >> USADA NO CADASTRO DE BENS
    BEGIN TRANSACTION
    GO 

    IF OBJECT_ID('dbo.hashCode') IS NOT NULL
      DROP FUNCTION dbo.hashCode;
    GO

    CREATE FUNCTION dbo.hashCode
    (
      @code VARCHAR(30)
    ) RETURNS BIGINT
    WITH EXECUTE AS CALLER
    AS
    BEGIN
      DECLARE @h BIGINT = 0
            , @char CHAR(1)
            , @len INT = LEN(@code)
            , @pos INT = 1;
    
      WHILE @pos <= @len
      BEGIN
        SET @char = SUBSTRING(@code, @pos, 1);
        SET @h = 31 * @h + ASCII(@char); 
        SET @pos = @pos + 1;
      END
    
      RETURN(@h);
    END;
    GO

    COMMIT
-- #FIM :: CRIA FUNCAO QUE GERA O HASCODE >> USADA NO CADASTRO DE BENS
GO


-- #INI :: CRIA FUNCAO QUE CONVERTE INICIAIS MAIUSCULAS >> USADA NA MIGRACAO DE CARACTERISTICAS
    BEGIN TRANSACTION
    GO

    IF OBJECT_ID('dbo.InitCap') IS NOT NULL
      DROP FUNCTION dbo.InitCap;
    GO
    
    CREATE FUNCTION dbo.InitCap 
    (
      @texto VARCHAR(8000)
    )
    RETURNS VARCHAR(8000)
    WITH EXECUTE AS CALLER
    AS
    BEGIN
      DECLARE @retorno VARCHAR(8000) = LOWER(@texto)
            , @char CHAR(1)
            , @alphanum BIT = 0
            , @len INT = LEN(@texto)
            , @pos INT = 1;
    
      WHILE @pos <= @len
      BEGIN
        SET @char = SUBSTRING(@texto, @pos, 1);
    
        IF @pos = 1 OR @alphanum = 0
            SET @retorno = STUFF(@retorno, @pos, 1, UPPER(@char));
    
        SET @pos = @pos + 1;
    
        -- Define if the current character is non-alphanumeric
        IF ASCII(@char) <= 47 OR (ASCII(@char) BETWEEN 58 AND 64) OR (ASCII(@char) BETWEEN 91 AND 96) OR (ASCII(@char) BETWEEN 123 AND 126)
            SET @alphanum = 0;
        ELSE
            SET @alphanum = 1;
    
      END
    
      RETURN(@retorno);
    END;
    GO

    COMMIT
-- #FIM :: CRIA FUNCAO QUE CONVERTE INICIAIS MAIUSCULAS >> USADA NA MIGRACAO DE CARACTERISTICAS
GO


-- #INI :: ALTERACOES ESPECIFICAS PARA MIGRACAO
    BEGIN TRANSACTION
    GO 
    
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'agente_id' AND Object_ID = Object_ID(N'Pessoa'))
      ALTER TABLE dbo.Pessoa                 ADD agente_id VARCHAR(30) NULL;
    ELSE
      PRINT 'Coluna agente_id já existe na tabela Pessoa'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'agente_id' AND Object_ID = Object_ID(N'Parceiro'))
      ALTER TABLE dbo.Parceiro               ADD agente_id VARCHAR(30) NULL;
    ELSE
      PRINT 'Coluna agente_id já existe na tabela Parceiro'
    GO
      
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'codigoasi' AND Object_ID = Object_ID(N'Transferencia'))
      ALTER TABLE dbo.Transferencia          ADD codigoasi VARCHAR(30) NULL;
    ELSE
      PRINT 'Coluna codigoasi já existe na tabela Transferencia'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'vl_unitario' AND Object_ID = Object_ID(N'Transferenciaitem'))
      ALTER TABLE dbo.Transferenciaitem      ADD vl_unitario NUMERIC(15,4) NULL;
    ELSE
      PRINT 'Coluna vl_unitario já existe na tabela Transferenciaitem'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'cd_conta' AND Object_ID = Object_ID(N'Transferenciaitem'))
      ALTER TABLE dbo.Transferenciaitem      ADD cd_conta VARCHAR(10) NULL;
    ELSE
      PRINT 'Coluna cd_conta já existe na tabela Transferenciaitem'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'codigo' AND Object_ID = Object_ID(N'Baixa'))
      ALTER TABLE dbo.Baixa                  ADD codigo BIGINT NULL;
    ELSE
      PRINT 'Coluna codigo já existe na tabela Baixa'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'vl_unitario' AND Object_ID = Object_ID(N'Baixapatrimonioitem'))
      ALTER TABLE dbo.Baixapatrimonioitem    ADD vl_unitario NUMERIC(15,4) NULL;
    ELSE
      PRINT 'Coluna vl_unitario já existe na tabela Baixapatrimonioitem'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'codigo' AND Object_ID = Object_ID(N'Observacao'))
      ALTER TABLE dbo.Observacao             ADD codigo NUMERIC(10,0) NULL;
    ELSE
      PRINT 'Coluna codigo já existe na tabela Observacao'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'bempatrimonial_id' AND Object_ID = Object_ID(N'Contacontabilmovimento'))
      ALTER TABLE dbo.Contacontabilmovimento ADD bempatrimonial_id BIGINT NULL;
    ELSE
      PRINT 'Coluna bempatrimonial_id já existe na tabela Contacontabilmovimento'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'transferencia_id' AND Object_ID = Object_ID(N'Contacontabilmovimento'))
      ALTER TABLE dbo.Contacontabilmovimento ADD transferencia_id BIGINT NULL;
    ELSE
      PRINT 'Coluna transferencia_id já existe na tabela Contacontabilmovimento'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'sq_requisicao_item' AND Object_ID = Object_ID(N'RequisicaoConsumoItem'))
      ALTER TABLE dbo.RequisicaoConsumoItem  ADD sq_requisicao_item NUMERIC(10,0);
    ELSE
      PRINT 'Coluna sq_requisicao_item já existe na tabela RequisicaoConsumoItem'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'sq_baixa_item' AND Object_ID = Object_ID(N'BaixaAlmoxarifadoItem'))
      ALTER TABLE dbo.BaixaAlmoxarifadoItem  ADD sq_baixa_item VARCHAR(10) NULL;
    ELSE
      PRINT 'Coluna sq_baixa_item já existe na tabela BaixaAlmoxarifadoItem'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'cd_conta' AND Object_ID = Object_ID(N'BaixaAlmoxarifadoItem'))
      ALTER TABLE dbo.BaixaAlmoxarifadoItem  ADD cd_conta VARCHAR(15) NULL;
    ELSE
      PRINT 'Coluna cd_conta já existe na tabela BaixaAlmoxarifadoItem'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'cd_conta' AND Object_ID = Object_ID(N'EntradaAlmoxarifadoItem'))
      ALTER TABLE dbo.EntradaAlmoxarifadoItem  ADD cd_conta VARCHAR(15) NULL;
    ELSE
      PRINT 'Coluna cd_conta já existe na tabela EntradaAlmoxarifadoItem'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'cd_conta' AND Object_ID = Object_ID(N'DevolucaoItem'))
      ALTER TABLE dbo.DevolucaoItem  ADD cd_conta VARCHAR(15) NULL;
    ELSE
      PRINT 'Coluna cd_conta já existe na tabela DevolucaoItem'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'cd_conta' AND Object_ID = Object_ID(N'RequisicaoConsumoItem'))
      ALTER TABLE dbo.RequisicaoConsumoItem  ADD cd_conta VARCHAR(15) NULL;
    ELSE
      PRINT 'Coluna cd_conta já existe na tabela RequisicaoConsumoItem'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'valor' AND Object_ID = Object_ID(N'AtendimentoReqConsumoItem'))
      ALTER TABLE dbo.AtendimentoReqConsumoItem  ADD valor NUMERIC(15,4) NULL;
    ELSE
      PRINT 'Coluna valor já existe na tabela AtendimentoReqConsumoItem'
    GO

    IF NOT EXISTS(SELECT *  FROM sys.objects WHERE Object_Name(Object_ID)=N'DF_Dominio_isPadrao')
      ALTER TABLE dbo.Dominio                ADD CONSTRAINT DF_Dominio_isPadrao DEFAULT 0 FOR isPadrao; 
    ELSE
      PRINT 'Constraint DF_Dominio_isPadrao já existe na coluna isPadrao'
    GO
    
    IF EXISTS(SELECT * FROM sys.columns WHERE Name = N'nome' AND Object_ID = Object_ID(N'Localizacao'))
      ALTER TABLE dbo.Localizacao  ALTER COLUMN nome VARCHAR(250) NULL;
    ELSE
      PRINT 'Coluna nome não existe na tabela Localizacao'
    GO
 
    -- VERIFICAR SE ESSES INDICES SERAO MANTIDOS NO MODELO
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix01_entrada_cd' AND Object_ID = Object_ID(N'Entrada'))
      CREATE NONCLUSTERED INDEX ix01_entrada_cd           ON dbo.Entrada (codigo ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix01_entrada_cd já existe na tabela Entrada'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix01_entrada_cd' AND Object_ID = Object_ID(N'EntradaPatrimonio'))
      CREATE NONCLUSTERED INDEX ix01_entpat_eoid          ON dbo.EntradaPatrimonio (estruturaorganizacional_id ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix01_entrada_cd já existe na tabela EntradaPatrimonio'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix01_estrutura_id' AND Object_ID = Object_ID(N'EstruturaOrganizacional'))
      CREATE NONCLUSTERED INDEX ix01_estrutura_id         ON dbo.EstruturaOrganizacional (id ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix01_estrutura_id já existe na tabela EstruturaOrganizacional'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix02_estrutura_cd' AND Object_ID = Object_ID(N'EstruturaOrganizacional'))
      CREATE NONCLUSTERED INDEX ix02_estrutura_cd         ON dbo.EstruturaOrganizacional (codigo ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix02_estrutura_cd já existe na tabela EstruturaOrganizacional'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix01_baixa_cd' AND Object_ID = Object_ID(N'Baixa'))
      CREATE NONCLUSTERED INDEX ix01_baixa_cd             ON dbo.Baixa (codigo ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix01_baixa_cd já existe na tabela Baixa'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix01_baixapat_cd' AND Object_ID = Object_ID(N'BaixaPatrimonio'))
      CREATE NONCLUSTERED INDEX ix01_baixapat_cd          ON dbo.BaixaPatrimonio (codigo ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix01_baixapat_cd já existe na tabela BaixaPatrimonio'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix01_histpat_dtop' AND Object_ID = Object_ID(N'HistoricoBemPatrimonial'))
      CREATE NONCLUSTERED INDEX ix01_histpat_dtop         ON dbo.HistoricoBemPatrimonial (dataoperacao ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix01_histpat_dtop já existe na tabela HistoricoBemPatrimonial'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix02_histpat_bpid' AND Object_ID = Object_ID(N'HistoricoBemPatrimonial'))
      CREATE NONCLUSTERED INDEX ix02_histpat_bpid         ON dbo.HistoricoBemPatrimonial (bempatrimonial_id ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix02_histpat_bpid já existe na tabela HistoricoBemPatrimonial'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix01_ctacont_cd' AND Object_ID = Object_ID(N'ContaContabil'))
      CREATE NONCLUSTERED INDEX ix01_ctacont_cd           ON dbo.ContaContabil (codigo ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix01_ctacont_cd já existe na tabela ContaContabil'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix01_ctacmov_mesctatp' AND Object_ID = Object_ID(N'ContaContabilMovimento'))
      CREATE NONCLUSTERED INDEX ix01_ctacmov_mesctatp     ON dbo.ContaContabilMovimento (datareferencia ASC, contacontabil_id ASC, tipomovimentocontacontabil_id ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix01_ctacmov_mesctatp já existe na tabela ContaContabilMovimento'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix02_ctacmov_dtref' AND Object_ID = Object_ID(N'ContaContabilMovimento'))
      CREATE NONCLUSTERED INDEX ix02_ctacmov_dtref        ON dbo.ContaContabilMovimento (datareferencia ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix02_ctacmov_dtref já existe na tabela ContaContabilMovimento'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix03_ctacmov_ctaid' AND Object_ID = Object_ID(N'ContaContabilMovimento'))
      CREATE NONCLUSTERED INDEX ix03_ctacmov_ctaid        ON dbo.ContaContabilMovimento (contacontabil_id ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix03_ctacmov_ctaid já existe na tabela ContaContabilMovimento'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix01_ctacsdo_mescta' AND Object_ID = Object_ID(N'ContaContabilSaldo'))
      CREATE NONCLUSTERED INDEX ix01_ctacsdo_mescta       ON dbo.ContaContabilSaldo (datareferencia ASC, contacontabil_id ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix01_ctacsdo_mescta já existe na tabela ContaContabilSaldo'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'idx01_transfitem_ctacd' AND Object_ID = Object_ID(N'TransferenciaItem'))
      CREATE NONCLUSTERED INDEX idx01_transfitem_ctacd    ON dbo.TransferenciaItem (cd_conta ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice idx01_transfitem_ctacd já existe na tabela TransferenciaItem'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'idx02_transfitem_undid' AND Object_ID = Object_ID(N'TransferenciaItem'))
      CREATE NONCLUSTERED INDEX idx02_transfitem_undid    ON dbo.TransferenciaItem (unidadeorigem_id ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice idx02_transfitem_undid já existe na tabela TransferenciaItem'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'idx03_transfitem_bemid' AND Object_ID = Object_ID(N'TransferenciaItem'))
      CREATE NONCLUSTERED INDEX idx03_transfitem_bemid    ON dbo.TransferenciaItem (bempatrimonial_id ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice idx03_transfitem_bemid já existe na tabela TransferenciaItem'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix02_entrada_cdasi' AND Object_ID = Object_ID(N'Entrada'))
      CREATE UNIQUE NONCLUSTERED INDEX ix02_entrada_cdasi ON dbo.Entrada (codigoasi ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix02_entrada_cdasi já existe na tabela Entrada'
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix01_dominio_chvcd' AND Object_ID = Object_ID(N'Dominio'))
      CREATE UNIQUE NONCLUSTERED INDEX ix01_dominio_chvcd ON dbo.Dominio (chave ASC, codigo ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix01_dominio_chvcd já existe na tabela Dominio'
    GO

    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix01_movest_material' AND Object_ID = Object_ID(N'MovimentoEstoque'))
      CREATE NONCLUSTERED INDEX ix01_movest_material ON dbo.MovimentoEstoque (material_id ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix01_movest_material já existe na tabela MovimentoEstoque'
    GO

    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix01_movest_data' AND Object_ID = Object_ID(N'MovimentoEstoque'))
      CREATE NONCLUSTERED INDEX ix01_movest_data ON dbo.MovimentoEstoque (datamovimento ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix01_movest_data já existe na tabela MovimentoEstoque'
    GO

    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix01_estoquesaldo_material' AND Object_ID = Object_ID(N'MaterialEstoqueSaldo'))
      CREATE NONCLUSTERED INDEX ix01_estoquesaldo_material ON dbo.MaterialEstoqueSaldo (material_id ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix01_estoquesaldo_material já existe na tabela MaterialEstoqueSaldo'
    GO

    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix01_estoquesaldo_data' AND Object_ID = Object_ID(N'MaterialEstoqueSaldo'))
      CREATE NONCLUSTERED INDEX ix01_estoquesaldo_data ON dbo.MaterialEstoqueSaldo (datareferencia ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix01_estoquesaldo_data já existe na tabela MaterialEstoqueSaldo'
    GO

    
    USE ASI_HOMOLOGA
    GO
    
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix01_seq_bem_perm' AND Object_ID = Object_ID(N'BP_DEPRECIACAO'))
      CREATE NONCLUSTERED INDEX ix01_seq_bem_perm         ON ASI_HOMOLOGA..BP_DEPRECIACAO (sq_bem_perm ASC) WITH (STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice ix01_entrada_cd já existe na tabela Entrada'
    GO
    
    USE GRPDB
    GO
    
    COMMIT
-- #FIM :: ALTERACOES ESPECIFICAS PARA MIGRACAO
GO


-- #INI :: ACRESCENTANDO DOMINIOS
    BEGIN TRANSACTION
    GO  
    -- ----------------------------------------------- --
    --  ATRIBUIR AO STARTUPLISTENER ESSAS ATUALIZACOES --
    -- ----------------------------------------------- --
    
    --> ATUALIZA TIPOMOVIMENTOCONTACONTABIL
    UPDATE dbo.dominio 
       SET DESCRICAO = 'Entrada orçamentária'
         , NOME      = 'ENTRADA_ORCAMENTARIA'
     WHERE CHAVE     = 'tipoMovimentoContaContabil'
       AND CODIGO    = 1;
    
    UPDATE dbo.dominio 
       SET DESCRICAO = 'Baixa orçamentária'
         , NOME      = 'BAIXA_ORCAMENTARIA'
     WHERE CHAVE     = 'tipoMovimentoContaContabil'
       AND CODIGO    = 2;
    
    
    --> ATUALIZA TIPOSTATUSDEPRECIACAO
    UPDATE dbo.dominio 
       SET DESCRICAO = 'Depreciação finalizada'
         , NOME      = 'NAO_DEPRECIAVEL_VALOR_RESIDUAL'
     WHERE CHAVE     = 'tipoStatusDepreciacao'
       AND CODIGO    = 3;
    
    UPDATE dbo.dominio 
       SET DESCRICAO = 'Não Depreciável'
         , NOME      = 'NAO_DEPRECIAVEL_MARCACAO_USUARIO'
     WHERE CHAVE     = 'tipoStatusDepreciacao'
       AND CODIGO    = 4;
    -- INSERT INTO dbo.dominio
    --    (datacriacao, dataedicao, datainativo, version, chave, descricao, nome, codigo, ispadrao) 
    -- VALUES 
    --    (GETDATE(), GETDATE(), NULL, 0, 'tipoStatusDepreciacao', 'Não Depreciado em exercício anterior', 'NAO_DEPRECIADO_EXERCICIO_ANTERIOR', 5, 0);
    
    
    --> ACRESCENTA TIPOSTATUSDEPRECIACAO
    DELETE FROM dbo.dominio WHERE chave='tipoDocumento'
    INSERT INTO dbo.dominio
         (datacriacao, dataedicao, datainativo, version, chave, descricao, nome, codigo, ispadrao) 
    VALUES 
         (GETDATE(), GETDATE(), NULL, 0, 'tipoDocumento', 'Nota de Empenho', 'EMPENHO', 1, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoDocumento', 'Processo de Entrada', 'PROCESSO_ENTRADA', 2, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoDocumento', 'Nota Fiscal', 'NOTA_FISCAL', 3, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoDocumento', 'Termo de doação', 'TERMO_DOACAO', 4, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoDocumento', 'Termo de cessão', 'TERMO_CESSAO', 5, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoDocumento', 'Ata de Reunião', 'ATA_REUNIAO', 6, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoDocumento', 'Atestado', 'ATESTADO', 7, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoDocumento', 'Contrato', 'CONTRATO', 8, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoDocumento', 'Requisição', 'REQUISICAO', 9, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoDocumento', 'Contrato', 'CONTRATO', 10, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoDocumento', 'Processo de Baixa', 'PROCESSO_BAIXA', 11, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoDocumento', 'Nº Processo', 'NUMERO_PROCESSO', 12, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoDocumento', 'Memorando', 'MEMORANDO', 13, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoDocumento', 'Portaria', 'PORTARIA', 14, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoDocumento', 'Despacho', 'DESPACHO', 15, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoDocumento', 'Ocorrência', 'OCORRENCIA', 16, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoDocumento', 'Nota de Sistema', 'NOTA_SISTEMA', 17, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoDocumento', 'Ato de destinação de mercadorias - ADM', 'ATO_DESTINACAO_MERCADORIAS_ADM', 18, 0);

    --> ACRESCENTA TIPOINVENTARIO
    INSERT INTO dbo.dominio 
         (datacriacao, dataedicao, datainativo, version, chave, descricao, nome, codigo, ispadrao) 
    VALUES 
         (GETDATE(), GETDATE(), NULL, 0, 'tipoInventario', 'POR U.A.', 'POR_UA', 6, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoInventario', 'POR U.L.', 'POR_UL', 7, 0);
       
    --> CRIA TIPOOBJETIVOINVENTARIO
    DELETE FROM dbo.dominio WHERE CHAVE='tipoObjetivoInventario';
    INSERT INTO dbo.dominio 
         (datacriacao, dataedicao, datainativo, version, chave, descricao, nome, codigo, ispadrao) 
    VALUES 
         (GETDATE(), GETDATE(), NULL, 0, 'tipoObjetivoInventario', 'INVENTÁRIO GERAL', 'INVENTARIO_GERAL', 1, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoObjetivoInventario', 'CRIAÇÃO DE ÓRGÃO/UG', 'CRIACAO_ORGAO_UG', 2, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoObjetivoInventario', 'EXTINÇAO DE ÓRGÃO/UG', 'EXTINCAO_ORGAO_UG', 3, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoObjetivoInventario', 'PARA LEVANTAMENTO', 'PARA_LEVANTAMENTO', 4, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoObjetivoInventario', 'PARA CONFERÊNCIA', 'PARA_CONFERENCIA', 5, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoObjetivoInventario', 'PARA INSPEÇÃO/AVALIAÇÃO', 'PARA_INSPECAO_AVALIACAO', 6, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoObjetivoInventario', 'PASSAGEM DE CARGA', 'PASSAGEM_CARGA', 7, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoObjetivoInventario', 'PARA BAIXA', 'PARA_BAIXA', 8, 0);
       
    --> ACRESCENTA TIPOOBJETIVOSAIDATEMPORARIA
    INSERT INTO dbo.dominio
         (datacriacao, dataedicao, datainativo, version, chave, descricao, nome, codigo, ispadrao) 
    VALUES 
         (GETDATE(), GETDATE(), NULL, 0, 'tipoObjetivoSaidaTemporaria', 'Manutenção', 'MANUTENCAO', 5, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoObjetivoSaidaTemporaria', 'Conserto na garantia', 'CONSERTO_GARANTIA', 6, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoObjetivoSaidaTemporaria', 'Reforma', 'REFORMA', 7, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoObjetivoSaidaTemporaria', 'Serviço', 'SERVIÇO', 8, 0);
        
    --> ACRESCENTA TIPOBAIXAALMOXARIFADO
    INSERT INTO dbo.dominio
         (datacriacao, dataedicao, datainativo, version, chave, descricao, nome, codigo, ispadrao) 
    VALUES 
         (GETDATE(), GETDATE(), NULL, 0, 'tipoBaixaAlmoxarifado', 'Inservível', 'INSERVIVEL', 7, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoBaixaAlmoxarifado', 'Validade', 'VALIDADE', 8, 0);
        
    --> ACRESCENTA TIPORECEBIMENTOALMOXARIFADO
    INSERT INTO dbo.dominio
         (datacriacao, dataedicao, datainativo, version, chave, descricao, nome, codigo, ispadrao) 
    VALUES
         (GETDATE(), GETDATE(), NULL, 0, 'tipoRecebimentoAlmoxarifado', 'Suprimento de fundos', 'SUPRIMENTO_FUNDOS', 7, 0),
         (GETDATE(), GETDATE(), NULL, 0, 'tipoRecebimentoAlmoxarifado', 'Aquisição execução anterior', 'AQUISICAO_EXECUCAO_ANTERIOR', 8, 0);  

    --> ACRESCENTA TIPOSITUACAOFISICA
    INSERT INTO dbo.dominio
         (datacriacao, dataedicao, datainativo, version, chave, descricao, nome, codigo, ispadrao) 
    VALUES
         (GETDATE(), GETDATE(), NULL, 0, 'tipoSituacaoFisica', 'Irrecuperável', 'IRRECUPERAVEL', 12, 0);

    --> ACRESCENTA TIPOSTATUSREQUISICAO
    INSERT INTO dbo.dominio
         (datacriacao, dataedicao, datainativo, version, chave, descricao, nome, codigo, ispadrao) 
    VALUES
         (GETDATE(), GETDATE(), NULL, 0, 'tipoStatusRequisicao', 'Pronta para autorizar', 'PRONTA_PARA_AUTORIZAR', 7, 0);
    
    --> CRIA TABELA MAPEAMENTO DE DOMINIOS ASI X GRP
    /*
       ATRIBUTO_GRP               CHAVE_DE_PARA                    CODIGO_ASI                   TABELA_DOMINIO_ASI
       -------------------------  -------------------------------- ---------------------------- ------------------------------------
       dominiotiporecebimento_id  'tipoRecebimento'                E1.CD_TP_FORMA_OP            asi_homologa..bp_tp_forma_op
       dominiotipobem_id          'tipoBem'                        E1.CD_TP_BEM                 asi_homologa..bp_tp_bem
       dominiotipodocumento_id    'tipoDocumento'                  D.CD_TP_DOCUMENTO            asi_homologa..cr_tp_documento
       dominiosituacaofisica_id   'tipoSituacaoFisica'             B.CD_TP_SIT_FISICA           asi_homologa..bp_tp_sit_fisica
       dominiostatus_id           'tipoStatusBem'                  B.CD_TP_STATUS               asi_homologa..bp_tp_status
       dominiotipoobjetivo_id     'tipoObjetivoSaidaTemporaria'    TRANSF.CD_MOTIVO_SAIDA_TEMP  asi_homologa..bp_motivo_saida_temp
       dominiosituacaobaixa_id    'tipoSituacaoBaixa'              BAIXA.CD_TP_SIT_BAIXA        asi_homologa..bp_tp_sit_baixa
       dominiotipobaixa_id        'tipoBaixa'                      BAIXA.CD_TP_FORMA_OP         asi_homologa..bp_tp_forma_op
       dominiotipodocumento_id    'tipoDocumento'                  DOC.CD_TP_DOCUMENTO          asi_homologa..cr_tp_documento

       dominiotiporecebimento_id  'tipoRecebimentoAlmoxarifado'    NR.CD_TIPO_MOVIMENTO         asi_homologa..ax_tipo_movimento
       dominiotipodocumento_id    'tipoDocumento'                  DOC.CD_TP_DOCUMENTO          asi_homologa..cr_tp_documento
       dominiotipobaixa_id        'tipoBaixaAlmoxarifado'          BA.CD_TIPO_MOVIMENTO         asi_homologa..ax_tipo_movimento
       dominiotipodocumento_id    'tipoDocumento'                  DOC.CD_TP_DOCUMENTO          asi_homologa..cr_tp_documento
       tipostatusrequisicao_id    'tipoStatusRequisicao'           R.CD_SITUACAO_REQ_ATUAL      asi_homologa..ax_situacao_req
    */

    PRINT 'Criando tabela de mapeamento de dominios...'
    IF EXISTS(SELECT * FROM sys.tables WHERE Object_ID = Object_ID(N'dominio_asi_grp'))
        DROP TABLE dominio_asi_grp
    GO

    SELECT id     AS id_grp
         , chave  AS tp_grp
         , codigo AS cd_grp
         , nome   AS nm_grp
         , chave  AS cd_asi
         , nome   AS nm_asi
    INTO dbo.dominio_asi_grp
    FROM dbo.dominio
    WHERE chave='tipoAbrangencia'
       OR chave='tipoPorte'
       OR chave='tipoBem'
       OR chave='tipoRecebimento'
       OR chave='tipoRecebimentoAlmoxarifado'
       OR chave='tipoSituacaoFisica'
       OR chave='tipoStatusBem'
       OR chave='tipoDocumento'
       OR chave='tipoObjetivoSaidaTemporaria'
       OR chave='tipoSituacaoBaixa'
       OR chave='tipoBaixa'
       OR chave='tipoBaixaAlmoxarifado'
       OR chave='tipoStatusRequisicao'
       OR chave='tipoInventario'
       OR chave='tipoObjetivoInventario';


    --> INICIALIZA OS VALORES [ASI] DO MAPEAMENTO
    UPDATE dbo.dominio_asi_grp 
    SET cd_asi = 'NULL', nm_asi = 'NULL';
    
    
    --===========================================================================--
    --> DE-PARA TIPOABRANGENCIA
    --===========================================================================--
    UPDATE dbo.dominio_asi_grp SET cd_asi = '01', nm_asi = 'LOCAL' 
    WHERE  tp_grp = 'tipoAbrangencia' AND cd_grp = 1;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '02', nm_asi = 'REGIONAL' 
    WHERE  tp_grp = 'tipoAbrangencia' AND cd_grp = 2;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '03', nm_asi = 'NACIONAL' 
    WHERE  tp_grp = 'tipoAbrangencia' AND cd_grp = 3;
    
    
    --===========================================================================--
    --> DE-PARA TIPOPORTE
    --===========================================================================--
    UPDATE dbo.dominio_asi_grp SET cd_asi = '01', nm_asi = 'MICRO-EMPRESA' 
    WHERE  tp_grp = 'tipoPorte' AND cd_grp = 1;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '02', nm_asi = 'PEQUENA' 
    WHERE  tp_grp = 'tipoPorte' AND cd_grp = 2;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '03', nm_asi = 'MÉDIA' 
    WHERE  tp_grp = 'tipoPorte' AND cd_grp = 3;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '04', nm_asi = 'GRANDE' 
    WHERE  tp_grp = 'tipoPorte' AND cd_grp = 4;
    
    
    --===========================================================================--
    --> DE-PARA TIPOBEM
    --===========================================================================--
    /*
      SELECT * FROM ASI_HOMOLOGA..bp_tp_bem 
      WHERE CD_TP_BEM IN (
           SELECT DISTINCT E1.CD_TP_BEM FROM ASI_HOMOLOGA..BP_ENTRADA E1
      )
      ORDER BY 1, 2
    */
    UPDATE dbo.dominio_asi_grp SET cd_asi = '01', nm_asi = 'PRÓPRIO CONTABILIZADO' 
    WHERE  tp_grp = 'tipoBem' AND cd_grp = 1;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '02', nm_asi = 'PRÓPRIO CONTROLADO' 
    WHERE  tp_grp = 'tipoBem' AND cd_grp = 2;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '03', nm_asi = 'DE TERCEIROS' 
    WHERE  tp_grp = 'tipoBem' AND cd_grp = 3;
    
    
    --===========================================================================--
    --> DE-PARA TIPORECEBIMENTO
    --===========================================================================--
    /*
      SELECT * FROM ASI_HOMOLOGA..BP_TP_FORMA_OP 
      WHERE CD_TP_FORMA_OP IN (
           SELECT DISTINCT E1.CD_TP_FORMA_OP FROM ASI_HOMOLOGA..BP_ENTRADA E1
      )
      ORDER BY 1, 2
    */
    UPDATE dbo.dominio_asi_grp SET cd_asi = '01', nm_asi = 'COMPRA'
    WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = 1;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '02', nm_asi = 'COMPRA EXTRA ORÇAMENTÁRIA'
    WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = 2;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '03', nm_asi = 'DOAÇÃO'
    WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = 3;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '04', nm_asi = 'CESSÃO DE USO'
    WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = 4;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '05', nm_asi = 'PERMUTA'
    -- WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = #;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '06', nm_asi = 'TRANSFERÊNCIA FORA DO SISTEMA'
    WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = 5;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '07', nm_asi = 'FABRICAÇÃO'
    -- WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '08', nm_asi = 'INCORPORAÇÃO PARA INTEGRALIZAÇÃO OU AUMENTO DE CAPITAL'
    -- WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = #;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '09', nm_asi = 'REPOSIÇÃO'
    WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = 6;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '24', nm_asi = 'OUTROS'
    -- WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '10', nm_asi = 'COMODATO'
    -- WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '11', nm_asi = 'LOCAÇÃO'
    -- WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '12', nm_asi = 'LEASING'
    -- WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '13', nm_asi = 'EMPRÉSTIMO'
    -- WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = #;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '31', nm_asi = 'INCORPORAÇÃO'
    WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = 7;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '32', nm_asi = 'DEVOLUÇÃO'
    -- WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = #;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '33', nm_asi = 'ENTRADA POR DECISÃO JUDICIAL'
    WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = 8;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '34', nm_asi = 'RESTO A PAGAR'
    WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = 9;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '36', nm_asi = 'CESSAO'
    WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = 10;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '38', nm_asi = 'AQUISIÇÃO EXTRA ORÇAMENTARIA/MULTA'
    WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = 11;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '35', nm_asi = 'BAIXA'
    -- WHERE  tp_grp = 'tipoRecebimento' AND cd_grp = #;
    
    
    --===========================================================================--
    --> DE-PARA TIPORECEBIMENTOALMOXARIFADO
    --===========================================================================--
    /*
      SELECT * FROM ASI_HOMOLOGA..AX_TIPO_MOVIMENTO 
      WHERE CD_TIPO_MOVIMENTO IN (
           SELECT DISTINCT NR.CD_TIPO_MOVIMENTO FROM ASI_HOMOLOGA..AX_NOTA_REC NR
      )
      ORDER BY 1, 2
    */

    UPDATE dbo.dominio_asi_grp SET cd_asi = '001', nm_asi = 'COMPRA'
    WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = 1;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '003', nm_asi = 'DOAÇÃO - ENTRADA'
    WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = 3;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '005', nm_asi = 'DEVOLUÇÃO'
    WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = 6;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '008', nm_asi = 'ENTRADA POR TRANSFERENCIA'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '009', nm_asi = 'ENTRADA POR ALTERAÇÃO DE CONTA'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '011', nm_asi = 'ALTERAÇÃO DE CONTA'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '012', nm_asi = 'ESTORNO/AJUSTE DE MOV.'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '013', nm_asi = 'CESSÃO - ENTRADA'
    WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = 4;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '014', nm_asi = 'FABRICAÇÃO PRÓPRIA'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '015', nm_asi = 'PERMUTA - ENTRADA'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '016', nm_asi = 'COMPRA EXTRA-ORÇAMENTÁRIA'
    WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = 2;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '018', nm_asi = 'SUPRIMENTO DE FUNDOS'
    WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = 7;

    SET IDENTITY_INSERT dbo.dominio_asi_grp ON;
    INSERT INTO dbo.dominio_asi_grp 
         ( id_grp, tp_grp, cd_grp, nm_grp, cd_asi, nm_asi ) 
    SELECT id, chave, codigo, nome, '041', 'SUPRIMENTO DE FUNDOS' 
    FROM DBO.DOMINIO WHERE CHAVE='tipoRecebimentoAlmoxarifado' AND CODIGO=7
    SET IDENTITY_INSERT dbo.dominio_asi_grp OFF;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '027', nm_asi = 'ESTORNO - ORCAMENTARIO'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '029', nm_asi = 'ENTRADA - IMPLANTACAO'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '032', nm_asi = 'ENTRADA - TRANSFERENCIA END.'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '033', nm_asi = 'D02'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '034', nm_asi = 'AQUISIÇÃO EXC. ANT.'
    WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = 8;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '035', nm_asi = 'TRANSF. ENTRE ÓRGÃO'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '100', nm_asi = 'COMPRA INTEGRADA'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '108', nm_asi = 'EMPRÉSTIMO'
    -- WHERE  tp_grp = 'tipoRecebimentoAlmoxarifado' AND cd_grp = #;
    
    
    --===========================================================================--
    --> DE-PARA TIPOSITUACAOFISICA
    --===========================================================================--
    /*
      SELECT * FROM ASI_HOMOLOGA..BP_TP_SIT_FISICA 
      WHERE CD_TP_SIT_FISICA IN (
           SELECT DISTINCT B.CD_TP_SIT_FISICA FROM ASI_HOMOLOGA..BP_BEM B
      )
      ORDER BY 1, 2
    */
    UPDATE dbo.dominio_asi_grp SET cd_asi = '02', nm_asi = 'BOM'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 1;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '03', nm_asi = 'REGULAR'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 2;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '04', nm_asi = 'INSERVÍVEL'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 3;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '05', nm_asi = 'PRECÁRIO'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 4;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '06', nm_asi = 'RUIM'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 5;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '07', nm_asi = 'ANTI ECONÔMICO'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 6;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '08', nm_asi = 'DANIFICADO'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 7;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '09', nm_asi = 'OBSOLETO'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 8;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '10', nm_asi = 'OCIOSO'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 9;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '11', nm_asi = 'IRRECUPERÁVEL'
    WHERE  tp_grp = 'tipoSituacaoFisica' AND cd_grp = 12;
    
    
    --===========================================================================--
    --> DE-PARA TIPOSTATUSBEM
    --===========================================================================--
    /*
      SELECT * FROM ASI_HOMOLOGA..BP_TP_STATUS 
      WHERE CD_TP_STATUS IN (
           SELECT DISTINCT B.CD_TP_STATUS FROM ASI_HOMOLOGA..BP_BEM B
      )
      ORDER BY 1, 2    
    */
    UPDATE dbo.dominio_asi_grp SET cd_asi = '01', nm_asi = 'UTILIZADO'
    WHERE  tp_grp = 'tipoStatusBem' AND cd_grp = 1;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '02', nm_asi = 'BAIXADO'
    WHERE  tp_grp = 'tipoStatusBem' AND cd_grp = 2;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '03', nm_asi = 'DISPONÍVEL'
    WHERE  tp_grp = 'tipoStatusBem' AND cd_grp = 3;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '04', nm_asi = 'EM PROCESSO DE BAIXA'
    WHERE  tp_grp = 'tipoStatusBem' AND cd_grp = 4;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '05', nm_asi = 'EM PROCESSO DE TRANSFERÊNCIA'
    WHERE  tp_grp = 'tipoStatusBem' AND cd_grp = 5;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '06', nm_asi = 'EM SAÍDA TEMPORÁRIA'
    WHERE  tp_grp = 'tipoStatusBem' AND cd_grp = 6;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '07', nm_asi = 'EXTRAVIADO'
    WHERE  tp_grp = 'tipoStatusBem' AND cd_grp = 7;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '08', nm_asi = 'INDISPONÍVEL'
    WHERE  tp_grp = 'tipoStatusBem' AND cd_grp = 8;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '09', nm_asi = 'SINDICÂNCIA'
    WHERE  tp_grp = 'tipoStatusBem' AND cd_grp = 9;
    
    
    --===========================================================================--
    --> DE-PARA TIPODOCUMENTO
    --===========================================================================--
    /*
      SELECT * FROM ASI_HOMOLOGA..CR_TP_DOCUMENTO 
      WHERE CD_TP_DOCUMENTO IN (
           SELECT DISTINCT D.CD_TP_DOCUMENTO FROM ASI_HOMOLOGA..BP_ENT_DOC D
      )
      UNION
      SELECT * FROM ASI_HOMOLOGA..CR_TP_DOCUMENTO 
      WHERE CD_TP_DOCUMENTO IN (
           SELECT DISTINCT DOC.CD_TP_DOCUMENTO FROM ASI_HOMOLOGA..BP_BAIXA_DOC DOC
      )
      ORDER BY 1, 2
    */
    UPDATE dbo.dominio_asi_grp SET cd_asi = '001', nm_asi = 'EMPENHO'
    WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 1;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '002', nm_asi = 'PROCESSO DE ENTRADA'
    WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 2;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '003', nm_asi = 'NOTA FISCAL'
    WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 3;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '004', nm_asi = 'REQUISICAO'
    WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 9;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '005', nm_asi = 'CONTRATO'
    WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 10;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '006', nm_asi = 'TERMO DE DOACAO'
    WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 4;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '007', nm_asi = 'TERMO DE PERMUTA'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = #;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '008', nm_asi = 'PROCESSO DE BAIXA'
    WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 11;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '009', nm_asi = 'TERMO DE CESSAO'
    WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 5;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '010', nm_asi = 'Nº DO PROCESSO'
    WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 12;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '011', nm_asi = 'NATUREZA DE DESPESA'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '012', nm_asi = 'PROCESSO DE COMPRA'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '013', nm_asi = 'EVENTO CONTÁBIL'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = #;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '014', nm_asi = 'MEMORANDO'
    WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 13;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '015', nm_asi = 'ATO DE DESTINAÇÃO DE MERCADORIAS - ADM'
    WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 18;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '016', nm_asi = 'INCORPORACÃO'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '017', nm_asi = 'TERMO D ADESÃO'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = #;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '018', nm_asi = 'DESPACHO'
    WHERE  tp_grp = 'tipoDocumento' AND cd_grp = 15;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '019', nm_asi = 'OCORRENCIA'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '020', nm_asi = 'NOTA DE SISTEMA'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '120', nm_asi = 'EDITAL'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '121', nm_asi = 'OFÍCIO'
    -- WHERE  tp_grp = 'tipoDocumento' AND cd_grp = #;
    
    
    --===========================================================================--
    --> DE-PARA TIPOOBJETIVOSAIDATEMPORARIA
    --===========================================================================--
    /*
      SELECT * FROM ASI_HOMOLOGA..BP_MOTIVO_SAIDA_TEMP 
      WHERE CD_MOTIVO_SAIDA_TEMP IN (
           SELECT DISTINCT TRANSF.CD_MOTIVO_SAIDA_TEMP FROM ASI_HOMOLOGA..BP_TRANSF_EXTERNA TRANSF
      )
      ORDER BY 1, 2
    */
    UPDATE dbo.dominio_asi_grp SET cd_asi = '01', nm_asi = 'COMODATO'
    WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = 1;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '02', nm_asi = 'CONSERTO'
    WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = 2;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '03', nm_asi = 'CONSERTO NA GARANTIA'
    WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = 6;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '04', nm_asi = 'EMPRÉSTIMO'
    WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = 3;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '05', nm_asi = 'EVENTO'
    WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = 4;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '06', nm_asi = 'MANUTENÇÃO'
    WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = 5;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '07', nm_asi = 'REFORMA'
    WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = 7;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '08', nm_asi = 'SERVIÇO'
    WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = 8;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '11', nm_asi = 'COMODATO'
    -- WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '12', nm_asi = 'CONSERTO'
    -- WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '13', nm_asi = 'CONSERTO NA GARANTIA'
    -- WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '14', nm_asi = 'EMPRÉSTIMO'
    -- WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '15', nm_asi = 'ORÇAMENTO'
    -- WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = #;

    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '16', nm_asi = 'CESSÃO'
    -- WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '17', nm_asi = 'REFORMA'
    -- WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '18', nm_asi = 'SERVIÇO'
    -- WHERE  tp_grp = 'tipoObjetivoSaidaTemporaria' AND cd_grp = #;
    

    --===========================================================================--
    --> DE-PARA TIPOSITUACAOBAIXA
    --===========================================================================--
    /*
      SELECT * FROM ASI_HOMOLOGA..BP_TP_SIT_BAIXA 
      WHERE CD_TP_SIT_BAIXA IN (
           SELECT DISTINCT BAIXA.CD_TP_SIT_BAIXA FROM ASI_HOMOLOGA..BP_BAIXA BAIXA
      )
      ORDER BY 1, 2    
    */
    UPDATE dbo.dominio_asi_grp SET cd_asi = '01', nm_asi = 'EM ANDAMENTO'
    WHERE  tp_grp = 'tipoSituacaoBaixa' AND cd_grp = 1;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '02', nm_asi = 'AUTORIZADA'
    WHERE  tp_grp = 'tipoSituacaoBaixa' AND cd_grp = 2;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '03', nm_asi = 'NÃO AUTORIZADA'
    WHERE  tp_grp = 'tipoSituacaoBaixa' AND cd_grp = 3;
    
    
    --===========================================================================--
    --> DE-PARA TIPOBAIXA
    --===========================================================================--
    /*
      SELECT * FROM ASI_HOMOLOGA..BP_TP_FORMA_OP 
      WHERE CD_TP_FORMA_OP IN (
           SELECT DISTINCT BAIXA.CD_TP_FORMA_OP FROM ASI_HOMOLOGA..BP_BAIXA BAIXA
      )
      ORDER BY 1, 2
    */
    UPDATE dbo.dominio_asi_grp SET cd_asi = '14', nm_asi = 'VENDA'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 1;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '15', nm_asi = 'DOAÇÃO'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 2;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '16', nm_asi = 'CESSÃO DE USO'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 3;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '17', nm_asi = 'EXTRAVIO OU SINISTRO'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 4;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '18', nm_asi = 'PERMUTA'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 5;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '19', nm_asi = 'TRANSFERÊNCIA FORA DO SISTEMA'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 6;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '20', nm_asi = 'CADASTRAMENTO INDEVIDO'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 7;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '23', nm_asi = 'MUDANÇA DE PERMANENTE/CONSUMO'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 8;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '25', nm_asi = 'ROUBO OU FURTO'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 9;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '26', nm_asi = 'OUTROS MOTIVOS'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 10;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '27', nm_asi = 'LEILÃO'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 11;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '28', nm_asi = 'DIVERSAS'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 12;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '29', nm_asi = 'RECUPERÁVEL'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 13;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '33', nm_asi = 'INUTILIZAÇÃO OU ABANDONO'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 14;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '35', nm_asi = 'CESSÃO'
    WHERE  tp_grp = 'tipoBaixa' AND cd_grp = 15;
    
    
    --===========================================================================--
    --> DE-PARA TIPOBAIXAALMOXARIFADO
    --===========================================================================--
    /*
      SELECT * FROM ASI_HOMOLOGA..AX_TIPO_MOVIMENTO 
      WHERE CD_TIPO_MOVIMENTO IN (
           SELECT DISTINCT BA.CD_TIPO_MOVIMENTO FROM ASI_HOMOLOGA..AX_BAIXA BA
      )
      ORDER BY 1, 2
    */
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '001', nm_asi = 'COMPRA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '003', nm_asi = 'DOAÇÃO - ENTRADA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '004', nm_asi = 'REQUISIÇÃO MATERIAL'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '005', nm_asi = 'DEVOLUÇÃO'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '006', nm_asi = 'REQUISIÇÃO POR TRANSFERENCIA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '007', nm_asi = 'EXTRAVIO'
    WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = 2;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '008', nm_asi = 'ENTRADA POR TRANSFERENCIA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '009', nm_asi = 'ENTRADA POR ALTERAÇÃO DE CONTA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '010', nm_asi = 'SAIDA POR ALTERAÇÃO DE CONTA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '011', nm_asi = 'ALTERAÇÃO DE CONTA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '012', nm_asi = 'ESTORNO/AJUSTE DE MOV.'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '013', nm_asi = 'CESSÃO - ENTRADA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '014', nm_asi = 'FABRICAÇÃO PRÓPRIA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '015', nm_asi = 'PERMUTA - ENTRADA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '016', nm_asi = 'COMPRA EXTRA-ORÇAMENTÁRIA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '018', nm_asi = 'SUPRIMENTO DE FUNDOS'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '020', nm_asi = 'LEILÃO'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '021', nm_asi = 'DOAÇÃO - BAIXA'
    WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = 1;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '022', nm_asi = 'PERMUTA - BAIXA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '023', nm_asi = 'CESSAO - BAIXA'
    WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = 6;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '024', nm_asi = 'CADASTRO INDEVIDO'
    WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = 3;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '025', nm_asi = 'ALIENAÇÃO'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '026', nm_asi = 'BAIXA-LEVANTAMENTO'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '027', nm_asi = 'ESTORNO - ORCAMENTARIO'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '028', nm_asi = 'ESTORNO SAIDA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '029', nm_asi = 'ENTRADA - IMPLANTACAO'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '030', nm_asi = 'SALDO ANTERIOR'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '031', nm_asi = 'SAIDA - TRANSFERENCIA END.'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '032', nm_asi = 'ENTRADA - TRANSFERENCIA END.'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '033', nm_asi = 'D02'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '034', nm_asi = 'AQUISIÇÃO EXC. ANT.'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '035', nm_asi = 'TRANSF. ENTRE ÓRGÃO'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '036', nm_asi = 'DIVERSAS'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '037', nm_asi = 'DEVOLUÇÃO AO FORNECEDOR'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '044', nm_asi = 'INSERVÍVEL'
    WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = 7;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '045', nm_asi = 'PERDA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '046', nm_asi = 'DESUSO'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '047', nm_asi = 'VALIDADE'
    WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = 8;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '100', nm_asi = 'COMPRA INTEGRADA'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '108', nm_asi = 'EMPRÉSTIMO'
    -- WHERE  tp_grp = 'tipoBaixaAlmoxarifado' AND cd_grp = #;
    
    
    --===========================================================================--
    --> DE-PARA TIPOSTATUSREQUISICAO
    --===========================================================================--
    /*
      SELECT * FROM ASI_HOMOLOGA..AX_SITUACAO_REQ 
      WHERE CD_SITUACAO_REQ IN (
           SELECT DISTINCT R.CD_SITUACAO_REQ_ATUAL FROM ASI_HOMOLOGA..AX_REQUISICAO R
      )
      ORDER BY 1, 2
    */
    UPDATE dbo.dominio_asi_grp SET cd_asi = '001', nm_asi = 'PRONTA PARA AUTORIZAR'
    WHERE  tp_grp = 'tipoStatusRequisicao' AND cd_grp = 7;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '002', nm_asi = 'PRONTA PARA ATENDER'
    WHERE  tp_grp = 'tipoStatusRequisicao' AND cd_grp = 2;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '003', nm_asi = 'ATENDIDA'
    WHERE  tp_grp = 'tipoStatusRequisicao' AND cd_grp = 5;
    -- WHERE  tp_grp = 'tipoStatusRequisicao' AND cd_grp = 4; -- REDMINE #4715 :: ATENDIDA >> FINALIZADA 
    
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '004', nm_asi = 'SEPARADA'
    -- WHERE  tp_grp = 'tipoStatusRequisicao' AND cd_grp = #;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '005', nm_asi = 'DEVOLVIDA'
    WHERE  tp_grp = 'tipoStatusRequisicao' AND cd_grp = 5;
    -- WHERE  tp_grp = 'tipoStatusRequisicao' AND cd_grp = 6; -- REDMINE #4715 :: DEVOLVIDA >> FINALIZADA 
        
    -- UPDATE dbo.dominio_asi_grp SET cd_asi = '006', nm_asi = 'PRONTA PARA RECEBER'
    -- WHERE  tp_grp = 'tipoStatusRequisicao' AND cd_grp = #;
    
    UPDATE dbo.dominio_asi_grp SET cd_asi = '007', nm_asi = 'FINALIZADA'
    WHERE  tp_grp = 'tipoStatusRequisicao' AND cd_grp = 5;
    
    COMMIT
-- #FIM :: ACRESCENTANDO DOMINIOS
GO


-- #INI :: CRIAÇÃO TABELA AUDITORIA DADOSBEMPATRIMONIAL :: NÃO ANOTADA NO JPA
    BEGIN TRANSACTION
    GO

    IF NOT EXISTS(SELECT * FROM sys.tables WHERE Object_ID = Object_ID(N'dadosbempatrimonial_aud'))
    BEGIN
      CREATE TABLE dbo.dadosbempatrimonial_aud  (
        id                            bigint IDENTITY(1,1) NOT NULL,
        version                       bigint,
        responsavel_id                bigint,
        detentor_id                   bigint,
        databloqueio                  date,
        datainativo                   date,
        datacriacao                   datetime,
        dataedicao                    datetime,
        copiadedadosfinal             bit,
        numeropatrimonial             varchar(50),
        valorliquido                  numeric(20,4),
        autor_id                      bigint,
        editor_id                     bigint,
        bempatrimonial_id             bigint,
        bempatrimonialprincipal_id    bigint,
        dominiolocalinconsistencia_id bigint,
        dominiosituacaofisica_id      bigint,
        dominiostatus_id              bigint,
        dominiostatusinventariobem_id bigint,
        unidadeatual_id               bigint,
        inventariobempatrimonial_id   bigint,
        inventariounidade_id          bigint,
        material_id                   bigint
      );
      PRINT 'Tabela dadosbempatrimonial_aud criada!'
    END
    ELSE
      PRINT 'Tabela dadosbempatrimonial_aud já existe!'

    COMMIT
-- #FIM :: CRIAÇÃO TABELA AUDITORIA DADOSBEMPATRIMONIAL :: NÃO ANOTADA NO JPA
GO


SET NOCOUNT OFF
--===========================================================================--
-- ##FIM :: AJUSTES INICIAIS PARA A MIGRACAO                                 --
--===========================================================================--