--===========================================================================--
-- ##INI :: IMPORTACAO TABELAS BASICAS                                       --
--===========================================================================--
SET NOCOUNT ON


-- #INI :: INICIALIZA AS TABELAS
    BEGIN TRANSACTION
    GO 

    DELETE FROM dbo.endereco;
    DELETE FROM dbo.bairro;
    DELETE FROM dbo.cidade;
    DELETE FROM dbo.estado;
    DELETE FROM dbo.regiao;
    DELETE FROM dbo.pais;
    DELETE FROM dbo.unidademedida;

    COMMIT
-- #FIM :: INICIALIZA AS TABELAS
GO


-- #INI :: IMPORTACAO DA TABELA DE PAISES >> PAIS
    SET IDENTITY_INSERT dbo.pais ON;
    BEGIN TRANSACTION
    GO
    
    INSERT INTO dbo.pais
         ( id
         , databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , codigo
         , nome
         , sigla
         , inativador_id
         , autor_id
         , editor_id 
         )
    SELECT PAIS.ID_PAIS       AS id
         , PAIS.DT_BLOQUEIO   AS databloqueio
         , NULL               AS datainativo
         , GETDATE()          AS datacriacao
         , GETDATE()          AS dataedicao
         , 0                  AS "version"
         , PAIS.CD_PAIS       AS codigo
         , PAIS.NM_PAIS       AS nome
         , PAIS.NM_SIGLA_PAIS AS sigla
         , NULL               AS inativador_id
         , NULL               AS autor_id
         , NULL               AS editor_id
    FROM ASI_HOMOLOGA..CR_PAIS PAIS
    ORDER BY PAIS.ID_PAIS;
    
    -- TOTALIZADORES
    PRINT 'IMPORTACAO DA TABELA DE PAISES >> PAIS';
    DECLARE @count_asi bigint = (SELECT COUNT(*) FROM ASI_HOMOLOGA..CR_PAIS);
    DECLARE @count_grp bigint = (SELECT COUNT(*) FROM DBO.PAIS);
    PRINT 'Total de registros do ASI ' + CAST(@count_asi AS NVARCHAR)
    PRINT 'Total de registros no GRP ' + CAST(@count_grp AS NVARCHAR)
    PRINT ' ';
    
    COMMIT
    SET IDENTITY_INSERT dbo.pais OFF;
-- #FIM :: IMPORTACAO DA TABELA DE PAISES >> PAIS
GO


-- #INI :: IMPORTACAO DA TABELA DE REGIOES >> REGIAO
    
    BEGIN TRANSACTION
    GO
    
    SET IDENTITY_INSERT dbo.regiao ON;
    INSERT INTO dbo.regiao
         ( id
         , databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , codigo
         , nome
         , inativador_id
         , autor_id
         , editor_id 
         , pais_id
         )
    SELECT REGIAO.ID_REGIAO_UF AS id
         , REGIAO.DT_BLOQUEIO  AS databloqueio
         , NULL                AS datainativo
         , GETDATE()           AS datacriacao
         , GETDATE()           AS dataedicao
         , 0                   AS "version"
         , REGIAO.CD_REGIAO    AS codigo
         , REGIAO.NM_REGIAO    AS nome
         , NULL                AS inativador_id
         , NULL                AS autor_id
         , NULL                AS editor_id
         , CASE REGIAO.NM_REGIAO 
             WHEN 'AMERICA DO NORTE' THEN 2 
             ELSE 1 
           END                 AS pais_id
    FROM ASI_HOMOLOGA..CR_REGIAO_UF REGIAO
    ORDER BY REGIAO.ID_REGIAO_UF;
    SET IDENTITY_INSERT dbo.regiao OFF;
    
    -- TOTALIZADORES
    PRINT 'IMPORTACAO DA TABELA DE REGIOES >> REGIAO';
    DECLARE @count_asi bigint = (SELECT COUNT(*) FROM ASI_HOMOLOGA..CR_REGIAO_UF);
    DECLARE @count_grp bigint = (SELECT COUNT(*) FROM dbo.regiao);
    PRINT 'Total de registros do ASI ' + CAST(@count_asi AS NVARCHAR)
    PRINT 'Total de registros no GRP ' + CAST(@count_grp AS NVARCHAR)
    PRINT ' ';
    
    COMMIT
-- #FIM :: IMPORTACAO DA TABELA DE REGIOES >> REGIAO
GO


-- #INI :: IMPORTACAO DAS UNIDADES FEDERATIVAS >> ESTADO
    BEGIN TRANSACTION
    GO
    
    SET IDENTITY_INSERT dbo.estado ON;
    INSERT INTO dbo.estado
         ( id
         , databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , version
         , codigo
         , nome
         , sigla
         , inativador_id
         , autor_id
         , editor_id 
         , regiao_id
         )
    SELECT UF.ID_UF       AS id
         , UF.DT_BLOQUEIO AS databloqueio
         , NULL           AS datainativo
         , GETDATE()      AS datacriacao
         , GETDATE()      AS dataedicao
         , 0              AS "version"
         , UF.CD_UF       AS codigo
         , UF.NM_UF       AS nome
         , UF.NM_SIGLA_UF AS sigla
         , NULL           AS inativador_id
         , NULL           AS autor_id
         , NULL           AS editor_id
         , CASE 
             WHEN UF.NM_SIGLA_UF IN ('MT','GO','MS','DF') 
               THEN 1 -- CENTRO OESTE
             WHEN UF.NM_SIGLA_UF IN ('ES','RJ','SP','MG')
               THEN 2 -- SUDESTE
             WHEN UF.NM_SIGLA_UF IN ('RS','PR','SC') 
               THEN 3 -- SUL
             WHEN UF.NM_SIGLA_UF IN ('CE','AL','PI','MA','PB','PE','BA','SE','RN')
               THEN 4 -- NORDESTE
             WHEN UF.NM_SIGLA_UF IN ('AC','AM','PA','RR','AP','RO','TO')
               THEN 5 -- NORTE
             WHEN UF.NM_SIGLA_UF IN ('US')
               THEN 6 -- AMERICA DO NORTE
             ELSE   7 -- CENTROESTE :: REGIAO DUPLICADA E DESCONHECIDA
            END           AS regiao_id
    FROM ASI_HOMOLOGA..CR_UF UF
    ORDER BY UF.ID_UF;
    SET IDENTITY_INSERT dbo.estado OFF;
    
    -- TOTALIZADORES
    PRINT 'IMPORTACAO DAS UNIDADES FEDERATIVAS >> ESTADO';
    DECLARE @count_asi bigint = (SELECT COUNT(*) FROM ASI_HOMOLOGA..CR_UF);
    DECLARE @count_grp bigint = (SELECT COUNT(*) FROM dbo.estado);
    PRINT 'Total de registros do ASI ' + CAST(@count_asi AS NVARCHAR)
    PRINT 'Total de registros no GRP ' + CAST(@count_grp AS NVARCHAR)
    PRINT ' ';
    
    COMMIT
-- #FIM :: IMPORTACAO DAS UNIDADES FEDERATIVAS >> ESTADO
GO


-- #INI :: IMPORTACAO DAS CIDADES >> CIDADE
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'Codigo' AND Object_ID = Object_ID(N'Estado'))
      ALTER TABLE dbo.Estado ALTER COLUMN Codigo VARCHAR(30) COLLATE SQL_Latin1_General_CP1_CI_AI 
    ELSE
      PRINT 'Coluna Codigo já existe na tabela Estado'
    GO
    
    BEGIN TRANSACTION
    GO
    
    SET IDENTITY_INSERT dbo.cidade ON;
    INSERT INTO dbo.cidade
         ( id
         , databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , version
         , codigo
         , codigoibge
         , nome
         , inativador_id
         , autor_id
         , editor_id 
         , estado_id
         )
    SELECT CID.ID_CIDADE   AS id
         , CID.DT_BLOQUEIO AS databloqueio
         , NULL            AS datainativo
         , GETDATE()       AS datacriacao
         , GETDATE()       AS dataedicao
         , 0               AS "version"
         , CID.CD_CIDADE   AS codigo
         , CID.CD_IBGE     AS codigoibge
         , CID.NM_CIDADE   AS nome
         , NULL            AS inativador_id
         , NULL            AS autor_id
         , NULL            AS editor_id
         , UF.ID           AS estado_id
    FROM ASI_HOMOLOGA..CR_CIDADE CID
    INNER JOIN dbo.ESTADO UF ON UF.CODIGO COLLATE SQL_Latin1_General_CP1_CI_AI = CID.CD_UF
    ORDER BY CID.CD_CIDADE;
    SET IDENTITY_INSERT dbo.cidade OFF;

    -- TOTALIZADORES
    PRINT 'IMPORTACAO DAS CIDADES >> CIDADE';
    DECLARE @count_asi bigint = (SELECT COUNT(*) FROM ASI_HOMOLOGA..CR_CIDADE CID INNER JOIN dbo.ESTADO UF ON UF.CODIGO COLLATE SQL_Latin1_General_CP1_CI_AI = CID.CD_UF);
    DECLARE @count_grp bigint = (SELECT COUNT(*) FROM dbo.cidade);
    PRINT 'Total de registros do ASI ' + CAST(@count_asi AS NVARCHAR)
    PRINT 'Total de registros no GRP ' + CAST(@count_grp AS NVARCHAR)
    PRINT ' ';
    
    COMMIT
-- #FIM :: IMPORTACAO DAS CIDADES >> CIDADE
GO


-- #INI :: CRIA UM BAIRRO PADRAO :: DEMAIS SERAO INSERIDOS PELA FUNCAO "GETIDBAIRRO()"
    BEGIN TRANSACTION
    GO
    
    SET IDENTITY_INSERT dbo.bairro ON;
    INSERT INTO dbo.bairro
         ( id
         , databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , version
         , codigo
         , nome
         , inativador_id
         , autor_id
         , editor_id 
         , cidade_id
         )
    SELECT 1               AS id
         , NULL            AS databloqueio
         , NULL            AS datainativo
         , GETDATE()       AS datacriacao
         , GETDATE()       AS dataedicao
         , 0               AS "version"
         , 1               AS codigo
         , '(MIGRACAO)'    AS nome
         , NULL            AS inativador_id
         , NULL            AS autor_id
         , NULL            AS editor_id
         , CID.ID_CIDADE   AS cidade_id
    FROM ASI_HOMOLOGA..CR_CIDADE CID
    WHERE CID.ID_CIDADE=2;
    SET IDENTITY_INSERT dbo.bairro OFF;
    
    -- TOTALIZADORES
    PRINT 'CRIA UM BAIRRO PADRAO';
    DECLARE @count_asi bigint = (SELECT COUNT(*) FROM ASI_HOMOLOGA..CR_CIDADE CID WHERE CID.ID_CIDADE=2);
    DECLARE @count_grp bigint = (SELECT COUNT(*) FROM dbo.bairro);
    PRINT 'Total de registros do ASI ' + CAST(@count_asi AS NVARCHAR)
    PRINT 'Total de registros no GRP ' + CAST(@count_grp AS NVARCHAR)
    PRINT ' ';
    
    COMMIT
-- #FIM :: CRIA UM BAIRRO PADRAO :: DEMAIS SERAO INSERIDOS PELA FUNCAO "GETIDBAIRRO()"
GO


-- #INI :: IMPORTACAO DAS INSTALACOES >> ENDERECO
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'Codigo' AND Object_ID = Object_ID(N'Cidade'))
      ALTER TABLE dbo.Cidade ALTER COLUMN Codigo VARCHAR(30) COLLATE SQL_Latin1_General_CP1_CI_AI 
    ELSE
      PRINT 'Coluna Codigo já existe na tabela Cidade'
    GO
    
    BEGIN TRANSACTION
    GO
    
    SET IDENTITY_INSERT dbo.endereco ON;
    INSERT INTO dbo.endereco
         ( id
         , databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , version
         , cep
         , codigo
         , complemento
         , latitude
         , logradouro
         , longitude
         , nome
         , numero
         , inativador_id
         , autor_id
         , editor_id
         , bairro_id
         , cidade_id
         , dominiotipoendereco_id
         , pessoa_id
         )
    SELECT ENDER.ID_INSTALACAO  AS id
         , ENDER.DT_BLOQUEIO    AS databloqueio
         , NULL                 AS datainativo
         , GETDATE()            AS datacriacao
         , GETDATE()            AS dataedicao
         , 0                    AS "version"
         , ENDER.NR_CEP         AS cep
         , ENDER.CD_INSTALACAO  AS codigo
         , ''                   AS complemento
         , NULL                 AS latitude
         , ENDER.DS_ENDERECO    AS logradouro
         , NULL                 AS longitude
         , ENDER.NM_INSTALACAO  AS nome
         , NULL                 AS numero
         , NULL                 AS inativador_id
         , NULL                 AS autor_id
         , NULL                 AS editor_id
         , 1                    AS bairro_id
         , CID.ID               AS cidade_id
         , NULL                 AS dominiotipoendereco_id
         , NULL                 AS pessoa_id
    FROM ASI_HOMOLOGA..CR_INSTALACAO ENDER
    INNER JOIN dbo.CIDADE CID ON CID.CODIGO COLLATE SQL_Latin1_General_CP1_CI_AI = ENDER.CD_CIDADE
    ORDER BY ENDER.CD_INSTALACAO;
    SET IDENTITY_INSERT dbo.endereco OFF;
    
    -- TOTALIZADORES
    PRINT 'IMPORTACAO DAS INSTALACOES >> ENDERECO';
    DECLARE @count_asi bigint = (SELECT COUNT(*) FROM ASI_HOMOLOGA..CR_INSTALACAO ENDER INNER JOIN dbo.CIDADE CID ON CID.CODIGO COLLATE SQL_Latin1_General_CP1_CI_AI = ENDER.CD_CIDADE);
    DECLARE @count_grp bigint = (SELECT COUNT(*) FROM dbo.endereco);
    PRINT 'Total de registros do ASI ' + CAST(@count_asi AS NVARCHAR)
    PRINT 'Total de registros no GRP ' + CAST(@count_grp AS NVARCHAR)
    PRINT ' ';
    
    COMMIT
-- #FIM :: IMPORTACAO DAS INSTALACOES >> ENDERECO
GO


-- #INI :: IMPORTACAO DAS UNIDADES DE MEDIDAS >> UNIDADEMEDIDA
    BEGIN TRANSACTION
    GO
    
    SET IDENTITY_INSERT dbo.unidademedida ON;
    INSERT INTO dbo.unidademedida
         ( id
         , databloqueio
         , datainativo
         , codigo
         , descricao
         , sigla
         , inativador_id
         , tiponumerico_id
         , quantidadepadrao
         )
    SELECT UM.ID_UM         AS id
         , NULL             AS databloqueio
         , NULL             AS datainativo
         , UM.CD_UM         AS codigo
         , UM.DS_UM         AS descricao
         , UM.SG_UM         AS sigla
         , NULL             AS inativador_id
         , (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoNumerico' AND CODIGO=1) AS tiponumerico_id -- 1-INTEIRO / 2-DECIMAL
         , UM.NR_FATOR_CONVERSAO AS quantidadepadrao
    FROM ASI_HOMOLOGA..CR_UM UM  -- #REVER NECESSIDADE DE FATOR DE CONVERSAO
    ORDER BY UM.ID_UM;
    SET IDENTITY_INSERT dbo.unidademedida OFF;
    
    -- TOTALIZADORES
    PRINT 'IMPORTACAO DAS UNIDADES DE MEDIDAS >> UNIDADEMEDIDA';
    DECLARE @count_asi bigint = (SELECT COUNT(*) FROM ASI_HOMOLOGA..CR_UM);
    DECLARE @count_grp bigint = (SELECT COUNT(*) FROM dbo.unidademedida);
    PRINT 'Total de registros do ASI ' + CAST(@count_asi AS NVARCHAR)
    PRINT 'Total de registros no GRP ' + CAST(@count_grp AS NVARCHAR)
    PRINT ' ';
    
    COMMIT
-- #FIM :: IMPORTACAO DAS UNIDADES DE MEDIDAS >> UNIDADEMEDIDA
GO


SET NOCOUNT OFF
--===========================================================================--
-- ##FIM :: IMPORTACAO TABELAS BASICAS                                       --
--===========================================================================--