--===========================================================================--
-- ##INI :: IMPORTACAO DAS UNIDADES DA ESTRUTURA ORGANIZACIONAL DO ORGAO     --
--===========================================================================--
SET NOCOUNT ON


-- #INI :: INICIALIZA AS TABELAS
    BEGIN TRANSACTION
    GO
    
    BEGIN TRY
        DELETE FROM dbo.seguranca_usuario_privilegio;
        DELETE FROM dbo.usuarioorganizacaoitem;
        DELETE FROM dbo.seguranca_usuario;
        DELETE FROM dbo.configuracaoparametrosistema;
        DELETE FROM dbo.configuracao;
        DELETE FROM dbo.estruturaorganizacional;
        DELETE FROM dbo.localizacao;
        DELETE FROM dbo.organizacao;   
        DELETE FROM dbo.centrocusto;
        DELETE FROM dbo.contacontabil;
        UPDATE dbo.menu SET editor_id = NULL;
    END TRY
    BEGIN CATCH
        PRINT CAST(ERROR_NUMBER() AS NVARCHAR) + '-' + ERROR_MESSAGE()
    END CATCH
        
    COMMIT
-- #FIM :: INICIALIZA AS TABELAS
GO


-- #INI :: IMPORTACAO DO ORGAO >> ORGANIZACAO
    BEGIN TRANSACTION
    GO
    
    UPDATE dbo.organizacao
       SET databloqueio = upd.databloqueio
         , datainativo = upd.datainicio
         , datacriacao = upd.datacriacao
         , dataedicao = upd.dataedicao
         , version = upd.version
         , codigo = upd.codigo
         , datafim = upd.datafim
         , datainicio = upd.datainicio
         , datareferenciavigente = upd.DATAREFERENCIAVIGENTE
         , nome = upd.nome
         , sigla = upd.sigla
    FROM ( SELECT ORG.ID_ORGAO AS id
         , ORG.DT_BLOQUEIO AS databloqueio
         , NULL            AS datainativo
         , GETDATE()       AS datacriacao
         , GETDATE()       AS dataedicao
         , 0               AS "version"
         , ORG.CD_ORGAO    AS codigo
         , NULL            AS datafim
         , CONVERT(DATETIME, '19730117', 112) AS datainicio
         , (SELECT DATEADD(MONTH, 1, CONCAT (RIGHT(CONCAT(REPLICATE('0', 4),CAST(UG.NR_ANO_FECHAMENTO AS VARCHAR)), 4),'-', RIGHT(CONCAT(REPLICATE('0', 4),CAST(UG.NR_MES_FECHAMENTO AS VARCHAR)), 2),'-', '01')) FROM ASI_HOMOLOGA..CR_UG UG WHERE DT_BLOQUEIO IS NULL) AS datareferenciavigente
         , ORG.NM_ORGAO    AS nome
         , ORG.SG_ORGAO    AS sigla
         , NULL            AS inativador_id
         , NULL            AS autor_id
         , NULL            AS editor_id
    FROM ASI_HOMOLOGA..CR_ORGAO ORG WHERE ORG.ID_ORGAO = 1) AS UPD
    WHERE dbo.organizacao.id = upd.id
    
    -- TOTALIZADORES
    PRINT 'IMPORTACAO DO ORGAO >> ORGANIZACAO';
    DECLARE @count_asi bigint = (SELECT COUNT(*) FROM ASI_HOMOLOGA..CR_ORGAO);
    DECLARE @count_grp bigint = (SELECT COUNT(*) FROM DBO.ORGANIZACAO);
    PRINT 'Total de registros do ASI ' + CAST(@count_asi AS NVARCHAR)
    PRINT 'Total de registros no GRP ' + CAST(@count_grp AS NVARCHAR)
    PRINT ' ';
    
    COMMIT
-- #FIM :: IMPORTACAO DO ORGAO >> ORGANIZACAO
GO


-- #INI :: IMPORTACAO DOS CENTROS DE CUSTO >> CENTROCUSTO
    BEGIN TRANSACTION
    GO
    
    SET IDENTITY_INSERT dbo.centrocusto ON;
    INSERT INTO dbo.centrocusto
         ( id
         , databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , version
         , codigo
         , descricao
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         , centrocustoparent_id
         )
    SELECT CC.ID_CENTRO_CUSTO AS id
         , NULL               AS databloqueio
         , NULL               AS datainativo
         , GETDATE()          AS datacriacao
         , GETDATE()          AS dataedicao
         , 0                  AS "version"
         , CC.CD_CENTRO_CUSTO AS codigo
         , CC.NM_CENTRO_CUSTO AS descricao
         , NULL               AS inativador_id
         , NULL               AS autor_id
         , NULL               AS editor_id
         , 1                  AS organizacao_id
         , NULL               AS centrocustoparent_id 
         -- DISTINCT CD_CENTRO_CUSTO_SUP RETORNOU NULL
    FROM ASI_HOMOLOGA..CR_CENTRO_CUSTO CC
    ORDER BY CC.ID_CENTRO_CUSTO;
    SET IDENTITY_INSERT dbo.centrocusto OFF;

    -- TOTALIZADORES
    PRINT 'IMPORTACAO DO ORGAO >> ORGANIZACAO';
    DECLARE @count_asi bigint = (SELECT COUNT(*) FROM ASI_HOMOLOGA..CR_CENTRO_CUSTO);
    DECLARE @count_grp bigint = (SELECT COUNT(*) FROM DBO.CENTROCUSTO);
    PRINT 'Total de registros do ASI ' + CAST(@count_asi AS NVARCHAR)
    PRINT 'Total de registros no GRP ' + CAST(@count_grp AS NVARCHAR)
    PRINT ' ';
    
    COMMIT
-- #FIM :: IMPORTACAO DOS CENTROS DE CUSTO >> CENTROCUSTO
GO


-- #INI :: IMPORTACAO DAS INSTALACOES >> LOCALIZACAO
    BEGIN TRANSACTION
    GO
    
    SET IDENTITY_INSERT dbo.localizacao ON;
    INSERT INTO dbo.localizacao
         ( id
         , databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , codigo
         , nome
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         , endereco_id
         )
    SELECT L.ID_LOCALIZ    AS id
         , L.DT_BLOQUEIO   AS databloqueio
         , NULL            AS datainativo
         , GETDATE()       AS datacriacao
         , GETDATE()       AS dataedicao
         , 0               AS "version"
         , L.CD_LOCAL      AS codigo
         , L.NM_LOCAL      AS nome
         , NULL            AS inativador_id
         , NULL            AS autor_id
         , NULL            AS editor_id
         , 1               AS organizacao_id
         , E.ID            AS endereco_id
    FROM ASI_HOMOLOGA..CR_LOCALIZ L
    INNER JOIN dbo.ENDERECO E ON E.CODIGO COLLATE SQL_Latin1_General_CP1_CI_AI=L.CD_INSTALACAO
    ORDER BY L.ID_LOCALIZ;
    SET IDENTITY_INSERT dbo.localizacao OFF;

    -- TOTALIZADORES
    PRINT 'IMPORTACAO DAS INSTALACOES >> LOCALIZACAO';
    DECLARE @count_asi bigint = (SELECT COUNT(*) FROM ASI_HOMOLOGA..CR_LOCALIZ L INNER JOIN dbo.ENDERECO E ON E.CODIGO COLLATE SQL_Latin1_General_CP1_CI_AI=L.CD_INSTALACAO);
    DECLARE @count_grp bigint = (SELECT COUNT(*) FROM DBO.LOCALIZACAO);
    PRINT 'Total de registros do ASI ' + CAST(@count_asi AS NVARCHAR)
    PRINT 'Total de registros no GRP ' + CAST(@count_grp AS NVARCHAR)
    PRINT ' ';
    
    COMMIT
-- #FIM :: IMPORTACAO DAS INSTALACOES >> LOCALIZACAO
GO


-- #INI :: IMPORTACAO DA UNIDADE GESTORA >> ESTRUTURA ORGANIZACIONAL
    BEGIN TRANSACTION
    GO
    
    INSERT INTO dbo.estruturaorganizacional
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , classificacao
         , codigo
         , datafim
         , datainicio
         , nome
         , sigla
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         , tipoestruturaorganizacional_id
         , unidadesuperior_id
         , localizacao_id
         )
    SELECT UG.DT_BLOQUEIO    AS databloqueio
         , NULL              AS datainativo
         , GETDATE()         AS datacriacao 
         , GETDATE()         AS dataedicao
         , 0                 as "version"
         , UG.CD_ORGAO       AS classificacao
         , UG.CD_UG          AS codigo
         , UG.DT_BLOQUEIO    AS datafim
         , COALESCE(UG.DT_BLOQUEIO,GETDATE()) AS datainicio
         , UG.NM_UG          AS nome
         , CASE WHEN UG.SG_UG IS NULL THEN 'SG' ELSE UG.SG_UG END AS sigla
         , NULL              AS inativador_id
         , NULL              AS autor_id
         , NULL              AS editor_id
         , 1                 AS organizacao_id
         , (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoEstruturaOrganizacional' AND CODIGO=1) AS tipoestruturaorganizacional_id -- 1;"Unidade gestora"; 2;"Unidade administrativa"; 3;"Unidade localizadora"; 4;"Unidade requisitante"
         , NULL              AS unidadesuperior_id
         , 1                 AS localizacao_id  -- COMO CR_UG NAO TEM LOCALIZACAO ATRIBUIMOS A MESMA DO ORGAO
    FROM ASI_HOMOLOGA..CR_UG UG
    ORDER BY UG.ID_UG;

    -- TOTALIZADORES
    PRINT 'IMPORTACAO DA UNIDADE GESTORA >> ESTRUTURA ORGANIZACIONAL';
    DECLARE @count_asi bigint = (SELECT COUNT(*) FROM ASI_HOMOLOGA..CR_UG);
    DECLARE @count_grp bigint = (SELECT COUNT(*) FROM DBO.ESTRUTURAORGANIZACIONAL WHERE TIPOESTRUTURAORGANIZACIONAL_ID = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoEstruturaOrganizacional' AND CODIGO=1));
    PRINT 'Total de registros do ASI ' + CAST(@count_asi AS NVARCHAR)
    PRINT 'Total de registros no GRP ' + CAST(@count_grp AS NVARCHAR)
    PRINT ' '
    
    COMMIT
-- #FIM :: IMPORTACAO DA UNIDADE GESTORA >> ESTRUTURA ORGANIZACIONAL
GO


-- #INI :: IMPORTACAO DA UNIDADE ADMINISTRATIVA >> ESTRUTURA ORGANIZACIONAL
    BEGIN TRANSACTION
    GO
    
    INSERT INTO dbo.estruturaorganizacional
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , classificacao
         , codigo
         , datafim
         , datainicio
         , nome
         , sigla
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         , tipoestruturaorganizacional_id
         , unidadesuperior_id
         , localizacao_id
         , unidadegestora_id
         )
    SELECT UA.DT_BLOQUEIO    AS databloqueio
         , NULL              AS datainativo
         , GETDATE()         AS datacriacao 
         , GETDATE()         AS dataedicao
         , 0                 as "version"
         , COALESCE(UA.CD_UA_SUP,UA.CD_UG) AS classificacao
         , UA.CD_UA          AS codigo
         , UA.DT_BLOQUEIO    AS datafim
         , COALESCE(UA.DT_BLOQUEIO,GETDATE()) AS datainicio
         , UA.NM_UA          AS nome
         , CASE WHEN UA.SG_UA IS NULL THEN 'SG' ELSE UA.SG_UA END AS sigla
         , NULL              AS inativador_id
         , NULL              AS autor_id
         , NULL              AS editor_id
         , 1                 AS organizacao_id
         , (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoEstruturaOrganizacional' AND CODIGO=2) AS tipoestruturaorganizacional_id -- 1;"Unidade gestora"; 2;"Unidade administrativa"; 3;"Unidade localizadora"; 4;"Unidade requisitante"
         , NULL              AS unidadesuperior_id
         , 1                 AS localizacao_id  -- COMO CR_UA NAO TEM LOCALIZACAO ATRIBUIMOS A MESMA DO ORGAO
         , (SELECT ID FROM dbo.ESTRUTURAORGANIZACIONAL WHERE CODIGO COLLATE SQL_Latin1_General_CP1_CI_AI = UA.CD_UG AND TIPOESTRUTURAORGANIZACIONAL_ID = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoEstruturaOrganizacional' AND CODIGO=1))
    FROM ASI_HOMOLOGA..CR_UA UA
    ORDER BY UA.ID_UA;

    -- TOTALIZADORES
    PRINT 'IMPORTACAO DA UNIDADE ADMINISTRATIVA >> ESTRUTURA ORGANIZACIONAL';
    DECLARE @count_asi bigint = (SELECT COUNT(*) FROM ASI_HOMOLOGA..CR_UA);
    DECLARE @count_grp bigint = (SELECT COUNT(*) FROM DBO.ESTRUTURAORGANIZACIONAL WHERE TIPOESTRUTURAORGANIZACIONAL_ID = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoEstruturaOrganizacional' AND CODIGO=2));
    PRINT 'Total de registros do ASI ' + CAST(@count_asi AS NVARCHAR)
    PRINT 'Total de registros no GRP ' + CAST(@count_grp AS NVARCHAR)
    PRINT ' ';
    
    COMMIT
-- #FIM :: IMPORTACAO DA UNIDADE ADMINISTRATIVA >> ESTRUTURA ORGANIZACIONAL
GO


-- #INI :: IMPORTACAO DA UNIDADE LOCALIZADORA >> ESTRUTURA ORGANIZACIONAL
    BEGIN TRANSACTION
    GO
    
    INSERT INTO dbo.estruturaorganizacional
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , classificacao
         , codigo
         , datafim
         , datainicio
         , nome
         , sigla
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         , tipoestruturaorganizacional_id
         , unidadesuperior_id
         , localizacao_id
         , unidadegestora_id
         )
    SELECT UL.DT_BLOQUEIO    AS databloqueio
         , NULL              AS datainativo
         , GETDATE()         AS datacriacao 
         , GETDATE()         AS dataedicao
         , 0                 AS "version"
         , UA.CODIGO         AS classificacao
         , UL.CD_UL          AS codigo
         , UL.DT_BLOQUEIO    AS datafim
         , COALESCE(UL.DT_BLOQUEIO,GETDATE()) AS datainicio
         , L.NOME          AS nome
         , CASE WHEN UL.SG_UL IS NULL THEN 'SG' ELSE UL.SG_UL END AS sigla
         , NULL              AS inativador_id
         , NULL              AS autor_id
         , NULL              AS editor_id
         , 1                 AS organizacao_id
         , (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoEstruturaOrganizacional' AND CODIGO=3) AS tipoestruturaorganizacional_id -- 1;"Unidade gestora"; 2;"Unidade administrativa"; 3;"Unidade localizadora"; 4;"Unidade requisitante"
         , UA.ID             AS unidadesuperior_id
         , L.ID              AS localizacao_id 
         , (SELECT ID FROM ESTRUTURAORGANIZACIONAL WHERE CODIGO COLLATE SQL_Latin1_General_CP1_CI_AI = UL.CD_UG AND TIPOESTRUTURAORGANIZACIONAL_ID = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoEstruturaOrganizacional' AND CODIGO=1))
    FROM ASI_HOMOLOGA..CR_UL UL
    INNER JOIN dbo.ESTRUTURAORGANIZACIONAL UA ON UA.CODIGO COLLATE SQL_Latin1_General_CP1_CI_AI = UL.CD_UA
    INNER JOIN dbo.LOCALIZACAO L ON L.CODIGO COLLATE SQL_Latin1_General_CP1_CI_AI = UL.CD_LOCAL
    ORDER BY UL.ID_UL;

    -- TOTALIZADORES
    PRINT 'IMPORTACAO DA UNIDADE LOCALIZADORA >> ESTRUTURA ORGANIZACIONAL';
    DECLARE @count_asi bigint = (SELECT COUNT(*) FROM ASI_HOMOLOGA..CR_UL);
    DECLARE @count_grp bigint = (SELECT COUNT(*) FROM DBO.ESTRUTURAORGANIZACIONAL WHERE TIPOESTRUTURAORGANIZACIONAL_ID = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoEstruturaOrganizacional' AND CODIGO=3));
    PRINT 'Total de registros do ASI ' + CAST(@count_asi AS NVARCHAR)
    PRINT 'Total de registros no GRP ' + CAST(@count_grp AS NVARCHAR)
    PRINT ' ';

    COMMIT
-- #FIM :: IMPORTACAO DA UNIDADE LOCALIZADORA >> ESTRUTURA ORGANIZACIONAL
GO


-- #INI :: IMPORTACAO UNIDADE REQUISITANTE >> ESTRUTURAORGANIZACIONAL
    BEGIN TRANSACTION
    GO
    
    /*
     - Conceito diferente de UG/UA/UL
     - Unidades Requisitantes com mesma UA e mesmo LOCAL que a UL, n�o ser�o criadas
     - Estamos considerando somente as Unidades Requisitantes que n�o possuem correspondente com UL
    */
    INSERT INTO dbo.estruturaorganizacional
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , classificacao
         , codigo
         , datafim
         , datainicio
         , nome
         , sigla
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         , tipoestruturaorganizacional_id
         , unidadesuperior_id
         , localizacao_id
         , unidadegestora_id
         )
    SELECT UR.DT_BLOQUEIO       AS databloqueio
         , NULL                 AS datainativo
         , GETDATE()            AS datacriacao 
         , GETDATE()            AS dataedicao
         , 0                    AS "version"
         , UR.CD_UA             AS classificacao
         , UR.CD_UR+'1'         AS codigo
         , UR.DT_BLOQUEIO       AS datafim
         , COALESCE(UR.DT_BLOQUEIO,GETDATE()) AS datainicio
         , UR.NM_UR             AS nome
         , CASE WHEN UR.SG_UR IS NULL THEN 'SG' ELSE UR.SG_UR END AS sigla
         , NULL                 AS inativador_id
         , NULL                 AS autor_id
         , NULL                 AS editor_id
         , 1                    AS organizacao_id
         , (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoEstruturaOrganizacional' AND CODIGO=4) AS tipoestruturaorganizacional_id -- 1;"Unidade gestora"; 2;"Unidade administrativa"; 3;"Unidade localizadora"; 4;"Unidade requisitante"
         , UA.ID                AS unidadesuperior_id
         , L.ID                 AS localizacao_id 
         , UA.UNIDADEGESTORA_ID AS unidadegestora_id
    FROM ASI_HOMOLOGA..AX_UR UR
    INNER JOIN dbo.ESTRUTURAORGANIZACIONAL UA ON UA.CODIGO COLLATE SQL_Latin1_General_CP1_CI_AI = UR.CD_UA
    INNER JOIN dbo.LOCALIZACAO L ON L.CODIGO COLLATE SQL_Latin1_General_CP1_CI_AI = UR.CD_LOCAL
    ORDER BY UR.ID_UR;

    -- TOTALIZADORES
    PRINT 'IMPORTACAO UNIDADE REQUISITANTE >> ESTRUTURAORGANIZACIONAL';
    DECLARE @count_asi bigint = (SELECT COUNT(*) FROM ASI_HOMOLOGA..AX_UR);
    DECLARE @count_grp bigint = (SELECT COUNT(*) FROM DBO.ESTRUTURAORGANIZACIONAL WHERE TIPOESTRUTURAORGANIZACIONAL_ID = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoEstruturaOrganizacional' AND CODIGO=4));
    PRINT 'Total de registros do ASI ' + CAST(@count_asi AS NVARCHAR)
    PRINT 'Total de registros no GRP ' + CAST(@count_grp AS NVARCHAR)
    PRINT ' ';
    
    COMMIT
-- #FIM :: IMPORTACAO UNIDADE REQUISITANTE >> ESTRUTURAORGANIZACIONAL
GO


-- #INI :: REPLICACAO ESTRUTURAORGANIZACIONAL >> ESTRUTURAORGANIZACIONALPATRIM
    BEGIN TRANSACTION
    GO
    
    INSERT INTO dbo.estruturaorganizacionalpatrim
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]    
         , isalmoxarifado
         , islocalizadoradebem
         , inativador_id
         , autor_id
         , editor_id
         , estruturaorganizacional_id
         )
    SELECT NULL      AS databloqueio
         , NULL      AS datainativo
         , GETDATE() AS datacriacao
         , GETDATE() AS dataedicao
         , 0         AS version
         , 0         AS isalmoxarifado
         , 0         AS islocalizadoradebem
         , NULL      AS inativador_id
         , NULL      AS autor_id
         , NULL      AS editor_id
         , E.ID      AS estruturaorganizacional_id
    FROM ESTRUTURAORGANIZACIONAL E;    

    -- TOTALIZADORES
    PRINT 'REPLICACAO ESTRUTURAORGANIZACIONAL >> ESTRUTURAORGANIZACIONALPATRIM';
    DECLARE @count_asi bigint = (SELECT COUNT(*) FROM DBO.ESTRUTURAORGANIZACIONAL);
    DECLARE @count_grp bigint = (SELECT COUNT(*) FROM DBO.ESTRUTURAORGANIZACIONALPATRIM);
    PRINT 'Total de registros de EO  ' + CAST(@count_asi AS NVARCHAR)
    PRINT 'Total de registros no EOP ' + CAST(@count_grp AS NVARCHAR)
    PRINT ' ';
    
    COMMIT
-- #FIM :: REPLICACAO ESTRUTURAORGANIZACIONAL >> ESTRUTURAORGANIZACIONALPATRIM
GO


-- #INI :: REPLICACAO ESTRUTURAORGANIZACIONAL >> ESTRUTURAORGANIZACIONALALMOX
    BEGIN TRANSACTION
    GO
    
    INSERT INTO dbo.estruturaorganizacionalalmox
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , diasrequisicao
         , isalmoxarifado
         , isundconsumidorarequisitante
         , inativador_id
         , autor_id
         , editor_id
         , estruturaorganizacional_id
         )
    SELECT NULL      AS databloqueio
         , NULL      AS datainativo
         , GETDATE() AS datacriacao
         , GETDATE() AS dataedicao
         , 0         AS "version"
         , NULL      AS diasrequisicao
         , 1         AS isalmoxarifado
         , 1         AS isundconsumidorarequisitante
         , NULL      AS inativador_id
         , NULL      AS autor_id
         , NULL      AS editor_id
         , E.ID      AS estruturaorganizacional_id
    FROM ESTRUTURAORGANIZACIONAL E;    

    -- TOTALIZADORES
    PRINT 'REPLICACAO ESTRUTURAORGANIZACIONAL >> ESTRUTURAORGANIZACIONALALMOX';
    DECLARE @count_asi bigint = (SELECT COUNT(*) FROM DBO.ESTRUTURAORGANIZACIONAL);
    DECLARE @count_grp bigint = (SELECT COUNT(*) FROM DBO.ESTRUTURAORGANIZACIONALALMOX);
    PRINT 'Total de registros de EO  ' + CAST(@count_asi AS NVARCHAR)
    PRINT 'Total de registros no EOA ' + CAST(@count_grp AS NVARCHAR)
    PRINT ' ';

    COMMIT
-- #FIM :: REPLICACAO ESTRUTURAORGANIZACIONAL >> ESTRUTURAORGANIZACIONALALMOX
GO


-- #INI :: AJUSTAR PARENT_ID COM BASE NO CD_UA_SUP IMPORTADO PARA CLASSIFICACAO
    BEGIN TRANSACTION
    GO
    
    PRINT 'Ajustando referência à unidade superior...'
    UPDATE dbo.estruturaorganizacional
       SET unidadesuperior_id = ep.parent_id
      FROM 
         ( SELECT f.id, p.id AS parent_id
             FROM dbo.estruturaorganizacional p  INNER JOIN 
                  dbo.estruturaorganizacional f  ON f.classificacao = p.codigo
         ) AS ep
     WHERE dbo.estruturaorganizacional.id = ep.id
       AND dbo.estruturaorganizacional.id <> 1;

    COMMIT
-- #FIM :: AJUSTAR PARENT_ID COM BASE NO CD_UA_SUP IMPORTADO PARA CLASSIFICACAO
GO


-- #INI :: AJUSTAR CENTROCUSTO_ID COM BASE NO CD_CENTRO_CUSTO DA UNIDADE REQUISITANTE
    BEGIN TRANSACTION
    GO
    
    PRINT 'Ajustando referência ao centro de custo...'
    UPDATE dbo.estruturaorganizacionalalmox
       SET centrocusto_id = c.cc_id
      FROM 
         ( SELECT EA.ID AS UR_ID, CC.ID AS CC_ID
           FROM ASI_HOMOLOGA..AX_UR UR
           INNER JOIN dbo.ESTRUTURAORGANIZACIONAL EA ON (EA.classificacao COLLATE SQL_Latin1_General_CP1_CI_AI = UR.CD_UA AND EA.CODIGO COLLATE SQL_Latin1_General_CP1_CI_AI= UR.CD_UR)
           INNER JOIN dbo.CENTROCUSTO CC ON (CC.CODIGO COLLATE SQL_Latin1_General_CP1_CI_AI = UR.CD_CENTRO_CUSTO)
         ) AS c
     WHERE estruturaorganizacionalalmox.estruturaOrganizacional_id = c.ur_id;

    COMMIT
-- #INI :: AJUSTAR CENTROCUSTO_ID COM BASE NO CD_CENTRO_CUSTO DA UNIDADE REQUISITANTE
GO


-- #INI :: PRECISEI JOGAR 1 NO CODIGO DAS URS POR QUE URS TINHAM CODIGOS DUPLICADOS COM ULS 
    BEGIN TRANSACTION
    GO
    
    /*
     * Alinhado com área de negócios de fazer esse update para jogar o código da classifição
     * sem o 1 para o usuário poder buscar pelo código antigo.
     */
    PRINT 'Ajustando código da classificação da UR...'
    UPDATE dbo.estruturaorganizacional
       SET classificacao = ep.codigo
      FROM 
         ( SELECT id, substring(codigo, 0, 5) as codigo
             FROM dbo.estruturaorganizacional                   
         ) AS ep
     WHERE dbo.estruturaorganizacional.id = ep.id  -- 1;"Unidade gestora"; 2;"Unidade administrativa"; 3;"Unidade localizadora"; 4;"Unidade requisitante"
       AND dbo.estruturaorganizacional.tipoestruturaorganizacional_id = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoEstruturaOrganizacional' AND CODIGO=4); 

    COMMIT
-- #FIM :: PRECISEI JOGAR 1 NO CODIGO DAS URS POR QUE URS TINHAM CODIGOS DUPLICADOS COM ULS
GO


-- #INI :: IMPORTACAO DAS CONTAS CONTABEIS >> CONTACONTABIL
    BEGIN TRANSACTION
    GO
    
    PRINT 'Inserindo contacontabil...'
    INSERT INTO dbo.contacontabil
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , codigo
         , codigosistemaorigem
         , descricao
         , percentualnaodepreciavel
         , taxadepreciacaomensal
         , vidautil
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         , contacontabilsaldoanterior_id
         , dominiotipocontacontabil_id
         , dominiotipomaterial_id
         )
    SELECT CC.DT_BLOQUEIO     AS databloqueio
         , NULL               AS datainativo
         , GETDATE()          AS datacriacao
         , GETDATE()          AS dataedicao
         , 0                  AS "version"
         , CC.CD_CONTA        AS codigo
         , CC.CD_CONTA        AS codigosistemaorigem
         , CC.NM_CONTA        AS descricao
         , (CC.VL_PERCENT_NAO_DEPRECIAVEL/100) AS percentualnaodepreciavel
         , CASE 
             WHEN CC.NR_VIDA_UTIL = 0 OR CC.NR_VIDA_UTIL IS NULL THEN 0.0
             ELSE 1/CC.NR_VIDA_UTIL
           END                AS taxadepreciacaomensal 
         , COALESCE(CC.NR_VIDA_UTIL, 0.0) AS vidautil
         , NULL               AS inativador_id
         , NULL               AS autor_id
         , NULL               AS editor_id
         , 1                  AS organizacao_id
         , NULL               AS contacontabilsaldoanterior_id 
         , (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoContaContabil' AND CODIGO=5) AS dominiotipocontacontabil_id -- Todas como contas do "Ativo"
         , CASE 
             WHEN CC.CD_SISTEMA = 'AX' THEN  -- Material de consumo
               (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMaterial' AND CODIGO= 1) 
             WHEN CC.CD_SISTEMA = 'PA' THEN  -- Material de permanente
               (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMaterial' AND CODIGO= 2)
           END                AS dominiotipomaterial_id
    FROM ASI_HOMOLOGA..CR_CONTA CC
    ORDER BY CC.ID_CONTA;

    -- TOTALIZADORES
    PRINT 'IMPORTACAO DAS CONTAS CONTABEIS >> CONTACONTABIL';
    DECLARE @count_asi bigint = (SELECT COUNT(*) FROM ASI_HOMOLOGA..CR_CONTA);
    DECLARE @count_grp bigint = (SELECT COUNT(*) FROM DBO.CONTACONTABIL);
    PRINT 'Total de registros do ASI ' + CAST(@count_asi AS NVARCHAR)
    PRINT 'Total de registros no GRP ' + CAST(@count_grp AS NVARCHAR)
    PRINT ' ';

    COMMIT
-- #FIM :: IMPORTACAO DAS CONTAS CONTABEIS >> CONTACONTABIL
GO


-- #INI :: DEFINE UMA CONFIGURACAO PADRAO DO ORGAO
    BEGIN TRANSACTION
    GO
    
    DECLARE @conta_id bigint = (SELECT TOP 1 ID FROM dbo.contacontabil WHERE descricao LIKE '%ESTOQUE%');
    
    IF @conta_id IS NOT NULL
    BEGIN
        SET IDENTITY_INSERT dbo.configuracao ON;
        INSERT INTO dbo.configuracao
             (id, databloqueio, datainativo, datacriacao, dataedicao, version,
              inativador_id, autor_id, editor_id, anexoimagem_id, organizacao_id) 
        VALUES
             (1, NULL, NULL, GETDATE(), GETDATE(), 0,
              NULL, NULL, NULL, NULL, 1);
        SET IDENTITY_INSERT dbo.configuracao OFF;
    
        INSERT INTO dbo.configuracaoparametrosistema
             (datacriacao, dataedicao, [version], grupo, chave, valor, configuracao_id) 
        VALUES 
             (GETDATE(), GETDATE(), 0, NULL, 'EXECUTAR_SCRIPT_MENU-1', 'false', 1),
             (GETDATE(), GETDATE(), 0, NULL, 'EXECUTAR_SCRIPT_MENU-2', 'false', 1),
             (GETDATE(), GETDATE(), 0, NULL, 'EXECUTAR_SCRIPT_MENU-3', 'false', 1),
             (GETDATE(), GETDATE(), 0, NULL, 'EXECUTAR_SCRIPT_MENU-4', 'false', 1),
             (GETDATE(), GETDATE(), 0, NULL, 'EXECUTAR_SCRIPT_MENU-5', 'false', 1),
             (GETDATE(), GETDATE(), 0, NULL, 'EXECUTAR_SCRIPT_MENU-6', 'false', 1),
             (GETDATE(), GETDATE(), 0, NULL, 'EXECUTAR_SCRIPT_MENU-7', 'false', 1),
             (GETDATE(), GETDATE(), 0, NULL, 'EXECUTAR_SCRIPT_MENU', 'false', 1),
             (GETDATE(), GETDATE(), 0, NULL, 'EXECUTAR_DOMINIOS', 'false', 1),
             (GETDATE(), GETDATE(), 0, NULL, 'EXECUTAR_DEFAULT_FILE', 'false', 1),
             (GETDATE(), GETDATE(), 0, NULL, 'EXECUTAR_MODULO', 'false', 1),
             (GETDATE(), GETDATE(), 0, NULL, 'EXECUTAR_INTERNACIONALIZACAO', 'true', 1),
             (GETDATE(), GETDATE(), 0, NULL, 'PATRIMONIO_MASCARA_NUMERO_PATRIMONIO', '999999', 1),
             (GETDATE(), GETDATE(), 0, NULL, 'CONTA_CONTABIL_ALMOXARIFADO', CAST(@conta_id as nvarchar), 1),
             (GETDATE(), GETDATE(), 0, NULL, 'TIPO_AVALIACAO_MONETARIA_ESTOQUE', '2', 1),
             (GETDATE(), GETDATE(), 0, NULL, 'RELATORIO_PRIMEIRO_TITULO', 'Nome do órgão', 1),
             (GETDATE(), GETDATE(), 0, NULL, 'RELATORIO_SEGUNDO_TITULO', 'Diretoria de Administração', 1),
             (GETDATE(), GETDATE(), 0, NULL, 'RELATORIO_TERCEIRO_TITULO', 'Coordenação-Geral de Administração Patrimonial', 1),
             (GETDATE(), GETDATE(), 0, NULL, 'MASCARA_NUMERO_IDENTIFICACAO', 'AAAA999999', 1),
             (GETDATE(), GETDATE(), 0, NULL, 'TIPO_DEP_REAVALIACAO_REDUCAO', '1', 1),
             (GETDATE(), GETDATE(), 0, NULL, 'TIPO_MOV_REAVALIACAO_REDUCAO', '2', 1);
    END
    ELSE
    BEGIN
      PRINT 'Conta [BENS MÓVEIS EM ESTOQUE] não encontrada!';
      PRINT ' ';
    END;

    -- TOTALIZADORES
    PRINT 'DEFINE UMA CONFIGURACAO PADRAO DO ORGAO';
    DECLARE @count_grp bigint = (SELECT COUNT(*) FROM DBO.CONFIGURACAO);
    PRINT 'Total de registros no GRP ' + CAST(@count_grp AS NVARCHAR)
    PRINT ' ';
    
    COMMIT
-- #FIM :: DEFINE UMA CONFIGURACAO PADRAO DO ORGAO
GO


-- #INI :: DEFINE UM MAPA ORGANIZACIONAL PADRAO DO ORGAO
    BEGIN TRANSACTION
    GO
    
    PRINT 'Inserindo mapaorganizacional...'
    INSERT INTO dbo.mapaorganizacional
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , codigo
         , datafim
         , datainicio
         , nome
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         )
    VALUES 
         ( NULL
         , NULL
         , GETDATE()
         , GETDATE()
         , 0
         , '2016000001'
         , NULL
         , GETDATE()
         , 'MAPA - MIGRACAO ASI (2016)'
         , NULL
         , NULL
         , NULL
         , 1
         );

    -- TOTALIZADORES
    PRINT 'DEFINE UM MAPA ORGANIZACIONAL PADRAO DO ORGAO';
    DECLARE @count_grp bigint = (SELECT COUNT(*) FROM dbo.mapaorganizacional);
    PRINT 'Total de registros no GRP ' + CAST(@count_grp AS NVARCHAR)
    PRINT ' ';
    
    COMMIT
-- #FIM :: DEFINE UM MAPA ORGANIZACIONAL PADRAO DO ORGAO
GO


SET NOCOUNT OFF
--===========================================================================--
-- ##FIM :: IMPORTACAO DAS ESTRUTURAS DO ORGAO                               --
--===========================================================================--