--===========================================================================--
-- ##INI :: IMPORTACAO TABELAS DE MOVIMENTO DE MATERIAS PERMANENTES          --
--===========================================================================--
SET NOCOUNT ON


-- #INI :: MOVIMENTACAO DE DAS ENTRADAS >> ENTRADA
    BEGIN TRANSACTION
    GO
    
    /*
      OBSERVAÇÕES:
      ============
      1. O código da entrada foi formado por CD_ENTRADA+CD_UG para garantir unicidade,
         pois no ASI a chave é CD_ORGAO + CD_UG + CD_ENTRADA;
      2. Tempo estimado de execução de ~12 minutos;
      3. Foi necessário filtrar somente registros com DT_CONTABIL, porque o ASI considera
         os registros sem conteúdo neste campo como "apagados". Já no GRP a data contábil
         é obrigatória.
    */

    --> REGISTRO DE ENTRADA
    DECLARE @datacriacao     DATETIME
          , @dataedicao      DATETIME
          , @codigo          VARCHAR(30)
          , @codigoasi       VARCHAR(30)
          , @datacontabil    DATETIME
          , @datarecebimento DATETIME
          , @datareferencia  DATETIME
          , @valortotalnota  NUMERIC(10,2)
          , @organizacao     BIGINT
          , @tipoentrada     BIGINT
          , @tiporecebimento BIGINT
          , @fornecedor      BIGINT;
    
    DECLARE ent_cursor CURSOR FAST_FORWARD FOR  
        SELECT DISTINCT  E1.DT_INCLUSAO  AS datacriacao
         , E1.DT_INCLUSAO                AS dataedicao
         , E1.CD_ENTRADA                 AS codigo
         , E1.CD_ENTRADA+E1.CD_UG        AS codigoasi
         , E1.DT_CONTABIL                AS datacontabil
         , E1.DT_AQUISICAO               AS datarecebimento
         , E1.DT_INCLUSAO                AS datareferencia
         , VT.VL_ENTRADA/100             AS valortotalnota
         , 1                             AS organizacao_id
         , CASE 
             WHEN TP.IN_ORCAMENTARIO='S' THEN (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoEntrada' AND CODIGO=1)
             ELSE (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoEntrada' AND CODIGO=2)
           END                           AS dominiotipoentrada_id
         , (SELECT ID_GRP FROM DBO.DOMINIO_ASI_GRP WHERE TP_GRP='tipoRecebimento' AND CD_ASI=E1.CD_TP_FORMA_OP) AS dominiotiporecebimento_id -- "asiweb.bp_tp_forma_op"
         , P.ID fornecedor_id
        FROM ASI_HOMOLOGA..BP_ENTRADA E1
        JOIN ASI_HOMOLOGA..BP_TP_FORMA_OP_POSSIVEL TP ON TP.CD_TP_OPERACAO=E1.CD_TP_OPERACAO AND TP.CD_TP_FORMA_OP=E1.CD_TP_FORMA_OP AND TP.CD_TP_BEM=E1.CD_TP_BEM
        JOIN ASI_HOMOLOGA..BP_ENT_ITEM E2 ON E1.CD_ORGAO=E2.CD_ORGAO AND E1.CD_UG=E2.CD_UG AND E1.CD_ENTRADA=E2.CD_ENTRADA
        JOIN ASI_HOMOLOGA..BP_BEM E3 ON E2.CD_ORGAO=E3.CD_ORGAO AND E2.CD_UG=E3.CD_UG AND E2.CD_ENTRADA=E3.CD_ENTRADA AND E2.SQ_ITEM=E3.SQ_ITEM
        JOIN ASI_HOMOLOGA..CR_BEM_SERVICO BS ON BS.CD_BEM_SERVICO=E3.CD_BEM_SERVICO     
        JOIN ( SELECT BP_BEM.CD_ORGAO, BP_BEM.CD_UG, BP_BEM.CD_ENTRADA, SUM(BP_BEM.VL_UNITARIO) AS VL_ENTRADA 
               FROM ASI_HOMOLOGA..BP_BEM 
               GROUP BY BP_BEM.CD_ORGAO, BP_BEM.CD_UG, BP_BEM.CD_ENTRADA
              ) AS VT ON VT.CD_ORGAO=E1.CD_ORGAO AND VT.CD_UG=E1.CD_UG AND VT.CD_ENTRADA=E1.CD_ENTRADA
         LEFT JOIN DBO.PARCEIRO P ON P.CODIGO=E1.CD_AGENTE;
         
    PRINT 'Importando os registros consolidados de entrada (header)...'
    OPEN ent_cursor   
    FETCH NEXT FROM ent_cursor 
     INTO @datacriacao    
        , @dataedicao     
        , @codigo         
        , @codigoasi      
        , @datacontabil   
        , @datarecebimento 
        , @datareferencia 
        , @valortotalnota 
        , @organizacao    
        , @tipoentrada    
        , @tiporecebimento
        , @fornecedor;
        
    WHILE @@FETCH_STATUS = 0   
    BEGIN   
        INSERT INTO dbo.entrada
             ( datacriacao, dataedicao, [version], codigo, codigoasi
             , datacontabil, datarecebimento, datareferencia, valortotalnota
             , organizacao_id, dominiotipoentrada_id, dominiotiporecebimento_id, fornecedor_id
             )
        VALUES
             ( @datacriacao, @dataedicao, 0, @codigo, @codigoasi
             , @datacontabil, @datarecebimento, @datareferencia, @valortotalnota
             , @organizacao, @tipoentrada, @tiporecebimento, @fornecedor );
    
        FETCH NEXT FROM ent_cursor 
         INTO @datacriacao
            , @dataedicao
            , @codigo
            , @codigoasi
            , @datacontabil
            , @datarecebimento
            , @datareferencia
            , @valortotalnota
            , @organizacao
            , @tipoentrada
            , @tiporecebimento
            , @fornecedor;
    END
    
    CLOSE ent_cursor   
    DEALLOCATE ent_cursor

    PRINT 'Criando os registros de referencia em entradapatrimonio...'
    INSERT INTO dbo.entradapatrimonio
         ( id
         , dominiotipobem_id
         , estruturaorganizacional_id
         )
    SELECT DISTINCT E.ID AS id
         , (SELECT ID_GRP FROM DBO.DOMINIO_ASI_GRP WHERE TP_GRP='tipoBem' AND CD_ASI=E1.CD_TP_BEM) AS dominiotipobem_id -- "asiweb..bp_tp_bem"
         , (SELECT ID FROM DBO.ESTRUTURAORGANIZACIONAL E WHERE E.CODIGO=B.CD_UL_ENTRADA AND E.TIPOESTRUTURAORGANIZACIONAL_ID = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoEstruturaOrganizacional' AND CODIGO=3)) AS estruturaorganizacional_id 
    FROM ASI_HOMOLOGA..BP_ENTRADA E1
    INNER JOIN DBO.ENTRADA E ON (E.CODIGOASI=E1.CD_ENTRADA+E1.CD_UG)
    LEFT JOIN ASI_HOMOLOGA..BP_BEM B ON (E.CODIGOASI=B.CD_ENTRADA+B.CD_UG);

    /*
     A estratégia usada para incluir os itens, foi agrupá-los por valor unitário para
     bater com o total, pois um dos itens do mesmo material pode ter valor unitario
     diferente por causa do resto de uma dízima periódica.
    */
    PRINT 'Criando os registros de itens da referencia em entradapatrimonio...'
    INSERT INTO dbo.entradapatrimonioitem
         ( datacriacao
         , dataedicao
         , [version]
         , emplaquetado
         , quantidade
         , sequencia
         , valor
         , valorunitario
         , entrada_id
         , material_id
         )
    SELECT E.DATACRIACAO                  AS datacriacao
         , E.DATAEDICAO                   AS dataedicao
         , 0                              AS "version"
         , 1                              AS emplaquetado
         , COUNT(*)                       AS quantidade
         , B.SQ_ITEM                      AS sequencia
         , ((B.VL_UNITARIO/100)*COUNT(*)) AS valor
         , (B.VL_UNITARIO/100)            AS valorunitario
         , E.ID                           AS entrada_id
         , M.ID                           AS material_id
    FROM DBO.ENTRADA E
    JOIN ASI_HOMOLOGA..BP_ENT_ITEM I ON E.CODIGOASI=I.CD_ENTRADA+I.CD_UG
    JOIN ASI_HOMOLOGA..BP_BEM B ON I.CD_ORGAO=B.CD_ORGAO AND I.CD_UG=B.CD_UG AND I.CD_ENTRADA=B.CD_ENTRADA AND I.SQ_ITEM=B.SQ_ITEM
    JOIN DBO.MATERIAL M ON (M.CODIGO=B.CD_BEM_SERVICO)
    GROUP BY B.CD_BEM_SERVICO, B.VL_UNITARIO, E.DATACRIACAO, E.DATAEDICAO, B.SQ_ITEM, E.ID, M.ID;

    PRINT 'Importando os documentos de entrada...'
    SET IDENTITY_INSERT dbo.documento ON;
    INSERT INTO dbo.documento
         ( id
         , databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , dataemissao
         , numero
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         , dominiotipodocumento_id
         , emitente_id
         )
    SELECT (ROW_NUMBER() OVER (ORDER BY D.CD_ENTRADA)) + M.MAX AS id
         , NULL                      AS databloqueio
         , NULL                      AS datainativo
         , GETDATE()                 AS datacriacao
         , GETDATE()                 AS dataedicao
         , 0                         AS "version"
         , D.DT_EMISSAO              AS dataemissao
         , D.NR_DOCUMENTO            AS numero
         , NULL                      AS inativador_id
         , NULL                      AS autor_id
         , NULL                      AS editor_id
         , 1                         AS organizacao_id
         , (SELECT ID_GRP FROM DBO.DOMINIO_ASI_GRP WHERE TP_GRP='tipoDocumento' AND CD_ASI=D.CD_TP_DOCUMENTO) AS dominiotipodocumento_id -- "asiweb.cr_tp_documento"
         , P.ID                      AS emitente_id
    FROM ASI_HOMOLOGA..BP_ENT_DOC D
    JOIN (SELECT COALESCE(MAX(ID),0) AS MAX FROM DBO.DOCUMENTO) M ON 1=1
    LEFT JOIN PARCEIRO P ON P.CODIGO = D.CD_AGENTE;
    SET IDENTITY_INSERT dbo.documento OFF;

    PRINT 'Criando a referencia dos documentos das entradas...'
    INSERT INTO dbo.entradadocumento
         ( id
         , entrada_id
         , entradainativo_id
         )
    SELECT (ROW_NUMBER() OVER (ORDER BY D.CD_ENTRADA)) + M.MAX AS id
         , E.ID AS entrada_id
         , NULL AS entradainativo_id
    FROM ASI_HOMOLOGA..BP_ENT_DOC D
    JOIN (SELECT COALESCE(MAX(ID),0) AS MAX FROM DBO.ENTRADADOCUMENTO) M ON 1=1
    LEFT JOIN DBO.ENTRADA E ON E.CODIGOASI=D.CD_ENTRADA+D.CD_UG
    ORDER BY D.CD_ENTRADA;
    
    /*
     CONSIDERAÇÕES:
     1. Como o banco do ASI mantém apenas MES/ANO da data de início da depreciacao, foram 
        acrescentados a todos os meses o dia 01;
     2. Para buscar o VALOR_LIQUIDO do bem, buscamos o último registro da tabela de depreciações,
        e calculamos a diferença entre o valor unitário do bem e VL_ACUMULADO da depreciação;
     3. A vida útil restante também foi obtida da tabela de depreciações;
     4. O conteúdo do campo SQ_BEM_PERM do ASI foi migrado para atributo CODIGO, para fazer o
        relacionamento do bem com algumas tabelas do ASI.
    */
    PRINT 'Desabilitando as constraints de bempatrimonial...'
    ALTER TABLE bempatrimonial NOCHECK CONSTRAINT ALL;

    PRINT 'Importando os registros de bempatrimonial...'
    INSERT INTO dbo.bempatrimonial
         ( datacriacao
         , dataedicao
         , [version]
         , bemprincipal
         , codigo         
         , datafinalgarantia
         , datainicialgarantia
         , datainiciodepreciacao
         , numeropatrimonial
         , numeropatrimonialnumerico
         , valoraquisicao
         , valorliquido
         , vidautiloriginal
         , vidautilatual
         , organizacao_id
         , dominiosituacaofisica_id
         , dominiostatus_id
         , entradaitem_id
         , unidadeatual_id
         , unidadecompra_id
         , fornecedor_id
         , material_id
         , depreciacaoacumulada
         , naodepreciar
         , percentualnaodepreciavel
         , reavaliacaoacumulada
         , reavaliacaoperiodo
         , reducaoacumulada
         , reducaoperiodo
         , valorbrutoatual
         , valorbrutocalculo
         , datacontabil
         )
    SELECT DISTINCT 
           ISNULL(I.DATACRIACAO,GETDATE())     AS datacriacao
         , ISNULL(I.DATAEDICAO,GETDATE())      AS dataedicao
         , 0                                   AS "version"
         , 0                                   AS bemprincipal
         , B.SQ_BEM_PERM                       AS codigo         
         , B.DT_GARANTIA_FIM                   AS datafinalgarantia
         , B.DT_GARANTIA_INI                   AS datainicialgarantia
         , CONVERT(DATETIME,B.DT_ANOMES_INICIO_DEP+'01') AS datainiciodepreciacao
         , B.CD_BEM_PERM                       AS numeropatrimonial
         , DBO.HASHCODE(B.CD_BEM_PERM)         AS numeropatrimonialnumerico
         , (V.VL_UNITARIO/100)                 AS valoraquisicao
         , 0.0                                 AS valorliquido       -- ATUALIZADO EM SEGUIDA
         , B.NR_VIDA_UTIL                      AS vidautiloriginal
         , B.NR_VIDA_UTIL                      AS vidautilatual      -- ATUALIZADO EM SEGUIDA
         , 1                                   AS organizacao_id
         , (SELECT ID_GRP FROM DBO.DOMINIO_ASI_GRP WHERE TP_GRP='tipoSituacaoFisica' AND CD_ASI=B.CD_TP_SIT_FISICA) AS dominiosituacaofisica_id  -- "asiweb.bp_tp_sit_fisica"
         , (SELECT ID_GRP FROM DBO.DOMINIO_ASI_GRP WHERE TP_GRP='tipoStatusBem' AND CD_ASI=B.CD_TP_STATUS) AS dominiostatus_id  -- "asiweb.bp_tp_status"
         , I.ID                                AS entradaitem_id
         , UND_ATUAL.ID                        AS unidadeatual_id
         , UND_ENTRADA.ID                      AS unidadecompra_id
         , E.FORNECEDOR_ID                     AS fornecedor_id
         , I.MATERIAL_ID                       AS material_id
         , 0
         , 0    
         , B.VL_PERCENT_NAO_DEPRECIAVEL/100    AS PERCENTUALNAODEPRECIAVEL
         , 0
         , 0
         , 0
         , 0
         , (V.VL_UNITARIO/100)
         , (V.VL_UNITARIO/100)
         , E.DATACONTABIL
    FROM ASI_HOMOLOGA..BP_BEM B
    LEFT JOIN DBO.ENTRADA E ON (E.CODIGOASI=B.CD_ENTRADA+B.CD_UG)
    LEFT JOIN DBO.MATERIAL M ON (M.CODIGO=B.CD_BEM_SERVICO)
    LEFT JOIN DBO.ENTRADAPATRIMONIOITEM I ON (I.ENTRADA_ID= E.ID AND I.SEQUENCIA=B.SQ_ITEM AND I.VALORUNITARIO = (B.VL_UNITARIO/100) AND I.MATERIAL_ID=M.ID)
    LEFT JOIN DBO.ESTRUTURAORGANIZACIONAL UND_ATUAL ON (UND_ATUAL.CODIGO=B.CD_UL_ATUAL AND UND_ATUAL.TIPOESTRUTURAORGANIZACIONAL_ID = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoEstruturaOrganizacional' AND CODIGO=3))
    LEFT JOIN DBO.ESTRUTURAORGANIZACIONAL UND_ENTRADA ON (UND_ENTRADA.CODIGO=B.CD_UL_ENTRADA AND UND_ENTRADA.TIPOESTRUTURAORGANIZACIONAL_ID = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoEstruturaOrganizacional' AND CODIGO=3))
    LEFT JOIN ASI_HOMOLOGA..BP_BEM V ON V.SQ_BEM_PERM=B.SQ_BEM_PERM;

    COMMIT
    PRINT 'Finalizada migração de entrada de bens!'
-- #INI :: MOVIMENTACAO DE DAS ENTRADAS >> ENTRADA
GO


-- #INI :: AJUSTES TABELA BEMPATRIMONIAL >> COLUNA CODIGO
    BEGIN TRANSACTION
    GO

    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'codigo_tmp' AND Object_ID = Object_ID(N'BemPatrimonial'))
      ALTER TABLE dbo.bempatrimonial ADD codigo_tmp VARCHAR(30)
    ELSE
      PRINT 'Coluna codigo_tmp já existe na tabela BemPatrimonial'
    GO
    COMMIT

    BEGIN TRANSACTION
    GO

    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix01_bempat_cd' AND Object_ID = Object_ID(N'BemPatrimonial'))
    BEGIN
      UPDATE dbo.bempatrimonial SET codigo_tmp = codigo;
      ALTER TABLE dbo.bempatrimonial DROP COLUMN codigo;
      ALTER TABLE dbo.bempatrimonial ADD codigo BIGINT;
      UPDATE dbo.bempatrimonial SET codigo = CONVERT(BIGINT, codigo_tmp);
      CREATE NONCLUSTERED INDEX ix01_bempat_cd ON dbo.BemPatrimonial (codigo ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    END
    ELSE
      PRINT 'Indice ix01_bempat_cd já existe na tabela BemPatrimonial'
    GO
    COMMIT
    PRINT 'Finalizado ajustes em BemPatrimonial.Codigo!'
-- #FIM :: AJUSTES TABELA BEMPATRIMONIAL >> COLUNA CODIGO
GO


-- #INI :: AJUSTES DADOS TABELA DEPRECIACAO >> BEMPATRIMONIAL
    BEGIN TRANSACTION
    GO

    PRINT 'Ajustando dados de depreciação em bempatrimonial...'
    DECLARE @bem_cd  BIGINT
          , @bem_id  BIGINT;
    
    --> RECUPERA APENAS OS BENS PATRIMONIAIS QUE ESTAO NA TABELA DE DEPRECIACAO
    DECLARE cur_depre  CURSOR FAST_FORWARD FOR
        SELECT B.CODIGO, B.ID
        FROM   DBO.BEMPATRIMONIAL B 
        WHERE EXISTS (SELECT 'X' FROM ASI_HOMOLOGA..BP_DEPRECIACAO DP WHERE DP.SQ_BEM_PERM = B.CODIGO);
        
    OPEN cur_depre
    FETCH NEXT FROM cur_depre INTO @bem_cd, @bem_id
    WHILE @@FETCH_STATUS = 0
    BEGIN
        SELECT D.SQ_BEM_PERM                      AS codigo
             , ((D.VL_UNITARIO)-(D.VL_DEPR_ACUM)) AS valorliquido
             , D.NR_MESES_DEPRECIADOS             AS vidautilacumulada
             , D.VL_DEPR_ACUM                     AS depreciacaoacumulada
             , D.VL_UNITARIO                      AS valorbrutocalculo
          INTO #tmp_depre
          FROM ASI_HOMOLOGA..BP_DEPRECIACAO D              
         WHERE D.SQ_BEM_PERM = @bem_cd 
           AND D.DT_ANOMES_FECHAMENTO = (SELECT MAX(DP.DT_ANOMES_FECHAMENTO) 
                                         FROM   ASI_HOMOLOGA..BP_DEPRECIACAO DP 
                                         WHERE  DP.SQ_BEM_PERM = D.SQ_BEM_PERM)

        UPDATE dbo.BemPatrimonial
           SET valorliquido         = (SELECT valorliquido         FROM #tmp_depre WHERE codigo = @bem_cd)
             , vidautilacumulada    = (SELECT vidautilacumulada    FROM #tmp_depre WHERE codigo = @bem_cd)
             , depreciacaoacumulada = (SELECT depreciacaoacumulada FROM #tmp_depre WHERE codigo = @bem_cd)
             , valorbrutocalculo    = (SELECT valorbrutocalculo    FROM #tmp_depre WHERE codigo = @bem_cd)
             , valorbrutoatual      = (SELECT valorbrutoatual      FROM #tmp_depre WHERE codigo = @bem_cd)
         WHERE ID = @bem_id
        
        FETCH NEXT FROM cur_depre INTO @bem_cd, @bem_id
        DROP TABLE #tmp_depre
      END
      
    CLOSE cur_depre
    DEALLOCATE cur_depre
    COMMIT
    PRINT 'Finalizado ajustes de depreciação em BemPatrimonial!'
-- #FIM :: AJUSTES DADOS TABELA DEPRECIACAO >> BEMPATRIMONIAL
GO


-- #INI :: AJUSTES VIDA UTIL DO BEM >> BEMPATRIMONIAL
    BEGIN TRANSACTION
    GO

    PRINT 'Ajustando vida útil do bempatrimonial...'
    DECLARE @bem_id  BIGINT
          , @vidautil   INT;

    DECLARE cur_vidautil CURSOR FAST_FORWARD FOR
        SELECT B.ID, B.VIDAUTILATUAL
        FROM   DBO.BEMPATRIMONIAL B 
        WHERE  VIDAUTILACUMULADA > VIDAUTILATUAL;
    
    OPEN cur_vidautil
    FETCH NEXT FROM cur_vidautil INTO @bem_id, @vidautil
    WHILE @@FETCH_STATUS = 0
    BEGIN
        UPDATE dbo.BemPatrimonial
           SET vidautilacumulada = @vidautil
         WHERE id = @bem_id;
             
        FETCH NEXT FROM cur_vidautil INTO @bem_id, @vidautil
    END      
    CLOSE cur_vidautil
    DEALLOCATE cur_vidautil
    COMMIT
    PRINT 'Finalizado ajustes de vidautil em BemPatrimonial!'
-- #FIM :: AJUSTES VIDA UTIL DO BEM >> BEMPATRIMONIAL
GO


-- #INI :: ATRIBUINDO DETENTOR AOS BENS QUE POSSUEM DETENTOR ATRIBUIDO
    BEGIN TRANSACTION
    GO

    UPDATE dbo.BemPatrimonial
       SET detentor_id = P.COLABORADOR_ID
      FROM
         ( SELECT C.id AS COLABORADOR_ID, B.SQ_BEM_PERM
           FROM ASI_HOMOLOGA..BP_BEM B
           INNER JOIN PESSOA P ON (P.CODIGO COLLATE SQL_Latin1_General_CP1_CI_AI = B.CD_DETENTOR)    
           INNER JOIN PARCEIRO C ON (C.PESSOA_ID = P.ID)       
           WHERE B.CD_DETENTOR IS NOT NULL           
         ) AS P
    WHERE CODIGO=P.SQ_BEM_PERM

    COMMIT
    PRINT 'Finalizado atribuindo detentor a BemPatrimonial!'
-- #FIM :: ATRIBUINDO DETENTOR AOS BENS QUE POSSUEM DETENTOR ATRIBUIDO
GO


-- #INI :: RETORNANDO AS CONSTRAINTS DE BEMPATRIMONIAL
    BEGIN TRANSACTION
    GO

    ALTER TABLE bempatrimonial WITH CHECK CHECK CONSTRAINT ALL;

    COMMIT
-- #FIM :: RETORNANDO AS CONSTRAINTS DE BEMPATRIMONIAL
GO


-- #INI :: MIGRACAO DOS VALORES DAS CARACTERISTICAS GENERICAS DOS BENS
    BEGIN TRANSACTION
    GO

    PRINT 'Importando as características genéricas de bens...'
    DECLARE @bem_id       BIGINT
          , @dominio_id   BIGINT
          , @conteudo     VARCHAR(MAX)
          , @tipoDado     VARCHAR(2)
          , @valorText    VARCHAR(4000)
          , @valorNumero  NUMERIC(15,2)
          , @caracteristica_id BIGINT
          , @chavedominio VARCHAR(50);
    
    -- RECUPERA AS CARACTERISTICAS DOS BENS
    DECLARE cur_cargen CURSOR FAST_FORWARD FOR  
        -- Na tabela de caracteristica existem mais campos que guardam valor, mas fiz uma busca e 
        -- nenhum no sistema e diferente de null: cg.ds_car_date as valorData, cg.ds_car_blob as arquivo
        SELECT CARASI.TP_DADO   AS tipoDado
             , CG.DS_CAR_CHAR   AS valorText
             , CG.DS_CAR_NUMBER AS valorNumero
             , BEM.ID           AS bem_id
             , MC.ID            AS caracteristica_id
             , CAR.CHAVEDOMINIO AS chavedominio
        FROM ASI_HOMOLOGA..BP_ENT_CAR_GEN CG
        INNER JOIN ASI_HOMOLOGA..CR_CARACTERISTICA CARASI ON (CARASI.CD_CARACTERISTICA = CG.CD_CARACTERISTICA)        
        INNER JOIN ASI_HOMOLOGA..BP_BEM B ON (B.CD_ORGAO = CG.CD_ORGAO AND B.CD_UG = CG.CD_UG AND B.CD_ENTRADA = CG.CD_ENTRADA AND B.SQ_ITEM = CG.SQ_ITEM)
        INNER JOIN DBO.BEMPATRIMONIAL BEM ON (BEM.CODIGO = B.SQ_BEM_PERM)
        INNER JOIN DBO.MATERIAL M ON (M.CODIGO COLLATE SQL_Latin1_General_CP1_CI_AI = B.CD_BEM_SERVICO) 
        INNER JOIN DBO.MATERIALCARACTERISTICA MC ON (MC.MATERIAL_ID = M.ID AND MC.CODIGO COLLATE SQL_Latin1_General_CP1_CI_AI= CG.CD_CARACTERISTICA)
        INNER JOIN DBO.CARACTERISTICA CAR ON (CAR.ID = MC.CARACTERISTICA_ID)

    OPEN cur_cargen   
    FETCH NEXT FROM cur_cargen 
        INTO @tipoDado
           , @valorText
           , @valorNumero
           , @bem_id
           , @caracteristica_id
           , @chavedominio
           
    WHILE @@FETCH_STATUS = 0
    BEGIN
        IF @chavedominio IS NOT NULL
        BEGIN
            SET @dominio_id = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE=@chavedominio AND CODIGO=CONVERT(BIGINT, @valorText));
            SET @conteudo = @dominio_id;
        END    
        ELSE
            SET @dominio_id = NULL;
        
        IF @tipoDado = 'NM'
        BEGIN 
            IF (@valorNumero-FLOOR(@valorNumero)) > 0
                SET @conteudo = (SELECT (CAST(@valorNumero AS varchar(MAX))))
            ELSE IF (@valorNumero-FLOOR(@valorNumero)) <= 0
                SET @conteudo = (SELECT (CAST(FLOOR(@valorNumero) AS varchar(MAX))))
            ELSE
                SET @conteudo = @valorText
        END
            
        INSERT INTO dbo.BemPatrimonialCaracteristica
             ( datacriacao, dataedicao, [version], valormaterialcaracteristica
             , bempatrimonial_id, materialcaracteristica_id )
        VALUES
             ( GETDATE(), GETDATE(), 0, @conteudo
             , @bem_id, @caracteristica_id )
        
        FETCH NEXT FROM cur_cargen 
            INTO @tipoDado
               , @valorText
               , @valorNumero
               , @bem_id
               , @caracteristica_id
               , @chavedominio
    END

    CLOSE cur_cargen
    DEALLOCATE cur_cargen
    COMMIT
    PRINT 'Finalizado migração de valores das características genéricas!'
-- #FIM :: MIGRACAO DOS VALORES DAS CARACTERISTICAS GENERICAS DOS BENS
GO


-- #INI :: MIGRACAO DOS VALORES DAS CARACTERISTICAS ESPECIFICAS DOS BENS
    BEGIN TRANSACTION
    GO

    PRINT 'Importando as características especificas de bens...'
    DECLARE @bem_id       BIGINT
          , @valorText    VARCHAR(4000)
          , @caracteristica_id BIGINT;
    
    -- RECUPERA AS CARACTERISTICAS DOS BENS
    DECLARE cur_carespec CURSOR FAST_FORWARD FOR                 
        -- Na tabela de caracteristicas especificas so existe valor no campo ce.ds_car_char, todas as caracteristicas desse tipo sao do tipo character
        SELECT CE.DS_CAR_CHAR AS valorText
             , BEM.ID         AS bem_id
             , MC.ID          AS caracteristica_id
        FROM ASI_HOMOLOGA..BP_ENT_CAR_BEM CE
        INNER JOIN ASI_HOMOLOGA..CR_CARACTERISTICA CARASI ON (CARASI.CD_CARACTERISTICA = CE.CD_CARACTERISTICA)                
        INNER JOIN DBO.BEMPATRIMONIAL BEM ON (BEM.CODIGO = CE.SQ_BEM_PERM)
        INNER JOIN DBO.MATERIAL M ON (M.ID = BEM.MATERIAL_ID) 
        INNER JOIN DBO.MATERIALCARACTERISTICA MC ON (MC.MATERIAL_ID = M.ID AND MC.CODIGO COLLATE SQL_Latin1_General_CP1_CI_AI = CE.CD_CARACTERISTICA)        
      
    OPEN cur_carespec
    FETCH NEXT FROM cur_carespec
        INTO @valorText
           , @bem_id
           , @caracteristica_id

    WHILE @@FETCH_STATUS = 0
    BEGIN
        INSERT INTO dbo.BemPatrimonialCaracteristica
             ( datacriacao, dataedicao, [version], valormaterialcaracteristica                           
             , bempatrimonial_id, materialcaracteristica_id )
        VALUES
             ( GETDATE(), GETDATE(), 0, @valorText
             , @bem_id, @caracteristica_id )
        
        FETCH NEXT FROM cur_carespec
            INTO @valorText
            , @bem_id
            , @caracteristica_id
    END
    CLOSE cur_carespec
    DEALLOCATE cur_carespec
    COMMIT
    PRINT 'Finalizado migração de valores das características específicas!'
-- #FIM :: MIGRACAO DOS VALORES DAS CARACTERISTICAS ESPECIFICAS DOS BENS
GO


-- #INI :: AJUSTA ATRIBUTOS 'ISLOCALIZADORADEBENS'/'ISALMOXARIFADO'/'ISUNDCONSUMIDORAREQUISITANTE'
    BEGIN TRANSACTION  
    GO

    PRINT 'Marcando o atribudo [ÉLocalizadoraDeBem] se existir bens não baixados na unidade atual...'
    UPDATE dbo.estruturaorganizacionalpatrim
       SET islocalizadoradebem = 1
      FROM  
         ( SELECT BP.UNIDADEATUAL_ID, COUNT(*) AS QTD
           FROM   BEMPATRIMONIAL BP   -- VERIFICA SE A ESTRUTURA POSSUI BENS NÃO BAIXADOS
           WHERE  BP.DOMINIOSTATUS_ID <> (SELECT ID FROM DOMINIO WHERE CHAVE='tipoStatusBem' AND CODIGO=2) -- BAIXADO
           GROUP  BY BP.UNIDADEATUAL_ID
         ) AS B
     WHERE B.UNIDADEATUAL_ID=ESTRUTURAORGANIZACIONAL_ID
       AND B.QTD > 0;
    GO

    PRINT 'Marcando o atribudo [ÉAlmoxarifado] das unidade informadas na entrada de bens...'
    UPDATE dbo.estruturaorganizacionalpatrim
       SET isalmoxarifado = 1
      FROM
         ( SELECT EO.ID AS ALM_ID
           FROM dbo.ESTRUTURAORGANIZACIONAL EO
           WHERE EO.DATAFIM IS NULL
           AND EO.ID IN (SELECT DISTINCT(UNIDADECOMPRA_ID) FROM dbo.BEMPATRIMONIAL)
         ) AS B
    WHERE ESTRUTURAORGANIZACIONAL_ID=B.ALM_ID;    
    GO
    
    UPDATE dbo.estruturaorganizacionalalmox
       SET isalmoxarifado = 1
      FROM
         ( SELECT EO.ID AS ALM_ID
           FROM dbo.ESTRUTURAORGANIZACIONAL EO
           WHERE EO.DATAFIM IS NULL
           AND EO.ID IN (SELECT DISTINCT(UNIDADECOMPRA_ID) FROM dbo.BEMPATRIMONIAL)
         ) AS B
    WHERE ESTRUTURAORGANIZACIONAL_ID=B.ALM_ID;
    GO

    /*  
    PRINT 'Marcando o atribudo [ÉUnidadeConsumidoraRequisitante] se corresponder a uma UR do ASI...';
    UPDATE ESTRUTURAORGANIZACIONALALMOX
       SET ISUNDCONSUMIDORAREQUISITANTE=1
      FROM  
         ( SELECT UL.ID FROM ASI_HOMOLOGA..AX_UR URQ
           INNER JOIN ASI_HOMOLOGA..CR_UL ULC ON URQ.CD_ORGAO = ULC.CD_ORGAO AND URQ.CD_UA = ULC.CD_UA AND URQ.CD_LOCAL = ULC.CD_LOCAL
           INNER JOIN dbo.ESTRUTURAORGANIZACIONAL UL ON UL.CODIGO = ULC.CD_UL
           INNER JOIN dbo.DOMINIO D ON UL.TIPOESTRUTURAORGANIZACIONAL_ID = D.ID AND D.CHAVE = 'tipoEstruturaOrganizacional' AND D.CODIGO=3
         ) AS U
     WHERE U.ID = ESTRUTURAORGANIZACIONAL_ID;
    */

    PRINT 'Marcando o atribudo [ÉUnidadeConsumidoraRequisitante] se foi cadastrada como UR...'
    UPDATE dbo.estruturaorganizacionalalmox
       SET isundconsumidorarequisitante = 1
      FROM  
         ( SELECT UR.ID FROM dbo.ESTRUTURAORGANIZACIONAL UR 
           INNER JOIN dbo.DOMINIO D ON UR.TIPOESTRUTURAORGANIZACIONAL_ID = D.ID AND D.CHAVE = 'tipoEstruturaOrganizacional' AND D.CODIGO=4
         ) AS U
     WHERE U.ID = ESTRUTURAORGANIZACIONAL_ID;
    GO     
    
    --> ATUALIZANDO AS UNIDADES ALMOXARIFADO E ALMOXARIFADO PADRÃO
    PRINT 'Inicializando o atribudo [ÉAlmoxarifadoPadrão] ...'
    UPDATE dbo.estruturaorganizacionalpatrim 
        SET isalmoxarifadopadrao = isalmoxarifado
      WHERE isalmoxarifadopadrao IS NULL;
    GO

    COMMIT
    PRINT 'Finalizado ajustes dos atributos Localizadora/Almoxarifado/UnidadeRequisitante!'
-- #FIM :: AJUSTA ATRIBUTOS 'ISLOCALIZADORADEBENS'/'ISALMOXARIFADO'/'ISUNDCONSUMIDORAREQUISITANTE
GO


-- #INI :: AJUSTES PARA SETAR OS BENS PRINCIPAIS COMO PRINCIPAIS
    BEGIN TRANSACTION 
    GO

    PRINT 'Marcando o atribudo [BemPrincipal] se houver referencia em outros bens...'
    UPDATE BEMPATRIMONIAL
       SET BEMPRINCIPAL = 1 
     WHERE CODIGO IN (SELECT SQ_BEM_PERM_SUP 
                        FROM ASI_HOMOLOGA..BP_BEM BEM 
                       WHERE BEM.SQ_BEM_PERM_SUP IS NOT NULL)
    GO
    COMMIT
    PRINT 'Finalizado ajustes de bem principal!'
-- #FIM :: AJUSTES PARA SETAR OS BENS PRINCIPAIS COMO PRINCIPAIS 
GO


-- #INI :: AJUSTES PARA SETAR OS BENS PRINCIPAIS NOS SEUS FILHOS
    BEGIN TRANSACTION
    GO

    PRINT 'Ajustando a referencia ao BemPrincipal ...'
    UPDATE dbo.bempatrimonial  
       SET bempatrimonialprincipal_id = D.PAI
      FROM
         ( SELECT BEM.SQ_BEM_PERM                           AS FILHO
                , (SELECT ID FROM BEMPATRIMONIAL BSUP 
                   WHERE BSUP.CODIGO = BEM.SQ_BEM_PERM_SUP) AS PAI
             FROM BEMPATRIMONIAL B INNER JOIN ASI_HOMOLOGA..BP_BEM BEM ON (BEM.SQ_BEM_PERM = B.CODIGO)
            WHERE SQ_BEM_PERM_SUP IS NOT NULL             
         ) AS D
       WHERE D.FILHO=CODIGO
    GO
    COMMIT
    PRINT 'Finalizado ajustes de bem principal nos seus filhos!'
-- #FIM :: AJUSTES PARA SETAR OS BENS PRINCIPAIS NOS SEUS FILHOS
GO


-- #INI :: AJUSTES PARA CRIAR O CODIGO UNICO DA TRANSFERENCIA
    USE ASI_HOMOLOGA
    GO

    BEGIN TRANSACTION
    GO

    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'codigomigracao' AND Object_ID = Object_ID(N'BP_TRANSF_INT_BEM'))
      ALTER TABLE dbo.BP_TRANSF_INT_BEM ADD codigomigracao VARCHAR(30) NOT NULL DEFAULT ' ';
    ELSE
      PRINT 'Coluna codigomigracao já existe na tabela BP_TRANSF_INT_BEM'
    GO

    PRINT 'Desabilitando constraints da tabela BP_TRANSF_INT_BEM'
    ALTER TABLE BP_TRANSF_INT_BEM NOCHECK CONSTRAINT ALL;
    COMMIT

    USE GRPDB
    GO

    BEGIN TRANSACTION
    GO

    PRINT 'Criando tabela temporária para migração das transferências...'
    IF EXISTS(SELECT * FROM sys.tables WHERE Object_ID = Object_ID(N'TMP_BP_TRANS_INT_BEM'))
        DROP TABLE TMP_BP_TRANS_INT_BEM
    GO

    -- Foi preciso buscar o valor unitário da tabela BP_BEM pois nesta versão do modelo ASI
    -- a tabela BP_TRANSF_INT_BEM não possui a coluna VL_UNITARIO.
    -- A implicação é que iremos assumir o valor atual do bem ao invés de o valor no momento
    -- em que foi realizada a transferência interna.
    SELECT T.CD_ORGAO+'.'+T.CD_UG+'.'+T.CD_TRANSFERENCIA+'.'+CONVERT(VARCHAR(6),COALESCE(I.DT_CONTABIL, T.DT_AUTORIZACAO),112) AS CODIGOMIGRACAO
         , I.CD_ORGAO
         , I.CD_UG
         , I.CD_TRANSFERENCIA
         , I.SQ_BEM_PERM
         , I.DT_CONTABIL
         , T.DT_AUTORIZACAO
         , T.CD_UL_D
         , T.CD_UL_O
         , T.DT_INCLUSAO
         , T.DT_TRANSFERENCIA
         , B.VL_UNITARIO
         , I.CD_CONTA_O
         , I.CD_CONTA_D
    INTO TMP_BP_TRANS_INT_BEM
    FROM ASI_HOMOLOGA..BP_TRANSF_INT_BEM I
    INNER JOIN ASI_HOMOLOGA..BP_TRANSF_INTERNA T ON (T.CD_ORGAO=I.CD_ORGAO AND T.CD_UG=I.CD_UG AND T.CD_TRANSFERENCIA=I.CD_TRANSFERENCIA)
    INNER JOIN ASI_HOMOLOGA..BP_BEM B ON (B.CD_ORGAO=I.CD_ORGAO AND B.CD_UG=I.CD_UG AND B.SQ_BEM_PERM=I.SQ_BEM_PERM)

    CREATE INDEX IX_TMP_BP_TRANS_INT_BEM_CODIGOMIGRACAO ON DBO.TMP_BP_TRANS_INT_BEM(CODIGOMIGRACAO ASC);
    CREATE INDEX IX_TMP_BP_TRANS_INT_BEM_SQ_BEM_PERM ON DBO.TMP_BP_TRANS_INT_BEM (SQ_BEM_PERM ASC);
    CREATE INDEX IX_TMP_BP_TRANS_INT_BEM_CD_UL_O ON DBO.TMP_BP_TRANS_INT_BEM (CD_UL_O ASC);
    COMMIT 
    PRINT 'Finalizada criação da tabela temporária de transferência!'
-- #FIM :: AJUSTES PARA CRIAR O CODIGO UNICO DA TRANSFERENCIA
GO


-- #INI :: MOVIMENTACAO DE TRANSFERENCIA INTERNA >> TRANSFERENCIA

    -- Necessário adicionar o campo com a conta contábil destino porque o material pode sofrer alteração
    -- de conta contábil com o tempo, e se for a conta errada para o movimentos o rmb fica errado
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'cd_conta_d' AND Object_ID = Object_ID(N'TransferenciaItem'))
        ALTER TABLE dbo.TransferenciaItem ADD cd_conta_d VARCHAR(30) NULL
    ELSE
        PRINT 'Coluna cd_conta_d já existe na tabela TransferenciaItem'
    GO

    BEGIN TRANSACTION
    GO
 
    IF EXISTS(SELECT * FROM sys.tables WHERE Object_ID = Object_ID(N'TMP_BP_TRANS_INT_BEM'))
    BEGIN
        PRINT 'Inserindo registros de Transferencia...'
        INSERT INTO dbo.transferencia
             ( datacriacao, dataedicao, [version], codigo, codigoasi
             , datatransferencia, referenciavigente, unidadedestino_id, organizacao_id )
        SELECT DISTINCT COALESCE(I.DT_INCLUSAO,I.DT_TRANSFERENCIA) AS datacriacao
             , GETDATE()          AS dataedicao
             , 0                  AS "version"
             , I.CD_TRANSFERENCIA AS codigo
             , I.CODIGOMIGRACAO   AS codigoasi
             , CAST(ISNULL(I.DT_CONTABIL, I.DT_AUTORIZACAO) AS date) AS datatransferencia
             , CAST(ISNULL(I.DT_CONTABIL, I.DT_AUTORIZACAO) AS date) AS referenciavigente
             , E.ID               AS unidadedestino_id
             , 1                  AS organizacao_id
        FROM DBO.TMP_BP_TRANS_INT_BEM I
        INNER JOIN DBO.ESTRUTURAORGANIZACIONAL E ON (E.CODIGO = I.CD_UL_D AND E.TIPOESTRUTURAORGANIZACIONAL_ID = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoEstruturaOrganizacional' AND CODIGO=3))
        WHERE I.DT_AUTORIZACAO IS NOT NULL

        PRINT 'Inserindo registros de TransferenciaItem...'
        INSERT INTO dbo.transferenciaitem
             ( datacriacao, dataedicao, [version], transferencia_id
             , bempatrimonial_id, unidadeorigem_id, vl_unitario, cd_conta, cd_conta_d )
        SELECT DISTINCT T.DATACRIACAO, T.DATAEDICAO, 0, T.ID
             , B.ID, E.ID, I.VL_UNITARIO/100, I.CD_CONTA_O, I.CD_CONTA_D
        FROM DBO.TMP_BP_TRANS_INT_BEM I 
        INNER JOIN DBO.TRANSFERENCIA T ON (T.CODIGOASI = I.CODIGOMIGRACAO)
        INNER JOIN DBO.BEMPATRIMONIAL B ON (B.CODIGO = I.SQ_BEM_PERM)
        INNER JOIN DBO.ESTRUTURAORGANIZACIONAL E ON (E.CODIGO = I.CD_UL_O AND E.TIPOESTRUTURAORGANIZACIONAL_ID = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoEstruturaOrganizacional' AND CODIGO=3))

        -- Foi preciso buscar o valor unitário da tabela BP_BEM pois nesta versão do modelo ASI
        -- a tabela BP_TRANSF_CONTA_BEM não possui a coluna VL_UNITARIO.
        -- A implicação é que iremos assumir o valor atual do bem ao invés de o valor no momento
        -- em que foi realizada a transferência entre contas.
        PRINT 'Inserindo registros de TransferenciaContaContabil...'
        INSERT INTO transferenciacontacontabil
             ( datacriacao, dataedicao, [version], datatransferencia, organizacao_id
             , contadestino_id, contaorigem_id, material_id, codigo, valortransferencia, quantidadematerial )
        SELECT DISTINCT TC.DT_TRANSFERENCIA, GETDATE(), 0, TC.DT_TRANSFERENCIA, 1
             , CD.ID, CO.ID, M.ID, T.CD_TRANSFERENCIA, SUM(B.VL_UNITARIO/100), 0.0  
        FROM ASI_HOMOLOGA..BP_TRANSF_CONTA_BEM T
        INNER JOIN ASI_HOMOLOGA..BP_BEM B ON (B.CD_ORGAO=T.CD_ORGAO AND B.CD_UG=T.CD_UG AND B.SQ_BEM_PERM=T.SQ_BEM_PERM)
        INNER JOIN ASI_HOMOLOGA..BP_TRANSF_CONTA TC ON (TC.CD_TRANSFERENCIA = T.CD_TRANSFERENCIA)
        INNER JOIN DBO.MATERIAL M ON (M.CODIGO = TC.CD_BEM_SERVICO)
        INNER JOIN DBO.CONTACONTABIL CO ON (CO.CODIGO = T.CD_CONTA_O)
        INNER JOIN DBO.CONTACONTABIL CD ON (CD.CODIGO = TC.CD_CONTA_D)
        GROUP BY TC.DT_TRANSFERENCIA, CD.ID, CO.ID, M.ID, T.CD_TRANSFERENCIA
    END
    ELSE
        PRINT 'Tabela temporária para migração das transferências NÃO foi criada'
    GO

    COMMIT
    PRINT 'Finalizada migração de transferência!'
-- #FIM :: MOVIMENTACAO DE TRANSFERENCIA INTERNA >> TRANSFERENCIA
GO


-- #INI :: MOVIMENTACAO DE TRANSFERENCIA EXTERNA >> SAIDATEMPORARIA
    BEGIN TRANSACTION
    GO

    PRINT 'Inserindo registros de SaidaTemporaria...'
    INSERT INTO dbo.saidatemporaria
         ( datacriacao
         , dataedicao
         , [version]
         , codigo
         , datasaida
         , previsaoretorno
         , organizacao_id
         , destino_id
         , dominiotipoobjetivo_id
         , dominiotipoprojeto_id
         , estruturaorganizacional_id
         , portador_id
         , responsavel_id
         )
    SELECT COALESCE(TRANSF.DT_INCLUSAO,GETDATE()) AS datacriacao
         , GETDATE()
         , 0
         , CAST(TRANSF.CD_TRANSFERENCIA AS VARCHAR(30))
         , TRANSF.DT_TRANSFERENCIA
         , TRANSF.DT_PREV_RETORNO
         , 1
         , DESTINO.ID
         , (SELECT ID_GRP FROM DBO.DOMINIO_ASI_GRP WHERE TP_GRP='tipoObjetivoSaidaTemporaria' AND CD_ASI=TRANSF.CD_MOTIVO_SAIDA_TEMP) AS dominiotipoobjetivo_id -- "asi_homologa..bp_motivo_saida_temp"
         , NULL
         , (SELECT ID FROM DBO.ESTRUTURAORGANIZACIONAL EO WHERE EO.CODIGO = TRANSF.CD_UG)
         , PORTADOR.ID
         , RESPONSAVEL.ID
    FROM ASI_HOMOLOGA..BP_TRANSF_EXTERNA TRANSF
    LEFT JOIN PARCEIRO DESTINO ON (DESTINO.CODIGO = TRANSF.CD_FORNEC_ORGEXT)
    LEFT JOIN PARCEIRO PORTADOR ON (PORTADOR.CODIGO = TRANSF.CD_PORTADOR)
    LEFT JOIN PARCEIRO RESPONSAVEL ON (RESPONSAVEL.CODIGO = TRANSF.CD_FUN_AUTORIZ_SAIDA)
    ORDER BY TRANSF.CD_TRANSFERENCIA;

    PRINT 'Inserindo registros de SaidaTemporariaItem...'
    INSERT INTO dbo.saidatemporariaitem
         ( datacriacao
         , dataedicao
         , version
         , dataretorno
         , garantiafinal
         , garantiainicial
         , sequencia
         , solucaoconserto
         , valorconserto 
         , bempatrimonial_id
         , dominiostatusanterior_id
         , dominiostatusretorno_id
         , recebedor_id
         , saidatemporaria_id
         , saidatemporariainativo_id
         )
    SELECT S.DATACRIACAO
         , S.DATAEDICAO
         , 0
         , TRANSFBEM.DT_RECEBIMENTO
         , TRANSFBEM.DT_GARANTIA_FIM
         , TRANSFBEM.DT_GARANTIA_INI
         , NULL
         , NULL
         , TRANSFBEM.vl_conserto
         , B.ID
         , NULL
         , NULL
         , RECEBEDOR.ID
         , S.ID
         , NULL
    FROM ASI_HOMOLOGA..BP_TRANSF_EXT_BEM TRANSFBEM
    INNER JOIN DBO.SAIDATEMPORARIA S ON (S.CODIGO = TRANSFBEM.CD_TRANSFERENCIA)
    LEFT JOIN DBO.BEMPATRIMONIAL B ON (B.CODIGO = TRANSFBEM.SQ_BEM_PERM)
    LEFT JOIN PARCEIRO RECEBEDOR ON (RECEBEDOR.CODIGO = TRANSFBEM.CD_RECEBEDOR)
    ORDER BY TRANSFBEM.CD_TRANSFERENCIA;
    
    COMMIT
    PRINT 'Finalizada migração de saída temporária!'
-- #FIM :: MOVIMENTACAO DE TRANSFERENCIA EXTERNA >> SAIDATEMPORARIA
GO


-- #INI :: DEFINICAO DE DENTENTOR DE BEM >> DEFINICAODETENTOR
    BEGIN TRANSACTION
    GO

    PRINT 'Importando registros de DefinicaoDetentor...'
    INSERT INTO dbo.definicaodetentor
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , version
         , dataatribuicao
         , datafimatribuicao
         , detentorresponsavel
         , numerotermo
         , referenciavigente
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         , colaborador_id
         )
    SELECT NULL                                AS databloqueio
         , NULL                                AS datainativo
         , COALESCE(TD.DT_INCLUSAO,GETDATE())  AS datacriacao
         , GETDATE()                           AS dataedicao
         , 0                                   AS "version"
         , TD.DT_TRANSFERENCIA                 AS dataatribuicao
         , NULL                                AS datafimatribuicao
         , CASE WHEN TD.IN_RESPONSAVEL='S' 
             THEN 0
             ELSE 1
           END                                 AS detentorresponsavel
         , CONVERT(BIGINT,TD.CD_TRANSFERENCIA) AS numerotermo
         , TD.DT_TRANSFERENCIA                 AS referenciavigente
         , NULL                                AS inativador_id
         , NULL                                AS autor_id
         , NULL                                AS editor_id
         , 1                                   AS organizacao_id
         , PAR.ID                              AS colaborador_id
    FROM ASI_HOMOLOGA..BP_TRANSF_DETENTOR TD
    INNER JOIN ASI_HOMOLOGA..CR_FUNCIONARIO FC ON TD.CD_DETENTOR_D=FC.CD_FUNCIONARIO
    INNER JOIN dbo.PESSOA P ON P.CODIGO=FC.CD_FUNCIONARIO
    INNER JOIN dbo.PARCEIRO PAR ON PAR.PESSOA_ID=P.ID
    WHERE TD.CD_DETENTOR_D IS NOT NULL;

    PRINT 'Importando registros de DefinicaoDetentorItem...'
    INSERT INTO dbo.definicaodetentoritem
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , version
         , dataatribuicao
         , datafimatribuicao
         , sequencia
         , inativador_id
         , autor_id
         , editor_id
         , bempatrimonial_id
         , definicaodetentor_id
         )
    SELECT NULL                               AS databloqueio
         , NULL                               AS datainativo
         , COALESCE(TD.DT_INCLUSAO,GETDATE()) AS datacriacao
         , GETDATE()                          AS dataedicao
         , NULL                               AS "version"
         , TD.DT_TRANSFERENCIA                AS dataatribuicao
         , NULL                               AS datafimatribuicao
         , TI.SQ_BEM_PERM                     AS sequencia
         , NULL                               AS inativador_id
         , NULL                               AS autor_id
         , NULL                               AS editor_id
         , BP.ID                              AS bempatrimonial_id
         , DD.ID                              AS definicaodetentor_id
    FROM ASI_HOMOLOGA..BP_TRANSF_DETENTOR TD
    INNER JOIN ASI_HOMOLOGA..BP_TRANSF_BEM_DET TI ON TD.CD_ORGAO=TI.CD_ORGAO AND TD.CD_TRANSFERENCIA=TI.CD_TRANSFERENCIA AND TD.CD_UG=TI.CD_UG
    INNER JOIN dbo.DEFINICAODETENTOR DD ON DD.NUMEROTERMO=TD.CD_TRANSFERENCIA
    INNER JOIN dbo.BEMPATRIMONIAL BP ON BP.CODIGO=TI.SQ_BEM_PERM
    WHERE TD.CD_DETENTOR_D IS NOT NULL
    ORDER BY TD.CD_TRANSFERENCIA;
    
    COMMIT
    PRINT 'Finalizada migração de definição de detentor!'
-- #FIM :: DEFINICAO DE DENTENTOR DE BEM >> DEFINICAODETENTOR
GO


-- #INI :: ADICAO A BEM PRINCIPAL >> ADICAOBEMPRINCIPAL
    BEGIN TRANSACTION
    GO
    
    PRINT 'Importando registros de AdicaoBemPrincipal...'
    INSERT INTO dbo.adicaobemprincipal
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , version
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         , bempatrimonial_id
         )
    SELECT NULL      AS databloqueio
         , NULL      AS datainativo
         , COALESCE(BH.DT_HISTORICO,GETDATE()) AS datacriacao
         , GETDATE() AS dataedicao
         , 0         AS "version"
         , NULL      AS inativador_id
         , NULL      AS autor_id
         , NULL      AS editor_id
         , 1         AS organizacao_id
         , PRI.ID    AS bempatrimonial_id
    FROM ASI_HOMOLOGA..BP_BEM BP 
    INNER JOIN ASI_HOMOLOGA..BP_HISTORICO BH ON BP.SQ_BEM_PERM=BH.SQ_BEM_PERM AND BP.CD_ORGAO=BH.CD_ORGAO
    INNER JOIN dbo.BEMPATRIMONIAL PRI ON PRI.CODIGO=BP.SQ_BEM_PERM_SUP
    WHERE BH.CD_TP_HISTORICO = '01'
    ORDER BY BH.DT_HISTORICO;

    PRINT 'Importando registros de AdicaoBemPrincipalItem...'
    INSERT INTO dbo.adicaobemprincipalitem
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , version
         , datareferencia
         , inativador_id
         , autor_id
         , editor_id
         , adicaobemprincipal_id
         , adicaobemprincipalinativado_id
         , bempatrimonial_id
         )
    SELECT NULL            AS databloqueio
         , NULL            AS datainativo
         , COALESCE(BH.DT_HISTORICO,GETDATE()) AS datacriacao
         , GETDATE()       AS dataedicao
         , 0               AS "version"
         , BH.DT_HISTORICO AS datareferencia
         , NULL            AS inativador_id
         , NULL            AS autor_id
         , NULL            AS editor_id
         , ADC.ID          AS adicaobemprincipal_id
         , NULL            AS adicaobemprincipalinativado_id
         , ADI.ID          AS bempatrimonial_id
    FROM ASI_HOMOLOGA..BP_BEM BP 
    INNER JOIN ASI_HOMOLOGA..BP_HISTORICO BH ON BP.SQ_BEM_PERM=BH.SQ_BEM_PERM AND BP.CD_ORGAO=BH.CD_ORGAO
    INNER JOIN dbo.BEMPATRIMONIAL ADI ON ADI.CODIGO=BP.SQ_BEM_PERM
    INNER JOIN dbo.BEMPATRIMONIAL PRI ON PRI.CODIGO=BP.SQ_BEM_PERM_SUP
    INNER JOIN dbo.ADICAOBEMPRINCIPAL ADC ON ADC.BEMPATRIMONIAL_ID=PRI.ID
    WHERE BH.CD_TP_HISTORICO = '01'
    ORDER BY BH.DT_HISTORICO;

    COMMIT
    PRINT 'Finalizada migração de adição a bem principal!'
-- #FIM :: ADICAO A BEM PRINCIPAL >> ADICAOBEMPRINCIPAL
GO


-- #INI :: MOVIMENTACAO DE BAIXAS >> BAIXA
    
    -- FOI NECESSARIO CRIAR ESSE CAMPO PARA A MIGRACAO POIS A CONTA DO MATERIAL NA EPOCA DA BAIXA
    -- PODE SER DIFERENTE DA QUE ESTA NO BEM NO MOMENTO
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'conta_contabil_o' AND Object_ID = Object_ID(N'BaixaPatrimonioItem'))
        ALTER TABLE dbo.BaixaPatrimonioItem    ADD conta_contabil_o VARCHAR(10);
    ELSE
        PRINT 'Coluna conta_contabil_o já existe na tabela BaixaPatrimonioItem'
    GO

    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'Codigo' AND Object_ID = Object_ID(N'Documento'))
      ALTER TABLE dbo.Documento ADD Codigo VARCHAR(30)
    ELSE
      PRINT 'Coluna Codigo já existe na tabela Documento'
    GO

    BEGIN TRANSACTION
    GO
    
    PRINT 'Inserindo registros de Baixa...'
    INSERT INTO dbo.baixa
         ( datacriacao 
         , dataedicao
         , version
         , organizacao_id
         , codigo  
         )
    SELECT GETDATE()
         , GETDATE()
         , 0
         , 1
         , BAIXA.SQ_BAIXA
    FROM ASI_HOMOLOGA..BP_BAIXA BAIXA
    ORDER BY BAIXA.NR_BAIXA;
    GO
    
    PRINT 'Inserindo registros de BaixaPatrimonio...'
    INSERT INTO dbo.baixapatrimonio
         ( id
         , codigo
         , datacontabilizacao
         , dataconclusao      
         , datainicio
         , destino_id
         , portador_id
         , dominiosituacaobaixa_id
         , dominiotipobaixa_id
         , dominiotipobem_id
         )
    SELECT B.ID
         , BAIXA.NR_BAIXA
         , BAIXA.DT_CONTABIL
         , BAIXA.DT_CONTABIL
         , BAIXA.DT_INICIO_BAIXA
         , PDESTINO.ID
         , PPORTADOR.ID
         , (SELECT ID_GRP FROM DBO.DOMINIO_ASI_GRP WHERE TP_GRP='tipoSituacaoBaixa' AND CD_ASI=BAIXA.CD_TP_SIT_BAIXA) AS dominiosituacaobaixa_id  -- "asi_homologa..bp_tp_sit_baixa"
         , (SELECT ID_GRP FROM DBO.DOMINIO_ASI_GRP WHERE TP_GRP='tipoBaixa' AND CD_ASI=BAIXA.CD_TP_FORMA_OP) AS dominiotipobaixa_id -- "asi_homologa..bp_tp_forma_op"
         , (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoBem' AND CODIGO=1) AS dominiotipobem_id -- 1;"Próprio contabilizado"
    FROM DBO.BAIXA B
    INNER JOIN ASI_HOMOLOGA..BP_BAIXA BAIXA ON (B.CODIGO = BAIXA.SQ_BAIXA) 
    LEFT JOIN DBO.PARCEIRO PDESTINO ON (PDESTINO.CODIGO = BAIXA.CD_DESTINO)
    LEFT JOIN DBO.PARCEIRO PPORTADOR ON (PPORTADOR.CODIGO = BAIXA.CD_PORTADOR)
    ORDER BY B.CODIGO;  

    PRINT 'Inserindo registros de BaixaPatrimonioItem...'
    INSERT INTO dbo.baixapatrimonioitem
         ( datacriacao 
         , dataedicao
         , version
         , baixa_id   
         , bempatrimonial_id
         , dataestorno
         , vl_unitario
         , conta_contabil_o 
         )
    SELECT B.DATACRIACAO
         , B.DATAEDICAO
         , 0
         , B.id
         , BEM.ID 
         , BAIXA_ITEM.DT_ESTORNO
         , BP.VL_UNITARIO/100
         , BAIXA_ITEM.CD_CONTA  
    FROM ASI_HOMOLOGA..BP_BAIXA_ITEM BAIXA_ITEM
    INNER JOIN ASI_HOMOLOGA..BP_BEM BP ON (BP.CD_ORGAO=BAIXA_ITEM.CD_ORGAO AND BP.CD_UG=BAIXA_ITEM.CD_UG AND BP.SQ_BEM_PERM=BAIXA_ITEM.SQ_BEM_PERM)
    INNER JOIN DBO.BAIXA B ON (B.CODIGO = BAIXA_ITEM.SQ_BAIXA)
    INNER JOIN DBO.BEMPATRIMONIAL BEM ON (BEM.CODIGO = BAIXA_ITEM.SQ_BEM_PERM);
    
    PRINT 'Inserindo registros de Observações da Baixa...'
    SET IDENTITY_INSERT dbo.observacao ON
    INSERT INTO dbo.observacao
         ( id
         , datacriacao
         , dataedicao
         , [version]
         , descricao
         , codigo
         )
    SELECT (ROW_NUMBER() OVER (ORDER BY O.DT_OBSERVACAO)) AS id
         , O.DT_OBSERVACAO AS datacriacao
         , GETDATE()       AS dataedicao
         , 0               AS "version"
         , O.DS_OBSERVACAO AS descricao
         , O.SQ_BAIXA      AS codigo
    FROM ASI_HOMOLOGA..BP_BAIXA_OBS O
    SET IDENTITY_INSERT dbo.observacao OFF

    PRINT 'Estabelecendo a referência de Observações e Baixas...'
    INSERT INTO dbo.baixaobservacao
         ( id
         , baixa_id
         )
    SELECT O.ID AS id
         , B.ID AS baixa_id
    FROM DBO.OBSERVACAO O
    INNER JOIN DBO.BAIXA B ON (B.CODIGO=O.CODIGO)

    PRINT 'Inserindo registros de Documentos da Baixa...'
    INSERT INTO dbo.documento
         ( datacriacao 
         , dataedicao
         , [version]
         , dataemissao
         , numero
         , dominiotipodocumento_id
         , emitente_id
         , codigo
         )
    SELECT COALESCE(D.DT_EMISSAO, GETDATE()) AS datacriacao
         , GETDATE()         AS dataedicao
         , 0                 AS "version"
         , D.DT_EMISSAO      AS dataemissao
         , D.NR_DOCUMENTO    AS numero
         , (SELECT ID_GRP FROM DBO.DOMINIO_ASI_GRP WHERE TP_GRP='tipoDocumento' AND CD_ASI=D.CD_TP_DOCUMENTO) AS dominiotipodocumento_id  -- "asi_homologa..cr_tp_documento"
         , P.ID              AS emitente_id
         , D.SQ_BAIXA        AS codigo
    FROM ASI_HOMOLOGA..BP_BAIXA_DOC D
    LEFT JOIN PARCEIRO P ON (P.CODIGO=D.CD_AGENTE)
    ORDER BY D.SQ_BAIXA

    PRINT 'Estabelecendo a referência de Documentos e Baixas...'
    INSERT INTO dbo.baixadocumento
         ( id
         , baixa_id
         )
    SELECT D.ID AS id
         , B.ID AS baixa_id
    FROM DBO.DOCUMENTO D 
    INNER JOIN dbo.BAIXA B ON (B.CODIGO=D.CODIGO)
    WHERE D.CODIGO IS NOT NULL
    ORDER BY D.CODIGO;
    GO

    UPDATE dbo.documento SET codigo = NULL
    COMMIT
    PRINT 'Finalizada migração de baixa de bens!'
-- #FIM :: MOVIMENTACAO DE BAIXAS >> BAIXA
GO


-- #INI :: MIGRACAO DAS DEPRECIACOES >> DEPRECIACAO
    BEGIN TRANSACTION
    GO

    PRINT 'Desabilitando constraints da tabela Depreciacao...'
    ALTER TABLE Depreciacao NOCHECK CONSTRAINT ALL;

    PRINT 'Desabilitando constraints da tabela DepreciacaoItem...'
    ALTER TABLE DepreciacaoItem NOCHECK CONSTRAINT ALL;

    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'anomes' AND Object_ID = Object_ID(N'DepreciacaoItem'))
      ALTER TABLE dbo.DepreciacaoItem ADD anomes char(6);
    ELSE
      PRINT 'Coluna anomes já existe na tabela DepreciacaoItem'
    GO

    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix01_anomes_bem' AND Object_ID = Object_ID(N'DepreciacaoItem'))
      CREATE INDEX ix01_anomes_bem ON dbo.DepreciacaoItem (anomes, bempatrimonial_id);
    ELSE
      PRINT 'Indice ix01_anomes_bem já existe na tabela DepreciacaoItem'
    GO
        
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix02_anomes_conta' AND Object_ID = Object_ID(N'DepreciacaoItem'))
      CREATE INDEX ix02_anomes_conta ON dbo.DepreciacaoItem (anomes, contacontabil_id)
    ELSE
      PRINT 'Indice ix02_anomes_conta já existe na tabela DepreciacaoItem'
    GO
    
    PRINT 'Inserindo registros de Depreciacao...'
    INSERT INTO dbo.depreciacao
         ( datacriacao 
         , dataedicao
         , [version]
         , datadepreciacao
         , mesreferencia
    --   , qtitensnaodepreciadovlrresidual -- FAZER UPDATE DEPOIS PARA ISSO
         , organizacao_id           
         )
    SELECT GETDATE()
         , GETDATE()
         , 0
         , D.DT_ANOMES_FECHAMENTO AS DATADEPRECIACAO
         , D.DT_ANOMES_FECHAMENTO AS MESREFERENCIA
         , 1
    FROM ASI_HOMOLOGA..BP_DEPRECIACAO D 
    GROUP BY D.DT_ANOMES_FECHAMENTO
    ORDER BY D.DT_ANOMES_FECHAMENTO;
    
    PRINT 'Inserindo registros de Depreciacao...'
    INSERT INTO dbo.depreciacaoitem
         ( datacriacao 
         , dataedicao
         , version
         , percentualnaodepreciavel
         , taxadepreciacao
         , valorbrutoatual
         , valorbrutocalculo
         , vldepreciacaoacumuladaanterior           
         , valordepreciacaoacumuladaatual
         , valordepreciacaoatual
         , valordepreciavel
         , valorliquidoanterior
         , valorliquidoatual
         , valorresidual
         , vidautilatual
         , vidautilrestanteanterior
         , vidautilrestanteatual
         , bempatrimonial_id    
         , contacontabil_id
         , depreciacao_id
         , anomes
         )
    SELECT DN.DATACRIACAO                  AS datacriacao 
         , DN.DATAEDICAO                   AS dataedicao
         , 0                               AS "version"
         , D.VL_PERCENT_NAO_DEPRECIAVEL    AS percentualnaodepreciavel
         , CASE WHEN D.VL_UNITARIO > 0 
             THEN (D.VL_DEPR_MENSAL/D.VL_UNITARIO)
             ELSE 0 
           END                             AS taxa
         , ((D.VL_UNITARIO-D.VL_DEPR_ACUM)+D.VL_DEPR_MENSAL) AS valorbrutoatual
         , ((D.VL_UNITARIO-D.VL_DEPR_ACUM)+D.VL_DEPR_MENSAL) AS valorbrutocalculo
         , CASE WHEN D.VL_DEPR_MENSAL > 0 
             THEN (D.VL_DEPR_ACUM - D.VL_DEPR_MENSAL)
             ELSE D.VL_DEPR_ACUM 
           END                             AS vldepreciacaoacumuladaanterior
         , D.VL_DEPR_ACUM                  AS valordepreciacaoacumuladaatual
         , D.VL_DEPR_MENSAL                AS valordepreciacaoatual
         , (D.VL_UNITARIO-D.VL_RESIDUAL)   AS valordepreciavel
         , (D.VL_UNITARIO-D.VL_DEPR_ACUM+D.VL_DEPR_MENSAL) as valorliquidoanterior
         , (D.VL_UNITARIO-D.VL_DEPR_ACUM)  AS valorliquidoatual
         , D.VL_RESIDUAL                   AS valorresidual
         , D.NR_VIDA_UTIL                  AS vidautilatual
         , CASE WHEN D.NR_MESES_DEPRECIADOS > 0
             THEN (D.NR_VIDA_UTIL-D.NR_MESES_DEPRECIADOS)+1
             ELSE D.NR_VIDA_UTIL
           END                             AS vidautilrestanteanterior
         , CASE WHEN D.NR_MESES_DEPRECIADOS > 0 
             THEN (D.NR_VIDA_UTIL-D.NR_MESES_DEPRECIADOS)
             ELSE D.NR_VIDA_UTIL
           END                             AS vidautilrestanteatual
         , B.ID                            AS bempatrimonial_id
         , C.ID                            AS contacontabil_id
         , DN.ID                           AS depreciacao_id
         , D.DT_ANOMES_FECHAMENTO AS anomes
    FROM ASI_HOMOLOGA..BP_DEPRECIACAO D
    INNER JOIN DBO.DEPRECIACAO DN ON (DN.MESREFERENCIA = CONVERT(VARCHAR(6),D.DT_ANOMES_FECHAMENTO, 112))
    INNER JOIN DBO.CONTACONTABIL C ON (C.CODIGO = D.CD_CONTA)
    INNER JOIN DBO.BEMPATRIMONIAL B ON (B.CODIGO = D.SQ_BEM_PERM);

    COMMIT
    PRINT 'Finalizada migração de depreciação de bens!'
-- #FIM :: MIGRACAO DAS DEPRECIACOES >> DEPRECIACAO
GO


-- #INI :: MIGRACAO REAVALIACAO/REDUCAO >> BEMPATRIMONIALALTERACAO E BEMPATRIMONIALALTERACAOITEM
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'cd_conta_d' AND Object_ID = Object_ID(N'BemPatrimonialAlteracaoItem'))
      ALTER TABLE dbo.BemPatrimonialAlteracaoItem ADD cd_conta_d VARCHAR(30);
    ELSE
      PRINT 'Coluna cd_conta_d já existe na tabela BemPatrimonialAlteracaoItem'
    GO

    BEGIN TRANSACTION
    GO
    
    DECLARE @row_id        BIGINT
          , @bem_alteracao BIGINT
          , @dom_mot_alter BIGINT
          , @dom_vlt_bruto BIGINT
          , @dom_vlt_liqui BIGINT
          , @dom_vida_util BIGINT
      
    SET @dom_mot_alter = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=3)
    SET @dom_vlt_bruto = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='campoReferencia' AND CODIGO= 2)
    SET @dom_vlt_liqui = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='campoReferencia' AND CODIGO=11)
    SET @dom_vida_util = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='campoReferencia' AND CODIGO= 1)
     
    SELECT (ROW_NUMBER() OVER (ORDER BY BEM.ID)) AS row_id
         , RE.DT_REAVALIACAO                  AS datainclusao
         , RE.DT_REAVALIACAO                  AS dataalteracao
         , BEM.ID                             AS bempatrimonial_id
    --   , RE.ID_REAVALIACAO                  AS id_reavaliacao
         , RI.VL_UNITARIO_O                   AS vl_bruto_o
         , RI.VL_UNITARIO_D                   AS vl_bruto_d
         , RI.CD_CONTA                        AS cd_conta
         , RI.VL_INCORPORADO_D                AS vl_liquido
         , RI.VL_RESIDUAL_O                   AS vl_depr_acum
         , RI.NR_VIDA_UTIL_D                  AS nr_vida_util_d
         , RI.NR_VIDA_UTIL_O                  AS nr_vida_util_o
    INTO #TMP_REAVALICAO
    FROM ASI_HOMOLOGA..BP_REAVALIACAO RE
    INNER JOIN ASI_HOMOLOGA..BP_REAVALIACAO_ITEM RI ON (RE.CD_ORGAO=RI.CD_ORGAO AND RE.CD_UG=RI.CD_UG AND RE.CD_REAVALIACAO=RI.CD_REAVALIACAO)
    INNER JOIN GRPDB..BEMPATRIMONIAL BEM ON (BEM.CODIGO = RI.SQ_BEM_PERM)

    DECLARE cur_reavaliacao CURSOR FAST_FORWARD FOR
        SELECT row_id FROM #TMP_REAVALICAO

    PRINT 'Registrando as reavaliações...'
    OPEN cur_reavaliacao
    FETCH NEXT FROM cur_reavaliacao INTO @row_id
    
    WHILE @@FETCH_STATUS = 0
    BEGIN
        INSERT INTO dbo.BemPatrimonialAlteracao
             ( databloqueio
             , datainativo
             , datacriacao
             , dataedicao
             , [version]
             , dataalteracao
             , inativador_id
             , autor_id
             , editor_id
             , organizacao_id
             , bempatrimonial_id
             , motivoalteracao_id
             )
        SELECT NULL                AS databloqueio
             , NULL                AS datainativo
             , R.DATAINCLUSAO      AS datacriacao
             , GETDATE()           AS dataedicao
             , 0                   AS "version"
             , R.DATAALTERACAO     AS dataalteracao
             , NULL                AS inativador_id
             , NULL                AS autor_id
             , NULL                AS editor_id
             , 1                   AS organizacao_id
             , R.BEMPATRIMONIAL_ID AS bempatrimonial_id
             , @dom_mot_alter      AS motivoalteracao_id
        FROM #TMP_REAVALICAO R
        WHERE R.ROW_ID = @row_id;
        SET @bem_alteracao = (SELECT IDENT_CURRENT(N'BemPatrimonialAlteracao'))

        PRINT 'Registrando alteração de valor bruto da reavaliação (' + CAST(@bem_alteracao AS NVARCHAR) + ')...'
        INSERT INTO dbo.BemPatrimonialAlteracaoItem
             ( databloqueio
             , datainativo
             , datacriacao
             , dataedicao
             , version
             , descricaocampo
             , valoranterior
             , valorarquivoanterior
             , valorarquivoatualizado
             , valoratualizado
             , inativador_id
             , autor_id
             , editor_id
             , bempatrimonial_id
             , bempatrimonialalteracao_id
             , bempatrimalteracaoinativo_id
             , campoalterado_id
             , cd_conta_d
             )
        SELECT NULL                AS databloqueio
             , NULL                AS datainativo
             , R.DATAINCLUSAO      AS datacriacao
             , GETDATE()           AS dataedicao
             , 0                   AS version
             , NULL                AS descricaocampo
             , CONVERT(NVARCHAR,R.VL_BRUTO_O) AS valoranterior
             , NULL                AS valorarquivoanterior
             , NULL                AS valorarquivoatualizado
             , CONVERT(NVARCHAR,R.VL_BRUTO_D) AS valoratualizado
             , NULL                AS inativador_id
             , NULL                AS autor_id
             , NULL                AS editor_id
             , R.BEMPATRIMONIAL_ID AS bempatrimonial_id
             , @bem_alteracao      AS bempatrimonialalteracao_id
             , NULL                AS bempatrimalteracaoinativo_id
             , @dom_vlt_bruto      AS campoalterado_id
             , R.CD_CONTA          AS cd_conta_d
        FROM #TMP_REAVALICAO R
        WHERE R.ROW_ID = @row_id;
        
        PRINT 'Registrando alteração de valor líquido da reavaliação ('+ CAST(@bem_alteracao AS NVARCHAR) + ')...'
        INSERT INTO dbo.BemPatrimonialAlteracaoItem
             ( databloqueio
             , datainativo
             , datacriacao
             , dataedicao
             , version
             , descricaocampo
             , valoranterior
             , valorarquivoanterior
             , valorarquivoatualizado
             , valoratualizado
             , inativador_id
             , autor_id
             , editor_id
             , bempatrimonial_id
             , bempatrimonialalteracao_id
             , bempatrimalteracaoinativo_id
             , campoalterado_id
             , cd_conta_d
             )
        SELECT NULL                AS databloqueio
             , NULL                AS datainativo
             , R.DATAINCLUSAO      AS datacriacao
             , GETDATE()           AS dataedicao
             , 0                   AS version
             , NULL                AS descricaocampo
             , CONVERT(NVARCHAR,R.VL_LIQUIDO) AS valoranterior
             , NULL                AS valorarquivoanterior
             , NULL                AS valorarquivoatualizado
             , CONVERT(NVARCHAR,(R.VL_BRUTO_D-R.VL_DEPR_ACUM)) AS valoratualizado
             , NULL                AS inativador_id
             , NULL                AS autor_id
             , NULL                AS editor_id
             , R.BEMPATRIMONIAL_ID AS bempatrimonial_id
             , @bem_alteracao      AS bempatrimonialalteracao_id
             , NULL                AS bempatrimalteracaoinativo_id
             , @dom_vlt_liqui      AS campoalterado_id
             , R.CD_CONTA          AS cd_conta_d
        FROM #TMP_REAVALICAO R
        WHERE R.ROW_ID = @row_id;
       
        PRINT 'Registrando alteração de vida útil da reavaliação ('+ CAST(@bem_alteracao AS NVARCHAR) +')...'
        INSERT INTO dbo.BemPatrimonialAlteracaoItem
             ( databloqueio
             , datainativo
             , datacriacao
             , dataedicao
             , version
             , descricaocampo
             , valoranterior
             , valorarquivoanterior
             , valorarquivoatualizado
             , valoratualizado
             , inativador_id
             , autor_id
             , editor_id
             , bempatrimonial_id
             , bempatrimonialalteracao_id
             , bempatrimalteracaoinativo_id
             , campoalterado_id
             , cd_conta_d
             )
        SELECT NULL                AS databloqueio
             , NULL                AS datainativo
             , R.DATAINCLUSAO      AS datacriacao
             , GETDATE()           AS dataedicao
             , 0                   AS version
             , NULL                AS descricaocampo
             , CONVERT(NVARCHAR,R.NR_VIDA_UTIL_O) AS valoranterior
             , NULL                AS valorarquivoanterior
             , NULL                AS valorarquivoatualizado
             , CONVERT(NVARCHAR,R.NR_VIDA_UTIL_D) AS valoratualizado
             , NULL                AS inativador_id
             , NULL                AS autor_id
             , NULL                AS editor_id
             , R.BEMPATRIMONIAL_ID AS bempatrimonial_id
             , @bem_alteracao      AS bempatrimonialalteracao_id
             , NULL                AS bempatrimalteracaoinativo_id
             , @dom_vida_util      AS campoalterado_id
             , R.CD_CONTA          AS cd_conta_d
        FROM #TMP_REAVALICAO R
        WHERE R.ROW_ID = @row_id;

        FETCH NEXT FROM cur_reavaliacao INTO @row_id
    END
    
    CLOSE cur_reavaliacao
    DEALLOCATE cur_reavaliacao
    COMMIT
    PRINT 'Finalizada migração de reavaliação de bens!'
-- #FIM :: MIGRACAO REAVALIACAO/REDUCAO >> BEMPATRIMONIALALTERACAO E BEMPATRIMONIALALTERACAOITEM 
GO


-- #INI :: MIGRACAO DE MOVIMENTOS DE ENTRADA >> CONTACONTABILMOVIMENTO E CONTACONTABILMOVPATRIMONIO
    PRINT 'Desabilitando constraints da tabela ContaContabilMovimento'
    ALTER TABLE dbo.ContaContabilMovimento NOCHECK CONSTRAINT ALL;

    PRINT 'Desabilitando constraints da tabela ContaContabilMovPatrimonio'
    ALTER TABLE dbo.ContaContabilMovPatrimonio NOCHECK CONSTRAINT ALL;

    BEGIN TRANSACTION
    GO
    
    DECLARE @dom_tipent_o BIGINT
          , @dom_movent_o BIGINT
          , @dom_movent_e BIGINT
          , @ccm_id       BIGINT
          , @row_id       BIGINT
    
    SET @dom_tipent_o = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoEntrada' AND CODIGO = 1)
    SET @dom_movent_o = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoMovimentoContaContabil' AND CODIGO = 1)
    SET @dom_movent_e = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoMovimentoContaContabil' AND CODIGO = 3)

    PRINT 'Criando tabela temporária com os bens da entrada...'
    SELECT (ROW_NUMBER() OVER (ORDER BY B.ID)) AS row_id
         , E.DATACONTABIL  AS datareferencia
         , I.VALORUNITARIO AS valor
         , (SELECT CAST(VALOR AS BIGINT) FROM DBO.CONFIGURACAOPARAMETROSISTEMA WHERE CHAVE='CONTA_CONTABIL_ALMOXARIFADO') AS contacontabil_id
         , CASE 
             WHEN E.DOMINIOTIPOENTRADA_ID = @dom_tipent_o THEN @dom_movent_o
             ELSE @dom_movent_e
           END    AS tipomovimentocontacontabil_id
         , E.ID   AS entrada_id
         , B.ID   AS bempatrimonial_id
    INTO #TMP_BENS_ENTRADA
    FROM DBO.BEMPATRIMONIAL B
    INNER JOIN DBO.ENTRADAPATRIMONIOITEM I ON (B.ENTRADAITEM_ID = I.ID)
    INNER JOIN DBO.ENTRADA E ON (E.ID = I.ENTRADA_ID)
    WHERE E.DATACONTABIL IS NOT NULL
    ORDER BY E.ID

    DECLARE cur_entrada CURSOR FAST_FORWARD FOR
        SELECT row_id FROM #TMP_BENS_ENTRADA

    OPEN cur_entrada
    FETCH NEXT FROM cur_entrada INTO @row_id
    
    WHILE @@FETCH_STATUS = 0
    BEGIN
        PRINT 'Registrando ContaContabilMovimento (' + CAST(@row_id AS NVARCHAR) + ')...'
        INSERT INTO dbo.ContaContabilMovimento
             ( datacriacao
             , dataedicao
             , [version]
             , datareferencia
             , valor
             , contacontabil_id
             , tipomovimentocontacontabil_id
             , entrada_id
             )
        SELECT GETDATE()           AS datacriacao
             , GETDATE()           AS dataedicao
             , 0                   AS "version"
             , R.DATAREFERENCIA    AS datareferencia
             , R.VALOR             AS valor
             , R.CONTACONTABIL_ID  AS contacontabil_id
             , R.TIPOMOVIMENTOCONTACONTABIL_ID AS tipomovimentocontacontabil_id
             , R.ENTRADA_ID        AS entrada_id
        FROM #TMP_BENS_ENTRADA R
        WHERE R.ROW_ID = @row_id;
        SET @ccm_id = (SELECT IDENT_CURRENT(N'ContaContabilMovimento'))
        
        
        PRINT 'Registrando ContaContabilMovPatrimonio (' + CAST(@row_id AS NVARCHAR) + ')...'
        INSERT INTO dbo.contacontabilmovpatrimonio
            ( id, reavaliacao, valordepreciacao, valorreavaliacao, valorreducao, bempatrimonial_id, transferencia_id )
        SELECT @ccm_id, 0, 0, 0, 0, R.BEMPATRIMONIAL_ID, NULL
        FROM #TMP_BENS_ENTRADA R
        WHERE R.ROW_ID = @row_id;
        
        FETCH NEXT FROM cur_entrada INTO @row_id
    END
    
    CLOSE cur_entrada
    DEALLOCATE cur_entrada
    COMMIT
    PRINT 'Finalizada migração dos movimentos de entrada de bens!'
-- #FIM :: MIGRACAO DE MOVIMENTOS DE ENTRADA >> CONTACONTABILMOVIMENTO E CONTACONTABILMOVPATRIMONIO
GO


-- #INI :: MIGRACAO DAS TRANSFERENCIAS INTERNAS >> CONTACONTABILMOVIMENTO E CONTACONTABILMOVPATRIMONIO
    BEGIN TRANSACTION
    GO
    
    DECLARE @dom_tpentr_o BIGINT
          , @dom_movent_o BIGINT
          , @dom_movent_e BIGINT
          , @dom_mvent_id BIGINT
          , @dom_movsai_e BIGINT
          , @cta_estoq_id BIGINT
          , @cta_estoq_cd VARCHAR(30)
          , @contadest_id BIGINT
          , @tran_item_id BIGINT
          , @row_id       BIGINT
          , @ccm_id       BIGINT

    -- RECUPERA O ID DO DOMINIO TIPO DE ENTRADA ORCAMENTARIA
    SET @dom_tpentr_o = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE = 'tipoEntrada' AND CODIGO = 1);                       

    -- RECUPERA O ID DO DOMINIO TIPO MOVIMENTO DE ENTRADA ORCAMENTARIA
    SET @dom_movent_o = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoMovimentoContaContabil' AND CODIGO = 1);

    -- RECUPERA O ID DO DOMINIO TIPO MOVIMENTO DE ENTRADA EXTRA-ORCAMENTARIA
    SET @dom_movent_e = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoMovimentoContaContabil' AND CODIGO = 3);

    -- RECUPERA O ID DO DOMINIO TIPO MOVIMENTO DE SAIDA EXTRA-ORCAMENTARIA
    SET @dom_movsai_e = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoMovimentoContaContabil' AND CODIGO = 4);                 

    -- RECUPERA O ID DA CONTA CONTACONTABIL ESTOQUE
    SET @cta_estoq_id = (SELECT CAST(VALOR AS BIGINT) FROM DBO.CONFIGURACAOPARAMETROSISTEMA WHERE CHAVE='CONTA_CONTABIL_ALMOXARIFADO')

    IF (@cta_estoq_id IS NOT NULL)
    BEGIN
        -- RECUPERA O CD DA CONTA CONTACONTABIL ESTOQUE
        SET @cta_estoq_cd = (SELECT CODIGO FROM DBO.CONTACONTABIL WHERE ID = @cta_estoq_id);
     
        PRINT 'Criando tabela temporária com os bens transferidos...'
        SELECT ROW_NUMBER() OVER(ORDER BY B.ID) AS row_id
             , T.REFERENCIAVIGENTE     AS datareferencia
             , TI.VL_UNITARIO          AS valor
             , CC.ID                   AS contacontabil_id
             , B.ID                    AS bempatrimonial_id
             , T.ID                    AS transferencia_id
             , E.DOMINIOTIPOENTRADA_ID AS tipomovimentocontacontabil_id
             , TI.ID                   AS transferenciaitem_id
        INTO #TMP_TRANSF_BENS
        FROM DBO.TRANSFERENCIA T 
        INNER JOIN DBO.TRANSFERENCIAITEM TI ON (TI.TRANSFERENCIA_ID = T.ID) 
        INNER JOIN DBO.BEMPATRIMONIAL B ON (B.ID = TI.BEMPATRIMONIAL_ID)   
        INNER JOIN DBO.ENTRADAPATRIMONIOITEM EI ON (EI.ID = B.ENTRADAITEM_ID)
        INNER JOIN DBO.ENTRADA E ON (E.ID = EI.ENTRADA_ID)
        INNER JOIN DBO.ESTRUTURAORGANIZACIONAL EO ON (EO.ID = TI.UNIDADEORIGEM_ID)
         LEFT JOIN DBO.CONTACONTABIL CC ON (TI.CD_CONTA_D = CC.CODIGO)
        WHERE TI.CD_CONTA = @cta_estoq_cd
        
        DECLARE cur_transf CURSOR FAST_FORWARD FOR
            SELECT row_id, contacontabil_id, transferenciaitem_id 
            FROM   #TMP_TRANSF_BENS
        
        OPEN cur_transf
        FETCH NEXT FROM cur_transf 
            INTO @row_id, @contadest_id, @tran_item_id
        
        WHILE @@FETCH_STATUS = 0
        BEGIN
            IF (@contadest_id IS NOT NULL)
            BEGIN 
                SET @dom_mvent_id = (SELECT tipomovimentocontacontabil_id FROM #TMP_TRANSF_BENS WHERE ROW_ID = @row_id);
                IF (@dom_mvent_id = @dom_tpentr_o)
                    SET @dom_mvent_id = @dom_movent_o
                ELSE
                    SET @dom_mvent_id = @dom_movent_e

                PRINT 'Inserindo movimento de entrada na conta destino...'
                INSERT INTO dbo.ContaContabilMovimento
                     ( datacriacao, dataedicao, [version], datareferencia
                     , valor, contacontabil_id, tipomovimentocontacontabil_id
                     , entrada_id, transferenciaconta_id, bempatrimonial_id, transferencia_id )
                SELECT GETDATE(), GETDATE(), 0, R.DATAREFERENCIA
                     , R.VALOR, R.CONTACONTABIL_ID, @dom_mvent_id
                     , NULL, NULL, R.BEMPATRIMONIAL_ID, R.TRANSFERENCIA_ID
                FROM #TMP_TRANSF_BENS R
                WHERE R.ROW_ID = @row_id;
                SET @ccm_id = (SELECT IDENT_CURRENT(N'ContaContabilMovimento'))

                PRINT 'Inserindo referência movimento de entrada da transferência...'
                INSERT INTO dbo.contacontabilmovpatrimonio
                     ( id, bempatrimonial_id, transferencia_id )               
                SELECT @ccm_id, R.BEMPATRIMONIAL_ID, R.TRANSFERENCIA_ID
                FROM #TMP_TRANSF_BENS R
                WHERE R.ROW_ID = @row_id;
                
                PRINT 'Inserindo movimento de saída da conta de origem...'
                INSERT INTO dbo.contacontabilmovimento
                     ( datacriacao, dataedicao, [version], datareferencia
                     , valor, contacontabil_id, tipomovimentocontacontabil_id
                     , entrada_id, transferenciaconta_id, bempatrimonial_id, transferencia_id )
                SELECT GETDATE(), GETDATE(), 0, R.DATAREFERENCIA
                     , -R.VALOR, R.CONTACONTABIL_ID, @dom_movsai_e
                     , NULL, NULL, R.BEMPATRIMONIAL_ID, R.TRANSFERENCIA_ID
                FROM #TMP_TRANSF_BENS R
                WHERE R.ROW_ID = @row_id;
                SET @ccm_id = (SELECT IDENT_CURRENT(N'ContaContabilMovimento'))

                PRINT 'Inserindo referência movimento de saída da transferência...'
                INSERT INTO dbo.contacontabilmovpatrimonio
                     ( id, bempatrimonial_id, transferencia_id )               
                SELECT @ccm_id, R.BEMPATRIMONIAL_ID, R.TRANSFERENCIA_ID
                FROM #TMP_TRANSF_BENS R
                WHERE R.ROW_ID = @row_id;
            END
            ELSE
                PRINT 'Transferência item sem conta destino ('+ CAST(@tran_item_id AS NVARCHAR) +')...'

            FETCH NEXT FROM cur_transf 
                INTO @row_id, @contadest_id, @tran_item_id
        END
        CLOSE cur_transf
        DEALLOCATE cur_transf
    END
    ELSE
        PRINT 'Conta contábil de estoque não está definida em ConfiguracaoParametroSistema'
    GO

    COMMIT
    PRINT 'Finalizada migração dos movimentos das transferências!'
-- #FIM :: MIGRACAO DAS TRANSFERENCIAS INTERNAS >> CONTACONTABILMOVIMENTO E CONTACONTABILMOVPATRIMONIO
GO


-- #INI :: MIGRACAO DAS BAIXAS >> CONTACONTABILMOVIMENTO E CONTACONTABILMOVPATRIMONIO
    BEGIN TRANSACTION
    GO
    
    DECLARE @dom_movent BIGINT
          , @dom_movsai BIGINT
          , @dt_estorno DATETIME
          , @row_id     BIGINT
          , @ccm_id     BIGINT
                  
    -- RECUPERA O ID DO DOMINIO ENTRADA ORCAMENTARIA
    SET @dom_movent = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE  = 'tipoMovimentoContaContabil' AND CODIGO = 1);
 
    -- RECUPERA O ID DO DOMINIO BAIXA ORCAMENTARIA
    SET @dom_movsai = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE  = 'tipoMovimentoContaContabil' AND CODIGO = 2);
 
    PRINT 'Criando tabela temporária com os bens baixados...'
    SELECT ROW_NUMBER() OVER(ORDER BY B.ID) AS row_id
         , B.DATACONTABILIZACAO             AS datareferencia
         , BI.DATAESTORNO                   AS dataestorno
         , BI.VL_UNITARIO                   AS valor
         , CC.ID                            AS contacontabil_id
         , B.ID                             AS baixa_id
         , BEM.ID                           AS bempatrimonial_id
         , COALESCE(DI.VALORDEPRECIACAOACUMULADAATUAL, 0) AS valordepreciacao
    INTO #TMP_BAIXA_BENS
    FROM DBO.BAIXAPATRIMONIO B 
    INNER JOIN DBO.BAIXAPATRIMONIOITEM BI ON (BI.BAIXA_ID = B.ID) 
    INNER JOIN DBO.BEMPATRIMONIAL BEM ON (BEM.ID = BI.BEMPATRIMONIAL_ID)
     LEFT JOIN DBO.DEPRECIACAOITEM DI ON (DI.BEMPATRIMONIAL_ID = BEM.ID AND ANOMES = CONVERT(VARCHAR(6), B.DATACONTABILIZACAO, 112))
    INNER JOIN DBO.CONTACONTABIL CC ON (CC.CODIGO = BI.CONTA_CONTABIL_O)

    DECLARE cur_baixa CURSOR FAST_FORWARD FOR
        SELECT row_id, dataestorno 
        FROM  #TMP_BAIXA_BENS
      
    OPEN cur_baixa
    FETCH NEXT FROM cur_baixa
        INTO @row_id, @dt_estorno
  
    WHILE @@FETCH_STATUS = 0
    BEGIN
        IF (@dom_movent IS NOT NULL) AND (@dom_movsai IS NOT NULL)
        BEGIN
            PRINT 'Inserindo movimento de saída da conta origem...'
            INSERT INTO dbo.ContaContabilMovimento
                 ( datacriacao, dataedicao, [version], datareferencia 
                 , valor, contacontabil_id, tipomovimentocontacontabil_id, baixa_id )
            SELECT GETDATE(), GETDATE(), 0, R.DATAREFERENCIA
                 , -R.VALOR, R.CONTACONTABIL_ID, @dom_movsai, R.BAIXA_ID
            FROM #TMP_BAIXA_BENS R
            WHERE R.ROW_ID = @row_id
            SET @ccm_id = (SELECT IDENT_CURRENT(N'ContaContabilMovimento'))

            PRINT 'Inserindo referência movimento de saída da baixa...'
            INSERT INTO dbo.ContaContabilMovPatrimonio
                 ( id, valordepreciacao, valorreavaliacao, valorreducao, bempatrimonial_id)
            SELECT @ccm_id, -R.VALORDEPRECIACAO, 0, 0, R.BEMPATRIMONIAL_ID
            FROM #TMP_BAIXA_BENS R
            WHERE R.ROW_ID = @row_id
        END
        ELSE IF (@dt_estorno IS NOT NULL)
        BEGIN
            -- O ASI CONSIDERA UMA BAIXA ESTORNADA COMO UMA ENTRADA
            -- REGISTRAMOS SAIDA DE TODAS AS BAIXAS E UMA ENTRADA
            -- SOMENTE DAS BAIXAS ESTORNADAS

            PRINT 'Inserindo movimento de entrada por estorno de baixa...'
            INSERT INTO dbo.ContaContabilMovimento
                 ( datacriacao, dataedicao, [version], datareferencia 
                 , valor, contacontabil_id, tipomovimentocontacontabil_id, baixa_id )
            SELECT GETDATE(), GETDATE(), 0, R.DATAESTORNO
                 , R.VALOR, R.CONTACONTABIL_ID, @dom_movent, R.BAIXA_ID
            FROM #TMP_BAIXA_BENS R
            WHERE R.ROW_ID = @row_id
            SET @ccm_id = (SELECT IDENT_CURRENT(N'ContaContabilMovimento'))

            PRINT 'Inserindo referência movimento de estorno da baixa...'
            INSERT INTO dbo.ContaContabilMovPatrimonio
                 ( id, valordepreciacao, valorreavaliacao, valorreducao, bempatrimonial_id)
            SELECT @ccm_id, R.VALORDEPRECIACAO, 0, 0, R.BEMPATRIMONIAL_ID
            FROM #TMP_BAIXA_BENS R
            WHERE R.ROW_ID = @ccm_id
        END
        FETCH NEXT FROM cur_baixa 
            INTO @ccm_id, @dt_estorno
    END
    CLOSE cur_baixa
    DEALLOCATE cur_baixa
    
    COMMIT
    PRINT 'Finalizada migração dos movimentos das baixas!'
-- #FIM :: MIGRACAO DAS BAIXAS >> CONTACONTABILMOVIMENTO E CONTACONTABILMOVPATRIMONIO
GO


-- #INI :: MIGRACAO DAS TRANSFERENCIAS ENTRE CONTAS >> CONTACONTABILMOVIMENTO
    BEGIN TRANSACTION
    GO
        
    DECLARE @vlr_deprec NUMERIC(10,4)
          , @data_refer DATETIME
          , @dom_movsai BIGINT
          , @dom_movent BIGINT
          , @ccm_id     BIGINT
          , @row_id     BIGINT

    -- RECUPERA O ID DO DOMINIO ENTRADA EXTRA-ORCAMENTARIA
    SET @dom_movent = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE  = 'tipoMovimentoContaContabil' AND CODIGO = 3);    

    -- RECUPERA O ID DO DOMINIO SAÍDA EXTRA-ORCAMENTARIA
    SET @dom_movsai = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE  = 'tipoMovimentoContaContabil' AND CODIGO = 4);             

    PRINT 'Criando tabela temporária com transferências entre contas...'
    SELECT ROW_NUMBER() OVER(ORDER BY TN.ID) AS row_id
         , TN.DATACRIACAO           AS datacriacao
         , TN.DATATRANSFERENCIA     AS datareferencia
         , TN.VALORTRANSFERENCIA    AS valor
         , TN.CONTAORIGEM_ID        AS contaorigem_id
         , TN.CONTADESTINO_ID       AS contadestino_id
         , CO.CODIGO                AS codigocontaorigem
         , CD.CODIGO                AS codigocontadestino
         , TN.ID                    AS transferenciaconta_id
    INTO #TMP_TRANSF_CC
    FROM DBO.TRANSFERENCIACONTACONTABIL TN
    INNER JOIN DBO.CONTACONTABIL CO ON (CO.ID = TN.CONTAORIGEM_ID)
    INNER JOIN DBO.CONTACONTABIL CD ON (CD.ID = TN.CONTADESTINO_ID)

    DECLARE cur_transf_cc CURSOR FAST_FORWARD FOR
        SELECT row_id, datareferencia FROM #TMP_TRANSF_CC

    OPEN cur_transf_cc
        FETCH NEXT FROM cur_transf_cc INTO @row_id, @data_refer
        
    WHILE @@FETCH_STATUS = 0
    BEGIN
        PRINT 'Inserindo movimento de saída da conta origem...'
        INSERT INTO dbo.ContaContabilMovimento
             ( datacriacao, dataedicao, [version], datareferencia 
             , valor, contacontabil_id, tipomovimentocontacontabil_id
             , entrada_id, baixa_id, transferenciaconta_id )
        SELECT R.DATACRIACAO, GETDATE(), 0, R.DATAREFERENCIA
             , -R.VALOR, R.CONTAORIGEM_ID, @dom_movsai
             , NULL, NULL, R.TRANSFERENCIACONTA_ID
        FROM #TMP_TRANSF_CC R
        WHERE R.ROW_ID = @row_id
        SET @ccm_id = (SELECT IDENT_CURRENT(N'ContaContabilMovimento'))

        SET @vlr_deprec = ( SELECT COALESCE(SUM(D.VL_DEPR_ACUM),0) AS valordepreciacao
                            FROM ASI_HOMOLOGA..BP_DEPRECIACAO D
                            WHERE D.DT_ANOMES_FECHAMENTO = CONVERT(VARCHAR(6), @data_refer, 112)
                            AND D.CD_CONTA = (SELECT R.CODIGOCONTAORIGEM FROM #TMP_TRANSF_CC R WHERE R.ROW_ID = @row_id)
                          )

        PRINT 'Inserindo referência movimento de saída da conta...'
        INSERT INTO dbo.ContaContabilMovPatrimonio
             ( id, valordepreciacao, valorreavaliacao, valorreducao, bempatrimonial_id )
        VALUES
             ( @ccm_id, @vlr_deprec, 0, 0, NULL )

        PRINT 'Inserindo movimento de entrada na conta destino...'
        INSERT INTO dbo.ContaContabilMovimento
             ( datacriacao, dataedicao, [version], datareferencia 
             , valor, contacontabil_id, tipomovimentocontacontabil_id
             , entrada_id, baixa_id, transferenciaconta_id )
        SELECT R.DATACRIACAO, GETDATE(), 0, R.DATAREFERENCIA
             , -R.VALOR, R.CONTADESTINO_ID, @dom_movent
             , NULL, NULL, R.TRANSFERENCIACONTA_ID
        FROM #TMP_TRANSF_CC R
        WHERE R.ROW_ID = @row_id
        SET @ccm_id = (SELECT IDENT_CURRENT(N'ContaContabilMovimento'))

        PRINT 'Inserindo referência movimento de entrada na conta...'
        INSERT INTO dbo.ContaContabilMovPatrimonio
             ( id, valordepreciacao, valorreavaliacao, valorreducao, bempatrimonial_id )
        VALUES
             ( @ccm_id, @vlr_deprec, 0, 0, NULL )

        FETCH NEXT FROM cur_transf_cc INTO @row_id, @data_refer
        END
    CLOSE cur_transf_cc
    DEALLOCATE cur_transf_cc
    COMMIT
    PRINT 'Finalizada migração dos movimentos das transferências entre contas!'
-- #FIM :: MIGRACAO DAS TRANSFERENCIAS ENTRE CONTAS >> CONTACONTABILMOVIMENTO
GO


-- #INI :: MIGRACAO DAS REAVALIACOES/REDUCOES >> CONTACONTABILMOVIMENTO E CONTACONTABILMOVPATRIMONIO
    BEGIN TRANSACTION
    GO

    DECLARE @dom_movent     BIGINT
          , @dom_movsai     BIGINT
          , @dom_refalt     BIGINT
          , @dom_vlrbru     BIGINT
          , @vl_anterior    NUMERIC(10,4)
          , @vl_atualizado  NUMERIC(10,4)
          , @vl_reducao     NUMERIC(10,4)
          , @vl_reavaliacao NUMERIC(10,4)
          , @row_id         BIGINT
          , @ccm_id         BIGINT

    -- RECUPERA O IDs DE DOMINIOS
    SET @dom_movent = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoMovimentoContaContabil' AND CODIGO = 3)
    SET @dom_movsai = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoMovimentoContaContabil' AND CODIGO = 4)
    SET @dom_refalt = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'classeReferencia' AND CODIGO = 8)
    SET @dom_vlrbru = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'campoReferencia'  AND CODIGO = 2)

    PRINT 'Criando tabela temporária com reavaliações...'
    SELECT AI.ID                               AS row_id
         , AI.BEMPATRIMONIAL_ID                AS bempatrimonial_id
         , A.DATAALTERACAO                     AS datareferencia
         , CONVERT(NUMERIC(10,4),AI.VALORANTERIOR)   AS valoranterior
         , CONVERT(NUMERIC(10,4),AI.VALORATUALIZADO) AS valoratualizado
         , CC.ID                               AS contacontabil_id
         , DI.VALORDEPRECIACAOACUMULADAATUAL   AS depreciacaoacumulada
    INTO #TMP_REAVALICAO
    FROM BEMPATRIMONIALALTERACAOITEM AI
    INNER JOIN BEMPATRIMONIALALTERACAO A ON (A.ID = AI.BEMPATRIMONIALALTERACAO_ID)          
    INNER JOIN BEMPATRIMONIAL BEM ON (BEM.ID = AI.BEMPATRIMONIAL_ID)
    INNER JOIN DEPRECIACAOITEM DI ON (DI.BEMPATRIMONIAL_ID = BEM.ID AND DI.ANOMES = CONVERT(VARCHAR(6), A.DATAALTERACAO, 112))
    INNER JOIN CONTACONTABIL CC ON (CC.CODIGO = AI.CD_CONTA_D)
    WHERE CAMPOALTERADO_ID = @dom_vlrbru
        
    DECLARE cur_reavaliacao CURSOR FAST_FORWARD FOR
        SELECT row_id, valoranterior, valoratualizado
        FROM #TMP_REAVALICAO
    
    OPEN cur_reavaliacao
    FETCH NEXT FROM cur_reavaliacao 
        INTO @row_id, @vl_anterior, @vl_atualizado
    
    WHILE @@FETCH_STATUS = 0
    BEGIN
        IF (@vl_anterior > @vl_atualizado)
        BEGIN
            SET @vl_reducao = @vl_anterior - @vl_atualizado;
            SET @vl_reavaliacao = 0;
        END
        ELSE
        BEGIN
            SET @vl_reavaliacao = @vl_atualizado - @vl_anterior;
            SET @vl_reducao = 0;
        END

        PRINT 'Inserindo movimento de saída da conta...'
        INSERT INTO dbo.ContaContabilMovimento
             ( datacriacao, dataedicao, [version], datareferencia
             , valor, contacontabil_id, tipomovimentocontacontabil_id
             , idclassereferencia, dominioclassereferencia_id )
        SELECT GETDATE(), GETDATE(), 0, R.DATAREFERENCIA
             , -@vl_anterior, R.CONTACONTABIL_ID, @dom_movsai
             , @row_id, @dom_refalt
        FROM #TMP_REAVALICAO R
        WHERE R.ROW_ID = @row_id
        SET @ccm_id = (SELECT IDENT_CURRENT(N'ContaContabilMovimento'))

        PRINT 'Inserindo referência movimento de saída da reavaliação/redução...'
        IF (@vl_reducao > 0)
            INSERT INTO dbo.ContaContabilMovPatrimonio
                 ( id, valordepreciacao, valorreavaliacao, valorreducao, bempatrimonial_id, reavaliacao )
            SELECT @ccm_id, 0, 0, -@vl_reducao, R.BEMPATRIMONIAL_ID, 1
            FROM #TMP_REAVALICAO R
            WHERE R.ROW_ID = @row_id
        ELSE
            INSERT INTO dbo.ContaContabilMovPatrimonio
                 ( id, valordepreciacao, valorreavaliacao, valorreducao, bempatrimonial_id, reavaliacao )
            SELECT @ccm_id, 0, 0, 0, R.BEMPATRIMONIAL_ID, 1
            FROM #TMP_REAVALICAO R
            WHERE R.ROW_ID = @row_id

        PRINT 'Inserindo movimento de entrada na conta...'
        INSERT INTO dbo.ContaContabilMovimento
             ( datacriacao, dataedicao, [version], datareferencia
             , valor, contacontabil_id, tipomovimentocontacontabil_id
             , idclassereferencia, dominioclassereferencia_id )
        SELECT GETDATE(), GETDATE(), 0, R.DATAREFERENCIA
             , @vl_atualizado, R.CONTACONTABIL_ID, @dom_movent
             , @row_id, @dom_refalt
        FROM #TMP_REAVALICAO R
        WHERE R.ROW_ID = @row_id
        SET @ccm_id = (SELECT IDENT_CURRENT(N'ContaContabilMovimento'))

        PRINT 'Inserindo referência movimento de saída da reavaliação/redução...'
        IF (@vl_reavaliacao > 0)
            INSERT INTO dbo.ContaContabilMovPatrimonio
                 ( id, valordepreciacao, valorreavaliacao, valorreducao, bempatrimonial_id, reavaliacao )
            SELECT @ccm_id, R.DEPRECIACAOACUMULADA, @vl_reavaliacao, 0, R.BEMPATRIMONIAL_ID, 1
            FROM #TMP_REAVALICAO R
            WHERE R.ROW_ID = @row_id
        ELSE
            INSERT INTO dbo.ContaContabilMovPatrimonio
                 ( id, valordepreciacao, valorreavaliacao, valorreducao, bempatrimonial_id, reavaliacao )
            SELECT @ccm_id, R.DEPRECIACAOACUMULADA, 0, 0, R.BEMPATRIMONIAL_ID, 1
            FROM #TMP_REAVALICAO R
            WHERE R.ROW_ID = @row_id

         FETCH NEXT FROM cur_reavaliacao 
            INTO @row_id, @vl_anterior, @vl_atualizado
    END
    CLOSE cur_reavaliacao
    DEALLOCATE cur_reavaliacao
    COMMIT
    PRINT 'Finalizada migração dos movimentos das reavaliações!'
-- #FIM :: MIGRACAO DAS REAVALIACOES/REDUCOES >> CONTACONTABILMOVIMENTO E CONTACONTABILMOVPATRIMONIO
GO


-- #INI :: RETORNANDO AS CONSTRAINTS DESABILITADAS
    USE ASI_HOMOLOGA
    GO
    
    BEGIN TRANSACTION
    GO

    PRINT 'Retornando as constraints da tabela BP_TRANSF_INT_BEM'
    ALTER TABLE BP_TRANSF_INT_BEM WITH CHECK CHECK CONSTRAINT ALL;
    COMMIT

    USE GRPDB
    GO

    BEGIN TRANSACTION
    GO

    ALTER TABLE dbo.Depreciacao WITH CHECK CHECK CONSTRAINT ALL;
    ALTER TABLE dbo.DepreciacaoItem WITH CHECK CHECK CONSTRAINT ALL;
    ALTER TABLE dbo.ContaContabilMovimento WITH CHECK CHECK CONSTRAINT ALL;
    ALTER TABLE dbo.ContaContabilMovPatrimonio WITH CHECK CHECK CONSTRAINT ALL;
    
    COMMIT
    PRINT 'Finalizada reabilitação das constraints de depreciação e contacontabilmovimento!'
-- #FIM :: RETORNANDO AS CONSTRAINTS DESABILITADAS
GO


-- #INI :: GERACAO DO HISTORICO DOS BENS PATRIMONIAIS >> HISTORICOBEMPATRIMONIAL
    IF EXISTS(SELECT * FROM sys.columns WHERE Name = N'descricao' AND Object_ID = Object_ID(N'HistoricoBemPatrimonial'))
      ALTER TABLE dbo.HistoricoBemPatrimonial ALTER COLUMN descricao VARCHAR(400);
    ELSE
      PRINT 'Coluna descricao não existe na tabela HistoricoBemPatrimonial'
    GO

    /*
     ======================================
     SCRIPT QUE GERA HISTORICOS DO BEM
     ======================================
    */
    BEGIN TRANSACTION
    GO

    DECLARE @hst_alteracao                       BIGINT
          , @hst_entrada                         BIGINT
          , @hst_baixa                           BIGINT
          , @hst_adicao_bem                      BIGINT
          , @hst_transf_interna                  BIGINT
          , @hst_atribuir_detentor               BIGINT
          , @mot_ajuste                          BIGINT
          , @mot_atual_monetaria                 BIGINT
          , @mot_reavaliacao                     BIGINT
          , @mot_inventario                      BIGINT
          , @mot_depreciacao                     BIGINT
          , @mot_incorporacao                    BIGINT
          , @mot_inicio_baixa                    BIGINT
          , @mot_baixa                           BIGINT
          , @mot_baixa_nao_autorizada            BIGINT
          , @mot_status_na_baixa                 BIGINT
          , @mot_transf_bem_principal            BIGINT
          , @mot_transf_interna                  BIGINT
          , @mot_detentor_bem_principal          BIGINT
          , @mot_responsavel_bem_principal       BIGINT
          , @mot_atribuicao_bem_principal        BIGINT
          , @mot_atribuicao_bens                 BIGINT
          , @mot_estorno_baixa                   BIGINT
          , @mot_estorno_item_baixa              BIGINT
          , @mot_exclusao_baixa                  BIGINT
          , @mot_exclusao_item_baixa             BIGINT
          , @mot_definicao_detentor              BIGINT
          , @mot_exclusao_bem_vinculado          BIGINT
          , @mot_exclusao_vinculo                BIGINT
          , @mot_exclusao_detentor_desvinculo    BIGINT
          , @mot_exclusao_responsavel_desvinculo BIGINT
          , @mot_definicao_responsavel           BIGINT
          , @mot_saida_temporaria                BIGINT
          , @mot_transf_detentor                 BIGINT
          , @mot_exclusao_detentor_atrib_bem     BIGINT
          , @mot_exclusao_resp_atrib_bem         BIGINT
          , @mot_retorno_saida                   BIGINT
          , @mot_exclusao_resp_detentor_nao_resp BIGINT
          , @mot_contabilizacao                  BIGINT
          , @mot_descontabilizacao               BIGINT
          , @controle_id2                        BIGINT
          , @controle_id1                        BIGINT
          , @bens_filhos                         VARCHAR(MAX)

    SET @hst_alteracao         = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoHistorico' AND CODIGO=1)
    SET @hst_entrada           = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoHistorico' AND CODIGO=2)
    SET @hst_baixa             = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoHistorico' AND CODIGO=3)
    SET @hst_adicao_bem        = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoHistorico' AND CODIGO=4)
    SET @hst_transf_interna    = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoHistorico' AND CODIGO=5)
    SET @hst_atribuir_detentor = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoHistorico' AND CODIGO=6)

    SET @mot_ajuste                          = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=1)
    SET @mot_atual_monetaria                 = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=2)
    SET @mot_reavaliacao                     = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=3)
    SET @mot_inventario                      = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=4)
    SET @mot_depreciacao                     = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=5)
    SET @mot_incorporacao                    = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=6)
    SET @mot_inicio_baixa                    = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=7)
    SET @mot_baixa                           = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=8)
    SET @mot_baixa_nao_autorizada            = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=9)
    SET @mot_status_na_baixa                 = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=10)
    SET @mot_transf_bem_principal            = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=11)
    SET @mot_transf_interna                  = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=12)
    SET @mot_detentor_bem_principal          = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=13)
    SET @mot_responsavel_bem_principal       = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=14)
    SET @mot_atribuicao_bem_principal        = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=15)
    SET @mot_atribuicao_bens                 = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=16)
    SET @mot_estorno_baixa                   = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=17)
    SET @mot_estorno_item_baixa              = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=18)
    SET @mot_exclusao_baixa                  = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=19)
    SET @mot_exclusao_item_baixa             = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=20)
    SET @mot_definicao_detentor              = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=21)
    SET @mot_exclusao_bem_vinculado          = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=22)
    SET @mot_exclusao_vinculo                = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=23)
    SET @mot_exclusao_detentor_desvinculo    = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=24)
    SET @mot_exclusao_responsavel_desvinculo = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=25)
    SET @mot_definicao_responsavel           = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=26)
    SET @mot_saida_temporaria                = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=27)
    SET @mot_transf_detentor                 = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=28)
    SET @mot_exclusao_detentor_atrib_bem     = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=29)
    SET @mot_exclusao_resp_atrib_bem         = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=30)
    SET @mot_retorno_saida                   = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=31)
    SET @mot_exclusao_resp_detentor_nao_resp = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=32)
    SET @mot_contabilizacao                  = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=33)
    SET @mot_descontabilizacao               = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMotivoAlteracaoBem' AND CODIGO=34)
    
    PRINT 'Registrando histórico [incorporação]...'
    INSERT INTO dbo.historicobempatrimonial
         ( datacriacao, dataedicao, [version]
         , descricao
         , valoranterior, valornovo, numeroprocesso
         , dataoperacao, organizacao_id, bempatrimonial_id
         , dominiotipohistorico_id, dominiotipomotivoalteracao_id )
    SELECT GETDATE(), GETDATE(), 0
         , 'Entrada em: '+ CONVERT(VARCHAR, E.DATARECEBIMENTO, 103) +' - Estrutura organizacional: '+ eo.codigo+' - '+ eo.nome + ' – Número Patrimonial: '+ b.numeropatrimonial + ' - ' + m.descricao as descricao
         , NULL, B.NUMEROPATRIMONIAL, NULL
         , E.DATARECEBIMENTO, 1, B.ID
         , @hst_entrada, @mot_incorporacao
    FROM ENTRADA E
    INNER JOIN ENTRADAPATRIMONIO EP ON (EP.ID = E.ID)
    INNER JOIN ENTRADAPATRIMONIOITEM I ON (I.ENTRADA_ID = E.ID)
    INNER JOIN ESTRUTURAORGANIZACIONAL EO ON (EO.ID = EP.ESTRUTURAORGANIZACIONAL_ID)
    INNER JOIN BEMPATRIMONIAL B ON (B.ENTRADAITEM_ID = I.ID)
    INNER JOIN MATERIAL M ON (M.ID = B.MATERIAL_ID)
    ORDER BY E.ID;
        
    PRINT 'Registrando histórico [contabilização]...'
    INSERT INTO dbo.historicobempatrimonial
         ( datacriacao, dataedicao, [version]
         , descricao
         , valoranterior, valornovo, numeroprocesso
         , dataoperacao, organizacao_id, bempatrimonial_id
         , dominiotipohistorico_id, dominiotipomotivoalteracao_id )
    SELECT GETDATE(), GETDATE(), 0
         , 'Contabilização realizada em: '+ CONVERT(VARCHAR, E.DATACONTABIL, 103) as descricao
         , NULL, CONVERT(VARCHAR,E.DATACONTABIL, 103), NULL
         , E.DATACONTABIL, 1, B.ID
         , @hst_entrada, @mot_contabilizacao
    FROM ENTRADA E
    INNER JOIN ENTRADAPATRIMONIOITEM I ON (I.ENTRADA_ID = E.ID)
    INNER JOIN BEMPATRIMONIAL B ON (B.ENTRADAITEM_ID = I.ID)
    WHERE E.DATACONTABIL IS NOT null
    ORDER BY E.ID;

    PRINT 'Registrando histórico [baixa]...'
    INSERT INTO dbo.historicobempatrimonial
         ( datacriacao, dataedicao, [version]
         , descricao
         , valoranterior
         , valornovo
         , numeroprocesso, dataoperacao, organizacao_id, bempatrimonial_id
         , dominiotipohistorico_id, dominiotipomotivoalteracao_id )
    SELECT GETDATE(), GETDATE(), 0
         , 'Processo de Baixa autorizado' as descricao
         , (SELECT NOME FROM DBO.DOMINIO WHERE ID = BEM.DOMINIOSTATUS_ID) as valoranterior
         , (SELECT NOME FROM DBO.DOMINIO WHERE ID = BEM.DOMINIOSTATUS_ID) as valornovo
         , NULL, BP.DATACONTABILIZACAO, 1, B.ID
         , @hst_baixa, @mot_baixa
    FROM BAIXA B
    INNER JOIN BAIXAPATRIMONIO BP ON (BP.ID = B.ID)
    INNER JOIN BAIXAPATRIMONIOITEM I ON (I.BAIXA_ID = BP.ID)
    INNER JOIN BEMPATRIMONIAL BEM ON (BEM.ID = I.BEMPATRIMONIAL_ID);

    PRINT 'Registrando histórico [transferencia]...'
   
    INSERT INTO dbo.historicobempatrimonial
         ( datacriacao, dataedicao, [version]
         , descricao
         , valoranterior
         , valornovo
         , numeroprocesso, dataoperacao, organizacao_id, bempatrimonial_id
         , dominiotipohistorico_id, dominiotipomotivoalteracao_id )
    SELECT GETDATE(), GETDATE(), 0
         , 'Transferência foi realizada em: '
         + CONVERT(VARCHAR,t.datatransferencia, 103)
         + ' - Nova Estrutura Organizacional: '
         + ED.CODIGO + ' - ' + ED.NOME 
         + (CASE WHEN TERMO.ID IS NOT NULL THEN ' - Número do Termo: ' + TERMO.NUMEROTERMO ELSE '' END) as descricao
         , EO.NOME as valoranterior
         , ED.NOME as valornovo
         , NULL, T.DATATRANSFERENCIA, 1, I.BEMPATRIMONIAL_ID
         , @hst_transf_interna, @mot_transf_interna
    FROM TRANSFERENCIA T
    INNER JOIN TRANSFERENCIAITEM I ON (I.TRANSFERENCIA_ID = T.ID)
    INNER JOIN ESTRUTURAORGANIZACIONAL ED ON (ED.ID = T.UNIDADEDESTINO_ID)
    INNER JOIN ESTRUTURAORGANIZACIONAL EO ON (EO.ID = I.UNIDADEORIGEM_ID)
    LEFT JOIN TERMORESPONSABILIDADE TERMO ON (TERMO.TRANSFERENCIA_ID = T.ID AND TERMO.UNIDADEORIGEM_ID = I.UNIDADEORIGEM_ID);

    PRINT 'Registrando histórico [detentor]...'
    INSERT INTO dbo.historicobempatrimonial
         ( datacriacao, dataedicao, [version]
         , descricao
         , valoranterior
         , valornovo
         , numeroprocesso, dataoperacao, organizacao_id, bempatrimonial_id
         , dominiotipohistorico_id, dominiotipomotivoalteracao_id )
    SELECT GETDATE(), GETDATE(), 0
         , 'Novo Detentor: '+ p.nome + (CASE WHEN d.datafimatribuicao IS NOT NULL THEN (' atribuído para o período ' 
         + CONVERT(VARCHAR,d.dataatribuicao, 103) + ' a ' + CONVERT(VARCHAR,d.datafimatribuicao, 103)) ELSE '' END) as descricao
         , NULL   AS valoranterior
         , P.NOME AS valornovo    
         , NULL, DI.DATAATRIBUICAO, 1, DI.BEMPATRIMONIAL_ID
         , @hst_atribuir_detentor, @mot_definicao_detentor
    FROM DEFINICAODETENTOR D
    INNER JOIN DEFINICAODETENTORITEM DI ON (DI.DEFINICAODETENTOR_ID = D.ID)        
    INNER JOIN PARCEIRO PAR ON (PAR.ID = D.COLABORADOR_ID)
    INNER JOIN PESSOA P ON (P.ID = PAR.PESSOA_ID);

    PRINT 'Registrando histórico [saída temporária]...'
    INSERT INTO dbo.historicobempatrimonial
         ( datacriacao, dataedicao, [version]
         , descricao
         , valoranterior
         , valornovo
         , numeroprocesso, dataoperacao, organizacao_id, bempatrimonial_id
         , dominiotipohistorico_id, dominiotipomotivoalteracao_id )
    SELECT GETDATE(), GETDATE(), 0
         , 'Bem está em saída temporária com o objetivo: '
         + OBJETIVO.DESCRICAO + '.' 
         + CASE WHEN s.previsaoretorno is null then ' Sem previsão de retorno' else ' Data Prevista para Retorno: '
         + CONVERT(VARCHAR, s.previsaoretorno, 106) end + '.'  as descricao
         , NULL AS valoranterior
         , NULL AS valornovo    
         , NULL, S.DATASAIDA, 1, BEM.ID
         , @hst_alteracao, @mot_saida_temporaria
    FROM SAIDATEMPORARIA S
    INNER JOIN SAIDATEMPORARIAITEM SI ON (SI.SAIDATEMPORARIA_ID = S.ID)        
    INNER JOIN BEMPATRIMONIAL BEM ON (BEM.ID = SI.BEMPATRIMONIAL_ID)
    INNER JOIN DOMINIO OBJETIVO ON (OBJETIVO.ID = S.DOMINIOTIPOOBJETIVO_ID); 
        
    PRINT 'Registrando histórico [retorno de saída temporária]...'
    INSERT INTO dbo.historicobempatrimonial
         ( datacriacao, dataedicao, [version]
         , descricao
         , valoranterior
         , valornovo
         , numeroprocesso, dataoperacao, organizacao_id, bempatrimonial_id
         , dominiotipohistorico_id, dominiotipomotivoalteracao_id )
    SELECT GETDATE(), GETDATE(), 0
         , 'Alteração do status do bem devido ao retorno de saída temporária.' as descricao
         , NULL as valoranterior
         , NULL as valornovo
         , NULL, SI.DATARETORNO, 1, BEM.ID
         , @hst_alteracao, @mot_retorno_saida
    FROM SAIDATEMPORARIA S
    INNER JOIN SAIDATEMPORARIAITEM SI ON (SI.SAIDATEMPORARIA_ID = S.ID)        
    INNER JOIN BEMPATRIMONIAL BEM ON (BEM.ID = SI.BEMPATRIMONIAL_ID)
    INNER JOIN DOMINIO STATUS ON (STATUS.ID = BEM.DOMINIOSTATUS_ID)
    WHERE SI.DATARETORNO IS NOT NULL;

    SELECT * INTO #tmp_adcp FROM ADICAOBEMPRINCIPAL
    SELECT * INTO #tmp_adc_item FROM ADICAOBEMPRINCIPALITEM
        
    DECLARE cur_adcp CURSOR FAST_FORWARD FOR
        SELECT A.ID, B.BEMPATRIMONIAL_ID 
        FROM #TMP_ADCP A 
        INNER JOIN #TMP_ADC_ITEM B ON A.ID = B.ADICAOBEMPRINCIPAL_ID

    OPEN cur_adcp
    FETCH NEXT FROM cur_adcp 
        INTO @controle_id1, @controle_id2
    
    WHILE @@FETCH_STATUS = 0
    BEGIN
        SET @bens_filhos = ( SELECT CONCAT(B.NUMEROPATRIMONIAL, ' - ', M.DESCRICAO) 
                             FROM DBO.BEMPATRIMONIAL B INNER JOIN MATERIAL M ON (B.MATERIAL_ID = M.ID)
                             WHERE B.ID = @controle_id2
                           )

        PRINT 'Registrando histórico [bens filhos]...'
        INSERT INTO dbo.historicobempatrimonial
             ( datacriacao, dataedicao, [version]
             , descricao
             , valoranterior
             , valornovo
             , numeroprocesso, dataoperacao, organizacao_id, bempatrimonial_id
             , dominiotipohistorico_id, dominiotipomotivoalteracao_id )
        SELECT GETDATE(), GETDATE(), 0
             , 'Atribuição de bens filhos. Números Patrimoniais: '+ @bens_filhos
             , NULL AS valoranterior
             , NULL AS valornovo
             , NULL, A.DATACRIACAO, 1, A.BEMPATRIMONIAL_ID
             , @hst_adicao_bem, @mot_atribuicao_bens
        FROM ADICAOBEMPRINCIPAL A
        WHERE A.ID = @controle_id1;

        PRINT 'Registrando histórico [bens principal]...'
        INSERT INTO dbo.historicobempatrimonial
             ( datacriacao, dataedicao, [version]
             , descricao
             , valoranterior
             , valornovo
             , numeroprocesso, dataoperacao, organizacao_id, bempatrimonial_id
             , dominiotipohistorico_id, dominiotipomotivoalteracao_id )
        SELECT GETDATE(), GETDATE(), 0
             , (SELECT CONCAT('Bem Principal: ', B.NUMEROPATRIMONIAL, ' - ', M.DESCRICAO) FROM BEMPATRIMONIAL B INNER JOIN MATERIAL M ON (B.MATERIAL_ID=M.ID) WHERE B.ID = A.BEMPATRIMONIAL_ID) as descricao
             , NULL as valoranterior
             , (SELECT NUMEROPATRIMONIAL FROM BEMPATRIMONIAL WHERE ID = A.BEMPATRIMONIAL_ID) as valornovo
             , NULL, I.DATAREFERENCIA, 1, A.BEMPATRIMONIAL_ID
             , @hst_adicao_bem, @mot_atribuicao_bem_principal
        FROM ADICAOBEMPRINCIPAL A
        INNER JOIN ADICAOBEMPRINCIPALITEM I ON (I.ADICAOBEMPRINCIPAL_ID = A.ID)
        WHERE A.ID = @controle_id1
            
        FETCH NEXT FROM cur_adcp INTO @controle_id1, @controle_id2
    END
    CLOSE cur_adcp
    DEALLOCATE cur_adcp
    COMMIT
    PRINT 'Finalizada geração de históricos de BemPatrimonial!'
-- #FIM :: GERACAO DO HISTORICO DOS BENS PATRIMONIAIS >> HISTORICOBEMPATRIMONIAL
GO


SET NOCOUNT OFF
--===========================================================================--
-- ##FIM :: IMPORTACAO TABELAS DE MOVIMENTO DE MATERIAS PERMANENTES          --
--===========================================================================--