--===========================================================================--
-- ##INI :: IMPORTACAO TABELAS DE MOVIMENTO DE MATERIAS DE CONSUMO           --
--===========================================================================--


-- #INI :: DEFINE AS UNIDADES ALMOXARIFADO DO ORGAO >> LOCALESTOQUE
    BEGIN TRANSACTION
    GO
    
    PRINT 'Definindo os Locais de estoque...'
    INSERT INTO dbo.localestoque
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         , almoxarifado_id
         )
    SELECT NULL             AS databloqueio
         , NULL             AS datainativo 
         , GETDATE()        AS datacriacao
         , GETDATE()        AS dataedicao
         , 0                AS "version"
         , NULL             AS inativador_id
         , NULL             AS autor_id
         , NULL             AS editor_id
         , 1                AS organizacao_id
         , EA.ESTRUTURAORGANIZACIONAL_ID AS almoxarifado_id
    FROM DBO.ESTRUTURAORGANIZACIONALALMOX EA
    WHERE EA.ISALMOXARIFADO = 1;
    
    COMMIT
    PRINT 'Finalizada definição de locais de estoque!'
-- #FIM :: DEFINE AS UNIDADES ALMOXARIFADO DO ORGAO >> LOCALESTOQUE
GO


-- #INI :: MOVIMENTACAO DE ENTRADAS >> ENTRADA
    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'codigo' AND Object_ID = Object_ID(N'Documento'))
      ALTER TABLE dbo.Documento   ADD codigo VARCHAR(30) NULL;
    ELSE
      PRINT 'Coluna codigo já existe na tabela Documento'
    GO

    BEGIN TRANSACTION
    GO
    
    /*
        TODAS AS ENTRADAS DO ALMOXARIFADO TEM DT_CONTABIL
    */
    --> REGISTROS DE ENTRADA
    PRINT 'Importando os registros consolidados de entrada almoxarifado (header)...'
    INSERT INTO dbo.entrada
         ( datacriacao
         , dataedicao
         , [version]
         , codigo
         , codigoasi
         , datacontabil
         , datarecebimento
         , datareferencia
         , valortotalnota
         , organizacao_id
         , dominiotipoentrada_id
         , dominiotiporecebimento_id
         , fornecedor_id
         )
    SELECT DISTINCT
           NR.DT_INCLUSAO                AS datacriacao
         , NR.DT_INCLUSAO                AS dataedicao
         , 0                             AS "version"
         , NR.CD_NOTA_REC                AS codigo
         , NR.CD_ALMOXARIFADO + NR.CD_ORGAO + NR.CD_NOTA_REC AS codigoasi
         , NR.DT_CONTABIL                AS datacontabil
         , NR.DT_RECEBIMENTO             AS datarecebimento
         , NR.DT_CONTABIL                AS datareferencia
         , NRT.VL_TOTAL_ENTRADA/100      AS valortotalnota
         , 1                             AS organizacao_id
         , (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoEntrada' AND CODIGO=1) AS dominiotipoentrada_id
         , (SELECT ID_GRP FROM DBO.DOMINIO_ASI_GRP WHERE TP_GRP='tipoRecebimentoAlmoxarifado' AND CD_ASI COLLATE SQL_Latin1_General_CP1_CI_AI=NR.CD_TIPO_MOVIMENTO) AS dominiotiporecebimento_id -- "ASI_HOMOLOGA..ax_tipo_movimento"
         , PAR.ID                        AS fornecedor_id
    FROM ASI_HOMOLOGA..AX_NOTA_REC NR
    JOIN ( SELECT AX_NOTA_REC_ITEM.CD_ALMOXARIFADO, AX_NOTA_REC_ITEM.CD_ORGAO, AX_NOTA_REC_ITEM.CD_NOTA_REC,
            SUM(AX_NOTA_REC_ITEM.QT_NOTA_REC_ITEM) AS QT_TOTAL_ENTRADA, SUM(AX_NOTA_REC_ITEM.VL_NOTA_REC_ITEM) AS VL_TOTAL_ENTRADA
           FROM ASI_HOMOLOGA..AX_NOTA_REC_ITEM
          GROUP BY AX_NOTA_REC_ITEM.CD_ALMOXARIFADO, AX_NOTA_REC_ITEM.CD_ORGAO, AX_NOTA_REC_ITEM.CD_NOTA_REC) NRT 
    ON NRT.CD_ALMOXARIFADO =NR.CD_ALMOXARIFADO AND NRT.CD_ORGAO=NR.CD_ORGAO AND NRT.CD_NOTA_REC=NR.CD_NOTA_REC
    JOIN ASI_HOMOLOGA..AX_V_AGENTE_ENTRADA AE ON NR.CD_AGENTE_ENTRADA=AE.CD_AGENTE
    LEFT JOIN DBO.PESSOA P ON (P.AGENTE_ID COLLATE SQL_Latin1_General_CP1_CI_AI=NR.CD_AGENTE_ENTRADA)
    LEFT JOIN DBO.PARCEIRO PAR ON (PAR.PESSOA_ID = P.ID)    
    WHERE AE.CD_TIPO_AGENTE <> '2'; -- TIPO DE AGENTE <> 2 POIS OS TIPOS 2 SAO URS QUE SERÃO MIGRADAS COMO DEVOLUCOES DEVIDO AO FEEDBACK DO MPOG
    
    --> REGISTROS DA REFERENCIA NECESSARIA EM ENTRADA DE ALMOXARIFADO
    PRINT 'Registrando a referência em EntradaAlmoxarifado...'
    INSERT INTO dbo.entradaalmoxarifado
         ( id
         , notarecebimento
         , almoxarifado_id
         , dominiofinalidade_id
         ) 
    SELECT DISTINCT
           E.ID                            AS id
         , CAST(NRI.CD_NOTA_REC AS BIGINT) AS notarecebimento
         , (SELECT TOP 1 ESTRUTURAORGANIZACIONAL_ID FROM ESTRUTURAORGANIZACIONALALMOX WHERE ISALMOXARIFADO = 1) AS estruturaorganizacional_id
         , CASE 
             WHEN N.CD_FINALIDADE_MOV = '01' THEN (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoFinalidadeMaterial' AND CODIGO=1) 
             WHEN N.CD_FINALIDADE_MOV = '03' THEN (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoFinalidadeMaterial' AND CODIGO=2) 
           END                             AS dominiofinalidade_id
    FROM DBO.ENTRADA E
    INNER JOIN ASI_HOMOLOGA..AX_NOTA_REC N ON (E.CODIGOASI=N.CD_ALMOXARIFADO+N.CD_ORGAO+N.CD_NOTA_REC)
    INNER JOIN ASI_HOMOLOGA..AX_NOTA_REC_ITEM NRI ON (E.CODIGOASI=NRI.CD_ALMOXARIFADO+NRI.CD_ORGAO+NRI.CD_NOTA_REC);

    /*
     A estratégia usada para incluir os itens, foi agrupá-los por valor unitário para
     bater com o total, pois um dos itens do mesmo material pode ter valor unitario
     diferente por causa do resto de uma d�zima peri�dica.
     esta dando diferenca entre o total da nota e os itens rever cd_entrada com erro = 2001000186
    */
    --> REGISTROS DOS ITENS DA REFERENCIA NECESSARIA EM ENTRADA DE ALMOXARIFADO
    PRINT 'Importando os registros de EntradaAlmoxarifadoItem...'
    INSERT INTO dbo.entradaalmoxarifadoitem
         ( datacriacao
         , dataedicao
         , [version]
         , datavalidade
         , numerolote
         , quantidade
         , quantidadetipounidadeentrada
         , quantidadesaldo
         , valortotal
         , valorunitario
         , entrada_id
         , material_id
         , unidademedidaentrada_id
         , cd_conta
         )
    SELECT DISTINCT
           E.DATACRIACAO                            AS datacriacao
         , E.DATAEDICAO                             AS dataedicao
         , 0                                        AS "version"
         , I.DT_VALIDADE                            AS datavalidade
         , I.NR_LOTE                                AS numerolote
         , I.QT_NOTA_REC_ITEM                       AS quantidade
         , I.QT_NOTA_REC_ITEM                       AS quantidadetipounidadeentrada
         , I.QT_SALDO_ITEM                          AS quantidadesaldo
         , (I.VL_NOTA_REC_ITEM)/100                 AS valortotal
         , (I.VL_NOTA_REC_ITEM/I.QT_NOTA_REC_ITEM)/100 AS valorunitario
         , E.ID                                     AS entrada_id
         , M.ID                                     AS material_id   
         , (SELECT ID FROM DBO.UNIDADEMEDIDA WHERE CODIGO=I.CD_UM_ELEMENTAR) AS unidademedidaentrada_id
         , I.CD_CONTA
    FROM DBO.ENTRADA E
    JOIN ASI_HOMOLOGA..AX_NOTA_REC_ITEM I ON E.CODIGOASI=(I.CD_ALMOXARIFADO+I.CD_ORGAO+I.CD_NOTA_REC)
    JOIN DBO.MATERIAL M ON (M.CODIGO=I.CD_BEM_SERVICO);

    -- ATRIBUIR 0.0 ONDE A QUANTIDADESALDO FOR NULA
    PRINT 'Atualiza saldo de estoque...'
    UPDATE ENTRADAALMOXARIFADOITEM 
       SET QUANTIDADESALDO = 0.0 
     WHERE QUANTIDADESALDO IS NULL;

    PRINT 'Atualiza saldo de estoque...'
    UPDATE ENTRADAALMOXARIFADOITEM 
       SET VALORSOBRA = 0.0
     WHERE VALORSOBRA IS NULL;

    PRINT 'Importando registros de Documento...'
    INSERT INTO dbo.documento
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , dataemissao
         , numero
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         , dominiotipodocumento_id
         , emitente_id
         , codigo
         )
    SELECT NULL                      AS databloqueio
         , NULL                      AS datainativo
         , GETDATE()                 AS datacriacao
         , GETDATE()                 AS dataedicao
         , 0                         AS "version"
         , DOC.DT_DOCUMENTO          AS dataemissao
         , DOC.NR_DOCUMENTO          AS numero
         , NULL                      AS inativador_id
         , NULL                      AS autor_id
         , NULL                      AS editor_id
         , 1                         AS organizacao_id
         , (SELECT ID_GRP FROM DBO.DOMINIO_ASI_GRP WHERE TP_GRP='tipoDocumento' AND CD_ASI=DOC.CD_TP_DOCUMENTO) AS dominiotipodocumento_id -- "asi_homologa..cr_tp_documento"
         , NULL                      AS emitente_id
         , DOC.CD_ALMOXARIFADO + DOC.CD_ORGAO + DOC.CD_NOTA_REC AS codigo
    FROM ASI_HOMOLOGA..AX_DOC_NOTA_REC DOC
    ORDER BY DOC.CD_NOTA_REC;

    PRINT 'Estabelecendo a referência entre Documento e Entrada...'
    INSERT INTO dbo.entradadocumento
         ( id 
         , entrada_id
         , entradainativo_id
         )
    SELECT D.ID    AS id
         , (SELECT E.ID FROM dbo.ENTRADA E WHERE E.CODIGOASI=D.CODIGO) AS entrada_id
         , NULL    AS entradainativo_id
    FROM DBO.DOCUMENTO D
    WHERE CODIGO IS NOT NULL;
    
    COMMIT
    PRINT 'Finalizada migração de entradas de materiais!'
-- #FIM :: MOVIMENTACAO DE ENTRADAS >> ENTRADA
GO


-- #INI :: IMPORTACAO BAIXAS DE ESTOQUE >> BAIXA
    BEGIN TRANSACTION
    GO
    
    /*
      BAIXA ALMOXARIFADO
      ==================
      1. Somente uma baixa está sem data da baixa, deixei ela de fora pois a data é obrigatória, por isso a codicao dt_baixa is not null;
      2. dominiotipobaixa definido para 'tipoBaixa' do patrimonio, mudar/criar um dominio so da baixa almoxarifado
    */
    PRINT 'Importando os registros consolidados de baixa almoxarifado (header)...'
    INSERT INTO dbo.baixa
         ( datacriacao 
         , dataedicao
         , [version]
         , organizacao_id
         , codigo  
         ) 
    SELECT B.DT_BAIXA     AS datacriacao
         , GETDATE()      AS dataedicao
         , 0              AS "version"
         , 1              AS organizacao_id
         , CAST(B.CD_BAIXA+B.CD_UG AS BIGINT) AS codigo    
    FROM ASI_HOMOLOGA..AX_BAIXA B
    WHERE B.DT_BAIXA IS NOT NULL;

    INSERT INTO dbo.baixaalmoxarifado
         ( id
         , databaixa
         , numerobaixa
         , almoxarifado_id
         , destino_id
         , dominiotipobaixa_id
         , dominiosituacaoestorno_id
         )
    SELECT B.ID                AS id
         , B.DATACRIACAO       AS databaixa
         , CAST(BA.CD_BAIXA AS BIGINT) AS numerobaixa
         , (SELECT TOP 1 ESTRUTURAORGANIZACIONAL_ID FROM ESTRUTURAORGANIZACIONALALMOX WHERE ISALMOXARIFADO=1) AS almoxarifado_id 
         , PAR.ID              AS destino_id
         , (SELECT ID_GRP FROM DBO.DOMINIO_ASI_GRP WHERE cd_asi='tipoBaixaAlmoxarifado' AND CD_ASI=BA.CD_TIPO_MOVIMENTO) AS dominiotipobaixa_id  -- "ASI_HOMOLOGA..ax_tipo_movimento"
         , (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoStatusBaixaEstorno' AND CODIGO=1) AS dominiosituacaoestorno_id  -- por padrão ajustar no final
    FROM BAIXA B
    INNER JOIN ASI_HOMOLOGA..AX_BAIXA BA ON ((BA.CD_BAIXA+BA.CD_UG)=B.CODIGO)
    INNER JOIN DBO.PARCEIRO PAR ON (PAR.AGENTE_ID=BA.CD_ORGAO_EXTERNO); 

    PRINT 'Importando os registros consolidados de baixa almoxarifado (detail)...'
    INSERT INTO dbo.baixaalmoxarifadoitem
         ( datacriacao
         , dataedicao
         , version
         , quantidade
         , materialconsumo_id
         , dataestorno
         , baixaalmoxarifado_id
         , sq_baixa_item
         , total
         , cd_conta
         )
    SELECT B.DATACRIACAO                 AS datacricao
         , B.DATAEDICAO                  AS dataedicao
         , 0                             AS "version"
         , L.QT_ATENDIDO                 AS quantidade  -- QT_BAIXAR NÃO EXISTE EM AX_BAIXA_ITEM
         , M.ID                          AS materialconsumo_id
         , NULL                          AS dataestorno
         , B.ID                          AS baixaalmoxarifado_id
         , I.SQ_BAIXA_ITEM               AS sq_baixa_item
         , L.VL_BAIXA_ITEM               AS total
         , I.CD_CONTA                    AS cd_conta
    FROM ASI_HOMOLOGA..AX_BAIXA_ITEM I
    INNER JOIN ASI_HOMOLOGA..AX_BAIXA BA ON (BA.CD_ORGAO=I.CD_ORGAO AND BA.CD_ALMOXARIFADO=I.CD_ALMOXARIFADO AND BA.CD_UG=I.CD_UG AND BA.CD_BAIXA=I.CD_BAIXA)
    INNER JOIN ASI_HOMOLOGA..AX_BAIXA_UG_LOTE L ON (L.SQ_BAIXA_ITEM=I.SQ_BAIXA_ITEM AND L.CD_ORGAO=I.CD_ORGAO AND L.CD_UG=I.CD_UG AND L.CD_ALMOXARIFADO=I.CD_ALMOXARIFADO AND L.CD_BEM_SERVICO=I.CD_BEM_SERVICO AND L.CD_BAIXA=I.CD_BAIXA)
    INNER JOIN DBO.BAIXA B ON ((BA.CD_BAIXA+BA.CD_UG)=B.CODIGO)
    INNER JOIN DBO.BAIXAALMOXARIFADO A ON (A.ID=B.ID)
     LEFT JOIN DBO.MATERIAL M ON (M.CODIGO = I.CD_BEM_SERVICO);

    -- Refazer as migracoes de documentos (Apagar e criar denovo, apagando o codigo toda vez que inserir nas tabelas
    --> DOCUMENTOS DA BAIXAALMOXARIFADO
    PRINT 'Importando os documentos de baixa almoxarifado...'
    INSERT INTO dbo.documento
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , dataemissao
         , numero
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         , dominiotipodocumento_id
         , emitente_id
         , codigo
         )
    SELECT NULL              AS databloqueio
         , NULL              AS datainativo
         , GETDATE()         AS datacriacao
         , GETDATE()         AS dataedicao
         , 0                 AS "version"
         , DOC.DT_DOCUMENTO  AS dataemissao
         , DOC.NR_DOCUMENTO  AS numero
         , NULL              AS inativador_id
         , NULL              AS autor_id
         , NULL              AS editor_id
         , 1                 AS organizacao_id
         , (SELECT ID_GRP FROM DBO.DOMINIO_ASI_GRP WHERE CD_ASI='tipoDocumento' AND CD_ASI=DOC.CD_TP_DOCUMENTO) AS dominiotipodocumento_id  -- "ASI_HOMOLOGA..cr_tp_documento"
         , NULL              AS emitente_id
         , CONCAT(DOC.CD_BAIXA,DOC.CD_UG) AS codigo
    FROM ASI_HOMOLOGA..AX_DOC_BAIXA DOC
    WHERE DOC.DT_DOCUMENTO IS NOT NULL
    ORDER BY DOC.CD_BAIXA;
    
    --> REFERENCIA DOS DOCUMENTOS DAS BAIXASALMOXARIFADO
    PRINT 'Registrando a referência dos documentos às baixa almoxarifado...'
    INSERT INTO dbo.baixadocumento
        ( id
        , baixa_id
        )
    SELECT DOC.ID AS id
         , B.ID   AS baixa_id
    FROM DBO.DOCUMENTO DOC 
    INNER JOIN DBO.BAIXA B ON (CAST(B.CODIGO AS NVARCHAR) = DOC.CODIGO)
    WHERE DOC.CODIGO IS NOT NULL
    ORDER BY DOC.CODIGO;
    
    COMMIT
    PRINT 'Finalizada a migração de baixa almoxarifado...'
-- #FIM :: IMPORTACAO BAIXAS DE ESTOQUE >> BAIXA
GO


-- #INI :: IMPORTACAO REQUISICAO >> REQUISICAOCONSUMO
    BEGIN TRANSACTION
    GO
    
    PRINT 'Importando os registros consolidados de requisicao (header)...'
    INSERT INTO dbo.requisicaoconsumo
         ( datacriacao
         , dataedicao
         , datarequisicao
         , version 
         , numerorequisicao
         , organizacao_id
         , almoxarifado_id
         , centrocusto_id
         , tipostatusrequisicao_id
         , unidaderequisitante_id
         , datafinalizacaoatendimento
         )
    SELECT R.DT_INCLUSAO   AS datacriacao
         , GETDATE()       AS dataedicao
         , R.DT_REQUISICAO AS datarequisicao
         , 0               AS "version"
         , R.CD_REQUISICAO AS numerorequisicao
         , 1               AS organizacao_id
         , (SELECT TOP 1 ESTRUTURAORGANIZACIONAL_ID FROM DBO.ESTRUTURAORGANIZACIONALALMOX WHERE ISALMOXARIFADO = 1) AS almoxarifado_id
         , EA.CENTROCUSTO_ID AS centrocusto_id  -- REDMINE #3587
         , (SELECT ID_GRP FROM DBO.DOMINIO_ASI_GRP WHERE TP_GRP='tipoStatusRequisicao' AND CD_ASI=R.CD_SITUACAO_REQ_ATUAL) AS tipostatusrequisicao_id  -- "ASI_HOMOLOGA..ax_situacao_req"
         , E.ID            AS unidaderequisitante_id
         , CASE WHEN R.CD_SITUACAO_REQ_ATUAL='007'
             THEN R.DT_SITUACAO_REQ_ATUAL 
             ELSE NULL 
           END             AS datafinalizacaoatendimento
    FROM ASI_HOMOLOGA..AX_REQUISICAO R
    INNER JOIN ASI_HOMOLOGA..AX_UR UR ON (UR.CD_ORGAO = R.CD_ORGAO AND UR.CD_AGENTE = R.CD_AGENTE)
    INNER JOIN DBO.ESTRUTURAORGANIZACIONAL E ON (E.CODIGO = UR.CD_UR+'1')    
    INNER JOIN DBO.ESTRUTURAORGANIZACIONALALMOX EA ON (EA.ESTRUTURAORGANIZACIONAL_ID = E.ID AND EA.ISUNDCONSUMIDORAREQUISITANTE = 1)

    PRINT 'Importando os registros consolidados de requisicao (detail)...'
    INSERT INTO dbo.requisicaoconsumoitem
         ( datacriacao
         , dataedicao
         , version
         , quantidade
         , quantidadedevolvida
         , organizacao_id
         , materialconsumo_id
         , requisicaoconsumo_id
         , quantidadeatendida
         , sq_requisicao_item
         , cd_conta
         )
    SELECT R.DATACRIACAO        AS datacriacao
         , R.DATAEDICAO         AS dataedicao
         , 0                    AS "version"
         , I.QT_REQUISICAO      AS quantidade
         , 0                    AS quantidadedevolvida
         , R.ORGANIZACAO_ID     AS organizacao_id
         , M.ID                 AS materialconsumo_id
         , R.ID                 AS requisicaoconsumo_id
         , RUL.QT_ATENDIDO      AS quantidadeatendida
         , I.SQ_REQUISICAO_ITEM AS sq_requisicao_item
         , I.CD_CONTA           AS cd_conta
    FROM ASI_HOMOLOGA..AX_REQUISICAO_ITEM I 
    INNER JOIN REQUISICAOCONSUMO R ON (R.NUMEROREQUISICAO = I.CD_REQUISICAO)
    INNER JOIN MATERIAL M ON (M.CODIGO = I.CD_BEM_SERVICO)
    LEFT JOIN ASI_HOMOLOGA..AX_REQ_UG_LOTE RUL ON (I.CD_ORGAO = RUL.CD_ORGAO AND I.CD_ALMOXARIFADO = RUL.CD_ALMOXARIFADO AND I.CD_REQUISICAO = RUL.CD_REQUISICAO AND I.SQ_REQUISICAO_ITEM = RUL.SQ_REQUISICAO_ITEM)

    -- O ASI nao tem o conceito de "Atendido Parcialmente" entao verificar necessidade de criar este no GRP
    -- O comando abaixo atribui o novo dominio para as requisicoes com status atendida em que a quantidadeatendida eh menor que a quantidade
    UPDATE dbo.requisicaoconsumo
       SET tipostatusrequisicao_id = (SELECT ID FROM DOMINIO WHERE CHAVE='tipoStatusRequisicao' AND NOME='ATENDIDA_PARCIALMENTE')
     WHERE ID IN (SELECT DISTINCT R.ID FROM REQUISICAOCONSUMO R 
                  INNER JOIN REQUISICAOCONSUMOITEM RCI ON (RCI.REQUISICAOCONSUMO_ID = R.ID) 
                  WHERE R.TIPOSTATUSREQUISICAO_ID = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE ='tipoStatusRequisicao' AND CODIGO=3)
                    AND (RCI.QUANTIDADE <> RCI.QUANTIDADEATENDIDA));
    COMMIT
    PRINT 'Finalizada migração de requisicaoconsumo!'
-- #FIM :: IMPORTACAO REQUISICAO >> REQUISICAOCONSUMO
GO


-- #INI :: IMPORTACAO ATENDIMENTO REQUISICAO >> ATENDIMENTOREQUISICAOCONSUMO
    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'idx01_atendimentoreq_numero' AND Object_ID = Object_ID(N'AtendimentoRequisicaoConsumo'))
      CREATE NONCLUSTERED INDEX idx01_atendimentoreq_numero ON dbo.AtendimentoRequisicaoConsumo (numeroatendimento ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice idx01_atendimentoreq_numero já existe na tabela AtendimentoRequisicaoConsumo'
    GO

    IF NOT EXISTS(SELECT * FROM sys.indexes WHERE Name = N'idx01_requisicao_link' AND Object_ID = Object_ID(N'RequisicaoConsumoItem'))
      CREATE NONCLUSTERED INDEX idx01_requisicao_link ON dbo.RequisicaoConsumoItem (requisicaoconsumo_id, materialconsumo_id, sq_requisicao_item ASC) WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, SORT_IN_TEMPDB = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON);
    ELSE
      PRINT 'Indice idx01_requisicao_link já existe na tabela RequisicaoConsumoItem'
    GO

    BEGIN TRANSACTION
    GO

    PRINT 'Importando os registros de atendimento requisicao (header)...' 
    INSERT INTO dbo.atendimentorequisicaoconsumo
        ( databloqueio
        , datainativo
        , datacriacao
        , dataedicao
        , [version]
        , dataatendimento
        , numeroatendimento
        , inativador_id 
        , autor_id
        , editor_id
        , organizacao_id
        , requisicaoconsumo_id
        )
    SELECT NULL                    AS databloqueio
         , NULL                    AS datainativo
         , R.DT_CONFIRMADA         AS datacriacao
         , GETDATE()               AS dataedicao
         , 0                       AS "version"
         , R.DT_SITUACAO_REQ_ATUAL AS dataatendimento
         , AR.ID_REQ_UG_LOTE       AS numeroatendimento
         , NULL                    AS inativador_id
         , NULL                    AS autor_id
         , NULL                    AS editor_id
         , 1                       AS organizacao_id
         , RC.ID                   AS requisicao_id
    FROM ASI_HOMOLOGA..AX_REQ_UG_LOTE AR
    INNER JOIN ASI_HOMOLOGA..AX_REQUISICAO R ON (R.CD_REQUISICAO = AR.CD_REQUISICAO)
    INNER JOIN ASI_HOMOLOGA..AX_REQUISICAO_ITEM RI ON (RI.CD_ALMOXARIFADO = AR.CD_ALMOXARIFADO AND RI.CD_REQUISICAO = AR.CD_REQUISICAO AND RI.CD_ORGAO = AR.CD_ORGAO AND RI.SQ_REQUISICAO_ITEM = AR.SQ_REQUISICAO_ITEM)
    INNER JOIN REQUISICAOCONSUMO RC ON (RC.NUMEROREQUISICAO = R.CD_REQUISICAO)
    WHERE R.CD_SITUACAO_REQ_ATUAL IN ('002', '003', '007');

    PRINT 'Importando os registros de atendimento requisicao item (detail)...' 
    INSERT INTO dbo.atendimentoreqconsumoitem 
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , quantidade
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         , atendimentoreqconsumo_id
         , material_id
         , requisicaoconsumoitem_id
         , valor
         )
        SELECT NULL            AS databloqueio
             , NULL            AS datainativo
             , ARC.DATACRIACAO AS datacriacao
             , GETDATE()       AS dataedicao
             , 0               AS "version"
             , AR.QT_ATENDIDO  AS quantidade
             , NULL            AS inativador_id
             , NULL            AS autor_id
             , NULL            AS editor_id
             , 1               AS organizacao_id
             , ARC.ID          AS atendimentoreqconsumo_id
             , M.ID            AS material_id
             , RCI.ID          AS requisicaoconsumoitem_id
             , AR.VL_ATENDIDO/100 AS valor
       FROM ASI_HOMOLOGA..AX_REQ_UG_LOTE AR
       INNER JOIN DBO.ATENDIMENTOREQUISICAOCONSUMO ARC ON (CAST(ARC.NUMEROATENDIMENTO AS NUMERIC) = AR.ID_REQ_UG_LOTE)
       INNER JOIN DBO.MATERIAL M ON (M.CODIGO = AR.CD_BEM_SERVICO)
       INNER JOIN DBO.REQUISICAOCONSUMOITEM RCI ON (RCI.REQUISICAOCONSUMO_ID = ARC.REQUISICAOCONSUMO_ID AND RCI.MATERIALCONSUMO_ID = M.ID AND RCI.SQ_REQUISICAO_ITEM = AR.SQ_REQUISICAO_ITEM);

    COMMIT
    PRINT 'Finalizada migração de atendimento de requisições!'
-- #FIM :: IMPORTACAO ATENDIMENTO REQUISICAO >> ATENDIMENTOREQUISICAOCONSUMO
GO


-- #INI :: IMPORTACAO DAS DEVOLUÇÕES DE MATERIAIS >> DEVOLUCAO
    IF EXISTS(SELECT * FROM sys.columns WHERE Name = N'requisicao_id' AND Object_ID = Object_ID(N'Devolucao'))
      ALTER TABLE dbo.Devolucao ALTER COLUMN requisicao_id BIGINT NULL;
    ELSE
      PRINT 'Coluna requisicao_id não existe na tabela Devolucao'
    GO

    IF NOT EXISTS(SELECT * FROM sys.columns WHERE Name = N'devolucao_id' AND Object_ID = Object_ID(N'RequisicaoConsumo'))
      ALTER TABLE dbo.RequisicaoConsumo ADD devolucao_id BIGINT;
    ELSE
      PRINT 'Coluna devolucao_id já existe na tabela RequisicaoConsumo'
    GO

    BEGIN TRANSACTION
    GO

    PRINT 'Importando os registros de devolucao (header)...'
    INSERT INTO dbo.devolucao
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , codigo
         , datadevolucao
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         , requisicao_id
         )
    SELECT NULL           AS databloqueio
         , NULL           AS datainativo 
         , NR.DT_INCLUSAO AS datacriacao
         , NR.DT_INCLUSAO AS dataedicao    
         , 0              AS "version"
         , NR.CD_NOTA_REC AS codigo
         , NR.DT_CONTABIL AS datacontabil
         , NULL           AS inativador_id
         , NULL           AS autor_id
         , NULL           AS editor_id
         , 1              AS organizacao_id
         , NULL           AS requisicao_id
    FROM ASI_HOMOLOGA..AX_NOTA_REC NR
    JOIN ASI_HOMOLOGA..AX_V_AGENTE_ENTRADA AE ON (NR.CD_AGENTE_ENTRADA=AE.CD_AGENTE)
    WHERE AE.CD_TIPO_AGENTE = '2' 
    AND NR.CD_TIPO_MOVIMENTO IN ('005', '033');

    PRINT 'Importando os registros de devolucao (detail)...'
    INSERT INTO dbo.devolucaoitem
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , version
         , quantidade    
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         , devolucao_id
         , materialconsumo_id    
         )
    SELECT NULL               AS databloqueio
         , NULL               AS datainativo
         , D.DATACRIACAO      AS datacriacao
         , D.DATAEDICAO       AS dataedicao
         , 0                  AS "version"
         , I.QT_NOTA_REC_ITEM AS quantidade
         , NULL               AS inativador_id
         , NULL               AS autor_id
         , NULL               AS editor_id
         , 1                  AS organizacao_id
         , D.ID               AS devolucao_id
         , M.ID               AS materialconsumo_id
    FROM DBO.DEVOLUCAO D
    JOIN ASI_HOMOLOGA..AX_NOTA_REC_ITEM I ON (D.CODIGO=I.CD_NOTA_REC)
    JOIN DBO.MATERIAL M ON (M.CODIGO=I.CD_BEM_SERVICO);     
        
    PRINT 'Vinculando requisicoes de consumo à devoluções...'
    DECLARE @row_id           BIGINT
          , @dt_requisicao    DATETIME
          , @nr_requisicao    VARCHAR(10)

    -- REQUISICOES QUE DEVERAO SER VINCULADAS AS DEVOLUCOES
    SELECT ROW_NUMBER() OVER(ORDER BY NR.DT_INCLUSAO) AS row_id
         , NR.DT_INCLUSAO                             AS datarequisicao                          
         , (SELECT TOP 1 ID FROM ORGANIZACAO)         AS organizacao
         , (SELECT TOP 1 ESTRUTURAORGANIZACIONAL_ID FROM dbo.ESTRUTURAORGANIZACIONALALMOX WHERE ISALMOXARIFADO = 1) AS almoxarifado_id 
         , (SELECT ID FROM DBO.CENTROCUSTO WHERE CODIGO = CC.CD_CENTRO_CUSTO) AS centrocusto
         , (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoStatusRequisicao' AND CODIGO=5) AS tipostatusrequisicao -- DEVOLVIDA
         , E.ID           AS unidaderequisitante
         , NR.DT_INCLUSAO AS datafinalizacao
         , D.ID           AS devolucao
    INTO #tmp_dev
    FROM ASI_HOMOLOGA..AX_NOTA_REC NR
    JOIN ASI_HOMOLOGA..AX_V_AGENTE_ENTRADA AE ON NR.CD_AGENTE_ENTRADA=AE.CD_AGENTE
    JOIN ASI_HOMOLOGA..AX_UR UR ON (UR.CD_UR = AE.CD_AGENTE_EXT)
    JOIN ASI_HOMOLOGA..CR_CENTRO_CUSTO CC ON (CC.CD_CENTRO_CUSTO = UR.CD_CENTRO_CUSTO)     
    INNER JOIN ESTRUTURAORGANIZACIONAL E ON (E.CODIGO = UR.CD_UR+'1')    
    INNER JOIN ESTRUTURAORGANIZACIONALALMOX EA ON (EA.ESTRUTURAORGANIZACIONAL_ID = E.ID AND EA.ISUNDCONSUMIDORAREQUISITANTE=1)
    INNER JOIN DEVOLUCAO D ON (D.CODIGO = NR.CD_NOTA_REC)
    WHERE AE.CD_TIPO_AGENTE = '2' 
    AND NR.CD_TIPO_MOVIMENTO IN ('005', '033')
       
    DECLARE cur_tmp_dev CURSOR FAST_FORWARD FOR
        SELECT row_id FROM #tmp_dev
    
    OPEN cur_tmp_dev
    FETCH NEXT FROM cur_tmp_dev 
        INTO @row_id
    
    WHILE @@FETCH_STATUS = 0
    BEGIN
        SET @nr_requisicao = (SELECT CAST(DATEPART(YEAR,DATAREQUISICAO) AS NVARCHAR) FROM #tmp_dev WHERE ROW_ID = @row_id) 
                           + REPLICATE('0',6) 
                           + (SELECT CAST(SUBSTRING(MAX(NUMEROREQUISICAO),5,6)+1 AS NVARCHAR) 
                              FROM DBO.REQUISICAOCONSUMO 
                              WHERE DATEPART(YEAR,DATAREQUISICAO) = DATEPART(YEAR, @dt_requisicao))

        INSERT INTO dbo.requisicaoconsumo
             ( datacriacao, dataedicao, datarequisicao, version, numerorequisicao
             , organizacao_id, almoxarifado_id, centrocusto_id, tipostatusrequisicao_id
             , unidaderequisitante_id, datafinalizacaoatendimento, devolucao_id )
        SELECT R.DATAREQUISICAO, R.DATAREQUISICAO, R.DATAREQUISICAO, 0, @nr_requisicao
             , R.ORGANIZACAO, R.ALMOXARIFADO_ID, R.CENTROCUSTO, R.TIPOSTATUSREQUISICAO
             , R.UNIDADEREQUISITANTE, R.DATAREQUISICAO, R.DEVOLUCAO
        FROM #TMP_DEV R
        WHERE R.ROW_ID = @row_id
        
        FETCH NEXT FROM cur_tmp_dev 
            INTO @row_id
    END
    CLOSE cur_tmp_dev
    DEALLOCATE cur_tmp_dev

    INSERT INTO dbo.requisicaoconsumoitem
         ( datacriacao
         , dataedicao
         , [version]
         , quantidade
         , quantidadedevolvida
         , organizacao_id
         , materialconsumo_id
         , requisicaoconsumo_id
         , quantidadeatendida
         )
    SELECT D.DATACRIACAO      AS datacriacao
         , D.DATAEDICAO       AS dataedicao
         , 0                  AS "version"
         , I.QT_NOTA_REC_ITEM AS quantidade
         , I.QT_NOTA_REC_ITEM AS quantidadedevolvida  
         , 1                  AS organizacao_id
         , M.ID               AS materialconsumo_id     
         , (SELECT ID FROM DBO.REQUISICAOCONSUMO WHERE DEVOLUCAO_ID = D.ID) as requisicao_id
         , I.QT_NOTA_REC_ITEM AS quantidadeatendida 
    FROM DBO.DEVOLUCAO D
    JOIN ASI_HOMOLOGA..AX_NOTA_REC_ITEM I ON D.CODIGO=I.CD_NOTA_REC
    JOIN DBO.MATERIAL M ON (M.CODIGO=I.CD_BEM_SERVICO);  
        
    PRINT 'Ajustando referência de requisicoes nas devoluções...'
    UPDATE DBO.DEVOLUCAO
       SET REQUISICAO_ID = (SELECT ID FROM REQUISICAOCONSUMO R WHERE R.DEVOLUCAO_ID = DBO.DEVOLUCAO.ID);
       
    IF EXISTS(SELECT * FROM sys.columns WHERE Name = N'requisicao_id' AND Object_ID = Object_ID(N'Devolucao'))
      ALTER TABLE dbo.Devolucao ALTER COLUMN requisicao_id BIGINT NOT NULL;
    ELSE
      PRINT 'Coluna requisicao_id não existe na tabela Devolucao'
    GO

    IF EXISTS(SELECT * FROM sys.columns WHERE Name = N'devolucao_id' AND Object_ID = Object_ID(N'RequisicaoConsumo'))
      ALTER TABLE dbo.RequisicaoConsumo DROP COLUMN devolucao_id;
    ELSE
      PRINT 'Coluna devolucao_id não existe na tabela RequisicaoConsumo'
    GO

    COMMIT
    PRINT 'Finalizada migração de devoluções de materiais!'
-- #FIM :: IMPORTACAO DAS DEVOLUÇÕES DE MATERIAIS >> DEVOLUCAO
GO


-- ############                                                                                 ##########
-- ############  REVER COM CONCEITO DE CLASSIFICACAO POR NATUREZA (ESTOCAVEL E NAO ESTOCAVEL)   ##########
-- ############                                                                                 ##########
-- #INI :: IMPORTACAO SALDOS DE ESTOQUE >> MOVIMENTOESTOQUESALDOMEDIO
    BEGIN TRANSACTION
    GO

    PRINT 'Importando os registros de materialestoquesaldomedio ...'
    INSERT INTO dbo.materialestoquesaldomedio
         ( datacriacao
         , dataedicao
         , [version]
         , quantidadeatual
         , organizacao_id
         , material_id
         )
    SELECT GETDATE()                     AS datacriacao
         , GETDATE()                     AS dataedicao
         , 0                             AS "version"
         , SUM(EST.QT_ESTOQUE)           AS quantidadeatual
         , 1                             AS organizacao_id
         , M.ID                          AS material_id
    FROM ASI_HOMOLOGA..AX_ESTOQUE_END EST
    INNER JOIN DBO.MATERIAL M ON (M.CODIGO = EST.CD_BEM_SERVICO)
    GROUP BY M.ID, EST.CD_BEM_SERVICO;

    PRINT 'Atualizando o valormedio do material...'
    UPDATE MATERIALESTOQUESALDOMEDIO
       SET VALORUNITARIOMEDIO = ESTOQUE.PRECO
         , RESTOMEDIO = ESTOQUE.RESTO
    FROM (SELECT MATERIAL_ID
               , (SELECT TOP 1 PR_MEDIO/100   FROM ASI_HOMOLOGA..AX_ESTOQUE_UG WHERE CD_BEM_SERVICO = M.CODIGO ORDER BY CD_NATUREZA_CLASS) AS PRECO
               , (SELECT TOP 1 VL_RESIDUO/100 FROM ASI_HOMOLOGA..AX_ESTOQUE_UG WHERE CD_BEM_SERVICO = M.CODIGO ORDER BY CD_NATUREZA_CLASS) AS RESTO
          FROM MATERIALESTOQUESALDOMEDIO E 
          INNER JOIN MATERIAL M ON (M.ID = E.MATERIAL_ID AND DOMINIOTIPOMATERIAL_ID = (SELECT id FROM DOMINIO WHERE CHAVE='tipoMaterial' and codigo = 1))
         ) AS ESTOQUE
    WHERE MATERIALESTOQUESALDOMEDIO.MATERIAL_ID = ESTOQUE.MATERIAL_ID;
    COMMIT
    PRINT 'Finalizada migração de saldo médido de estoque!'
-- #FIM :: IMPORTACAO SALDOS DE ESTOQUE >> MOVIMENTOESTOQUESALDOMEDIO
GO


-- #INI :: IMPORTACAO DOS MOVIMENTO DE ESTOQUE >> MOVIMENTOESTOQUE
    BEGIN TRANSACTION
    GO

    PRINT 'Importando movimentos do estoque para geracao dos saldos mensais...'

    PRINT '    --> Movimentos do estoque para entradas...'
    INSERT INTO dbo.movimentoestoque
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , datamovimento
         , quantidade
         , valorfifo
         , valormedia
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         , atendimentoreqconsumoitem_id
         , baixaalmoxarifadoitem_id
         , devolucaoitem_id
         , entradaalmoxarifadoitem_id
         , localestoque_id
         , material_id
         , materialestoquesaldo_id
         , tipomovimento_id
         )
    SELECT NULL                          AS databloqueio
         , NULL                          AS datainativo
         , E.DATACRIACAO                 AS datacriacao
         , GETDATE()                     AS dataedicao
         , 0                             AS "version"
         , E.DATACONTABIL                AS datamovimento
         , EAI.QUANTIDADE                AS quantidade
         , EAI.VALORTOTAL                AS valorfifo
         , EAI.VALORTOTAL                AS valormedia
         , NULL                          AS inativador_id
         , NULL                          AS autor_id
         , NULL                          AS editor_id
         , 1                             AS organizacao_id
         , NULL                          AS atendimentoreqconsumoitem_id
         , NULL                          AS baixaalmoxarifadoitem_id
         , NULL                          AS devolucaoitem_id
         , EAI.ID                        AS entradaalmoxarifadoitem_id
         , (SELECT TOP 1 ID FROM DBO.LOCALESTOQUE) AS localestoque_id
         , EAI.MATERIAL_ID               AS material_id
         , NULL                          AS materialestoquesaldo_id
         , (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoMovimentoEstoque' AND CODIGO = 1) AS tipomovimento_id
    FROM DBO.ENTRADAALMOXARIFADO EA
    INNER JOIN DBO.ENTRADA E ON (E.ID = EA.ID)
    INNER JOIN DBO.ENTRADAALMOXARIFADOITEM EAI ON (EAI.ENTRADA_ID = E.ID)

    PRINT '    --> Movimentos do estoque para baixas...'
    INSERT INTO dbo.movimentoestoque
        ( databloqueio
        , datainativo
        , datacriacao
        , dataedicao
        , [version]
        , datamovimento
        , quantidade
        , valorfifo
        , valormedia
        , inativador_id
        , autor_id
        , editor_id
        , organizacao_id
        , atendimentoreqconsumoitem_id
        , baixaalmoxarifadoitem_id
        , devolucaoitem_id
        , entradaalmoxarifadoitem_id
        , localestoque_id
        , material_id
        , materialestoquesaldo_id
        , tipomovimento_id
        )
    SELECT NULL                          AS databloqueio
         , NULL                          AS datainativo
         , B.DATACRIACAO                 AS datacriacao
         , GETDATE()                     AS dataedicao
         , 0                             AS "version"
         , BA.DATABAIXA                  AS datamovimento
         , -BAI.QUANTIDADE               AS quantidade
         , -(BL.VL_BAIXA_ITEM/100)       AS valorfifo
         , -(BL.VL_BAIXA_ITEM/100)       AS valormedia
         , NULL                          AS inativador_id
         , NULL                          AS autor_id
         , NULL                          AS editor_id
         , 1                             AS organizacao_id
         , NULL                          AS atendimentoreqconsumoitem_id
         , BAI.ID                        AS baixaalmoxarifadoitem_id
         , NULL                          AS devolucaoitem_id
         , NULL                          AS entradaalmoxarifadoitem_id
         , (SELECT TOP 1 ID FROM DBO.LOCALESTOQUE) AS localestoque_id
         , BAI.MATERIALCONSUMO_ID        AS material_id
         , NULL                          AS materialestoquesaldo_id
         , (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoMovimentoEstoque' AND CODIGO = 2) AS tipomovimento_id
    FROM DBO.BAIXAALMOXARIFADO BA
    INNER JOIN DBO.BAIXA B ON (B.ID = BA.ID)
    INNER JOIN DBO.BAIXAALMOXARIFADOITEM BAI ON (BAI.BAIXAALMOXARIFADO_ID = B.ID)
    INNER JOIN ASI_HOMOLOGA..AX_BAIXA_ITEM BI ON (BI.CD_BAIXA+BI.CD_UG = CAST(B.CODIGO AS NVARCHAR) AND BAI.SQ_BAIXA_ITEM = BI.SQ_BAIXA_ITEM)
    INNER JOIN ASI_HOMOLOGA..AX_BAIXA_UG_LOTE BL ON (BL.CD_ORGAO = BI.CD_ORGAO AND BL.CD_ALMOXARIFADO = BI.CD_ALMOXARIFADO AND BL.CD_UG = BI.CD_UG AND BL.CD_BAIXA = BI.CD_BAIXA AND BL.CD_BEM_SERVICO = BI.CD_BEM_SERVICO AND BL.SQ_BAIXA_ITEM = BI.SQ_BAIXA_ITEM)
    ORDER BY BAI.ID;
    
    PRINT '    --> Movimentos do estoque para atendimento de requisição...'
    INSERT INTO dbo.movimentoestoque
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , datamovimento
         , quantidade
         , valorfifo
         , valormedia
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         , atendimentoreqconsumoitem_id
         , baixaalmoxarifadoitem_id
         , devolucaoitem_id
         , entradaalmoxarifadoitem_id
         , localestoque_id
         , material_id
         , materialestoquesaldo_id
         , tipomovimento_id
         )
    SELECT NULL                          AS databloqueio
         , NULL                          AS datainativo
         , A.DATACRIACAO                 AS datacriacao
         , GETDATE()                     AS dataedicao
         , 0                             AS "version"    
         , A.DATAATENDIMENTO             AS datamovimento
         , -AI.QUANTIDADE                AS quantidade
         , (CASE WHEN AI.QUANTIDADE = 0 THEN 0 ELSE -(AI.VALOR) END) AS valorfifo
         , (CASE WHEN AI.QUANTIDADE = 0 THEN 0 ELSE -(AI.VALOR) END) AS valormedia
         , NULL                          AS inativador_id
         , NULL                          AS autor_id
         , NULL                          AS editor_id
         , 1                             AS organizacao_id
         , AI.ID                         AS atendimentoreqconsumoitem_id
         , NULL                          AS baixaalmoxarifadoitem_id
         , NULL                          AS devolucaoitem_id
         , NULL                          AS entradaalmoxarifadoitem_id
         , (SELECT TOP 1 ID FROM DBO.LOCALESTOQUE) AS localestoque_id
         , AI.MATERIAL_ID                AS material_id
         , NULL                          AS materialestoquesaldo_id
         , (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoMovimentoEstoque' AND CODIGO = 3) AS tipomovimento_id
    FROM DBO.ATENDIMENTOREQUISICAOCONSUMO A
    INNER JOIN DBO.ATENDIMENTOREQCONSUMOITEM AI ON (A.ID = AI.ATENDIMENTOREQCONSUMO_ID);

    PRINT '    --> Movimentos do estoque para devoluções...'
    INSERT INTO dbo.movimentoestoque
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , datamovimento
         , quantidade
         , valorfifo
         , valormedia
         , inativador_id
         , autor_id
         , editor_id
         , organizacao_id
         , atendimentoreqconsumoitem_id
         , baixaalmoxarifadoitem_id
         , devolucaoitem_id
         , entradaalmoxarifadoitem_id
         , localestoque_id
         , material_id
         , materialestoquesaldo_id
         , tipomovimento_id
         )
    SELECT NULL                             AS databloqueio
         , NULL                             AS datainativo
         , D.DATACRIACAO                    AS datacriacao
         , D.DATAEDICAO                     AS dataedicao
         , 0                                AS "version"
         , D.DATADEVOLUCAO                  AS datamovimento
         , DI.QUANTIDADE                    AS quantidade
         , (I.VL_NOTA_REC_ITEM/100)         AS valorfifo
         , (I.VL_NOTA_REC_ITEM/100)         AS valormedia
         , NULL                             AS inativador_id
         , NULL                             AS autor_id
         , NULL                             AS editor_id
         , 1                                AS organizacao_id
         , NULL                             AS atendimentoreqconsumoitem_id
         , NULL                             AS baixaalmoxarifadoitem_id
         , DI.ID                            AS devolucaoitem_id
         , NULL                             AS entradaalmoxarifadoitem_id
         , (SELECT TOP 1 ID FROM DBO.LOCALESTOQUE) AS localestoque_id
         , DI.MATERIALCONSUMO_ID            AS material_id
         , NULL                             AS materialestoquesaldo_id
         , (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoMovimentoEstoque' AND CODIGO = 4) AS tipomovimento_id
    FROM DBO.DEVOLUCAO D
    INNER JOIN DBO.DEVOLUCAOITEM DI ON (D.ID = DI.DEVOLUCAO_ID)    
    INNER JOIN DBO.MATERIAL M ON (M.ID = DI.MATERIALCONSUMO_ID)
    INNER JOIN ASI_HOMOLOGA..AX_NOTA_REC_ITEM I ON (D.CODIGO=I.CD_NOTA_REC AND M.CODIGO = I.CD_BEM_SERVICO);
    
    COMMIT
    PRINT 'Finalizada migração de movimentos de estoque!'
-- #FIM :: IMPORTACAO DOS MOVIMENTO DE ESTOQUE >> MOVIMENTOESTOQUE
GO


-- #INI :: IMPORTACAO DOS MOVIMENTO DAS CONTAS DE ALMOXARIFADO >> CONTACONTABILMOVIMENTO
    BEGIN TRANSACTION
    GO
    
    /*
    PRINT 'Criando o contacontabilmovimento...'
    INSERT INTO dbo.contacontabilmovimento
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , datareferencia
         , valor
         , inativador_id
         , autor_id
         , editor_id
         , baixa_id
         , contacontabil_id
         , tipomovimentocontacontabil_id
         , entrada_id
         , transferenciaconta_id
         )
    SELECT NULL      AS databloqueio
         , NULL      AS datainativo
         , GETDATE() AS datacriacao
         , GETDATE() AS dataedicao
         , 0         AS "version"
         , CONVERT(DATETIME,CONCAT(S.AA_FECHAMENTO,S.MM_FECHAMENTO,'01'),112) AS datareferencia
         , CASE 
             WHEN S.VL_ENTRADA > 0 THEN (S.VL_ENTRADA/100)
             WHEN S.VL_SAIDA   > 0 THEN (ABS(S.VL_SAIDA/100)*(-1))
           END       AS valor
         , NULL      AS inativador_id
         , NULL      AS autor_id
         , NULL      AS editor_id
         , NULL      AS baixa_id
         , C.ID      AS contacontabil_id
         , CASE 
             WHEN S.VL_ENTRADA > 0 THEN (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoMovimentoContaContabil' AND CODIGO=1)  -- "Entrada orçamentária"
             WHEN S.VL_SAIDA   > 0 THEN (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoMovimentoContaContabil' AND CODIGO=2)  -- "Baixa orçamentária"
           END      AS tipomovimentocontacontabil_id
         , NULL     AS entrada_id
         , NULL     AS transferenciaconta_id
    FROM DBO.CONTACONTABIL C
    INNER JOIN ASI_HOMOLOGA..AX_V_SALDO_ANT S ON C.CODIGO=S.CD_CONTA
    JOIN (SELECT COALESCE(MAX(ID),0) AS MAX FROM DBO.CONTACONTABILMOVIMENTO) M ON 1=1
    WHERE (S.VL_ENTRADA+S.VL_SAIDA)>0
    ORDER BY S.CD_CONTA; 
    */
    
    PRINT 'Criando o contacontabilmovimento das entradas...'
    INSERT INTO dbo.contacontabilmovimento
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , datareferencia
         , valor
         , inativador_id
         , autor_id
         , editor_id
         , baixa_id
         , contacontabil_id
         , tipomovimentocontacontabil_id
         , entrada_id
         , transferenciaconta_id
         , idclassereferencia
         , dominioclassereferencia_id
         )
    SELECT NULL             AS databloqueio
         , NULL             AS datainativo
         , GETDATE()        AS datacriacao
         , GETDATE()        AS dataedicao
         , 0                AS "version"
         , M.DATAMOVIMENTO  AS datareferencia
         , M.VALORMEDIA     AS valor
         , NULL             AS inativador_id
         , NULL             AS autor_id
         , NULL             AS editor_id
         , NULL             AS baixa_id
         , C.ID             AS contacontabil_id
         , (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoMovimentoContaContabil' AND CODIGO=1) AS tipomovimentocontacontabil_id -- "Entrada orçamentária" 
         , NULL             AS entrada_id
         , NULL             AS transferenciaconta_id
         , M.ENTRADAALMOXARIFADOITEM_ID AS idclassereferencia
         , (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='classeReferencia' AND CODIGO = 12) as dominioclassereferencia_id
    FROM DBO.MOVIMENTOESTOQUE M
    INNER JOIN DBO.ENTRADAALMOXARIFADOITEM EI ON (EI.ID = M.ENTRADAALMOXARIFADOITEM_ID)
    INNER JOIN DBO.CONTACONTABIL C ON (C.CODIGO = EI.CD_CONTA)
    WHERE TIPOMOVIMENTO_ID = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoMovimentoEstoque' AND CODIGO = 1);
    
    PRINT 'Criando o contacontabilmovimento das baixas...'
    INSERT INTO dbo.contacontabilmovimento
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , datareferencia
         , valor
         , inativador_id
         , autor_id
         , editor_id
         , baixa_id
         , contacontabil_id
         , tipomovimentocontacontabil_id
         , entrada_id
         , transferenciaconta_id
         , idclassereferencia
         , dominioclassereferencia_id
         )
    SELECT NULL            AS databloqueio
         , NULL            AS datainativo
         , GETDATE()       AS datacriacao
         , GETDATE()       AS dataedicao
         , 0               AS "version"
         , M.DATAMOVIMENTO AS datareferencia
         , M.VALORMEDIA    AS valor
         , NULL            AS inativador_id
         , NULL            AS autor_id
         , NULL            AS editor_id
         , NULL            AS baixa_id
         , C.ID            AS contacontabil_id
         , (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoMovimentoContaContabil' AND CODIGO=4) AS tipomovimentocontacontabil_id -- "Entrada orçamentária" 
         , NULL            AS entrada_id
         , NULL            AS transferenciaconta_id
         , M.BAIXAALMOXARIFADOITEM_ID AS idclassereferencia
         , (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='classeReferencia' AND CODIGO = 13) as dominioclassereferencia_id
    FROM DBO.MOVIMENTOESTOQUE M
    INNER JOIN DBO.BAIXAALMOXARIFADOITEM BI ON (BI.ID = M.BAIXAALMOXARIFADOITEM_ID)
    INNER JOIN DBO.CONTACONTABIL C ON (C.CODIGO = BI.CD_CONTA)
    WHERE TIPOMOVIMENTO_ID = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoMovimentoEstoque' AND CODIGO = 2);

    PRINT 'Criando o contacontabilmovimento das devoluções...'
    INSERT INTO dbo.contacontabilmovimento
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , datareferencia
         , valor
         , inativador_id
         , autor_id
         , editor_id
         , baixa_id
         , contacontabil_id
         , tipomovimentocontacontabil_id
         , entrada_id
         , transferenciaconta_id
         , idclassereferencia
         , dominioclassereferencia_id
         )
    SELECT NULL            AS databloqueio
         , NULL            AS datainativo
         , GETDATE()       AS datacriacao
         , GETDATE()       AS dataedicao
         , 0               AS "version"
         , M.DATAMOVIMENTO AS datareferencia
         , M.VALORMEDIA    AS valor
         , NULL            AS inativador_id
         , NULL            AS autor_id
         , NULL            AS editor_id
         , NULL            AS baixa_id
         , C.ID            AS contacontabil_id
         , (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoMovimentoContaContabil' AND CODIGO=1) AS tipomovimentocontacontabil_id -- "Entrada orçamentária" 
         , NULL            AS entrada_id
         , NULL            AS transferenciaconta_id
         , M.DEVOLUCAOITEM_ID AS idclassereferencia
         , (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='classeReferencia' AND CODIGO = 6) as dominioclassereferencia_id
    FROM DBO.MOVIMENTOESTOQUE M
    INNER JOIN DBO.DEVOLUCAOITEM DI ON (DI.ID = M.DEVOLUCAOITEM_ID)
    INNER JOIN DBO.CONTACONTABIL C ON (C.CODIGO = DI.CD_CONTA)
    WHERE TIPOMOVIMENTO_ID = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoMovimentoEstoque' AND CODIGO = 4);

    PRINT 'Criando o contacontabilmovimento dos atendimentos...'
    INSERT INTO dbo.contacontabilmovimento
         ( databloqueio
         , datainativo
         , datacriacao
         , dataedicao
         , [version]
         , datareferencia
         , valor
         , inativador_id
         , autor_id
         , editor_id
         , baixa_id
         , contacontabil_id
         , tipomovimentocontacontabil_id
         , entrada_id
         , transferenciaconta_id
         , idclassereferencia
         , dominioclassereferencia_id
         )
    SELECT NULL            AS databloqueio
         , NULL            AS datainativo
         , GETDATE()       AS datacriacao
         , GETDATE()       AS dataedicao
         , 0               AS "version"
         , M.DATAMOVIMENTO AS datareferencia
         , M.VALORMEDIA    AS valor
         , NULL            AS inativador_id
         , NULL            AS autor_id
         , NULL            AS editor_id
         , NULL            AS baixa_id
         , C.ID            AS contacontabil_id
         , (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoMovimentoContaContabil' AND CODIGO=4) AS tipomovimentocontacontabil_id -- "Entrada orçamentária" 
         , NULL            AS entrada_id
         , NULL            AS transferenciaconta_id
         , M.ATENDIMENTOREQCONSUMOITEM_ID AS idclassereferencia
         , (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='classeReferencia' AND CODIGO = 14) as dominioclassereferencia_id
    FROM DBO.MOVIMENTOESTOQUE M
    INNER JOIN DBO.ATENDIMENTOREQCONSUMOITEM AI ON (AI.ID = M.ATENDIMENTOREQCONSUMOITEM_ID)
    INNER JOIN DBO.REQUISICAOCONSUMOITEM RI ON (RI.ID = AI.REQUISICAOCONSUMOITEM_ID)
    INNER JOIN DBO.CONTACONTABIL C ON (C.CODIGO = RI.CD_CONTA)
    WHERE TIPOMOVIMENTO_ID = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoMovimentoEstoque' AND CODIGO = 3);
    
    COMMIT
    PRINT 'Finalizada importação dos movimentos das contas do almoxarifado!'
-- #FIM :: IMPORTACAO DOS MOVIMENTO DAS CONTAS DE ALMOXARIFADO >> CONTACONTABILMOVIMENTO
GO


-- #INI :: GERACAO DE SALDOS INICIAIS DO ALMOXARIFADO CONFORME CLIENTE >> CONTACONTABILSALDO
    BEGIN TRANSACTION
    GO
    
    INSERT INTO dbo.contacontabilsaldo
         ( datacriacao
         , dataedicao
         , [version]
         , datareferencia
         , valor
         , contacontabil_id
         , depreciacaoacumulada
         , depreciacaomensal
         , reavaliacaoperiodo
         , reavaliacaoacumulada
         , reducaoperiodo
         , reducaoacumulada
         )
    SELECT GETDATE()
         , GETDATE()
         , 0
         , CONVERT(DATETIME,'20070501',112)  -- REVER CONFORME CLIENTE
         , F.VL_SALDO_ANT/100
         , C.ID
         , 0
         , 0
         , 0
         , 0
         , 0
         , 0
    FROM ASI_HOMOLOGA..AX_V_SALDO_ANT F 
    INNER JOIN CONTACONTABIL C ON (C.CODIGO = F.CD_CONTA)
    WHERE F.AA_FECHAMENTO = '2007' AND F.MM_FECHAMENTO = '06' AND F.VL_SALDO_ANT > 0;

    INSERT INTO dbo.contacontabilsaldo
         ( datacriacao
         , dataedicao
         , [version]
         , datareferencia
         , valor
         , contacontabil_id
         , depreciacaoacumulada
         , depreciacaomensal
         , reavaliacaoperiodo
         , reavaliacaoacumulada
         , reducaoperiodo
         , reducaoacumulada
         )
    SELECT GETDATE()
         , GETDATE()
         , 0
         , CONVERT(DATETIME,'20070501',112)  -- REVER CONFORME CLIENTE
         , 0
         , C.ID
         , 0
         , 0
         , 0
         , 0
         , 0
         , 0
    FROM DBO.CONTACONTABIL C 
    WHERE C.ID NOT IN (SELECT CONTACONTABIL_ID FROM DBO.CONTACONTABILSALDO)
    AND C.DOMINIOTIPOMATERIAL_ID = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE='tipoMaterial' AND CODIGO=1);
    
    COMMIT
    PRINT 'Finalizada carga dos saldos iniciais de contacontabilsaldo!'
-- #FIM :: GERACAO DE SALDOS INICIAIS DO ALMOXARIFADO CONFORME CLIENTE >> CONTACONTABILSALDO
GO


-- ############                                                                                 ##########
-- ############                     ))) RODAR PARA MES NAO CONTEMPLADO (((                      ##########
-- ############                 SE A INSERCAO_CONTACONTABILSALDO FOI EXECUTADA                  ##########
-- ############                                                                                 ##########
-- #INI :: GERACAO DE SALDOS COM DADADOS DEPRECIACAO >> CONTACONTABILSALDO
    BEGIN TRANSACTION
    GO

    /*
      SCRIPT QUE GERA O SALDO
      =======================
      1. Definir intervalo referencia (@ref_atu_fday/@ref_atu_lday) com o mês
         a partir do qual se deseja gerar os saldos, por exemplo: JAN-2016.
      2. Definir intervalo anterior (@ref_ant_fday e @ref_ant_lday) com o mês
         com saldo anterior, por exemplo: DEZ-2015, lembrando que os saldos
         ja devem estar gerados para esse mes.
      3. Definir @ref_fin_fday como o ultimo mes de referencia fechado.
    */
    DECLARE @dom_movent_o BIGINT
          , @dom_movent_e BIGINT
          , @dom_movbai_o BIGINT
          , @dom_movbai_e BIGINT
          , @dom_mat_cons BIGINT
          , @ref_ant_fday DATETIME
          , @ref_ant_lday DATETIME
          , @ref_atu_fday DATETIME
          , @ref_atu_lday DATETIME
          , @ref_fin_fday DATETIME
          , @ref_ant_prnt DATETIME = GETDATE()
          , @conta_cnt_cd NVARCHAR(30)
          , @conta_cnt_id BIGINT
          , @ccs_id       BIGINT

    SET @dom_movent_o = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoMovimentoContaContabil' AND CODIGO = 1)
    SET @dom_movent_e = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoMovimentoContaContabil' AND CODIGO = 3)
    SET @dom_movbai_o = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoMovimentoContaContabil' AND CODIGO = 2)
    SET @dom_movbai_e = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoMovimentoContaContabil' AND CODIGO = 4)
    SET @dom_mat_cons = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoMaterial' AND CODIGO = 1)
    SET @ref_fin_fday = '20100201'
    SET @ref_ant_fday = '20070501'
    SET @ref_ant_lday = DATEADD(MONTH, 1, @ref_ant_fday) - '00:00:01'

    SET @ref_atu_fday = '20070601'
    SET @ref_atu_lday = DATEADD(MONTH, 1, @ref_atu_fday) - '00:00:01'

    PRINT 'Gerando saldos entre [' + CONVERT(VARCHAR(8),@ref_atu_fday,112) + '] e [' + CONVERT(VARCHAR(8),@ref_fin_fday,112) + '] ...'
    WHILE DATEPART(MONTH,@ref_atu_fday) <= DATEPART(MONTH,@ref_fin_fday)
    BEGIN
        IF (@ref_atu_fday<>@ref_ant_prnt)
        BEGIN
            PRINT '  Mês: [' + CONVERT(VARCHAR(6), @ref_atu_fday, 112) + ']...'
            SET @ref_ant_prnt = @ref_atu_fday
        END

        SELECT CC.ID                    AS contacontabil_id
             , CC.CODIGO                AS contacontabil_cd
             , COALESCE(SDO.VALOR, 0)   AS saldoanterior
             , COALESCE(
                 (SELECT SUM(M.VALOR) -- ENTRADATOTAL
                    FROM CONTACONTABILMOVIMENTO M
                   WHERE CONTACONTABIL_ID = CC.ID 
                     AND M.DATAREFERENCIA BETWEEN @ref_atu_fday AND @ref_atu_lday
                     AND TIPOMOVIMENTOCONTACONTABIL_ID in (@dom_movent_o, @dom_movent_e))
                 , 0)                   AS entrada_grp
             , COALESCE(
                 (SELECT SUM(M.VALOR) -- SAIDA
                    FROM CONTACONTABILMOVIMENTO M 
                   WHERE CONTACONTABIL_ID = CC.ID
                     AND M.DATAREFERENCIA BETWEEN @ref_atu_fday AND @ref_atu_lday
                     AND TIPOMOVIMENTOCONTACONTABIL_ID in (@dom_movbai_o, @dom_movbai_e))
                 , 0)                   AS saida_grp
        INTO #TMP_SALDO
        FROM DBO.CONTACONTABIL CC
        LEFT JOIN CONTACONTABILSALDO SDO ON (SDO.CONTACONTABIL_ID = CC.ID AND SDO.DATAREFERENCIA BETWEEN @ref_ant_fday AND @ref_ant_lday)
        WHERE CC.DATAINATIVO IS NULL
        AND DOMINIOTIPOMATERIAL_ID = @dom_mat_cons
        ORDER BY CC.ID
        
        DECLARE cur_ccs CURSOR FAST_FORWARD FOR
            SELECT contacontabil_id, contacontabil_cd
            FROM #TMP_SALDO
        
        OPEN cur_ccs
        FETCH NEXT FROM cur_ccs
            INTO @conta_cnt_id, @conta_cnt_cd
        
        WHILE @@FETCH_STATUS = 0
        BEGIN
             PRINT '    Conta: ' + @conta_cnt_cd;
             INSERT INTO dbo.contacontabilsaldo
                  ( datacriacao
                  , dataedicao
                  , [version]
                  , datareferencia
                  , valor
                  , contacontabil_id
                  , depreciacaoacumulada
                  , depreciacaomensal
                  , reavaliacaoperiodo
                  , reavaliacaoacumulada
                  , reducaoperiodo
                  , reducaoacumulada
                  )
             SELECT GETDATE()
                  , GETDATE()
                  , 0
                  , @ref_atu_fday
                  , (R.SALDOANTERIOR+R.ENTRADA_GRP)+R.SAIDA_GRP
                  , R.CONTACONTABIL_ID
                  , 0
                  , 0
                  , 0
                  , 0
                  , 0
                  , 0
                  FROM #TMP_SALDO R
                  WHERE R.CONTACONTABIL_ID = @conta_cnt_id

            FETCH NEXT FROM cur_ccs
                INTO @conta_cnt_id, @conta_cnt_cd
        END
        CLOSE cur_ccs
        DEALLOCATE cur_ccs

        IF DATEPART(month,@ref_atu_fday) = DATEPART(month, @ref_fin_fday)
        BEGIN 
            -- Quando termina de gerar os saldos, atribuir o último no contacontabilsaldoanterior_id
            -- na tabela contacontabil (Necessários para o RMB e fechamento de mês)
            UPDATE dbo.contacontabil
               SET contacontabilsaldoanterior_id =
                 ( SELECT TOP 1 S.ID 
                   FROM DBO.CONTACONTABILSALDO S
                   WHERE S.CONTACONTABIL_ID = ID
                   ORDER BY DATAREFERENCIA DESC );
        END

        SET @ref_atu_fday = DATEADD(MONTH, 1, @ref_atu_fday);
        SET @ref_atu_lday = DATEADD(MONTH, 1, @ref_atu_fday) - '00:00:01';

        SET @ref_ant_fday = DATEADD(MONTH, 1, @ref_ant_fday);
        SET @ref_ant_lday = DATEADD(MONTH, 1, @ref_ant_fday) - '00:00:01';
        DROP TABLE #TMP_SALDO
    END
    COMMIT
    PRINT 'Finalizada geração de saldos de almoxarifado!'
-- #FIM :: GERACAO DE SALDOS COM DADOS DEPRECIACAO >> CONTACONTABILSALDO
GO


-- #INI :: INCLUI AS CONTAS SEM SALDO EM CADA REFERENCIA >> CONTACONTABILSALDO
    /*
    BEGIN TRANSACTION
    GO
    
    DECLARE @v_tipoMat int,
            @v_dataIni datetime,
            @v_dataFin datetime,
            @v_dataRef datetime,
            @v_tipo int,
            @v_idctb bigint

    --> RECUPERA O DOMINIO DAS CONTAS DE MATERIAL DE CONSUMO
    SET @v_tipo = (SELECT ID FROM dbo.DOMINIO WHERE CHAVE='tipoMaterial' AND CODIGO=1)

    --> RECUPERA O PRIMEIRO MES DE REFERENCIA DAS CONTAS DE MATERIAL DE CONSUMO
    SET @v_dataIni = (SELECT MIN(datareferencia) FROM dbo.CONTACONTABILSALDO)

    --> INICIALIZA A DATA DE REFERENCIA
    SET @v_dataRef = @v_dataIni

    --> RECUPERA O ULTIMO MES DE REFERENCIA DAS CONTAS DE MATERIAL DE CONSUMO
    SET @v_dataFin = (SELECT MAX(datareferencia) FROM dbo.CONTACONTABILSALDO)

    WHILE (@v_dataRef BETWEEN @v_dataIni AND @v_dataFin)
    BEGIN
        DECLARE cur_contas CURSOR FAST_FORWARD FOR SELECT ID FROM dbo.contacontabil WHERE dominiotipomaterial_id = @v_tipo
        OPEN cur_contas
        FETCH NEXT FROM cur_contas INTO @v_idctb
        WHILE @@FETCH_STATUS = 0
            IF (SELECT ID FROM dbo.contacontabilsaldo WHERE contacontabil_id=contas.id AND datareferencia=@v_dataRef) IS NULL
                BEGIN
                INSERT INTO dbo.contacontabilsaldo
                     ( 
                       datacriacao, dataedicao, 
                       [version], datareferencia, valor, contacontabil_id, depreciacaoacumulada, depreciacaomensal,
                       reavaliacaoperiodo, reavaliacaoacumulada, reducaoperiodo, reducaoacumulada    
                     )
                VALUES 
                     ( 
                       GETDATE(), GETDATE(), 0,
                       @v_dataRef, 0.0, @v_idctb, 0, 0, 0, 0, 0, 0
                     );
            END
        FETCH NEXT FROM cur_contas INTO @v_idctb
        END
        CLOSE cur_contas
        DEALLOCATE cur_contas
        @v_dataRef = DATEADD(MONTH,1,@v_dataRef)
    END
    
    COMMIT
    */
-- #FIM :: INCLUI AS CONTAS SEM SALDO EM CADA REFERENCIA >> CONTACONTABILSALDO
GO


--===========================================================================--
-- ##INI :: IMPORTACAO TABELAS DE MOVIMENTO DE MATERIAS DE CONSUMO           --
--===========================================================================--