--===========================================================================--
-- ##INI :: AJUSTE FINAIS DA MIGRACAO                                        --
--===========================================================================--
SET NOCOUNT ON


-- #INI :: AJUSTES TABELA BEMPATRIMONIAL >> COLUNA CODIGO
    BEGIN TRANSACTION
    GO

    IF EXISTS(SELECT * FROM sys.indexes WHERE Name = N'ix01_bempat_cd' AND Object_ID = Object_ID(N'BemPatrimonial'))
      DROP INDEX ix01_bempat_cd ON dbo.bempatrimonial
    ELSE
      PRINT 'Indice ix01_bempat_cd não existe na tabela BemPatrimonial'
    GO

    IF EXISTS(SELECT * FROM sys.columns WHERE Name = N'codigo_tmp' AND Object_ID = Object_ID(N'BemPatrimonial'))
    BEGIN
      ALTER TABLE dbo.bempatrimonial DROP COLUMN codigo;
      EXEC SP_RENAME 'bempatrimonial.codigo_tmp', 'codigo', 'COLUMN'
    END
    ELSE
      PRINT 'Coluna codigo_tmp não existe na tabela BemPatrimonial'
    GO

    COMMIT
-- #FIM :: AJUSTES TABELA BEMPATRIMONIAL >> COLUNA CODIGO
GO


-- #INI :: GERACAO DE CONFIGURACAO DE CONTAS PARA O RMA/RMB >> CONTACONTABILCONFIGREFERENCIA
    BEGIN TRANSACTION
    GO

    /*
    SCRIPT QUE GERA A REFERENCIA DE CONTAS
    ======================================
    1. Definir ref_atu_fday e ref_atu_lday com o período inicial que você quer gerar a referência
    2. Definir ref_ant_fday com o mês de referência anterior ao saldo que você quer gerar 
       (os saldos já devem estar gerados para esse mês)
    3. Apos o insert definir onde você quer quer a função pare de gerar a referência,
       ou seja o ultimo mes de referencia fechado no banco
    */
    DECLARE @ref_ant_fday DATETIME
          , @ref_atu_fday DATETIME
          , @ref_atu_lday DATETIME
          , @ref_fin_fday DATETIME
          , @imprimir_rma BIT
          , @imprimir_rmb BIT
          , @conta_cnt_id BIGINT
          , @conta_cnt_nm NVARCHAR(100)

    SET @ref_fin_fday = '20100201'
    SET @ref_ant_fday = '20070501'
    SET @ref_atu_fday = '20070601'
    SET @ref_atu_lday = DATEADD(MONTH, 1, @ref_atu_fday) - '00:00:01'

    WHILE DATEPART(month,@ref_atu_fday) <= DATEPART(month, @ref_fin_fday)
    BEGIN
      
        DECLARE cur_ctacontabil CURSOR FAST_FORWARD FOR
            SELECT C.ID, D.NOME FROM DBO.CONTACONTABIL C
            INNER JOIN DBO.DOMINIO D ON (D.ID = DOMINIOTIPOMATERIAL_ID)
            ORDER BY 1

        OPEN cur_ctacontabil
        FETCH NEXT FROM cur_ctacontabil INTO @conta_cnt_id, @conta_cnt_nm

        WHILE @@FETCH_STATUS = 0
        BEGIN
            --> A PRINCIPIO NAO APARECE NEM NO RMA NEM NO RMB
            SET @imprimir_rmb = 0;
            SET @imprimir_rma = 0;

            --> VERIFICA SE A CONTA TEM MOVIMENTO
            IF EXISTS (SELECT 'X' FROM CONTACONTABILMOVIMENTO M 
                        WHERE M.DATAREFERENCIA BETWEEN @ref_atu_fday AND @ref_atu_lday
                          AND M.CONTACONTABIL_ID = @conta_cnt_id)
            BEGIN
                IF (@conta_cnt_nm='PERMANENTE') SET @imprimir_rmb = 1;
                IF (@conta_cnt_nm='CONSUMO')    SET @imprimir_rma = 1;
            END

            --> OU SE A CONTA TEM SALDO
            IF EXISTS (SELECT 'X' FROM CONTACONTABILSALDO S 
                        WHERE S.DATAREFERENCIA BETWEEN @ref_atu_fday AND @ref_atu_lday
                          AND S.CONTACONTABIL_ID = @conta_cnt_id AND S.VALOR > 0.0)
            BEGIN
                IF (@conta_cnt_nm='PERMANENTE') SET @imprimir_rmb = 1;
                IF (@conta_cnt_nm='CONSUMO')    SET @imprimir_rma = 1;
            END

            --> INSERE UM REGISTRO EM CONFIGURACAO PARA CADA CONTA / MESREFERENCIA
            INSERT INTO dbo.contacontabilconfigreferencia
                 ( datacriacao, dataedicao, [version], apresentarma,
                   apresentarmb, apresentaRelatorioDepreciacao, mesreferencia, contacontabil_id )
            VALUES 
                 ( GETDATE(), GETDATE(), 0, @imprimir_rma,
                   @imprimir_rmb, @imprimir_rmb, @ref_atu_fday, @conta_cnt_id)

            FETCH NEXT FROM cur_ctacontabil INTO @conta_cnt_id, @conta_cnt_nm
        END
        CLOSE cur_ctacontabil
        DEALLOCATE cur_ctacontabil

        SET @ref_atu_fday = DATEADD(MONTH, 1, @ref_atu_fday)
        SET @ref_atu_lday = DATEADD(MONTH, 1, @ref_atu_fday)
    END
    
    COMMIT
-- #FIM :: GERACAO DE CONFIGURACAO DE CONTAS PARA O RMA/RMB >> CONTACONTABILCONFIGREFERENCIA
GO


-- #INI :: AJUSTA CONFIGURACAO DE CONTAS PARA ULTIMO MESREFERENCIA >> CONTACONTABILCONFIGREFERENCIA
    BEGIN TRANSACTION
    GO

    INSERT INTO dbo.contacontabilconfigreferencia
         ( datacriacao, dataedicao, [version], apresentarma,
           apresentarmb, apresentarelatoriodepreciacao, mesreferencia, contacontabil_id )
    SELECT GETDATE(), GETDATE(), 0, r.apresentarma, r.apresentarmb,
           r.apresentarmb, '20100201', r.contacontabil_id
    FROM CONTACONTABILCONFIGREFERENCIA R
    WHERE R.MESREFERENCIA = (SELECT MAX(MESREFERENCIA) FROM CONTACONTABILCONFIGREFERENCIA)

    COMMIT
-- #FIM :: AJUSTA CONFIGURACAO DE CONTAS PARA ULTIMO MESREFERENCIA >> CONTACONTABILCONFIGREFERENCIA
GO


-- #INI :: DEFINICAO DAS CARACTERISTICAS FILTRAVEIS PARA O MPOG (FUTURAMENTE O USUARIO PODERA ESCOLHER MAIS CARACTERISTICAS NO SOFTWARE)
    BEGIN TRANSACTION
    GO

    UPDATE dbo.caracteristica SET filtro = 0 WHERE ID = (SELECT TOP 1 ID FROM DBO.CARACTERISTICA WHERE descricao like 'DESCRICAO' AND DATABLOQUEIO IS NULL)
    UPDATE dbo.caracteristica SET filtro = 0 WHERE ID = (SELECT TOP 1 ID FROM DBO.CARACTERISTICA WHERE descricao like 'MARCA'     AND DATABLOQUEIO IS NULL)
    UPDATE dbo.caracteristica SET filtro = 0 WHERE ID = (SELECT TOP 1 ID FROM DBO.CARACTERISTICA WHERE descricao like 'MODELO'    AND DATABLOQUEIO IS NULL)
    UPDATE dbo.caracteristica SET filtro = 0 WHERE ID = (SELECT TOP 1 ID FROM DBO.CARACTERISTICA WHERE descricao like 'SERIE'     AND DATABLOQUEIO IS NULL)
    UPDATE dbo.caracteristica SET filtro = 0 WHERE ID = (SELECT TOP 1 ID FROM DBO.CARACTERISTICA WHERE descricao like 'PLACA'     AND DATABLOQUEIO IS NULL)
    UPDATE dbo.caracteristica SET filtro = 0 WHERE ID = (SELECT TOP 1 ID FROM DBO.CARACTERISTICA WHERE descricao like 'RENAVAM'   AND DATABLOQUEIO IS NULL)
    UPDATE dbo.caracteristica SET filtro = 0 WHERE ID = (SELECT TOP 1 ID FROM DBO.CARACTERISTICA WHERE descricao like 'TÍTULO'    AND DATABLOQUEIO IS NULL)
    UPDATE dbo.caracteristica SET filtro = 0 WHERE ID = (SELECT TOP 1 ID FROM DBO.CARACTERISTICA WHERE descricao like 'AUTOR'     AND DATABLOQUEIO IS NULL)
    
    COMMIT
-- #FIM :: DEFINICAO DAS CARACTERISTICAS FILTRAVEIS PARA O MPOG (FUTURAMENTE O USUARIO PODERA ESCOLHER MAIS CARACTERISTICAS NO SOFTWARE)
GO


-- #INI :: BLOQUEIO DE ALGUNS DOMINIOS :: REDMINE #4715
    BEGIN TRANSACTION
    GO

    UPDATE PUBLIC.DOMINIO SET databloqueio = GETDATE()
    WHERE CHAVE = 'tipoStatusRequisicao'
    AND NOME IN ('ATENDIDA','DEVOLVIDA');

    COMMIT
-- #FIM :: BLOQUEIO DE ALGUNS DOMINIOS :: REDMINE #4715
GO


-- #INI :: AJUSTES TIPOSTATUSBAIXAESTORNO >> BAIXAALMOXARIFADO
    BEGIN TRANSACTION
    GO

    DECLARE @dom_estorno_sem BIGINT
          , @dom_estorno_par BIGINT
          , @dom_estorno_tot BIGINT
          , @qtd_itens_baixa BIGINT
          , @qtd_itens_estor BIGINT
          , @baixa_id        BIGINT
      
    SET @dom_estorno_sem = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoStatusBaixaEstorno' AND NOME = 'SEM_ESTORNO')
    SET @dom_estorno_par = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoStatusBaixaEstorno' AND NOME = 'ESTORNO_PARCIAL')
    SET @dom_estorno_tot = (SELECT ID FROM DBO.DOMINIO WHERE CHAVE = 'tipoStatusBaixaEstorno' AND NOME = 'ESTORNO_TOTAL')

    DECLARE cur_baixa CURSOR FAST_FORWARD FOR
        SELECT BA.BAIXAALMOXARIFADO_ID        AS baixa_id
             , COUNT(BA.BAIXAALMOXARIFADO_ID) AS qtd_itens_baixa
             , COUNT(BA.DATAESTORNO)          AS qtd_itens_estor
        FROM BAIXAALMOXARIFADOITEM BA
        INNER JOIN BAIXAALMOXARIFADO B ON B.ID = BA.BAIXAALMOXARIFADO_ID
        GROUP BY BAIXAALMOXARIFADO_ID

    OPEN cur_baixa
    FETCH NEXT FROM cur_baixa
        INTO @baixa_id, @qtd_itens_baixa, @qtd_itens_estor

    WHILE @@FETCH_STATUS = 0
    BEGIN
        IF (@qtd_itens_baixa = @qtd_itens_estor) AND (@qtd_itens_estor > 0)
            UPDATE dbo.BaixaAlmoxarifado 
            SET    dominiosituacaoestorno_id = @dom_estorno_tot
            WHERE  id = @baixa_id
        ELSE IF (@qtd_itens_baixa > @qtd_itens_estor) AND (@qtd_itens_estor > 0)
            UPDATE dbo.BaixaAlmoxarifado 
            SET    dominiosituacaoestorno_id = @dom_estorno_par
            WHERE  id = @baixa_id
        ELSE
            UPDATE dbo.BaixaAlmoxarifado
            SET    dominiosituacaoestorno_id = @dom_estorno_sem
            WHERE  id = @baixa_id

        FETCH NEXT FROM cur_baixa 
            INTO @baixa_id, @qtd_itens_baixa, @qtd_itens_estor
    END
    CLOSE cur_baixa
    DEALLOCATE cur_baixa
    
    COMMIT
-- #FIM :: AJUSTES TIPOSTATUSBAIXAESTORNO >> BAIXAALMOXARIFADO
GO


-- #INI :: GARANTINDO QUE AS CONTAS SEM SALDO ANTERIOR, TENHAM UM SALDO ZERADO
    BEGIN TRANSACTION
    GO

    INSERT INTO dbo.contacontabilsaldo
         ( datacriacao, dataedicao, version, datareferencia, valor, contacontabil_id, depreciacaoacumulada,
           depreciacaomensal, reavaliacaoperiodo, reavaliacaoacumulada, reducaoperiodo, reducaoacumulada )
    SELECT '2010-02-01 00:00:00.000', '2010-02-01 00:00:00.000', 0, '2015-09-01 00:00:00.000', 0, ID, 0, 0, 0, 0, 0, 0
    FROM DBO.CONTACONTABIL
    WHERE CONTACONTABILSALDOANTERIOR_ID IS NULL;

    -- Referenciar um conta contabil ha um registro do conta contabil
    UPDATE dbo.contacontabil 
       SET contacontabilsaldoanterior_id = 
         ( SELECT TOP 1 S.ID FROM DBO.CONTACONTABILSALDO S 
           WHERE S.CONTACONTABIL_ID = ID 
           ORDER BY DATAREFERENCIA DESC );
    
    COMMIT
-- #INI :: GARANTINDO QUE AS CONTAS SEM SALDO ANTERIOR, TENHAM UM SALDO ZERADO
GO


SET NOCOUNT OFF
--===========================================================================--
-- ##FIM :: AJUSTE FINAIS DA MIGRACAO                                        --
--===========================================================================--