package br.com.centralit.controller;

import java.util.Collection;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import br.com.centralit.api.model.ClassificacaoMaterial;
import br.com.centralit.api.service.ClassificacaoMaterialService;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.json.ResponseBodyWrapper;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.view.ResultResponseVH;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * @since 23/12/2014 - 10:33:38
 *
 * @version 1.0.0
 *
 * @author wilker.machado
 *
 */
@Controller
@RequestMapping("/rest/classificacaoMaterial")
public class ClassificacaoMaterialController extends GenericController<ClassificacaoMaterial> {

	/** Atributo classificacaoMaterialService. */
	private ClassificacaoMaterialService classificacaoMaterialService;

	public ClassificacaoMaterialController() {
		super();
	}

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 *
	 * @param classificacaoMaterialService
	 */
	@Autowired
	public ClassificacaoMaterialController( ClassificacaoMaterialService classificacaoMaterialService ) {

		super(classificacaoMaterialService);

		this.classificacaoMaterialService = classificacaoMaterialService;
	}

	/**
	 *
	 * Método responsável por listar Grupos codigo 1 de classificação material
	 *
	 * @author wilker.machado
	 *
	 * @return <code>ResponseBodyWrapper</code>
	 */
	@RequestMapping(value = "/listarGrupos", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper listarGrupos() {

		ResultResponseVH<List<ClassificacaoMaterial>> resultResponseVH = new ResultResponseVH<List<ClassificacaoMaterial>>(classificacaoMaterialService.listarGruposClassificacaoMaterial());

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(resultResponseVH, this.getListView());

		return responseBody;
	}

	/**
	 * Método responsável por realizar consulta na arvore atraves de uma descrição que esta sendo informada
	 *
	 * @author wilker.machado
	 *
	 * @param descricao
	 *
	 * @return <code>ResponseBodyWrapper</code>
	 */
	@RequestMapping(value = "/listarGruposTree", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper listarGruposTree(@RequestParam(value = "descricao") String descricao) {

		ResultResponseVH<Collection<ClassificacaoMaterial>> resultResponseVH = new ResultResponseVH<Collection<ClassificacaoMaterial>>(classificacaoMaterialService.listarClassificacaoMaterialTree(descricao));

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(resultResponseVH, Views.ClassificacaoMaterialSearchView.class);

		return responseBody;
	}

	/**
	 * Método responsável por listar Grupos de acordo com o codigo eviado. Sempre listara o codigo acima do selecionado
	 *
	 * @author wilker.machado
	 *
	 * @param descricao
	 * @param codigo
	 * @return <code>ResponseBodyWrapper</code>
	 */
	@RequestMapping(value = "/listarClassificacaoMaterial", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper listarGruposBusca(@RequestParam(value = "descricao") String descricao, @RequestParam(value = "codigo") Long codigo) {

		ResultResponseVH<List<ClassificacaoMaterial>> resultResponseVH = new ResultResponseVH<List<ClassificacaoMaterial>>(classificacaoMaterialService.listarClassificacaoMaterialPorDescricaoECodigo(descricao, codigo));

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(resultResponseVH, this.getListView());

		return responseBody;
	}

	/**
	 * Método responsável por listar filhos de no selecionado
	 *
	 * @author wilker.machado
	 *
	 * @param objeto
	 * @return <code>ResponseBodyWrapper</code>
	 */
	@RequestMapping(method = RequestMethod.GET, value = "/listarFilhos")
	@ResponseBody
	public ResponseBodyWrapper listarFilhos(@RequestParam(value = "id") Long id) {

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(this.classificacaoMaterialService.listarFilhos(id), getListView());

		return responseBody;

	}

	/**
	 * Método responsável por salvar a entidade<code>FederalSupply</code>
	 *
	 * @author rogerio.costa
	 *
	 * @param objeto
	 *
	 * @throws Exception
	 */
	@RequestMapping(method = RequestMethod.POST, value = "/salvarClassificacaoMaterialTipoMaterial")
	@ResponseBody
	public ResponseBodyWrapper salvarClassificacaoMaterialTipoMaterial(@RequestBody ClassificacaoMaterial objeto) {

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(this.classificacaoMaterialService.salvarClassificacaoMaterialTipoMaterial(objeto), getEditView());

		return responseBody;

	}

	/**
	 * Método responsável por salvar a entidade<code>FederalSupply</code>
	 *
	 * @author rogerio.costa
	 *
	 * @param objeto
	 *
	 * @throws Exception
	 */
	@RequestMapping(method = RequestMethod.POST, value = "/salvarClassificacaoMaterialDetalhe")
	@ResponseBody
	public ResponseBodyWrapper salvarClassificacaoMaterialDetalhe(@RequestBody ClassificacaoMaterial objeto) {

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(this.classificacaoMaterialService.salvarClassificacaoMaterialDetalhe(objeto), getEditView());

		return responseBody;

	}

	@RequestMapping(method = RequestMethod.POST, value = "/findClassificacaoMaterialParent", produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findClassificacaoMaterialParent(@RequestBody ClassificacaoMaterial objeto) {

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(this.classificacaoMaterialService.getReference(objeto.getId()), this.getListView());

		return responseBody;

	}

	@RequestMapping(value = "/listarClassificacaoMaterialFilho", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper listarClassificacaoMaterialFilho(@RequestParam(value = "descricao") String descricao, @RequestParam(value = "id") Long id) {

		ResultResponseVH<List<ClassificacaoMaterial>> resultResponseVH = new ResultResponseVH<List<ClassificacaoMaterial>>(classificacaoMaterialService.listarClassificacaoMaterialPorDescricaoECodigo(descricao, id));

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(resultResponseVH, this.getListView());

		return responseBody;
	}

	@RequestMapping(value = "/listarFilhosPorDescricaoIdParent", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper listarFilhosPorDescricaoIdParent(@RequestParam(value = "descricao") String descricao, @RequestParam(value = "id") Long id) {

		ResultResponseVH<Collection<ClassificacaoMaterial>> resultResponseVH = new ResultResponseVH<Collection<ClassificacaoMaterial>>(classificacaoMaterialService.listarFilhosPorDescricaoIdParent(id, descricao));

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(resultResponseVH, this.getListView());

		return responseBody;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Class<Views.ClassificacaoMaterialListView> getListView() {

		return Views.ClassificacaoMaterialListView.class;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Class<Views.ClassificacaoMaterialEditView> getEditView() {

		return Views.ClassificacaoMaterialEditView.class;
	}
}
