package br.com.centralit.controller;

import java.util.Collection;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import br.com.centralit.api.model.ContaContabil;
import br.com.centralit.api.model.MyMensagemRetorno;
import br.com.centralit.api.model.MyRetornoStatus;
import br.com.centralit.api.service.ContaContabilService;
import br.com.centralit.api.service.OrganizacaoService;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.json.ResponseBodyWrapper;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.Organizacao;
import br.com.centralit.framework.view.ResultResponseVH;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 29/12/2014 - 17:37:26
 *
 * @version 1.0.0
 *
 * @author rogerio.costa
 *
 */
@Controller
@RequestMapping("/rest/contaContabil")
public class ContaContabilController extends GenericController<ContaContabil> {

	/** Atributo contaContabilService. */
	private ContaContabilService contaContabilService;
	
	@Autowired
	private OrganizacaoService organizacaoService;

	public ContaContabilController() {
		super();
	}

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 *
	 * @param contabilService
	 */
	@Autowired
	public ContaContabilController( ContaContabilService contabilService ) {

		super(contabilService);

		this.contaContabilService = contabilService;

	}

	/**
	 * Método responsável por listar a entidade<code>ContaContabil</code> por codigo ou descrição a partir da chave passada
	 *
	 * @author rogerio.costa
	 *
	 * @return Collection<Regiao>
	 */
	@RequestMapping(value = "/findContaContabil", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findContaContabil(@RequestParam(value = "chave") String chave) {

		Collection<ContaContabil> listaContaContabil = this.getContaContabilService().findContaContabil(chave);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(listaContaContabil, Views.ContaContabilAutoCompleteView.class);

		return responseBody;
	}

	/**
	 * Método responsável por listar a entidade<code>ContaContabil</code> por código e órgão do
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param codigo
	 *
	 * @param idOrganizacao
	 *
	 * @return Collection<Regiao>
	 */
	@RequestMapping(value = "/findContaContabilPorOrganizacao", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findContaContabilPorOrganizacao(@RequestParam(value = "codigo") String codigo, @RequestParam(value = "idOrganizacao") Long idOrganizacao) {

		Collection<ContaContabil> listaContaContabil = this.getContaContabilService().findContaContabilPorOrganizacao(codigo, idOrganizacao);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(listaContaContabil, Views.ContaContabilAutoCompleteView.class);

		return responseBody;
	}

	/**
	 * Método responsável por listar a entidade<code>ContaContabil</code> por códig, órgão, domínio
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param codigo
	 *
	 * @param idOrganizacao
	 *
	 * @param idDominio
	 *
	 * @return Collection<Regiao>
	 */
	@RequestMapping(value = "/findContaContabilPorOrganizacaoEDominio", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findContaContabilPorOrganizacaoEDominio(@RequestParam(value = "codigo") String codigo, @RequestParam(value = "idOrganizacao") Long idOrganizacao, @RequestParam(value = "idDominio") Long idDominio) {

		Collection<ContaContabil> listaContaContabil = this.getContaContabilService().findContaContabilPorOrganizacaoEDominio(codigo, idOrganizacao, idDominio);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(listaContaContabil, Views.ContaContabilAutoCompleteView.class);

		return responseBody;
	}

	/**
	 * Método responsável por listar a entidade<code>ContaContabil</code> por código, órgão, domínio que não esteja em uso em algum material
	 *
	 * @author rogerio.cassimiro
	 *
	 * @param codigo
	 *
	 * @param idOrganizacao
	 *
	 * @param idDominio
	 *
	 * @return Collection<Regiao>
	 */
	@RequestMapping(value = "/findContaContabilPorOrganizacaoEDominioSemUsoMaterial", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findContaContabilPorOrganizacaoEDominioSemUsoMaterial(@RequestParam(value = "codigo") String codigo, @RequestParam(value = "idOrganizacao") Long idOrganizacao, @RequestParam(value = "idDominio") Long idDominio) {

		Collection<ContaContabil> listaContaContabil = this.getContaContabilService().findContaContabilPorOrganizacaoEDominioSemUsoMaterial(codigo, idOrganizacao, idDominio);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(listaContaContabil, Views.ContaContabilAutoCompleteView.class);

		return responseBody;
	}

	/**
	 * Método responsável por listar a entidade<code>ContaContabil</code> por código, órgão, domínio que não esteja em uso em algum almoxarifado
	 *
	 * @author geovane.filho
	 *
	 * @param codigo
	 *
	 * @param idOrganizacao
	 *
	 * @param idDominio
	 *
	 * @return Collection
	 */
	@RequestMapping(value = "/findContaContabilPorOrganizacaoEDominioSemUsoAlmoxarifado", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findContaContabilPorOrganizacaoEDominioSemUsoAlmoxarifado(@RequestParam(value = "codigo") String codigo, @RequestParam(value = "idOrganizacao") Long idOrganizacao, @RequestParam(value = "idDominio") Long idDominio) {

		Collection<ContaContabil> listaContaContabil = this.getContaContabilService().findContaContabilPorOrganizacaoEDominioSemUsoAlmoxarifado(codigo, idOrganizacao, idDominio);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(listaContaContabil, Views.ContaContabilAutoCompleteView.class);

		return responseBody;
	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por
	 *
	 * @author rogerio.costa
	 *
	 * @param codigo
	 * @param idOrganizacao
	 * @param idTipoMaterial
	 * @return
	 */
	@RequestMapping(value = "/findContaContabilPorOrganizacaoTipoMaterial", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findContaContabilPorOrganizacaoTipoMaterial(@RequestParam(value = "codigo") String codigo, @RequestParam(value = "idOrganizacao") Long idOrganizacao,@RequestParam(value = "codigoTipoMaterial") Long codigoTipoMaterial) {

		Collection<ContaContabil> listaContaContabil = this.getContaContabilService().findContaContabilPorOrganizacaoTipoMaterial(codigo, idOrganizacao, codigoTipoMaterial);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(listaContaContabil, Views.ContaContabilAutoCompleteView.class);

		return responseBody;
	}

	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por obter a conta contábil definida na configuracaoparametrosistema
	 *
	 * @author rogerio.cassimiro
	 *
	 * @return ContaContabil
	 */
	@RequestMapping(value = "/obterContaContabilAlmoxarifadoConfiguracaoParametroSistema", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper obterContaContabilAlmoxarifadoConfiguracaoParametroSistema() {

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(this.getContaContabilService().obterContaContabilAlmoxarifadoConfiguracaoParametroSistema(), Views.ContaContabilAutoCompleteView.class);

		return responseBody;
	}

	/**
	 * Retorna o valor do atributo <code>contaContabilService</code>
	 *
	 * @return <code>ContaContabilService</code>
	 */
	public ContaContabilService getContaContabilService() {

		return contaContabilService;
	}

	@Override
	public Class<Views.ContaContabilListView> getListView() {

		return Views.ContaContabilListView.class;

	}

	@Override
	public Class<Views.ContaContabilEditView> getEditView() {

		return Views.ContaContabilEditView.class;

	}


	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por obter o valor total da conta contábil
	 *
	 * @author gilberto.nery
	 *
	 * @return ContaContabil
	 */
	@RequestMapping(value = "/obterValorTotal", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper getValorTotal(@RequestParam(value = "contaId") Long contaId) {

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(this.getContaContabilService().getValorAtual(contaId), Views.ContaContabilAutoCompleteView.class);

		return responseBody;
	}
	
	/**
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
	 *
	 * Método responsável por obter a conta contábil por inventário
	 *
	 * @author rogerio.cassimiro
	 *
	 * @return ContaContabil
	 */
	@RequestMapping(value = "/findContaContabilPorInventario", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findContaContabilPorInventario(@RequestParam(value = "busca") String busca, @RequestParam(value = "idInventario") Long idInventario) {

		ResultResponseVH<List<ContaContabil>> resultResponseVH = new ResultResponseVH<List<ContaContabil>>(this.contaContabilService.findContaContabilPorInventario(busca, idInventario));

		ResponseBodyWrapper responseBodyWrapper = new ResponseBodyWrapper(resultResponseVH, Views.ContaContabilAutoCompleteView.class);
		
		return responseBodyWrapper;
	}

	/**
	 * 
	 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
	 *
	 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>	
	 *
	 * Método responsável por trazer a conta contabil referenciada em um determinado material
	 *
	 * @author geovane.filho
	 *
	 * @param materialId id do material
	 * @return
	 */
	@RequestMapping(value = "/findContaContabilPorMaterial", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findContaContabilPorMaterial(@RequestParam(value = "materialId") Long materialId) {

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(this.getContaContabilService().findByMaterial(materialId), Views.ContaContabilEditView.class);

		return responseBody;
	}
	
	@RequestMapping(method = RequestMethod.POST, value = "")
	@ResponseBody
	public ResponseBodyWrapper save(@RequestBody ContaContabil entity) throws Exception {

		Organizacao organizacao = this.organizacaoService.find(this.getUsuario().getOrganizacao().getId());

		if (!organizacao.getEmFechamento()) {

			ResponseBodyWrapper responseBody = new ResponseBodyWrapper(this.contaContabilService.save(entity), getEditView());

			return responseBody;
			
		} else {
			MyRetornoStatus retorno = new MyRetornoStatus();
			retorno.data = entity;
			retorno.mensagens.add(new MyMensagemRetorno(MyMensagemRetorno.TipoMensagemRetorno.warning, "MSG.ORGANIZACAO_EM_FECHAMENTO_MES"));

			ResponseBodyWrapper responseBody = new ResponseBodyWrapper(retorno, getEditView());

			return responseBody;
		}
	}
	
	@RequestMapping(method = RequestMethod.POST, value = "/update")
	@ResponseBody
	public ResponseBodyWrapper update(@RequestBody ContaContabil objeto) {
		Organizacao organizacao = this.organizacaoService.find(this.getUsuario().getOrganizacao().getId());

		if (!organizacao.getEmFechamento()) {
			ResponseBodyWrapper responseBody = new ResponseBodyWrapper(this.contaContabilService.merge(objeto), getEditView());

			return responseBody;
		} else {
			MyRetornoStatus retorno = new MyRetornoStatus();
			retorno.data = objeto;
			retorno.mensagens.add(new MyMensagemRetorno(MyMensagemRetorno.TipoMensagemRetorno.warning, "MSG.ORGANIZACAO_EM_FECHAMENTO_MES"));

			ResponseBodyWrapper responseBody = new ResponseBodyWrapper(retorno, getEditView());

			return responseBody;
		}
	}
	
	@RequestMapping(method = RequestMethod.DELETE, value = "/{id}")
	@ResponseBody
	public ResponseBodyWrapper delete(@PathVariable("id") Long id) {
		
		Organizacao organizacao = this.organizacaoService.find(this.getUsuario().getOrganizacao().getId());

		if (!organizacao.getEmFechamento()) {
			ResponseBodyWrapper responseBody = new ResponseBodyWrapper(contaContabilService.removeById(id), getEditView());

			return responseBody;
		} else {
			MyRetornoStatus retorno = new MyRetornoStatus();
			retorno.data = id;
			retorno.mensagens.add(new MyMensagemRetorno(MyMensagemRetorno.TipoMensagemRetorno.warning, "MSG.ORGANIZACAO_EM_FECHAMENTO_MES"));

			ResponseBodyWrapper responseBody = new ResponseBodyWrapper(retorno, getEditView());

			return responseBody;
		}
	}
}