package br.com.centralit.controller;

import java.util.Calendar;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import br.com.centralit.api.model.ContaContabilMovimento;
import br.com.centralit.api.service.ContaContabilMovimentoService;
import br.com.centralit.api.service.ContaContabilSaldoService;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.OrganizacaoService;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.json.ResponseBodyWrapper;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.model.Organizacao;
import br.com.centralit.framework.util.UtilDate;


/**
 * <p><img src="http://centralit.com.br/images/logo_central.png"></p>
 *
 * <p><b>Company: </b> Central IT - Governança Corporativa - </p>
 *
 * <p><b>Title: </b></p>
 *
 * <p><b>Description: </b></p>
 *
 * <p><b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a></p>
 *
 * <p><b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a></p>
 *
 * @since 15/01/2015 - 18:11:02
 *
 * @version 1.0.0
 *
 * @author iago.almeida
 *
 */
@Controller
@RequestMapping("/rest/contaContabilMovimento")
public class ContaContabilMovimentoController extends GenericController<ContaContabilMovimento> {

	private ContaContabilMovimentoService contaContabilMovimentoService;

	@Autowired
	private ContaContabilSaldoService contaContabilSaldoService;

	@Autowired
	private OrganizacaoService organizacaoService;

	@Autowired
	private DominioService dominioService;

	public ContaContabilMovimentoController() {
		super();
	}

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * @param contaContabilMovimentoService
	 */
	@Autowired
	public ContaContabilMovimentoController(ContaContabilMovimentoService contaContabilMovimentoService) {
		super(contaContabilMovimentoService);
		this.contaContabilMovimentoService = contaContabilMovimentoService;
	}


	@Override
	public Class<Views.ContaContabilMovimentoListView> getListView(){

		return Views.ContaContabilMovimentoListView.class;

	}

	@Override
	public Class<Views.ContaContabilMovimentoEditView> getEditView(){

		return Views.ContaContabilMovimentoEditView.class;

	}



	/**
	 * Retorna o valor do atributo <code>contaContabilMovimentoService</code>
	 *
	 * @return <code>ContaContabilMovimentoService</code>
	 */
	public ContaContabilMovimentoService getContaContabilMovimentoService() {

		return contaContabilMovimentoService;
	}



	/**
	 * Define o valor do atributo <code>contaContabilMovimentoService</code>.
	 *
	 * @param contaContabilMovimentoService
	 */
	public void setContaContabilMovimentoService(ContaContabilMovimentoService contaContabilMovimentoService) {

		this.contaContabilMovimentoService = contaContabilMovimentoService;
	}

	/**
	 *
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por verificar se existe saldo fechado para a referência e a organização definida.
	 *
	 * @author geovane.filho
	 *
	 * @param request
	 *            Pedido de verificação de existencia de saldo contendo a organização e a data de referencia.
	 *
	 * @return <code>ResponseBodyWrapper</code> contendo <code>true</code> caso exista saldo para a referencia na organização e <code>false</code> caso contrário.
	 */
	@RequestMapping(value = "/existeSaldoReferencia", method = RequestMethod.POST)
	@ResponseBody
	public ResponseBodyWrapper existeSaldoReferencia(@RequestBody Map<String, String> request) {

		Organizacao organizacao = this.organizacaoService.find(Long.parseLong(request.get("organizacao")));
		String dataReferenciaStr = request.get("dataReferencia");

		Boolean existe = this.contaContabilSaldoService.existeSaldoOrganizacaoEDataReferencia(organizacao, UtilDate.getCalendarDaData(UtilDate.stringReferenciaToDate(dataReferenciaStr)));

		request.put("existe", existe.toString());

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(request, this.getEditView());

		return responseBody;
	}

	/**
	 * Método responsável por
	 *
	 * @author wilker.machado
	 *
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/existeMovimentacaoReferencia", method = RequestMethod.POST)
	@ResponseBody
	public ResponseBodyWrapper existeMovimentacaoReferencia(@RequestBody Map<String, String> request) {

		Organizacao organizacao = this.organizacaoService.find(Long.parseLong(request.get("organizacao")));
		String tipoMaterialNome = request.get("tipoMaterial");

		Dominio tipoMaterial = this.dominioService.findByChaveAndNome("tipoMaterial", tipoMaterialNome);

		Calendar refInicial = UtilDate.getCalendarDaData(UtilDate.stringReferenciaToDate(request.get("dataReferenciaInicial")));
		Calendar refFinal = UtilDate.calendarUltimoDiaDoMesUltimaHora(UtilDate.getCalendarDaData(UtilDate.stringReferenciaToDate(request.get("dataReferenciaFinal"))));

		Boolean existe = this.contaContabilMovimentoService.existeMovimentoOrganizacaoDataReferenciaTipoMaterial(organizacao, refInicial, refFinal, tipoMaterial.getId());

		request.put("existe", existe.toString());

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(request, this.getEditView());

		return responseBody;
	}
}
