package br.com.centralit.controller;

import java.math.BigDecimal;
import java.util.List;

import org.apache.commons.lang3.SerializationUtils;
import org.hibernate.exception.ConstraintViolationException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import br.com.centralit.api.framework.json.ViewsAdmMateriais;
import br.com.centralit.api.model.MaterialConsumo;
import br.com.centralit.api.service.MaterialConsumoService;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.json.ResponseBodyWrapper;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.view.ResultResponseVH;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 *
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 *
 * @since 25/05/2015 - 10:29:07
 *
 * @version 1.0.0
 *
 * @author rogerio.costa
 *
 */
@Controller
@RequestMapping("/rest/materialConsumo")
public class MaterialConsumoController extends GenericController<MaterialConsumo> {

	/** Atributo materialConsumoService. */
	private MaterialConsumoService materialConsumoService;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 *
	 * @param materialConsumoService
	 */
	@Autowired
	public MaterialConsumoController( MaterialConsumoService materialConsumoService ) {

		super(materialConsumoService);

		this.materialConsumoService = materialConsumoService;

	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por trazer o valor atual total de um determinado material de consumo
	 *
	 * @param materialId
	 *            Material a se buscar o valor atual total.
	 * @param organizacaoId
	 * @return
	 */
	@RequestMapping(value = "/valorAtualMaterial", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public BigDecimal getValorAtualMaterial(@RequestParam(value = "materialId") Long materialId, @RequestParam(value = "idOrganizacao") Long organizacaoId) {

		return this.materialConsumoService.getValorAtualMaterial(materialId, organizacaoId);
	}
	
	@RequestMapping(method = RequestMethod.POST, value = "")
	@ResponseBody
	public ResponseBodyWrapper save(@RequestBody MaterialConsumo entity) throws Exception{
		boolean uniqueConstraintViolate = true;
		
		MaterialConsumo materialConsumo = new MaterialConsumo();
		
		//TODO Solução temporária para evitar que sejam criados registros com o código duplicado. Alterar solução quando for implementado o cache compartilhado
		for(int i = 5; i > 0 && uniqueConstraintViolate; i--){
			try{
				materialConsumo = this.materialConsumoService.save(SerializationUtils.clone(entity));
				uniqueConstraintViolate = false;
			}catch(Throwable e){
				if(e.getCause() instanceof ConstraintViolationException){
					ConstraintViolationException ex = (ConstraintViolationException) e.getCause();
					if(ex.getSQLException().getMessage().contains("codigo_unico")){
						uniqueConstraintViolate = true;
					}else{
						throw e;
					}
				}else{
					throw e;
				}
			}
		}
		
		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(materialConsumo, getEditView());
		
		return responseBody;
	}

	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por trazer a quantidade atual de um determinado material de consumo
	 *
	 * @param materialId Material a se buscar a quantidade atual
	 * @param organizacaoId
	 * @return
	 */
	@RequestMapping(value = "/quantidadeAtualMaterial", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public BigDecimal getQtdAtualMaterial(@RequestParam(value = "materialId") Long materialId, @RequestParam(value = "idOrganizacao") Long organizacaoId) {
		return this.materialConsumoService.getQuantidadeAtualMaterial(materialId, organizacaoId);
	}

	@RequestMapping(value = "/findMateriaisConsumoByOrganizacao", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findMateriaisConsumoByOrganizacao(@RequestParam(value = "descricao") String descricao, @RequestParam(value = "idOrganizacao") Long idOrganizacao) {

		ResultResponseVH<List<MaterialConsumo>> resultResponseVH = new ResultResponseVH<List<MaterialConsumo>>(materialConsumoService.findMateriaisConsumoByOrganizacao(descricao, idOrganizacao));

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(resultResponseVH, ViewsAdmMateriais.MaterialConsumoAutoCompleteView.class);

		return responseBody;

	}


	@RequestMapping(value = "/findMateriaisConsumoInfo", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findMateriaisConsumoInfo(@RequestParam(value = "descricao") String descricao, @RequestParam(value = "idOrganizacao") Long idOrganizacao) {

		ResultResponseVH<List<MaterialConsumo>> resultResponseVH = new ResultResponseVH<List<MaterialConsumo>>(materialConsumoService.findMateriaisConsumoByOrganizacao(descricao, idOrganizacao));

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(resultResponseVH, Views.MaterialConsultaInfo.class);

		return responseBody;

	}

	@Override
	public Class<Views.MaterialEditView> getEditView() {

		return Views.MaterialEditView.class;
	}
}
