package br.com.centralit.controller;

import java.util.Collection;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import br.com.centralit.api.framework.json.ViewsAdmMateriais;
import br.com.centralit.api.model.Material;
import br.com.centralit.api.model.MaterialPermanente;
import br.com.centralit.api.service.MaterialImagemService;
import br.com.centralit.api.service.MaterialService;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.json.ResponseBodyWrapper;
import br.com.centralit.framework.json.Views;
import br.com.centralit.framework.model.Dominio;
import br.com.centralit.framework.util.UtilObjeto;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 *
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 *
 * <p>
 * <b>Title: </b>
 * </p>
 *
 * <p>
 * <b>Description: </b>
 * </p>
 *
 * @since 15/12/2014 - 09:38:48
 *
 * @version 1.0.0
 *
 * @author rogerio.costa
 *
 */
@Controller
@RequestMapping("/rest/material")
public class MaterialController extends GenericController<Material> {

	/** Atributo materialService. */
	private MaterialService materialService;

	/** Atributo materialImagemService. */
	@Autowired
	private MaterialImagemService materialImagemService;

	public MaterialController() {
		super();
	}

	@Autowired
	public MaterialController( MaterialService materialService ) {

		super(materialService);

		this.materialService = materialService;

	}

	/**
	 * Método responsável por listar a entidade<code>Material</code>
	 *
	 * @author rogerio.costa
	 *
	 * @return Collection<Regiao>
	 */
	@RequestMapping(value = "/findMaterialConsumo", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper listarMaterialConsumo(@RequestParam(value = "descricao") String descricao) {

		Collection<Material> listaMaterial = this.getMaterialService().listarMaterial(descricao, Dominio.TIPO_MATERIAL_CONSUMO);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(listaMaterial, Views.MaterialEditView.class);

		return responseBody;
	}

	/**
	 * Método responsável por listar a entidade<code>Pais</code>
	 *
	 * @author rogerio.costa
	 *
	 * @return Collection<Regiao>
	 */
	@RequestMapping(value = "/findMaterial", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper listarMaterial(@RequestParam(value = "descricao") String descricao) {

		Collection<Material> listaMaterial = this.getMaterialService().listarMaterial(descricao, null);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(listaMaterial, Views.MaterialAutoCompleteView.class);

		return responseBody;
	}

	/**
	 * Método responsável por listar a entidade<code>Material</code> Permanente
	 *
	 * @author wilker.machado
	 *
	 * @return Collection<Regiao>
	 */
	@RequestMapping(value = "/findMaterialPermanente", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findMaterialPermanente(@RequestParam(value = "descricao") String descricao) {

		Collection<Material> listaMaterial = this.getMaterialService().listarMaterial(descricao, Dominio.TIPO_MATERIAL_PERMANENTE);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(listaMaterial, Views.MaterialAutoCompleteView.class);

		return responseBody;
	}

	@RequestMapping(value = "/findPorIdClassificacaoMaterial", method = RequestMethod.GET)
	@ResponseBody
	public ResponseBodyWrapper obterPorIdClassificacaoMaterial(@RequestParam("idClassificacaoMaterial") Long idClassificacaoMaterial) {

		Material material = this.getMaterialService().obterPorIdClassificacaoMaterial(idClassificacaoMaterial);

		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(UtilObjeto.isReferencia(material) ? material : new MaterialPermanente(), this.getEditView());

		return responseBody;
	}
	
	@RequestMapping(value = "/getMaterialTransferencia", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper getMaterialTransferencia(@RequestParam(value = "id") Long id) {

		Material material = this.getMaterialService().find(id);
		
		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(material, ViewsAdmMateriais.MaterialTransferenciaView.class);

		return responseBody;
	}

	@RequestMapping(value = "/findMateriaisByOrganizacao", method = RequestMethod.GET, produces = "application/json")
	@ResponseBody
	public ResponseBodyWrapper findMateriaisByOrganizacao(@RequestParam(value = "codigoOuDescricao") String codigoOuDescricao, @RequestParam(value = "idOrganizacao") Long idOrganizacao) {
		
		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(getMaterialService().findMateriaisByOrganizacao(codigoOuDescricao, idOrganizacao), getAutoCompleteView());
		
		return responseBody;
	}

	/**
	 * Retorna o valor do atributo <code>materialService</code>
	 *
	 * @return <code>MaterialService</code>
	 */
	public MaterialService getMaterialService() {

		return materialService;
	}

	/**
	 * Retorna o valor do atributo <code>materialImagemService</code>
	 *
	 * @return <code>MaterialImagemService</code>
	 */
	public MaterialImagemService getMaterialImagemService() {

		return materialImagemService;
	}

	@Override
	public Class<Views.MaterialEditView> getEditView() {

		return Views.MaterialEditView.class;
	}
}
