package br.com.centralit.controller;

import java.io.IOException;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import br.com.centralit.api.model.MaterialImagem;
import br.com.centralit.api.service.DominioService;
import br.com.centralit.api.service.MaterialImagemService;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.model.Dominio;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 22/12/2014 - 15:36:32
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
@Controller
@RequestMapping("/rest/materialImagem")
public class MaterialImagemController extends GenericController<MaterialImagem> {

	/** Atributo materialImagemService. */
	private MaterialImagemService materialImagemService;

	@Autowired
	private DominioService dominioService;

	public MaterialImagemController() {

		super();
	}

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * 
	 * @param materialImagemService
	 */
	@Autowired
	private MaterialImagemController( MaterialImagemService materialImagemService ) {

		super(materialImagemService);

		this.materialImagemService = materialImagemService;

	}

	@RequestMapping(value = "/uploadMultipleFile", method = RequestMethod.POST)
	public @ResponseBody
	void uploadMultipleFileHandler(@RequestParam(value = "filename", required = false) String name, @RequestParam("file") MultipartFile file, @RequestParam(value = "idMaterial", required = false) Long idMaterial) {

		this.materialImagemService.save(file, idMaterial);

	}

	@RequestMapping(value = "/downloadImagem", method = RequestMethod.GET)
	public @ResponseBody
	void downloadImagem(@RequestParam(value = "idMaterialImagem") Long idMaterialImagem, HttpServletResponse response) throws IOException {

		MaterialImagem materialImagem = this.materialImagemService.find(idMaterialImagem);

		response.setContentType("application/octet-stream");

		response.setHeader("Content-Disposition", "attachment;filename=" + materialImagem.getDescricao());

		response.getOutputStream().write(materialImagem.getAnexo());
	}

	@RequestMapping(value = "/visualizarImagem", method = RequestMethod.GET)
	public @ResponseBody
	ResponseEntity<byte[]> visualizarImagem(@RequestParam(value = "idMaterialImagem") Long idMaterialImagem, HttpServletResponse response) throws IOException {

		MaterialImagem materialImagem = this.materialImagemService.find(idMaterialImagem);

		String extensao = materialImagem.getDescricao().contains(".") ? materialImagem.getDescricao().split("\\.")[1] : "";

		String mimeType = "";
		
		for (Dominio dominio : this.dominioService.listarPorChave(Dominio.TIPO_ANEXO)) {

			if (dominio.getNome().equalsIgnoreCase(extensao.trim())) {

				// mimeType do arquivo esta relacionado com o dominio via sql
				mimeType = dominio.getDescricao();

				break;
			}
		}

		HttpHeaders headers = new HttpHeaders();
		
		headers.add("content-disposition", "inline;filename=" + materialImagem.getDescricao());
		
		headers.setContentType(MediaType.parseMediaType(mimeType));
		
		ResponseEntity<byte[]> resp = new ResponseEntity<byte[]>(materialImagem.getAnexo(), headers, HttpStatus.OK);
		
		return resp;

	}

}
