package br.com.centralit.controller;

import org.apache.commons.lang3.SerializationUtils;
import org.hibernate.exception.ConstraintViolationException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import br.com.centralit.api.model.MaterialPermanente;
import br.com.centralit.api.service.MaterialPermanenteService;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.json.ResponseBodyWrapper;
import br.com.centralit.framework.json.Views;

/**
 * <p>
 * <img src="http://centralit.com.br/images/logo_central.png">
 * </p>
 * 
 * <p>
 * <b>Company: </b> Central IT - Governança Corporativa -
 * </p>
 * 
 * <p>
 * <b>Title: </b>
 * </p>
 * 
 * <p>
 * <b>Description: </b>
 * </p>
 * 
 * <p>
 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
 * </p>
 * 
 * <p>
 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
 * </p>
 * 
 * @since 28/05/2015 - 08:59:13
 * 
 * @version 1.0.0
 * 
 * @author rogerio.costa
 * 
 */
@Controller
@RequestMapping("/rest/materialPermanente")
public class MaterialPermanenteController extends GenericController<MaterialPermanente> {

	/** Atributo materialPermanenteService. */
	private MaterialPermanenteService materialPermanenteService;

	/**
	 * Responsável pela criação de novas instâncias desta classe.
	 * 
	 * @param materialPermanenteService
	 */
	@Autowired
	public MaterialPermanenteController( MaterialPermanenteService materialPermanenteService ) {

		super(materialPermanenteService);

		this.materialPermanenteService = materialPermanenteService;

	}

	/**
	 * Retorna o valor do atributo <code>materialPermanenteService</code>
	 * 
	 * @return <code>MaterialPermanenteService</code>
	 */
	public MaterialPermanenteService getMaterialPermanenteService() {

		return materialPermanenteService;
	}
	

	@Override
	public Class<Views.MaterialEditView> getEditView() {

		return Views.MaterialEditView.class;
	}
	
	@RequestMapping(method = RequestMethod.POST, value = "")
	@ResponseBody
	public ResponseBodyWrapper save(@RequestBody MaterialPermanente entity) throws Exception{
		boolean uniqueConstraintViolate = true;
		
		MaterialPermanente materialPermanente = new MaterialPermanente();
		
		//TODO Solução temporária para evitar que sejam criados registros com o código duplicado. Alterar solução quando for implementado o cache compartilhado
		for(int i = 5; i > 0 && uniqueConstraintViolate; i--){
			try{
				materialPermanente = this.materialPermanenteService.save(SerializationUtils.clone(entity));
				uniqueConstraintViolate = false;
			}catch(Throwable e){
				if(e.getCause() instanceof ConstraintViolationException){
					ConstraintViolationException ex = (ConstraintViolationException) e.getCause();
					if(ex.getSQLException().getMessage().contains("codigo_unico")){
						uniqueConstraintViolate = true;
					}else{
						throw e;
					}
				}else{
					throw e;
				}
			}
		}
		
		ResponseBodyWrapper responseBody = new ResponseBodyWrapper(materialPermanente, getEditView());
		
		return responseBody;
	}

}
