package br.com.centralit.controller;

import org.apache.commons.lang3.SerializationUtils;
import org.hibernate.exception.ConstraintViolationException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import br.com.centralit.api.framework.json.ViewsAdmMateriais;
import br.com.centralit.api.model.MyMensagemRetorno;
import br.com.centralit.api.model.MyRetornoStatus;
import br.com.centralit.api.model.TransferenciaContaContabil;
import br.com.centralit.api.service.OrganizacaoService;
import br.com.centralit.api.service.TransferenciaContaContabilService;
import br.com.centralit.framework.controller.GenericController;
import br.com.centralit.framework.json.ResponseBodyWrapper;
import br.com.centralit.framework.model.Organizacao;

@Controller
@RequestMapping("/rest/transferenciaContaContabil")
public class TransferenciaContaContabilController extends GenericController<TransferenciaContaContabil>{

	@Autowired
	private TransferenciaContaContabilService transferenciaContaContabilService;
	
	@Autowired
	private OrganizacaoService organizacaoService;
	
	@Autowired
	public TransferenciaContaContabilController(TransferenciaContaContabilService transferenciaContaContabilService) {
		super(transferenciaContaContabilService);
	}
	
	@Override
	public Class<ViewsAdmMateriais.TransferenciaContaContabilListView> getListView() {

		return ViewsAdmMateriais.TransferenciaContaContabilListView.class;

	}

	@Override
	public Class<ViewsAdmMateriais.TransferenciaContaContabilEditView> getEditView() {

		return ViewsAdmMateriais.TransferenciaContaContabilEditView.class;

	}
	
	/**
	 * <p>
	 * <b>Iniciativa(s):</b> <a href="LINK_PORTAL">NUMERO_INICIATIVA</a>
	 * </p>
	 *
	 * <p>
	 * <b>Regra(s) de negócio:</b> <a href="LINK_PORTAL">NUMERO_REGRA_DE_NEGOCIO</a>
	 * </p>
	 *
	 * Método responsável por realizar uma transferencia entre contas contábeis.
	 *
	 * @author geovane.filho
	 *
	 * @param transferenciaConta
	 * @return ResponseBodyWrapper
	 */
	@RequestMapping(value = "/realizarTransferencia", method = RequestMethod.POST)
	@ResponseBody
	public ResponseBodyWrapper realizarTransferencia(@RequestBody TransferenciaContaContabil transferenciaConta) {
		Organizacao organizacao = this.organizacaoService.find(this.getUsuario().getOrganizacao().getId());
		Boolean transferido = false;
		boolean uniqueConstraintViolate = true;

		if (!organizacao.getEmFechamento()) {
			
			//TODO Solução temporária para evitar que sejam criados registros com o código duplicado. Alterar solução quando for implementado o cache compartilhado
			for(int i = 5; i > 0 && uniqueConstraintViolate; i--){
				try{
					if (transferenciaConta.getMaterial() != null) {
						transferido = this.transferenciaContaContabilService.realizaTransferenciaMaterial(SerializationUtils.clone(transferenciaConta));
					} else {
						transferido = this.transferenciaContaContabilService.realizaTransferenciaAlmoxarifado(SerializationUtils.clone(transferenciaConta));
					}
					uniqueConstraintViolate = false;
				}catch(Throwable e){
					if(e.getCause() instanceof ConstraintViolationException){
						ConstraintViolationException ex = (ConstraintViolationException) e.getCause();
						if(ex.getSQLException().getMessage().contains("codigo_unico")){
							uniqueConstraintViolate = true;
						}else{
							throw e;
						}
					}else{
						throw e;
					}
				}
			}
			
			ResponseBodyWrapper responseBody = new ResponseBodyWrapper(transferido, getEditView());

			return responseBody;
		} else {
			MyRetornoStatus retorno = new MyRetornoStatus();
			retorno.data = transferido;
			retorno.mensagens.add(new MyMensagemRetorno(MyMensagemRetorno.TipoMensagemRetorno.warning, "MSG.ORGANIZACAO_EM_FECHAMENTO_MES"));

			ResponseBodyWrapper responseBody = new ResponseBodyWrapper(retorno, getEditView());

			return responseBody;
		}
	}
}
