'use strict';

citApp.controller('ClassificacaoMaterialController', ['$scope', 'ClassificacaoMaterialRepository', 'DominioRepository', '$translate', '$timeout',
        function ClassificacaoMaterialController($scope, ClassificacaoMaterialRepository, DominioRepository, $translate, $timeout) {

	$scope.classificacaoMaterial = {};

	// prepara para adicionar uma Classifica;áo de material tipo GRUPO ou no RAIZ
	$scope.novoGrupoClassificacao = function(){

		$scope.setLoadingGet(true);

		DominioRepository.findAllDominioByCodigo('tipoClassificacaoMaterial', 0).then(function(result) {

			$scope.resetForm(true);

			$scope.classificacaoMaterial.dominioTipoClassificacaoMaterial = result.originalElement;

			$scope.setLoading(false);

			$scope.scrollToCadastro('#containerClassificacaoMaterialEdit');
		});

	};

	// inicializa o formulario passando por parametro se esta editando ou visualizando o formulario
	$scope.resetForm = function(edit) {
		$scope.classificacaoMaterial = {};
		// caso seja false o formulario vem desabilitado
		$scope.edit = edit;
		$scope.limparSelected();
		$timeout(function(){
			$scope.classificacaoMaterialForm.$submitted = false;
			$scope.classificacaoMaterialForm.$setPristine();
		});
	};

	// Seleciona a Classificação de material
	$scope.setClassificacaoMaterial = function(classificacaoMaterial){

		$scope.setLoadingGet(true);

		$scope.limparSelected();

		ClassificacaoMaterialRepository.get(classificacaoMaterial.id).then(function(result) {

			$scope.classificacaoMaterial = result.originalElement;

			$scope.edit = true;

			$scope.setLoading(false);

			$scope.scrollToCadastro('#containerClassificacaoMaterialEdit');
		});

	};

	// verifica se esta adicionando um novo grupo, caso esteja nao vai apresentar o grupo pai na tela, se for qualquer no filho esse metodo retornará true
	$scope.isVisualizarGrupo = function(){
		var result = false;
		angular.forEach($scope.tipoClassificacaoMaterial, function(tipoClassificacaoMaterial, key) {
			if($scope.classificacaoMaterial.dominioTipoClassificacaoMaterial && $scope.classificacaoMaterial.dominioTipoClassificacaoMaterial.id == tipoClassificacaoMaterial.id){
				result = tipoClassificacaoMaterial.nome != 'CLASSE';
				$scope.classificacaoMaterial.dominioTipoClassificacaoMaterial = tipoClassificacaoMaterial.originalElement;
			}
		});
		return result;
	};

	DominioRepository.findAllDominio('tipoClassificacaoMaterial').then(function(result) {
		$scope.tipoClassificacaoMaterial = result;
	});

	DominioRepository.findAllDominio('tipoMaterial').then(function(result) {
		$scope.tipoMaterial = result;
	});

	$scope.saveOrUpdate = function(){
		$scope.classificacaoMaterialForm.$submitted = true;
		// verifica campos obrigatorios
		if($scope.classificacaoMaterialForm.$valid){
			$scope.setLoadingSalva(true);
			
			if($scope.classificacaoMaterial.classificacaoMaterialParent && $scope.classificacaoMaterial.classificacaoMaterialParent.classificacaoMaterialFilhos)
				delete $scope.classificacaoMaterial.classificacaoMaterialParent.classificacaoMaterialFilhos;
			
			ClassificacaoMaterialRepository.save($scope.classificacaoMaterial).then(function(result) {
				$scope.setLoading(false);
				$scope.showAlert("success", $translate.instant('ADMINISTRACAODEMATERIAIS.MSG.SUCESSO_CLASSIFICACAO'));
				$scope.classificacaoMaterial = {};
				$scope.resetForm(false);
				$scope.fetchResult(false);
			});
		}else{
			$scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'));
		}
	};

	// -------------------- LISTAGEM -------------------
	$scope.novoFilhoMaterial = function(classificacaoMaterial){

		$scope.limparSelected();

		$scope.edit = true;

		$scope.novoNoFilho(classificacaoMaterial);

		$scope.buscarTree(classificacaoMaterial.descricao);

	};

	// metodo responsavel inicializar e selecionar a classificaçãoMaterialParent(Pai) para poder incluir um novo filho
	$scope.novoNoFilho = function(classificacaoMaterial){

		$scope.limparSelected();

		$scope.classificacaoMaterial = {};

		$scope.classificacaoMaterial.classificacaoMaterialParent = classificacaoMaterial.originalElement ? classificacaoMaterial.originalElement : classificacaoMaterial;

		$scope.setLoadingGet(true);

		DominioRepository.findAllDominioByCodigo('tipoClassificacaoMaterial', (classificacaoMaterial.dominioTipoClassificacaoMaterial.codigo + 1)).then(function(result) {

			$scope.classificacaoMaterial.dominioTipoClassificacaoMaterial = result.originalElement;

			$scope.edit = true;

			$scope.setLoading(false);

			$scope.scrollToCadastro('#containerClassificacaoMaterialEdit');
		});

		$scope.classificacaoMaterialForm.$submitted = false;
		$scope.classificacaoMaterialForm.$setPristine();
	};

	// metodo responsavel por listar os filhos na arvore
	$scope.mostrarFilhos = function(classificacaoMaterialParent){

		// verificação para mostrar filhos >  caso seja false ira esconder os filhos e colocar o sinal de +
		if(classificacaoMaterialParent.mostrarFilhos){

			classificacaoMaterialParent.mostrarFilhos = false;

		}else{

			if (classificacaoMaterialParent.dominioTipoClassificacaoMaterial.codigo === 4){

				$scope.setClassificacaoMaterial(classificacaoMaterialParent);

			} else {

				var offsetWindow = angular.element(window).scrollTop();

				$scope.setLoadingPesquisa(true);

				// busca via rest os filhos do no selecionado
				ClassificacaoMaterialRepository.listarFilhos(classificacaoMaterialParent.originalElement ? classificacaoMaterialParent.originalElement.id : classificacaoMaterialParent.id).then(function(result) {

					classificacaoMaterialParent.classificacaoMaterialFilhos = result;

					classificacaoMaterialParent.mostrarFilhos = true;

					$scope.setLoading(false);

					$scope.scrollToCadastro('#containerClassificacaoMaterialEdit', offsetWindow);
				});

			}

		}

	};

	// recarregar a arvore de listagem
	$scope.fetchResult = function(exclusaoClassificacao) {

		if(exclusaoClassificacao){

			$scope.setLoadingAtualiza(true);

		} else {

			$scope.setLoadingPesquisa(true);
		}


		return ClassificacaoMaterialRepository.listarGrupos().then(function(result) {
			$scope.classificacaoMaterialList = result;

			$scope.setLoading(false);

		}, function() {
			$scope.classificacaoMaterialList = [];

		});
	};

	// metodo responsavel por preparar e chamar o dialog de confirmação para uma classificação de material
	$scope.remove = function(classificacaoMaterial){
		$scope.classificacaoMaterial = classificacaoMaterial;

		$scope.$openModalConfirm({message: $translate.instant('MSG.DESEJA_EXCLUIR_ITENS'), callback: $scope.excluir, item: $scope.classificacaoMaterial});
	};

	// metodo para remover uma classificação de material
	$scope.excluir = function(){

		var item = this.item;

		$scope.setLoadingPesquisa(true);
		ClassificacaoMaterialRepository.remove(item).then(function(result) {
			$scope.resetForm(false);
			// verifica de foi excluido com sucesso
			if(result){
				$scope.showAlert("success", $translate.instant('ADMINISTRACAODEMATERIAIS.MSG.EXCLUSAO_CLASSIFICASSAO_MATERIAL'));
			}
			$scope.setLoading(false);
			$scope.$modalConfirmInstance.dismiss('cancel');
			$scope.fetchResult(true);
		});

		$scope.$modalConfirmInstance.dismiss('cancel');

	};

	// metodo para buscar qualquer elemento na arvore de acordo com que o usuario digita (obs: acima de 2 caracteres)
	$scope.buscarTree = function(buscaFederalDescricao){
		
		//Deve remover caracteres especiais
		$scope.buscaFederalSupply = $scope.removerCaracteresEspeciais($scope.buscaFederalSupply);
		buscaFederalDescricao = $scope.buscaFederalSupply;
		
		// verifica se o usuario já digitou mais q dois caracteres
		if(buscaFederalDescricao && buscaFederalDescricao.length > 2) {

			$scope.setLoadingPesquisa(true);

			return ClassificacaoMaterialRepository.listarGruposTree(buscaFederalDescricao).then(function(result) {

				$scope.classificacaoMaterialList = result;

				$scope.abrirNosArvore();

				$scope.setLoading(false);

			}, function() {

				$scope.classificacaoMaterialList = [];

			});

		}else{
			$scope.fetchResult(false);
			$scope.setLoading(false);
		}
	};

	// metodo responsavl por abrir todos os nos da arvore para mostrar os filhos
	$scope.abrirNosArvore = function(){

		// percorre os nos CLASSIFICADORES - PERMANENTE OU CONSUMO
		angular.forEach($scope.classificacaoMaterialList, function(classificacao, key) {

			if(classificacao.classificacaoMaterialFilhos){

				classificacao.mostrarFilhos = true;

			} else {

				classificacao.mostrarFilhos = false;

			}

			// percorre os nos RAIZ - GRUPOS
			angular.forEach(classificacao.classificacaoMaterialFilhos, function(grupo, key) {

				if(grupo.classificacaoMaterialFilhos){

					grupo.mostrarFilhos = false;

				} else {

					grupo.mostrarFilhos = true;

				}

				// percorre os SUB-GRUPOS
				angular.forEach(grupo.classificacaoMaterialFilhos, function(subGrupo, key) {

					if(subGrupo.classificacaoMaterialFilhos){

						subGrupo.mostrarFilhos = false;

						grupo.mostrarFilhos = true;

					} else {

						subGrupo.mostrarFilhos = true;

					}

					// percorre os MATERIAIS
					angular.forEach(subGrupo.classificacaoMaterialFilhos, function(material, key) {

						if(material.classificacaoMaterialFilhos){

							material.mostrarFilhos = false;

							subGrupo.mostrarFilhos = true;

						} else {

							material.mostrarFilhos = true;

						}

					});
				});
			});
		});
	};

	// Limpar o item selecionado onde quer q ele esteja
	$scope.limparSelected = function(){

		// percorre os GRUPOS
		angular.forEach($scope.classificacaoMaterialList, function(grupo, key) {

			grupo.$selected = false;

			// percorre os SUB-GRUPOS
			angular.forEach(grupo.classificacaoMaterialFilhos, function(subGrupo, key) {

				subGrupo.$selected = false;

				// percorre os MATERIAIS
				angular.forEach(subGrupo.classificacaoMaterialFilhos, function(material, key) {

					material.$selected = false;

				});
			});

		});
	};

	$scope.removerCaracteresEspeciais = function(texto) {
		var textoNovo = texto;
		var regex = new RegExp("^[0-9a-zA-Z\b]+$");
		if (!regex.test(texto)) {
			textoNovo = texto.replace(/[!@#$%&*()_|+\-=?;:'"<>\{\}\[\]\\\/]/gi, '');
		}
		return textoNovo;
	};

	// BUSCA LISTA DE FederalSupply
	$scope.fetchResult(false);
}]);


