'use strict';

citApp.controller('ContaContabilController', ['$scope', 'ContaContabilRepository', 'DominioRepository', '$translate', '$timeout', 'MaterialRepository', 'ConfiguracaoParametroSistemaRepository', 'ContaContabilMovimentoRepository',
		function ContaContabilController($scope, ContaContabilRepository, DominioRepository, $translate, $timeout, MaterialRepository, ConfiguracaoParametroSistemaRepository, ContaContabilMovimentoRepository) {

	$scope.contaContabil = {
			configuracoesReferencia : [{
				mesReferencia : $scope.usuarioLogado.organizacao.dataReferenciaVigente,
				apresentaRMB : false,
				apresentaRelatorioInventario : false,
				apresentaRMA : false
			}]
	};

	$scope.dataReferencia = $scope.usuarioLogado.organizacao.dataReferenciaVigente;

	// Limpa formulário para novo cadastro
	$scope.resetForm = function() {
		$scope.limparContaContabil();
		$scope.edit = true;
		$scope.pgEdit = true;
		$scope.emUso = false;
		$timeout(function(){
			$scope.contaContabilForm.$submitted = false;
			$scope.contaContabilForm.$setPristine();
		});
	};

	// Atualiza pagina de pesquisa
	$scope.atualizaPaginaPesquisa = function () {
		angular.element('#searchContaContabil').scope().fetchResult();
	};

	// MODAL QUE CONFIRMA REMOVER DA CONTA CONTABIL
	$scope.remove = function(contaContabil){
		$scope.$openModalConfirm({
			message: $translate.instant('ADMINISTRACAODEMATERIAIS.MSG.CONFIRMA_EXCLUIR_CONTA_CONTABIL'),
			callback: function () {
				ContaContabilRepository.remove(contaContabil).then(function(result) {
					if (result.originalElement && result.originalElement.status != undefined && !result.originalElement.status && result.originalElement.mensagens) {
						$scope.setLoading(false);
						angular.forEach(result.originalElement.mensagens, function (mensagem) {
							$scope.showAlert(mensagem.tipoMensagem, $translate.instant(mensagem.mensagem));
		    	        });
					} else {
						$scope.contaContabil = {};

						$scope.$modalConfirmInstance.dismiss('cancel');
						$scope.showAlert("success", $translate.instant('ADMINISTRACAODEMATERIAIS.MSG.SUCESSO_CONTA_CONTABIL_EXCLUIDO'));
						angular.element('#searchContaContabil').scope().fetchResult();

						$scope.resetForm();
					}
				});
			}
		});
	};

	DominioRepository.findAllDominio('tipoContaContabil').then(function(result) {
		$scope.dominiosTipoContaContabil = result;
	});

	DominioRepository.findAllDominio('tipoMaterial').then(function(result) {
		$scope.dominiosTipoMaterial = result;
	});

	DominioRepository.findAllDominio('tipoCicloTaxaContabil').then(function(result) {
		$scope.dominiosTipoCicloTaxaContabil = result;
	});

	$scope.mudaTipoMaterial = function() {
		$timeout(function(){
			if ($scope.contaContabil.dominioTipoMaterial.codigo == 1) {
				$scope.contaContabil.dominioTipoContaContabil = null;
				$scope.contaContabil.taxaDepreciacaoMensal = null;
				$scope.contaContabil.percentualNaoDepreciavel = null;
				$scope.contaContabil.vidaUtil = null;

				angular.forEach($scope.contaContabil.configuracoesReferencia, function (conf) {
					conf.apresentaRMB = false;
					conf.apresentaRelatorioInventario = false;
		        });
			} else if ($scope.contaContabil.dominioTipoMaterial.codigo == 2) {
				angular.forEach($scope.contaContabil.configuracoesReferencia, function (conf) {
					conf.apresentaRMA = false;
		        });
			}
		}, 1000);
	};

	// SALVA a Conta Contabil
	$scope.saveOrUpdate = function(){
		$scope.contaContabilForm.$submitted = true;

		//verifica se o formulario está valido para salvar
		if($scope.contaContabilForm.$valid){

			$scope.setLoadingSalva(true);


			//Valida o tipo do material = PERMANENTE
			if($scope.contaContabil.dominioTipoMaterial.codigo == 2){

				//Faz o calculo da taxa de depreciacao mensal
				$scope.calcularTaxaDepreciacao();

			}

			ContaContabilRepository.save($scope.contaContabil).then(function(result) {
				if (result.originalElement.status != undefined && !result.originalElement.status && result.originalElement.mensagens) {
					$scope.setLoading(false);
					angular.forEach(result.originalElement.mensagens, function (mensagem) {
						$scope.showAlert(mensagem.tipoMensagem, $translate.instant(mensagem.mensagem));
	    	        });
				} else {
					$scope.contaContabil = result.originalElement;
					$scope.showAlert("success", $translate.instant('ADMINISTRACAODEMATERIAIS.MSG.SUCESSO_CONTA_CONTABIL'));
					$scope.contaContabilForm.$submitted = false;
					$scope.setLoading(false);
				}
			});

		}else{
			//Mensagem de erro de campos obrigatorios não preenchidos
			$scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
		}
	};

	// Limpa o formulario preenchido
	$scope.limparContaContabil = function(){
		$scope.contaContabil = {
			organizacao : $scope.usuarioLogado.organizacao,
			taxaDepreciacaoMensal : 0.0,
			configuracoesReferencia : [{
				mesReferencia : $scope.usuarioLogado.organizacao.dataReferenciaVigente,
				apresentaRMB : false,
				apresentaRelatorioInventario : false,
				apresentaRMA : false
			}]
		};
	};

	// Limpa o formulario preenchido
	$scope.validaReferencia = function(mesReferencia){
		var mesReferenciaData = converterStringUSemDate(mesReferencia);
		var referenciaVigente = $scope.usuarioLogado.organizacao.dataReferenciaVigente;
		if (referenciaVigente.getMonth() == mesReferenciaData.getMonth() && referenciaVigente.getYear() == mesReferenciaData.getYear()) {
			return false;
		}
		return true;
	};


	/**
	 * Funcao para calcular a taxa de depreciacao mensal com baso no calculo:
	 *  Taxa de Depreciação: taxaDepreciacao = (1/vidaUtil)
	 */
	$scope.calcularTaxaDepreciacao = function(){
		if($scope.contaContabil.vidaUtil && $scope.contaContabil.vidaUtil > 0){
			$scope.contaContabil.taxaDepreciacaoMensal = (1/$scope.contaContabil.vidaUtil);
		} else {
			$scope.contaContabil.taxaDepreciacaoMensal = 0.00000,0;
		}
	};

	// Consulta entidade e mostra no formulario
	$scope.getContaContabil = function(contaContabil, edit){
		$scope.setLoadingGet(true);
		$scope.emUso = false;
		var idConta = contaContabil.id;

		ContaContabilRepository.get(idConta).then(function(result) {
			$scope.contaContabil = result.originalElement;
			$scope.edit = edit;
			$scope.setLoading(false);
		});

		MaterialRepository.existeVinculo({"joinClass": 'contaContabil.id', "id": idConta}).then(function(result) {
			$timeout(function() {
				if (result) {
					$scope.emUso = true;
				}
			});
		});
		ConfiguracaoParametroSistemaRepository.getParametro("CONTA_CONTABIL_ALMOXARIFADO").then(function(result) {
			$timeout(function() {
				if (result.valor == idConta) {
					$scope.emUso = true;
				}
			});
		});
		ContaContabilMovimentoRepository.existeVinculo({"joinClass": 'contaContabil.id', "id": idConta}).then(function(result) {
			$timeout(function() {
				if (result) {
					$scope.emUso = true;
				}
			});
		});
		$scope.pgEdit = edit;
	};
}]);

citApp.filter('orderReferencia', [function(){ //Ordena a lista de integrantes em ordem alfabética
 return function(input) {
	    if (!angular.isObject(input)) return input;

	    var array = [];
	    for(var objectKey in input) {
	    	if (objectKey != "fill") {
	    		array.push(input[objectKey]);
	    	}
	    }

	    array.sort(function(a, b){
    	  var alc = a.mesReferencia,
    	      blc = b.mesReferencia;
    	  return alc > blc ? 1 : alc < blc ? -1 : 0;
    	});
	    return array;
	 };
}]);


