'use strict';

citApp.controller('ContaContabilListController', ['$scope', 'ContaContabilRepository', 'DominioRepository', '$translate',
		function ContaContabilListController($scope, ContaContabilRepository, DominioRepository, $translate) {
	$scope.$showAdvancedFilters = false;

	$scope.resetForm = function() {
		angular.element("#editContaContabil").scope().resetForm();
	};

	$scope.headers = [ {
		title : $translate.instant('LABEL.CODIGO'),
		value : 'codigo'
	}, {
		title : $translate.instant('LABEL.DESCRICAO'),
		value : 'descricao'
	}, {
		title : $translate.instant('ADMINISTRACAODEMATERIAIS.LABEL.TIPO_MATERIAL'),
		value : 'dominioTipoMaterial.descricao'
	}, {
		title : $translate.instant('LABEL.DATA_BLOQUEIO'),
		value : 'dataBloqueio',
		filter : 'dateBR'
	}];

	// default criteria that will be sent to the server
	$scope.filterCriteria = {
		start : 1,
		dir : 'asc',
		sort : 'id',
		limit : 10,
		fields: ['id','codigo','descricao', 'dominioTipoMaterial.descricao', 'dataBloqueio'],
		filters : [{type: 'string', field: 'codigo'},
		           {type: 'string', field: 'descricao'}, 
		           {type: 'string', field: 'dominioTipoMaterial.descricao', listaDominio : []},
		           {type: 'date', field: 'dataBloqueio'}]
	};

	DominioRepository.findAllDominio('tipoMaterial').then(function(result) {
		$scope.filterCriteria.filters[2].listaDominio = result;
	});

	// ABRI CONTA CONTABIL SELECIONADA
	$scope.abrirContaContabil = function(edit){
		var contaContabil = $scope.contaContabilChecked;

		if(!edit && !contaContabil) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_VISUALIZACAO'));
			return;
		}

		if(edit && !contaContabil) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO'));
			return;
		}

		angular.element('#editContaContabil').scope().getContaContabil(contaContabil, edit);
		$scope.$showPageEditWorkspace($scope.workspace);
	};
	
	// MODAL DE CONFIRMAÇÃO E REMOÇÃO DO REGISTRO
	$scope.remover = function(){

		var contaContabil = $scope.contaContabilChecked;

		if(contaContabil) {

			$scope.$openModalConfirm({
				message: $translate.instant("MSG.CONFIRMA_EXCLUSAO"),
				callback: function () {
					$scope.setLoadingRemove(true);

					ContaContabilRepository.remove(contaContabil).then(function(result) {
						if (result.originalElement && result.originalElement.status != undefined && !result.originalElement.status && result.originalElement.mensagens) {
							$scope.setLoading(false);
							angular.forEach(result.originalElement.mensagens, function (mensagem) {
								$scope.showAlert(mensagem.tipoMensagem, $translate.instant(mensagem.mensagem));
			    	        });
						} else {
							$scope.$modalConfirmInstance.dismiss('cancel');
							$scope.showAlert('success', $translate.instant('MSG.REGISTRO_EXCLUIDO'));
							$scope.fetchResult();
							$scope.setLoading(false);
						}
					});

				}
			});

		} else {

			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_SER_REMOVIDO'));
		}

	};
	
}]);


