'use strict';

citApp.controller('MaterialListController', ['$scope', '$timeout', 'MaterialRepository', 'ClassificacaoMaterialRepository', 'DominioRepository', '$translate', 
                                             function MaterialListController($scope, $timeout, MaterialRepository, ClassificacaoMaterialRepository, DominioRepository, $translate) {
	
	$scope.$showAdvancedFilters = false;

	$scope.buscaFederalSupply = "";
	$scope.mostraArvore = false;

	$scope.resetForm = function() {
		angular.element("#editMaterial").scope().resetForm();
	};

	$scope.headers = [ {
		title : $translate.instant('LABEL.CODIGO'),
		value : 'codigo'
	}, {
		title : $translate.instant('LABEL.DESCRICAO'),
		value : 'descricao'
	}, {
		title : $translate.instant('ADMINISTRACAODEMATERIAIS.LABEL.TIPO_MATERIAL'),
		value : 'dominioTipoMaterial.descricao'
	}];

	// default criteria that will be sent to the server
	$scope.filterCriteria = {
			start : 1,
			dir : 'asc',
			sort : 'codigo',
			limit : 10,
			fields: ['id','codigo', 'descricao', 'dominioTipoMaterial.descricao'],
			filters : [{type: 'string', field: 'codigo'},
			           {type: 'string', field: 'descricao'},
			           {type: 'string', field: 'dominioTipoMaterial.descricao', listaDominio : []}]
	};

	DominioRepository.findAllDominio('tipoMaterial').then(function(result) {
		$scope.filterCriteria.filters[2].listaDominio = result;
	});

	$scope.fetchResultMaterial = function() {
		delete $scope.materialChecked;
		if ($scope.mostraArvore) {
			inicializarArvore();
		} else {
			$scope.fetchResult();
		}
	};

	$scope.abrirMaterial = function(edit, material) {
		if (!material) {
			material = $scope.materialChecked;
		}

		if(!edit && !material) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_VISUALIZACAO'));
			return;
		}
		
		if(edit && !material) {
			$scope.showAlert('warning', $translate.instant('MSG.SELECIONE_UM_ITEM_PARA_EDICAO'));
			return;
		}		

		angular.element('#editMaterial').scope().getMaterial(material, edit);
		$scope.$showPageEditWorkspace($scope.workspace);
	};
	
	//Método responsável por realizar a busca da classificação material na arvore
	$scope.buscarTree = function(buscaFederalDescricao){

		$scope.buscaFederalSupply = buscaFederalDescricao;
		if (buscaFederalDescricao.length > 2) {

			$scope.setLoadingPesquisa(true);
			return ClassificacaoMaterialRepository.listarGruposTree(buscaFederalDescricao).then(function(result) {

				$scope.classificacaoMaterialList = result;
				$scope.abrirNosArvore();
				$scope.setLoading(false);
			}, function() {
				$scope.classificacaoMaterialList = [];
			});
		} else {
			$scope.fetchResult();
			$scope.isApresentarDadosMaterial =  false;
			$scope.resetForm();
		};
	};

	$scope.abrirNosArvore = function(){

		// percorre os nos CLASSIFICADORES - PERMANENTE OU CONSUMO
		angular.forEach($scope.classificacaoMaterialList, function(classificacao, key) {
			if(classificacao.classificacaoMaterialFilhos){
				classificacao.mostrarFilhos = true;
			} else {
				classificacao.mostrarFilhos = false;
			}

			// percorre os nos RAIZ - GRUPOS
			angular.forEach(classificacao.classificacaoMaterialFilhos, function(grupo, key) {
				if(grupo.classificacaoMaterialFilhos){
					grupo.mostrarFilhos = false;
				} else {
					grupo.mostrarFilhos = true;
				}

				// percorre os SUB-GRUPOS
				angular.forEach(grupo.classificacaoMaterialFilhos, function(subGrupo, key) {
					if(subGrupo.classificacaoMaterialFilhos){
						subGrupo.mostrarFilhos = false;
						grupo.mostrarFilhos = true;
					} else {
						subGrupo.mostrarFilhos = true;
					}

					// percorre os MATERIAIS
					angular.forEach(subGrupo.classificacaoMaterialFilhos, function(material, key) {
						if(material.classificacaoMaterialFilhos){
							material.mostrarFilhos = false;
							subGrupo.mostrarFilhos = true;
						} else {
							material.mostrarFilhos = true;
						}
					});
				});
			});
		});
	};

	$scope.novoFilho = function(value){

		$scope.openWorkspaceIfNotOpen($translate.instant('ADMINISTRACAODEMATERIAIS.LABEL.CLASSIFICACAO_MATERIAL'), '/cit-adm-materiais-web/html/classificacaoMaterial/classificacaoMaterial.html', 'mod-green');
		$timeout(function() {
			angular.element('#classificacaoMaterialControllerId').scope().novoFilhoMaterial(value.originalElement ? value.originalElement : value);
		}, 200);
	};

	$scope.mostrarFilhos = function(classificacaoMaterialParent) {

		if (classificacaoMaterialParent.mostrarFilhos) {
			classificacaoMaterialParent.mostrarFilhos = false;
		} else {
			$scope.setLoadingPesquisa(true);
			ClassificacaoMaterialRepository.listarFilhos(
					classificacaoMaterialParent.originalElement ? classificacaoMaterialParent.originalElement.id : classificacaoMaterialParent.id).then(function(result) {

						classificacaoMaterialParent.classificacaoMaterialFilhos = result;
						classificacaoMaterialParent.mostrarFilhos = true;

						$scope.setLoading(false);
					});
		};
	};

	$scope.fetchResult = function(page) {

		$scope.setLoadingPesquisa(true);
		return ClassificacaoMaterialRepository.listarGrupos().then(function(result) {
			$scope.classificacaoMaterialList = result;
			$scope.setLoading(false);
		}, function() {
			$scope.classificacaoMaterialList = [];
		});
	};

	$scope.mostrarArvore = function() {
		$scope.mostraArvore = !$scope.mostraArvore;
		delete $scope.materialChecked;

		if ($scope.mostraArvore) {
			inicializarArvore();
		} else {
			$scope.fetchResultMaterial();
		}
	};

	var inicializarArvore = function() {
		$scope.classificacaoMaterialList = [];

		// BUSCA TODAS AS CLASSIFICACAO MATERIAL NIVEL 0 (ZERO) PELO ÓRGÃO DO USUÁRIO
		ClassificacaoMaterialRepository.listarGrupos().then(function(result) {
			for(var i = 0; i < result.length; i++) {
				result[i].originalElement.sinalPositivo = result[i].originalElement.possuiFilho;
				$scope.classificacaoMaterialList.push(result[i].originalElement);
			}
		});
	};

	$scope.exibirMaterialArvore = function(item) {

		if (item.dominioTipoClassificacaoMaterial.codigo == 4) {
			angular.element('#editMaterial').scope().apresentarDadosMaterial(item);
			$scope.$showPageEditWorkspace($scope.workspace);
		};
	};

}]);