'use strict';

citApp.controller('TransferenciaContaContabilController', ['$scope', 'TransferenciaContaContabilRepository', '$filter', '$translate', '$timeout', 'MaterialRepository', 'ContaContabilRepository',
                                                           'DominioRepository', 'BemPatrimonialRepository', 'MaterialConsumoRepository',
		function TransferenciaContaContabilController($scope, TransferenciaContaContabilRepository, $filter, $translate, $timeout, MaterialRepository, ContaContabilRepository, DominioRepository,
				BemPatrimonialRepository, MaterialConsumoRepository) {

	// CONSTANTES DOMINIO
	var TIPO_MATERIAL = 'tipoMaterial';
	var CODIGO_MATERIAL_PERMANENTE = 2;

    $scope.transferenciaContaContabil = {
    		dataTransferencia : $filter('date')(new Date(), "dd/MM/yyyy")
    };

    // Limpa formulário para novo cadastro
    $scope.resetForm = function() {
        $scope.limparTransferenciaContaContabil();
        $scope.edit = true;
        $scope.pgEdit = true;

        $scope.limparCamposMaterial();

        $timeout(function(){
            $scope.transferenciaContaContabilForm.$submitted = false;
            $scope.transferenciaContaContabilForm.$setPristine();
        });
    };

    // Atualiza pagina de pesquisa
    $scope.atualizaPaginaPesquisa = function () {
        angular.element('#searchTransferenciaContaContabil').scope().fetchResult();
    };

    // MODAL QUE CONFIRMA REMOVER DA CIDADE
    $scope.remove = function(transferenciaContaContabil){
        $scope.transferenciaContaContabil = transferenciaContaContabil;
        $scope.$openModalConfirm({
            message: $translate.instant('MSG.CONFIRMA_EXCLUSAO'),
            callback: function () {
                TransferenciaContaContabilRepository.remove($scope.transferenciaContaContabil).then(function() {

                    $scope.$modalConfirmInstance.dismiss('cancel');
                    $scope.showAlert("success", $translate.instant('MSG.REGISTRO_EXCLUIDO'));
                    angular.element('#searchTransferenciaContaContabil').scope().fetchResult();

                    $scope.resetForm();
                });
            }
        });
    };

    // SALVA O TransferenciaContaContabil
    $scope.saveOrUpdate = function(){
        $scope.transferenciaContaContabilForm.$submitted = true;

        //verifica se o formulario está valido para salvar
        if($scope.transferenciaContaContabilForm.$valid){

            $scope.setLoadingSalva(true);

            if (!$scope.validarDataTransferencia($scope.transferenciaContaContabil.dataTransferencia)) {
				$scope.showAlert('error', $translate.instant('ADMINISTRACAODEMATERIAIS.VALIDACAO.DATA_TRANSFERENCIA_FORA_REFERENCIA'));
				$scope.setLoading(false);
				return ;
			}

            TransferenciaContaContabilRepository.realizarTransferencia($scope.transferenciaContaContabil).then(function(result) {
            	if (result.originalElement && result.originalElement.status != undefined && !result.originalElement.status && result.originalElement.mensagens) {
					$scope.setLoading(false);
					angular.forEach(result.originalElement.mensagens, function (mensagem) {
						$scope.showAlert(mensagem.tipoMensagem, $translate.instant(mensagem.mensagem));
	    	        });
				} else {
					if (result) {
	            		$scope.showAlert("success", $translate.instant('ADMINISTRACAODEMATERIAIS.MSG.TRANSFERENCIA_SUCESS'));
	                    $scope.transferenciaContaContabilForm.$submitted = false;
	                    angular.element('#searchTransferenciaContaContabil').scope().fetchResult();

	                    $scope.resetForm();
	            	} else {
	            		$scope.showAlert('error', $translate.instant('ADMINISTRACAODEMATERIAIS.MSG.TRANSFERENCIA_FAIL'), " ", false);
	            	}
					$scope.setLoading(false);
				}
            });
        }else{
            //Mensagem de erro de campos obrigatorios não preenchidos
            $scope.showAlert('error', $translate.instant('VALIDACAO.ALERTA_OBRIGATORIOS'), " ", false);
        }
    };

    // Limpa o formulario preenchido
    $scope.limparTransferenciaContaContabil = function(){
        $scope.transferenciaContaContabil = {
        		dataTransferencia : $filter('date')(new Date(), "dd/MM/yyyy")
        };
    };

    // Consulta entidade e mostra no formulario
    $scope.getTransferenciaContaContabil = function(transferenciaContaContabil, edit){
        $scope.setLoadingGet(true);

        TransferenciaContaContabilRepository.get(transferenciaContaContabil.id).then(function(result) {
            $scope.transferenciaContaContabil = result.originalElement;
            $scope.edit = edit;
            $scope.setLoading(false);
        });

        $scope.pgEdit = edit;
    };

    $scope.findAutoCompleteMaterial = function(value){
        return MaterialRepository.findAutoComplete('descricao', value).then(function(result) {
            return result;
        });
    };

    $scope.findMateriaisByOrganizacao = function(value){
    	return MaterialRepository.findMateriaisByOrganizacao(value, $scope.usuarioLogado.organizacao.id).then(function(result) {
    		return result;
    	});
    };

    $scope.getMaterialTransferencia = function(material) {
    	MaterialRepository.getMaterialTransferencia(material.id).then(function(result) {
    		$scope.transferenciaContaContabil.material = result.originalElement;

    		if ($scope.transferenciaContaContabil.material.dominioTipoMaterial.codigo == 1) {
    			MaterialConsumoRepository.getValorAtualMaterial($scope.transferenciaContaContabil.material.id, $scope.usuarioLogado.organizacao.id).then(function(result) {
    				if(!result){
        				$scope.transferenciaContaContabil.valorTransferencia = 0.00;
        			} else {
        				$scope.transferenciaContaContabil.valorTransferencia = result;
        			}
    			});
    			
    			MaterialConsumoRepository.getQtdAtualMaterial($scope.transferenciaContaContabil.material.id, $scope.usuarioLogado.organizacao.id).then(function(result) {
    				if(!result){
        				$scope.transferenciaContaContabil.quantidadeMaterial = 0;
        			} else {
        				$scope.transferenciaContaContabil.quantidadeMaterial = result;
        			}
    			});
    			
    		} else if ($scope.transferenciaContaContabil.material.dominioTipoMaterial.codigo == 2) {
    			BemPatrimonialRepository.getValorAtualMaterial($scope.transferenciaContaContabil.material.id, $scope.usuarioLogado.organizacao.id).then(function(result) {
        			if(!result){
        				$scope.transferenciaContaContabil.valorTransferencia = 0.00;
        			} else {
        				$scope.transferenciaContaContabil.valorTransferencia = result;
        			}
                });

        		BemPatrimonialRepository.getQtdAtualMaterial($scope.transferenciaContaContabil.material.id, $scope.usuarioLogado.organizacao.id).then(function(result) {
        			if(!result){
        				$scope.transferenciaContaContabil.quantidadeMaterial = 0;
        			} else {
        				$scope.transferenciaContaContabil.quantidadeMaterial = result;
        			}
                });

        		BemPatrimonialRepository.getValorDepreciacaoAcumulada($scope.transferenciaContaContabil.material.id, $scope.usuarioLogado.organizacao.id).then(function(result) {
        			if(!result){
        				$scope.transferenciaContaContabil.valorDepreciacaoAcumulada = 0.00;
        			} else {
        				$scope.transferenciaContaContabil.valorDepreciacaoAcumulada = result;
        			}
                });
    		}

        });
    };

    DominioRepository.findAllDominioByCodigo(TIPO_MATERIAL, CODIGO_MATERIAL_PERMANENTE).then(function(result) {
		$scope.dominioTipoMaterialPermanente = result.originalElement;
	});

    $scope.findAutoCompleteContaDestino = function(value){
    	var resultado = [];
    	if ($scope.transferenciaContaContabil.almoxarifado) {
    		return ContaContabilRepository.findContaContabilPorOrganizacaoEDominioSemUsoMaterial(value, $scope.usuarioLogado.organizacao.id, $scope.dominioTipoMaterialPermanente.id).then(function(result) {
    			angular.forEach(result, function (item) {
    				if (item.id != $scope.transferenciaContaContabil.contaOrigem.id) {
    					resultado.push(item);
    				}
    	        });
				return resultado;
		   });
    	} else {
    		if (!$scope.transferenciaContaContabil.material) {
    			$scope.showAlert('warning', $translate.instant('ADMINISTRACAODEMATERIAIS.MSG.SELECIONE_MATERIAL_PRIMEIRO'));
    			return resultado;
    		} else {
    			return ContaContabilRepository.findContaContabilPorOrganizacaoEDominioSemUsoAlmoxarifado(value, $scope.usuarioLogado.organizacao.id,
        				$scope.transferenciaContaContabil.material.dominioTipoMaterial.id).then(function(result) {
        			angular.forEach(result, function (item) {
        				if (item.id != $scope.transferenciaContaContabil.contaOrigem.id) {
        					resultado.push(item);
        				}
        	        });
                    return resultado;
                });
    		}
    	}
    };

	// Watch para limpar autocompletes
	$scope.$watch('transferenciaContaContabil.almoxarifado', function() {
		if (!$scope.transferenciaContaContabil.id) {
			$scope.transferenciaContaContabil.material = null;
			$scope.transferenciaContaContabil.contaDestino = null;
			$scope.limparCamposMaterial();
			
			if($scope.transferenciaContaContabil.almoxarifado) {

				ContaContabilRepository.obterContaContabilAlmoxarifadoConfiguracaoParametroSistema().then(function(result) {
					$scope.transferenciaContaContabil.contaOrigem = result.originalElement;

					//Atualiza valor total da conta
					$scope.calcularValorTransferencia(result.originalElement);

		        });

			} else {
				$scope.transferenciaContaContabil.contaOrigem = null;
			}
		}
	});

	// Watch para atualizar conta de origem do material selecionado
	$scope.$watch('transferenciaContaContabil.material', function() {

		if (!$scope.transferenciaContaContabil.id) {
			if($scope.transferenciaContaContabil.material) {
				$scope.transferenciaContaContabil.contaOrigem = $scope.transferenciaContaContabil.material.contaContabil;
			} else {
				$scope.transferenciaContaContabil.contaOrigem = null;
			}
		}

	 });

	$scope.limparCamposMaterial = function(){

		$scope.transferenciaContaContabil.valorTransferencia = 0;
		$scope.transferenciaContaContabil.quantidadeMaterial = 0;
		$scope.transferenciaContaContabil.valorDepreciacaoAcumulada = 0;

    };


    $scope.calcularValorTransferencia = function(contaOrigem){

    	$scope.transferenciaContaContabil.valorTransferencia = 0;

    	if($scope.transferenciaContaContabil.almoxarifado && contaOrigem && contaOrigem.id){

    		ContaContabilRepository.getValorTotal(contaOrigem.id).then(function(result) {
    			$scope.transferenciaContaContabil.valorTransferencia = result ? result : 0;
	        });

    	}
    };

}]);